#include <pygame/pygame.h>
#include <structmember.h>
#include "common.h"
SDL_Surface *g_screen;
#if SDL_BYTEORDER==SDL_BIG_ENDIAN
int COLOR_WHITE = 0xffffff00;
#else
int COLOR_WHITE=0xffffff;
#endif
PyObject *gmapi;
PyObject *gm6;
PyObject *gm6_Room_AddDrawnInstance;
PyObject *gm6_Room_AddInstance;
PyObject *gm6_Room_DrawBackgrounds;
PyObject *gm6_Room_DrawView;
PyObject *gm6_Room_HandleCollisions;
PyObject *gm6_Room_HandleKeypress;
PyObject *gm6_Room_HandleKeyrelease;
PyObject *gm6_Room_Leave;
PyObject *gm6_Room_MouseButtonDown;
PyObject *gm6_Room_MouseButtonUp;
PyObject *gm6_Room_MoveInstances;
PyObject *gm6_Room_RenderBackground;
PyObject *gm6_Room_ResortDrawnObjects;
PyObject *gm6_Room_SetBackgroundIndex;
PyObject *gm6_Room_StepAlarms;
PyObject *gm6_Room_StepEvents;

int seqi(PyObject *seq, int idx)
{
  return PyInt_AsLong(PySequence_Fast_GET_ITEM(seq,idx));
}

PyObject *load_membertuple(PyObject *self, PyObject *args)
{
  PyObject *state = PyTuple_GET_ITEM(args,0);
  int statelen = PyTuple_Size(state);
  int x = 0;
  while (x<statelen) {
    PyObject *item = PyTuple_GET_ITEM(state,x);
    PyObject_SetAttrString(self,PyString_AsString(PyTuple_GET_ITEM(item,0)),PyTuple_GET_ITEM(item,1));
    x++;
  }
  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *membertuple(PyObject *self, PyObject *args)
{
  PyMemberDef *memb = &self->ob_type->tp_members[0];
  int x = 0;
  while (memb->name) {
    x++;
    memb = &self->ob_type->tp_members[x];
  }
  PyObject *l = PyTuple_New(x);
  int y = 0;
  while (y<x) {
     memb = &self->ob_type->tp_members[y];
     PyTuple_SetItem(l,y,Py_BuildValue("(sO)",memb->name,PyObject_GetAttrString(self,memb->name)));
     y++;
  }
  PyObject *ret = Py_BuildValue("O()O",&GMRoom_Type, l);
  Py_DECREF(l);
  return ret;
}

PyObject *Truth(int cond)
{
  if (cond) {
    Py_INCREF(Py_True);
    return Py_True;
  }
  Py_INCREF(Py_False);
  return Py_False;
}

PyObject *Object_AsInt_richcompare(PyObject *a, PyObject *b, int cond)
{
  PyObject *aa = PyNumber_Int(a);
  if (!aa) return NULL;
  PyObject *bb = PyNumber_Int(b);
  if (!bb) {
    Py_DECREF(aa);
    return NULL;
  }
  if (aa->ob_type->tp_richcompare) {
    PyObject *ret = aa->ob_type->tp_richcompare(a,b,cond);
    Py_DECREF(aa);
    Py_DECREF(bb);
    return ret;
  }
  PyObject *ret = NULL;
  switch (cond) {
    case Py_LT:
      ret = Truth(PyInt_AsLong(aa)<PyInt_AsLong(bb));
      break;
    case Py_LE:
      ret = Truth(PyInt_AsLong(aa)<=PyInt_AsLong(bb));
      break;
    case Py_EQ:
      ret = Truth(PyInt_AsLong(aa)==PyInt_AsLong(bb));
      break;
    case Py_NE:
      ret = Truth(PyInt_AsLong(aa)!=PyInt_AsLong(bb));
      break;
    case Py_GT:
      ret = Truth(PyInt_AsLong(aa)>PyInt_AsLong(bb));
      break;
    case Py_GE:
      ret = Truth(PyInt_AsLong(aa)>=PyInt_AsLong(bb));
      break;
  }
  Py_DECREF(aa);
  Py_DECREF(bb);
  return ret;
}

int Object_AsInt_compare(PyObject *a, PyObject *b)
{
  // XXX GMVar_compare needs to decref aa and bb!!!
  PyObject *aa = PyNumber_Int(a);
  if (!aa) {
    fprintf(stderr,"XXX need to set an exception.\n");
    return -1;
  }
  PyObject *bb = PyNumber_Int(b);
  if (!bb) {
    fprintf(stderr,"XXX need to set an exception.\n");
    Py_DECREF(aa);
    return -1;
  }
  if (aa->ob_type->tp_compare) {
    int ret = aa->ob_type->tp_compare(aa,bb);
    Py_DECREF(aa);
    Py_DECREF(bb);
    return ret;
  }
  PyObject *eq = aa->ob_type->tp_richcompare(aa,bb,Py_EQ);
  if (eq) {
    eq = aa->ob_type->tp_richcompare(aa,bb,Py_LT);
    int truth = PyObject_IsTrue(eq);
    Py_DECREF(eq);
    if (truth) {
      Py_DECREF(aa);
      Py_DECREF(bb);
      return -1;
    }
    eq = aa->ob_type->tp_richcompare(aa,bb,Py_GT);
    truth = PyObject_IsTrue(eq);
    Py_DECREF(eq);
    if (truth) {
      Py_DECREF(aa);
      Py_DECREF(bb);
      return 1;
    }
  }
  Py_DECREF(aa);
  Py_DECREF(bb);
  return 0;
}

static int Floatlist8_compare(PyObject *a, PyObject *b)
{
  PyObject *ao = PyNumber_Float(a);
  double aa = PyFloat_AsDouble(ao);
  Py_DECREF(ao);
  PyObject *bo = PyNumber_Float(b);
  double bb = PyFloat_AsDouble(bo);
  Py_DECREF(bo);
  if (aa<bb) return -1;
  if (aa>bb) return 1;
  return 0;
}

static PyObject *Floatlist8_int(PyObject *o)
{
  return PyInt_FromLong(((struct Floatlist8 *)o)->f[0]);
}

static PyObject *Floatlist8_float(PyObject *o)
{
  return PyFloat_FromDouble(((struct Floatlist8 *)o)->f[0]);
}

static PyObject *Floatlist8_str(PyObject *o)
{
  struct Floatlist8 *fl = (struct Floatlist8 *)o;
  if (fl->f[0]==floor(fl->f[0])) {
    int i = fl->f[0];
    return PyString_FromFormat("%d",i);
  }
  return PyString_FromFormat("%f",fl->f[0]);
}

#define fl8_mathop_head \
  struct Floatlist8 * fl; \
  PyObject *ff; \
  if (o->ob_type==&Floatlist8_Type) { \
    fl = (struct Floatlist8 *)o; \
    ff = PyNumber_Float(f); \
  } else { \
    fl = (struct Floatlist8 *)f; \
    ff = PyNumber_Float(o); \
  } \
  struct Floatlist8 *ret = (struct Floatlist8 *)PyType_GenericNew(fl->ob_type,NULL,NULL); \
  ret->f[1] = fl->f[1]; \
  ret->f[2] = fl->f[2]; \
  ret->f[3] = fl->f[3]; \
  ret->f[4] = fl->f[4]; \
  ret->f[5] = fl->f[5]; \
  ret->f[6] = fl->f[6]; \
  ret->f[7] = fl->f[7]; 

static PyObject *Floatlist8_add(PyObject *o, PyObject *f)
{
  fl8_mathop_head
  ret->f[0] = fl->f[0] + PyFloat_AsDouble(ff);
  Py_DECREF(ff);
  return (PyObject *)ret;
}

static PyObject *Floatlist8_sub(PyObject *o, PyObject *f)
{
  /*
  struct Floatlist8 * fl = (struct Floatlist8 *)o;
  struct Floatlist8 *ret = (struct Floatlist8 *)PyType_GenericNew(o->ob_type,NULL,NULL);
  PyObject *ff = PyNumber_Float(f);
  ret->f[1] = fl->f[1];
  ret->f[2] = fl->f[2];
  ret->f[3] = fl->f[3];
  ret->f[4] = fl->f[4];
  ret->f[5] = fl->f[5];
  ret->f[6] = fl->f[6];
  ret->f[7] = fl->f[7];
  */
  fl8_mathop_head
  ret->f[0] = fl->f[0] - PyFloat_AsDouble(ff);
  Py_DECREF(ff);
  return (PyObject *)ret;
}

static PyObject *Floatlist8_mul(PyObject *o, PyObject *f)
{
  fl8_mathop_head
  ret->f[0] = fl->f[0]*PyFloat_AsDouble(ff);
  Py_DECREF(ff);
  return (PyObject *)ret;
}

static PyObject *Floatlist8_div(PyObject *o, PyObject *f)
{
  fl8_mathop_head
  ret->f[0] = fl->f[0]/PyFloat_AsDouble(ff);
  Py_DECREF(ff);
  return (PyObject *)ret;
}

static PyObject *Floatlist8_isub(PyObject *o, PyObject *f)
{
  struct Floatlist8 *fl = (struct Floatlist8 *)o;
  PyObject *s = PyNumber_Float(f);
  fl->f[0]-=PyFloat_AsDouble(s);
  Py_DECREF(s);
  Py_INCREF(o);
  return o;
}

static PyNumberMethods Floatlist8_numbermethods = {
        Floatlist8_add, Floatlist8_sub, Floatlist8_mul, Floatlist8_div, // + - * /
        0, 0, // remainder divmod
        0, 0, 0, 0, // pow neg pos abs
        0, 0, 0, 0, // nz inv << >>
        0, 0, 0, // & ^ |
        0, // coerce
        Floatlist8_int,
        0, // long
        Floatlist8_float, // 0
        0, 0, // oct hex
        0, // inplace add
        Floatlist8_isub,
        };

static PyObject *Beancounter_int(PyObject *o)
{
  return PyInt_FromLong(((struct Beancounter *)o)->count);
}

static PyNumberMethods Beancounter_numbermethods = {
        0, 0, 0, 0, /* + - * / */
        0, 0, /* remainder, divmod */
        0, 0, 0, 0, /* pow neg pos abs */
        0, 0, 0, 0, /* nonzero, invert, << >> */
        0, 0, 0, /* & ^ | */
        0, /* coerce */
        Beancounter_int,
        0, /* long */
        0, /* float */
        0, 0, /* oct hex */
        0, 0, /* +=, -= */
};

static Py_ssize_t Floatlist8_length(PyObject *o)
{
  return 8;
}

static PyObject *Floatlist8_item(PyObject *o, Py_ssize_t idx)
{
  if (idx<0 || idx>=8) return NULL;
  return PyFloat_FromDouble(((struct Floatlist8 *)o)->f[idx]);
}

static int Floatlist8_ass_item(PyObject *o, Py_ssize_t idx, PyObject *v)
{
  if (idx<0 || idx>=8) return -1;
  PyObject *tmp = PyNumber_Float(v);
  if (tmp) {
    double vd = PyFloat_AsDouble(tmp);
    Py_DECREF(tmp);
    ((struct Floatlist8 *)o)->f[idx] = vd;
    return 0;
  }
  return -1;
}

static PyObject *Floatlist8_reduce(PyObject *o, PyObject *args)
{
  struct Floatlist8 *fl = (struct Floatlist8 *)o;
  return Py_BuildValue("O()(ffffffff)", &Floatlist8_Type, fl->f[0], fl->f[1], fl->f[2], fl->f[3], fl->f[4], fl->f[5], fl->f[6], fl->f[7]);
};

static PyObject *Floatlist8_setstate(PyObject *o, PyObject *args)
{
  PyObject *state = PyTuple_GET_ITEM(args,0);
  struct Floatlist8 *fl = (struct Floatlist8 *)o;
  fl->f[0] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,0));
  fl->f[1] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,1));
  fl->f[2] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,2));
  fl->f[3] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,3));
  fl->f[4] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,4));
  fl->f[5] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,5));
  fl->f[6] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,6));
  fl->f[7] = PyFloat_AsDouble(PyTuple_GET_ITEM(state,7));
  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef Floatlist8_methods[] = {
        {"__reduce__", Floatlist8_reduce, METH_VARARGS, NULL,},
        {"__setstate__",Floatlist8_setstate, METH_VARARGS, NULL,},
        {NULL},
};

static PySequenceMethods Floatlist8_sequencemethods = {
        Floatlist8_length,
        0, // concat
        0, // repeat
        Floatlist8_item,
        0, //slice
        Floatlist8_ass_item,
        };

PyTypeObject Beancounter_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /* ob_size */
        "fasttint.Beancounter",
        sizeof(struct Beancounter),
        0, /* tp_itemsize */
        0,0,0,
        0, /*setattr */
        0, /* compare */
        0,
        &Beancounter_numbermethods,
        0,0,0,0,
        0, /* str */
        0,0,0,
        Py_TPFLAGS_DEFAULT,
        "A counter.",
        0,
        0,
        0,
        0,
        0,
        0,
        0, /* tp_methods */
        0,
        0,
        0,
        0,
        0, /* descr_get */
        0, /* descr_set */
        0, /* dictoffset */
        0, /* init */
        0, /* alloc */
        PyType_GenericNew, /* tp_new */
};

PyTypeObject Floatlist8_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.Floatlist8",    /*tp_name*/
        sizeof(struct Floatlist8), /*tp_basicsize*/
        0,      /* tp_itemsize */
        0,
        0,
        0,
        0, /* setattr */
        Floatlist8_compare,
        0,
        &Floatlist8_numbermethods,
        &Floatlist8_sequencemethods,
        0,
        0,
        0,
        Floatlist8_str, /* str */
        0,
        0,
        0,
        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES,
        "A list of 8 floats.",
        0,                         /* tp_traverse */
        0,                         /* tp_clear */
        0,                         /* tp_richcompare */
        0,                         /* tp_weaklistoffset */
        0,                         /* tp_iter */
        0,                         /* tp_iternext */
        Floatlist8_methods,             /* tp_methods */
        0,             /* tp_members */
        0,                         /* tp_getset */
        0,                         /* tp_base */
        0,                         /* tp_dict */
        0,                         /* tp_descr_get */
        0,                         /* tp_descr_set */
        0,                         /* tp_dictoffset */
        0,      /* tp_init */
        0,                         /* tp_alloc */
        PyType_GenericNew,                 /* tp_new */
}; // XXX what does this initialize to?

inline PyObject *Game_Sequence_Index(PyObject *self, char *seqname)
{
  if (!gm6) return NULL;
  int x = 0;
  PyObject *game = PyObject_GetAttrString(gm6,"game");
  PyObject *seq = PyObject_GetAttrString(game,seqname);
  Py_DECREF(game);
  if (!seqname) {
    return NULL;
  }
  int xmax = PyList_Size(seq);
  while (x<xmax) {
    if (PySequence_Fast_GET_ITEM(seq,x)==self) {
      PyObject *ret = PyInt_FromLong(x);
      Py_DECREF(seq);
      return ret;
    }
    x ++;
  }
  Py_DECREF(seq);
  return NULL;
}

inline PyObject *Game_Sequence_Index_F(PyObject *self, char *seqname)
{
  int x = 0;
  PyObject *game = PyObject_GetAttrString(gm6,"game");
  PyObject *seq = PyObject_GetAttrString(game,seqname);
  Py_DECREF(game);
  if (!seqname) {
    return NULL;
  }
  int xmax = PyList_Size(seq);
  while (x<xmax) {
    if (PySequence_Fast_GET_ITEM(seq,x)==self) {
      PyObject *ret = PyFloat_FromDouble(x);
      Py_DECREF(seq);
      return ret;
    }
    x ++;
  }
  Py_DECREF(seq);
  return NULL;
}

inline PyObject *Game_Sequence_Index_S(PyObject *self, char *seqname)
{
  int x = 0;
  PyObject *game = PyObject_GetAttrString(gm6,"game");
  PyObject *seq = PyObject_GetAttrString(game,seqname);
  Py_DECREF(game);
  if (!seqname) {
    return NULL;
  }
  int xmax = PyList_Size(seq);
  while (x<xmax) {
    if (PySequence_Fast_GET_ITEM(seq,x)==self) {
      PyObject *ret = PyString_FromFormat("%d",x);
      Py_DECREF(seq);
      return ret;
    }
    x ++;
  }
  Py_DECREF(seq);
  return PyString_FromString("-1");
}

inline PyObject *invoke_4(PyObject *func, PyObject *self, PyObject *arg2, PyObject *arg3, PyObject *arg4)
{
  PyObject *val = Py_BuildValue("(OOOO)",self,arg2,arg3,arg4);
  PyObject *ret = PyObject_CallObject(func, val);
  Py_DECREF(val);
  return ret;
}

inline void PushSelf(PyObject *o)
{
  PyObject *self = PyObject_GetAttrString(gmapi,"self");
  PyList_Append(gmapi_py_self_stack,self);
  Py_DECREF(self);
  PyObject_SetAttrString(gmapi,"self",o);
}

inline void PopSelf(void)
{
  int pos = PyList_Size(gmapi_py_self_stack)-1;
  PyObject *self = PyList_GetItem(gmapi_py_self_stack,pos);
  if (self) {
    PyObject_SetAttrString(gmapi,"self",self);
    PySequence_DelItem(gmapi_py_self_stack,pos);
  } else {
    fprintf(stderr,"Error in PyList_GetItem %d\n",pos);
  }
}

inline int color_as_int(PyObject *o)
{
  if (PySequence_Check(o)) {
    if (PySequence_Size(o)<3) return COLOR_WHITE;
    PyObject *r, *g, *b;
    r = PySequence_GetItem(o,0);
    g = PySequence_GetItem(o,1);
    b = PySequence_GetItem(o,2);
    int ret = (PyInt_AsLong(r)<<g_screen->format->Rshift)+(PyInt_AsLong(g)<<g_screen->format->Gshift)+(PyInt_AsLong(b)<<g_screen->format->Bshift);
    Py_DECREF(r); Py_DECREF(g); Py_DECREF(b);
    return ret;
  }
  return PyInt_AsLong(o);
}

PyObject *GetFirstInstance(PyObject *o)
{
  PyObject *ret;
  PyObject *instances = PyObject_GetAttrString(o,"instances");
  if (PySequence_Size(instances)>0) {
    ret = PySequence_GetItem(instances, 0);
    Py_DECREF(instances);
    return ret;
  }
  Py_DECREF(instances);
  int x = 0;
  PyObject *children = PyObject_GetAttrString(o,"children");
  /* It possibly should iterate over the list and select the lowest-ID'd
        instance. */
  int seqlen = PySequence_Size(children);
  while (x<seqlen) {
    PyObject *child = PySequence_GetItem(children,x);
    ret = GetFirstInstance(child);
    Py_DECREF(child);
    if (ret) {
      Py_DECREF(children);
      return ret;
    }
    x++;
  }
  Py_DECREF(children);
  return NULL;
}

SDL_Surface *RGBA_copy(SDL_Surface *surf)
{
  /* DisplayFormatAlpha crashes on PPC MacOS.  Don't know what it does
   on Intel MacOS. */
  #ifdef __ppc__
  return SDL_ConvertSurface(surf, g_screen->format,SDL_SRCALPHA);
  #else
  return SDL_DisplayFormatAlpha(surf);
  #endif
}
