/* Utility function forward declarations. */
#define GM_WHITE 0xffffff
int Object_AsInt_compare(PyObject *a, PyObject *b);
PyObject *Object_AsInt_richcompare(PyObject *a, PyObject *b, int cond);
PyObject *membertuple(PyObject *self, PyObject *args);
PyObject *load_membertuple(PyObject *self, PyObject *args);
extern PyTypeObject RoomTileData_Type;
int seqi(PyObject *seq, int idx);
inline PyObject *Game_Sequence_Index(PyObject *self, char *seqname);
inline PyObject *Game_Sequence_Index_F(PyObject *self, char *seqname);
inline PyObject *Game_Sequence_Index_S(PyObject *self, char *seqname);
inline PyObject *invoke(PyObject *func, PyObject *self);
inline PyObject *invoke_4(PyObject *func, PyObject *self, PyObject *arg2, PyObject *arg3, PyObject *arg4);
inline void PushSelf(PyObject *o);
inline void PopSelf(void);
inline int color_as_int(PyObject *o);
void fasttint_init_rooms(void);
/* useful internal variables */
extern double g_view_xscale, g_view_yscale;
// XXX temporary.  An intlist8 will possibly have higher performance where relevant.
#define Intlist8 Floatlist8
#define PyNone_New() Py_None; Py_INCREF(Py_None);
extern int COLOR_WHITE;

/* Shared variables for callbacks into Python modules */
extern PyObject *gmapi;
extern PyObject *gmapi__dict__;
extern PyObject *gmapi_draw_sprite_ext;
extern PyObject *gmapi_py_ParticleSystem;
extern PyObject *gmapi_py_self_stack;
extern PyObject *gmapi_py_draw_props;
extern PyObject *gm6;
extern PyObject *gm6_Room_Compile;
extern PyObject *gm6_Room_Create;
extern PyObject *gm6_Room_InitializeBackgrounds;
extern PyObject *gm6_Room_AddInstance;
extern PyObject *gm6_Room_AddDrawnInstance;
extern PyObject *gm6_Room_ResortDrawnObjects;
extern PyObject *gm6_Room_SetBackgroundIndex;
extern PyObject *gm6_Room_StepAlarms;
extern PyObject *gm6_Room_MoveInstances;
extern PyObject *gm6_Room_HandleCollisions;
extern PyObject *gm6_Room_StepEvents;
extern PyObject *gm6_Room_DrawBackgrounds;
extern PyObject *gm6_Room_RenderBackground;
extern PyObject *gm6_Room_Leave;
extern PyObject *gm6_Room_MouseButtonDown;
extern PyObject *gm6_Room_MouseButtonUp;
extern PyObject *gm6_Room_HandleKeyrelease;
extern PyObject *gm6_Room_HandleKeypress;
extern PyObject *gm6_Room_DrawView;
extern PyObject *pygame_draw_rect;

/* Common structures used for fasttint */

struct RoomTileData {
        PyObject_HEAD
        int x, y, background_idx, tilex, tiley, w, h, layer_depth, id, locked;
};

struct Floatlist8 {
        PyObject_HEAD
        double f[8];
};

struct Beancounter {
        PyObject_HEAD
        int count;
};

struct GMRoom {
        PyObject_HEAD
        int id;
        PyObject *name;
        int room_width, room_height, snapx, snapy, isogrid, room_speed, room_persistent, bgcolor, draw_bgcolor;
        // 'caption' and 'room_caption' are the same AFAIK.
        PyObject *room_caption;
        float frame_duration; // Not exported.
        PyObject *creation_code; // Not exported.
        struct RoomBackgroundData *bg0; // Not exported
        struct RoomBackgroundData *bg1; // Not exported
        struct RoomBackgroundData *bg2; // Not exported
        struct RoomBackgroundData *bg3; // Not exported
        struct RoomBackgroundData *bg4; // Not exported
        struct RoomBackgroundData *bg5; // Not exported
        struct RoomBackgroundData *bg6; // Not exported
        struct RoomBackgroundData *bg7; // Not exported
        int view_enabled; // Not exported
        int numviews; // Not exported
        struct Floatlist8 *view_xview, *view_yview, *view_object, *view_hborder, *view_vborder, *view_wview, *view_hview, *view_xport, *view_yport, *view_wport, *view_hport, *background_index;
        PyObject *initial_view_data;
        struct Floatlist8 *background_x, *background_y, *background_visible,
            *background_width, *background_height,
            *background_xscale, *background_yscale,
            *background_hspeed, *background_vspeed,
            *background_blend, *background_alpha,
            *background_htiled, *background_vtiled;
        struct Intlist8 *view_visible;
        // XXX background_index could become an intlist for a slight
        // performance boost, possibly offset by the compatibility hit.
        PyObject *instances;
        PyObject *tiles;
        int tile_w, tile_h, tile_hsep, tile_vsep, tile_hoffset, tile_voffset;
        int created; // exported, though not a GM-native var.
        PyObject *all_objects, *drawn_objects;
        PyObject *deactivated_objects, *keypressed_objects, *keyreleased_objects, *mouse_leftpressed_objects, *mouse_rightpressed_objects, *collisioned_objects, *stepped_objects, *moving_objects, *alarmed_objects, *animated_objects;
        PyObject *key_objects, *mouse_objects;
        int view_current;
        PyObject *cached_bg, *cached_bg_scaled;
        PyObject *creation_code_compiled;
};
extern PyTypeObject GMRoom_Type;

extern PyTypeObject Floatlist8_Type;
extern PyTypeObject Beancounter_Type;

struct BackgroundData {
        PyObject_HEAD
        PyObject *name;
        PyObject *surf;
        PyObject *scaled_data_cache;
        int w,h,transparent;
        // use_video_memory and load_only_on_use omitted.
        // smooth_edges, preload_texture, use_as_tile_set omitted
        // tile_width, tile_height, hoff, voff, hsep, vsep omitted
};
/* For MacOS compilation */
#ifndef Py_VISIT
#define Py_VISIT(op)                                                    \
        do {                                                            \
                if (op) {                                               \
                        int vret = visit((PyObject *)(op), arg);        \
                        if (vret)                                       \
                                return vret;                            \
                }                                                       \
        } while (0)

#endif
#ifndef Py_CLEAR
#define Py_CLEAR(op)                            \
        do {                                    \
                if (op) {                       \
                        PyObject *tmp = (PyObject *)(op);       \
                        (op) = NULL;            \
                        Py_DECREF(tmp);         \
                }                               \
        } while (0)

#endif

int Room_DrawView(struct GMRoom *room, PyObject *surface_py, int x, PyObject *depthsorted);
PyObject *GetFirstInstance(PyObject *o);
extern SDL_Surface *g_screen;
SDL_Surface *RGBA_copy(SDL_Surface *surf);
SDL_Surface *flip_vh(SDL_Surface *src);
SDL_Surface *flip_v(SDL_Surface *src);
SDL_Surface *flip_h(SDL_Surface *src);
PyObject *rotoscale(PyObject *naught, PyObject *args);
PyObject *scale(PyObject *naught, PyObject *args);
PyObject *py_flip_vh(PyObject *naught, PyObject *args);
PyObject *py_flip_v(PyObject *naught, PyObject *args);
PyObject *py_flip_h(PyObject *naught, PyObject *args);
inline PyObject *PySurface_FromSurface(SDL_Surface *surf);
