

#include <pygame/pygame.h>
#include <structmember.h>

#include <stdio.h>
#include <math.h>
#include <SDL_rotozoom.h>
#include <SDL_imageFilter.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#if __ppc__
#include <altivec.h>
#define SDL_Display_FormatAlpha crasher
#else
#include <mmintrin.h>
#endif
#define NOPARTICLES

#include "common.h"

static PyObject *g_screen_pyobj;
static PyTypeObject *g_cobject_type_pointer;
PyObject *gmapi__dict__;
PyObject *gmapi_draw_sprite_ext;
static PyObject *gm6_SpriteData_load_images;
static PyObject *gm6_BackgroundData_load_images;
PyObject *gm6_Room_Compile = NULL;
PyObject *gm6_Room_Create = NULL;
PyObject *gm6_Room_InitializeBackgrounds = NULL;
PyObject *gmapi_py_ParticleSystem = NULL;
PyObject *gmapi_py_draw_props = NULL;
PyObject *gmapi_py_self_stack = NULL;
PyObject *pygame_draw_rect = NULL;
PyObject *pygame_Surface = NULL;

typedef struct {
        PyObject        *shape;
        SDL_Surface     *shapesurf;
        int             blend;
        int             animate;
        int             stretch;
        int             random; // XXX?
        float           size;
        float           size_min, size_max;
        float           size_range;
        float           size_incr;
        float           size_wiggle;
        float           size_xscale, size_yscale;
        unsigned int    startcolor, halfcolor, endcolor;
        float           startalpha, halfalpha, endalpha;
        //float           angle_min, angle_max;
        //float           angle_incr, angle_wiggle, angle_relative;
        float           speed_min, speed_max, speed_range, speed_incr, speed_wiggle;
        float           dir_min, dir_max, dir_range, dir_incr, dir_wiggle;
        int             life_min, life_max, life_range;
        /* XXX I haven't implemented death_type */
        float           grav_amount, grav_dir;
        float           orientation_min, orientation_max, orientation_incr, orientation_wiggle, orientation_relative;
        PyObject        *cached_surfaces;
} ParticleType_object;

static ParticleType_object *PyObject_AsParticleType(PyObject *o);

void debug(const char *s)
{
  printf(s);
  fflush(stdout);
}

void debug_log(const char *s)
{
  FILE *f = fopen("debug.log","a");
  fprintf(f,"fasttint: %s\n",s);
  fclose(f);
}

#define warning printf

/*
 * Returns color multiplied by alpha.  An alpha'd fill can then use
 * bg = multalpha(bg,255-alpha)+multalpha(fg,alpha).  Which has the
 * advantange that multalpha(fg,alpha) can be cached.
 * 
 * If I ever figure out how to do saturated MMXesque math in assembler,
 * this will be even better.  However, as my sample PC is getting acceptable
 * rates, PPC AltiVec is probably a better target for this.
 */
inline unsigned int multalpha(unsigned int color, unsigned char alpha)
{
  /* Can make this faster with saturated vector math? */
  int fg_rb = color & 0xff00ff;
  fg_rb *= alpha;
  fg_rb >>= 8;
  /* Turning this into a short slows it down slightly. */
  int fg_g = color & 0xff00;
  fg_g *= alpha;
  fg_rb &= 0xff00ff;
  fg_rb += (fg_g>>8)&0xff00;
  return fg_rb;
}

unsigned int blend(unsigned int color, unsigned int bg, unsigned int alpha)
{
    int negalpha = 256-alpha;
    int r = color & 0xff;
    r *= alpha;
    int e = bg & 0xff;
    e *= negalpha;
    r += e;
    int ret = r>>8;

    r = color&0xff00;
    r *= alpha;
    r >>= 8;
    e = bg&0xff00;
    e *= negalpha;
    r += e>>8;
    ret += r&0xff00;
    r = color&0xff0000;
    r *= alpha;
    e = bg&0xff0000;
    e *= negalpha;
    r += e;
    r >>=8;
    ret += r&0xff0000;
    return ret;
}

unsigned int interpolate_unformatted(unsigned int startcolor, unsigned int endcolor, float pos)
{
    int alpha = 256*pos;
    return blend(startcolor,endcolor,alpha);
}

int color_for_format(unsigned int color, SDL_PixelFormat *fmt)
{
  return SDL_MapRGB(fmt,color&0xff,color>>8,color>>16);
  /*unsigned int ret = (color<<fmt->Rshift)&fmt->Rmask;
  ret |= (color<<fmt->Gshift)&fmt->Gmask;
  ret |= (color<<fmt->Bshift)&fmt->Bmask;
  return ret;*/
}

unsigned int interpolate_fmt(unsigned int startcolor, unsigned int endcolor, float pos, SDL_PixelFormat *fmt)
{
    int alpha = 255*pos;
    return color_for_format(blend(startcolor,endcolor,alpha),fmt);
}

#if __ppc__ && 0
/* The AltiVec gradient code is actually slower than the C for some reason.
Benchmark results, 10K iterations of draw_rectangle_color w/0.6 alpha:
# 1.38-1.39 with plain C, -funroll-loops, -O99 -O3
# 1.42 secs in C, 10K 0.6a
# 1.49-1.51 secs on Mac altivec 10K 0.6a
# 1.52-1.54 altivec w/unrolled loops
# 1.55-1.57 with dst
 */
inline static void fast_draw_gradient_v_low(SDL_Surface *dst,int x1,int y1,int x2,int y2,int col_t,int col_b,int alpha)
{
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch;
    int x = y*stride;
    int xmax;
    float gradpos = 1.0;
    unsigned char negalpha = 255-alpha;
    vector unsigned char destcol_v;
    if (dst->h<=y2) y2 = dst->h-1;
    void *ptr = dst->pixels;
    vector unsigned char px;
    int width_shifted;
    switch (dst->format->BytesPerPixel) {
      case 4:
	x += x1<<2;
        width_shifted = width<<2;
        break;
      case 2:
	x += x1<<1;
        width_shifted = width<<1;
        break;
      default:
        fprintf(stderr,"Attempt to draw gradient on surface of invalid size (%d bytes per pixel)\n",dst->format->BytesPerPixel);
        return;
    }
    int prefetch_stride = (width_shifted>>4)<<3;
    if (y2-y1<256) {
      prefetch_stride |= (y2-y1)<<8;
    }
    prefetch_stride |= stride<<16;
    //vec_dst((vector signed char *)(dst->pixels+x),prefetch_stride,0);
    int startx = x;
    vector unsigned char pxmul_e,pxmul_o;
    vector unsigned char negalpha_v = vec_lde(0,&negalpha);
    negalpha_v = (vector unsigned char)(
	negalpha,negalpha,negalpha,negalpha,
	negalpha,negalpha,negalpha,negalpha,
	negalpha,negalpha,negalpha,negalpha,
	negalpha,negalpha,negalpha,negalpha);
    vector unsigned char merger = (vector unsigned char)(0,0x10,2,0x12,4,0x14,6,0x16,8,0x18,0xa,0x1a,0xc,0x1c,0xe,0x1e);
    char *ptrmax;
    while (y<y2) {
        //xmax = x + width_shifted;
        destcol = multalpha(interpolate_fmt(col_t,col_b,gradpos,dst->format),alpha);
        destcol_v = (vector unsigned char)(vector unsigned int)(destcol,destcol,destcol,destcol);
        ptr = &dst->pixels[x];
	ptrmax = ptr+width_shifted;
        while (ptr<ptrmax) {
            px = vec_ld(0,(vector unsigned char *)ptr); // LSU 3
            pxmul_e = vec_mule(px,negalpha_v); // VCIU 3
            pxmul_o = vec_mulo(px,negalpha_v); // VCIU 3
            px = vec_perm(pxmul_e,pxmul_o,merger); // VPERM 1
            px = vec_add(px,destcol_v); // VSIU 1
            vec_st(px,0,(vector unsigned char *)ptr); // LSU 3
	    ptr += sizeof(px);
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
    vec_dss(0);
}
#else
/* Not PPC */
inline static void fast_draw_gradient_v_low(SDL_Surface *dst,int x1,int y1,int x2,int y2,int col_t,int col_b,int alpha)
{
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    unsigned int negalpha = 255-alpha;
    if (dst->h<=y2) y2 = dst->h-1;
    while (y<y2) {
        xmax = x + width;
        destcol = multalpha(interpolate_fmt(col_t,col_b,gradpos,dst->format),alpha);
        while (x<xmax) {
            dstpx[x] = destcol+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
}
#endif

static PyObject *fast_draw_gradients_vc(PyObject *self, PyObject *args)
{
    /* Old code took 21.8808698654.
     * Using multalpha instead of blend takes it down to 
     *  13.3176190853 - 13.3490550518 for 10K fills.
     *  13.19-13.22 secs for 10K fills if I use fg_rb+=(fg_g>>8)&0xff instead
     *  of (fg_g&0xff0000)>>8
     * inlining multalpha brings it down to 11.64-11.67.
     */
    SDL_Surface *dst;
    PyObject *pysurf;
    unsigned int x1, y1, x2, y2, col_t1, col_b1, col_2;
    float alpha1, alpha2;
    if (!PyArg_ParseTuple(args, "OiiiiIIfIf",
        &pysurf, &x1, &y1, &x2, &y2,
        &col_t1, &col_b1, &alpha1,
        &col_2, &alpha2)) {
        return NULL;
    }
    float basealpha = 1.0;
    basealpha -= alpha2;
    alpha1 *= basealpha;
    basealpha -= alpha1;
    unsigned char alpha1c = alpha1*256;
    unsigned char alpha2c = alpha2*256;
    dst = PySurface_AsSurface(pysurf);
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    unsigned int negalpha = basealpha*256;
    if (dst->h<=y2) y2 = dst->h-1;
    unsigned int col2_mult = multalpha(col_2,alpha2c);
    while (y<y2) {
        xmax = x + width;
        destcol = multalpha(interpolate_fmt(col_t1,col_b1,gradpos,dst->format),alpha1c);
        destcol += col2_mult;
        while (x<xmax) {
            dstpx[x] = destcol+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *fast_draw_gradients_vv(PyObject *self, PyObject *args)
{
    /* Old code took 21.8808698654.
     * Using multalpha instead of blend takes it down to 
     *  13.3176190853 - 13.3490550518 for 10K fills.
     *  13.19-13.22 secs for 10K fills if I use fg_rb+=(fg_g>>8)&0xff instead
     *  of (fg_g&0xff0000)>>8
     * inlining multalpha brings it down to 11.64-11.67.
     */
    SDL_Surface *dst;
    PyObject *pysurf;
    unsigned int x1, y1, x2, y2, col_t1, col_b1, col_t2,col_b2;
    float alpha1, alpha2;
    if (!PyArg_ParseTuple(args, "OiiiiIIfIIf",
        &pysurf, &x1, &y1, &x2, &y2,
        &col_t1, &col_b1, &alpha1,
        &col_t2, &col_b2, &alpha2)) {
        return NULL;
    }
    float basealpha = 1.0;
    basealpha -= alpha2;
    alpha1 *= basealpha;
    basealpha -= alpha1;
    unsigned char alpha1c = alpha1*256;
    unsigned char alpha2c = alpha2*256;
    dst = PySurface_AsSurface(pysurf);
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    unsigned int negalpha = basealpha*256;
    if (dst->h<=y2) y2 = dst->h-1;
    while (y<y2) {
        xmax = x + width;
        destcol = multalpha(interpolate_fmt(col_t1,col_b1,gradpos,dst->format),alpha1c);
        destcol += multalpha(interpolate_fmt(col_t2,col_b2,gradpos,dst->format),alpha2c);
        while (x<xmax) {
            dstpx[x] = destcol+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *fast_draw_gradients_vcvvc(PyObject *self, PyObject *args)
{
    /* Old code took 21.8808698654.
     * Using multalpha instead of blend takes it down to 
     *  13.3176190853 - 13.3490550518 for 10K fills.
     *  13.19-13.22 secs for 10K fills if I use fg_rb+=(fg_g>>8)&0xff instead
     *  of (fg_g&0xff0000)>>8
     * inlining multalpha brings it down to 11.64-11.67.
     */
    SDL_Surface *dst;
    PyObject *pysurf;
    unsigned int x1, y1, x2, y2, col_t1, col_b1, col_2, col_t3, col_b3,
        col_t4, col_b4, col_5;
    float alpha1, alpha2, alpha3, alpha4, alpha5;
    if (!PyArg_ParseTuple(args, "OiiiiIIfIfIIfIIfIf", 
        &pysurf, &x1, &y1, &x2, &y2,
        &col_t1, &col_b1, &alpha1,
        &col_2, &alpha2,
        &col_t3, &col_b3, &alpha3,
        &col_t4, &col_b4, &alpha4,
        &col_5, &alpha5)) {
        return NULL;
    }
    /* Now, sum up the alphas. */
    /* Alpha3 is unchanged. */
    /* Alpha2 must be multiplied by 1-alpha3 */
    /* alpha1 must be multiplied by 1-current */
    /* I'm pretty sure that this is the right alpha */
    /* Orig
    unsigned char alpha1c,alpha2c,alpha3c;
    float totalalpha = alpha3+(alpha2*(1.0-alpha3));
    alpha1c = alpha1*(1.0-totalalpha)*255;
    totalalpha += alpha1*(1.0-totalalpha);
    alpha2c = alpha2*(1.0-alpha3)*255;
    alpha3c = alpha3*255;
    unsigned char alpha = totalalpha*255;*/
    float basealpha = 1.0;
    basealpha -= alpha5;
    alpha4 *= basealpha;
    basealpha -= alpha4;
    alpha3 *= basealpha;
    basealpha -= alpha3;
    alpha2 *= basealpha;
    basealpha -= alpha2;
    alpha1 *= basealpha;
    basealpha -= alpha1;
    unsigned char alpha1c = alpha1*256;
    unsigned char alpha2c = alpha2*256;
    unsigned char alpha3c = alpha3*256;
    unsigned char alpha4c = alpha4*256;
    unsigned char alpha5c = alpha5*256;
    dst = PySurface_AsSurface(pysurf);
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    unsigned int negalpha = basealpha*256;
    if (dst->h<=y2) y2 = dst->h-1;
    unsigned int col2_mult = multalpha(col_2,alpha2c);
    unsigned int col5_mult = multalpha(col_5,alpha5c);
    while (y<y2) {
        xmax = x + width;
        destcol = multalpha(interpolate_fmt(col_t1,col_b1,gradpos,dst->format),alpha1c);
        destcol += col2_mult;
        destcol += multalpha(interpolate_fmt(col_t3,col_b3,gradpos,dst->format),alpha3c);
        destcol += multalpha(interpolate_fmt(col_t4,col_b4,gradpos,dst->format),alpha4c);
        destcol += col5_mult;
        while (x<xmax) {
            dstpx[x] = destcol+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *fast_draw_gradients_vcv(PyObject *self, PyObject *args)
{
    /* Old code took 21.8808698654.
     * Using multalpha instead of blend takes it down to 
     *  13.3176190853 - 13.3490550518 for 10K fills.
     *  13.19-13.22 secs for 10K fills if I use fg_rb+=(fg_g>>8)&0xff instead
     *  of (fg_g&0xff0000)>>8
     * inlining multalpha brings it down to 11.64-11.67.
     */
    SDL_Surface *dst;
    PyObject *pysurf;
    unsigned int x1, y1, x2, y2, col_t1, col_b1, col_2, col_t3, col_b3;
    float alpha1, alpha2, alpha3;
    if (!PyArg_ParseTuple(args, "OiiiiIIfIfIIf", 
        &pysurf, &x1, &y1, &x2, &y2,
        &col_t1, &col_b1, &alpha1,
        &col_2, &alpha2,
        &col_t3, &col_b3, &alpha3)) {
        return NULL;
    }
    /* Now, sum up the alphas. */
    /* Alpha3 is unchanged. */
    /* Alpha2 must be multiplied by 1-alpha3 */
    /* alpha1 must be multiplied by 1-current */
    /* I'm pretty sure that this is the right alpha */
    unsigned char alpha1c,alpha2c,alpha3c;
    float totalalpha = alpha3+(alpha2*(1.0-alpha3));
    alpha1c = alpha1*(1.0-totalalpha)*255;
    totalalpha += alpha1*(1.0-totalalpha);
    alpha2c = alpha2*(1.0-alpha3)*255;
    alpha3c = alpha3*255;
    unsigned char alpha = totalalpha*255;
    dst = PySurface_AsSurface(pysurf);
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    unsigned int negalpha = 255-alpha;
    if (dst->h<=y2) y2 = dst->h-1;
    unsigned int col2_mult = multalpha(col_2,alpha2c);
    while (y<y2) {
        xmax = x + width;
        destcol = multalpha(interpolate_fmt(col_t1,col_b1,gradpos,dst->format),alpha1c);
        destcol += col2_mult;
        destcol += multalpha(interpolate_fmt(col_t3,col_b3,gradpos,dst->format),alpha3c);
        while (x<xmax) {
            dstpx[x] = destcol+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *fast_draw_gradient_v(PyObject *self, PyObject *args)
{
    /* Old code took 21.8808698654.
     * Using multalpha instead of blend takes it down to 
     *  13.3176190853 - 13.3490550518 for 10K fills.
     *  13.19-13.22 secs for 10K fills if I use fg_rb+=(fg_g>>8)&0xff instead
     *  of (fg_g&0xff0000)>>8
     * inlining multalpha brings it down to 11.64-11.67.
     */
    SDL_Surface *dst;
    PyObject *pysurf;
    unsigned int x1, y1, x2, y2, col_t, col_b, alpha;
    if (!PyArg_ParseTuple(args, "OiiiiIIi", &pysurf, &x1, &y1, &x2, &y2,
        &col_t, &col_b, &alpha)) {
        return NULL;
    }
    dst = PySurface_AsSurface(pysurf);
    fast_draw_gradient_v_low(dst,x1,y1,x2,y2,col_t,col_b,alpha);
    Py_INCREF(Py_None);
    return Py_None;
}

inline static void fast_draw_gradient_h_opaque(SDL_Surface *dst, int x1, int y1,int x2, int y2, unsigned int col_l, unsigned int col_r)
{
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    if (dst->w<=x2) x2 = dst->w-1;
    float colorstep = 1.0/(x2-x1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    float gradpos = 1.0;
    if (dst->h<=y2) y2 = dst->h-1;
    int bound=(y2*stride)+x2;
    int i;
    int starti = (y1*stride)+x1;
    int imax = (y1*stride)+x2;
    while (starti<imax) {
        destcol = interpolate_fmt(col_l,col_r,gradpos,dst->format);
        i = starti;
        while (i<bound) {
            dstpx[i]=destcol;
            i += stride;
        }
        starti++;
        gradpos -= colorstep;
    }
}

inline static void fast_draw_gradient_v_opaque(SDL_Surface *dst, int x1, int y1,int x2, int y2, unsigned int col_t, unsigned int col_b)
{
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax;
    float gradpos = 1.0;
    if (dst->h<=y2) y2 = dst->h-1;
    while (y<y2) {
        xmax = x + width;
        destcol = interpolate_fmt(col_t,col_b,gradpos,dst->format);
        while (x<xmax) {
            dstpx[x] = destcol;
            x++;
        }
        startx += stride;
        x = startx;
        y ++;
        gradpos -= colorstep;
    }
}

inline static void fast_draw_gradient_v_opaque_outline(SDL_Surface *dst, int x1, int y1,int x2, int y2, unsigned int col_t, unsigned int col_b)
{
    unsigned int *dstpx = (unsigned int *)dst->pixels;
    int y = y1;
    if (dst->w<=x2) x2 = dst->w-1;
    int width = x2-x1;
    float colorstep = 1.0/(y2-y1);
    unsigned int destcol;
    int stride = dst->pitch>>2;
    int x = x1+(y*stride);
    int startx = x;
    int xmax=x+width;
    float gradpos = 1.0;
    if (dst->h<=y2) y2 = dst->h-1;
    unsigned int col_t_px = color_for_format(col_t,dst->format);
    unsigned int col_b_px = color_for_format(col_b,dst->format);
    while (x<xmax) {
      dstpx[x++] = col_t_px;
    }
    x = x1+(y2*stride);
    xmax = x+width;
    while (x<xmax) {
      dstpx[x++] = col_b_px;
    }
    x = startx+stride;
    while (x<xmax) {
        destcol = interpolate_fmt(col_t,col_b,gradpos,dst->format);
        dstpx[x] = destcol;
        dstpx[x+width] = destcol;
        x += stride;
        gradpos -= colorstep;
    }
}

static inline void fill_translucent_low(SDL_Surface *dst, int x1, int y1, int x2, int y2, unsigned int color, unsigned int alpha)
{
    register unsigned int *dstpx = (unsigned int *)dst->pixels;
    if (dst->w<=x2) x2 = dst->w-1;
    int stride = dst->pitch>>2;
    int startx = x1+(y1*stride);
    int width = x2-x1;
    if (dst->h<=y2) y2 = dst->h-1;
    int endx = x2+(y2*stride);
    register int x = startx;
    int xmax;
    color = multalpha(color,alpha);
    unsigned char negalpha = 255-alpha;
    while (x<endx) {
        xmax = x+width;
        while (x<xmax) {
            dstpx[x] = color+multalpha(dstpx[x],negalpha);
            x++;
        }
        startx += stride;
        x = startx;
    }
}

static PyObject *fast_draw_rectangle_color(PyObject *self, PyObject *args)
{
  unsigned int col_tl,col_tr,col_br,col_bl;
  int x1, y1, x2, y2;
  float alphaf;
  int outline;
  if (!PyArg_ParseTuple(args,"iiiiIIIIfi",&x1,&y1,&x2,&y2,&col_tl,&col_tr,&col_br,&col_bl,&alphaf,&outline)) {
    return NULL;
  }
  int tmp;
  if (x2<x1) {
    tmp = x1;
    x1 = x2;
    x2 = tmp;
  }
  if (y2<y1) {
    tmp = y1;
    y1 = y2;
    y2 = tmp;
  }
  int alpha = 255*alphaf;
  if (col_tl==col_tr && col_bl==col_br) {
    if (col_tl!=col_bl) {
      if (alpha!=255) {
        if (!outline) {
          fast_draw_gradient_v_low(g_screen, x1,y1,x2,y2, col_tl, col_bl, alpha);
        } else {
          warning("BUG: need to draw translucent outline vgrad.\n");
        }
      } else {
        if (!outline) {
          fast_draw_gradient_v_opaque(g_screen, x1,y1,x2,y2,col_tl,col_bl);
        } else {
          fast_draw_gradient_v_opaque_outline(g_screen,x1,y1,x2,y2,col_tl,col_bl);
        }
      }
    } else { /* col_tl==col_bl - a homogenous field. */
      if (alpha<255) {
        fill_translucent_low(g_screen,x1,y1,x2,y2,col_tl,alpha);
      } else {
        warning("BUG: should actually fill.\n");
      }
    }
  } else {
    if (alpha==255 && !outline) {
      fast_draw_gradient_h_opaque(g_screen,x1,y1,x2,y2,col_tl,col_tr);
    } else {
      warning("Horizontal gradient, complex gradient, or no gradient. %d/%d\n",alpha,outline);
    }
  }
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *fill_translucent(PyObject *self, PyObject *args)
{
    /* Might be able to optimize by omitting self? */
    /* Could probably also improve by using MMX and/or AltiVec, but that's likely to be Work. */
    unsigned int color;
    unsigned int alpha;
    unsigned int x1;
    unsigned int x2;
    unsigned int y1;
    unsigned int y2;
    PyObject *pysurf;
    SDL_Surface *dst;
    if (!PyArg_ParseTuple(args, "Oiiiiii", &pysurf, &x1, &y1, &x2, &y2,
        &color, &alpha)) {
        return NULL;
    }
    dst = PySurface_AsSurface(pysurf);
    fill_translucent_low(dst,x1,y1,x2,y2,color,alpha);
    Py_INCREF(Py_None);
    return Py_None;
}

inline unsigned int DesiredAlpha(ParticleType_object *pt, float pos)
{
  if (pos>0.5) {
    pos-=0.5;
    pos *= 2;
    unsigned int ret=255*((pt->startalpha*pos)+(pt->halfalpha*(1.0-pos)));
    return ret;
  }
  pos *= 2;
  return 255*((pt->endalpha*(1.0-pos))+
        (pt->halfalpha*pos));
}

inline unsigned int DesiredTint_type(ParticleType_object *pt,float pos)
{
  if (pos>0.5) {
    return interpolate_unformatted(pt->startcolor,pt->halfcolor,
      (pos-0.5)*2);
  }
  return interpolate_unformatted(
      pt->halfcolor,pt->endcolor,
      pos*2);
}

inline void Translucentify(SDL_Surface *surf, unsigned char desiredalpha)
{
  unsigned int *pixels = surf->pixels;
  unsigned int imax = (surf->h * surf->pitch)>>2;
  unsigned int i = 0;
  unsigned int px;
  unsigned int colormask = surf->format->Rmask|surf->format->Gmask|surf->format->Bmask;
  unsigned int alphamask = surf->format->Amask;
  if (surf->format->Ashift>=24) {
    while(i<imax) {
      px = pixels[i]>>8;
      px *= desiredalpha;
      pixels[i] = (pixels[i]&colormask)|(px&alphamask);
      i++;
    }
  } else {
    while (i<imax) {
      px = pixels[i]&alphamask;
      px *= desiredalpha;
      px >>= 8;
      pixels[i] = (pixels[i]&colormask)|(px&alphamask);
      i ++;
    }
  }
}

typedef unsigned int v8qi __attribute__ ((mode(V8QI)));

void additive_blit(SDL_Surface *src, SDL_Surface *dst, int srcstart, int dststart, int height, int width)
{
  int h = height;
  while (h) {
    SDL_imageFilterAdd(
      &(((unsigned char *)src->pixels)[srcstart]),
      &(((unsigned char *)dst->pixels)[dststart]),
      &(((unsigned char *)dst->pixels)[dststart]),width);
    dststart += dst->pitch;
    srcstart += src->pitch;
    h--;
  }
  /* For some reason, this causes all manner of stupid graphical bugs.  Why? 
  int h = height;
  v8qi *start, *end;
  int wstart = width/sizeof(v8qi);
  while (h) {
    start = &src->pixels[srcstart];
    end = &dst->pixels[dststart];
    int w = wstart;
    while (w) {
      *end = __builtin_ia32_paddusb(*start, *end);
      start++;
      end++;
      w--;
    }
    srcstart += src->pitch;
    dststart += dst->pitch;
    h--;
  }*/
}

void flagged_blit(SDL_Surface *src, SDL_Surface *screensurf, 
        SDL_Rect *dst, int additive)
{
  if (!additive) {
    SDL_BlitSurface(src,NULL,screensurf,dst);
  } else {
    //unsigned int *pixels = screensurf->pixels;
    int i = 0;
    int astart;
    if (dst->y>0) {
      if (dst->y>=screensurf->h) return;
      astart = (dst->x<<2)+(dst->y*screensurf->pitch);
    } else {
      astart = dst->x<<2;
    }
    if (dst->x>=screensurf->w) return;
    int spstart = 0;
    int imax = i+src->h;
    if (imax+dst->y>=screensurf->h) {
      imax = screensurf->h-dst->y;
    }
    int srcwidth = src->w<<2;
    if (src->w+dst->x>=screensurf->w) {
      srcwidth = (screensurf->w-(src->w+dst->x))<<2;
    }
    if (dst->x<0) {
      srcwidth+=dst->x<<2;
      astart-=dst->x<<2;
      spstart -= dst->x<<2;
    }
    if (srcwidth<=0) return;
    switch (additive) {
      case 1: /* pygame.BLEND_ADD? */
      case 128: /* bm_add */
        additive_blit(src,screensurf,spstart,astart,imax-i,srcwidth);
        return;
      default:
        warning("Error: Unexpected additive blend mode %d\n",additive);
        SDL_BlitSurface(src,NULL,screensurf,dst);
    }
  }
}

inline int blend_mode_flatten(int bm)
{
  switch (bm) {
    case 1:
    case 128:
      return 1;
    default:
      return 0;
  }
}

/* Create a new CParticle! */
struct CParticle {
        float   x,y;
        float   direction,speed;
        float xvel,yvel;
        /* Optimize out the trig functions from common motion by making
           direction set an x/y vector?  Might not be desirable, though,
           especially on wiggling particles which will be changing their
           vector every step. */
        struct Beancounter *beancounter;
        PyObject *type;
        float   size;
        int     ttl;
        unsigned int startcolor, halfcolor, endcolor;
        float lifespan;
        float angle;
};

static PyObject *CParticle_attach_beancounter(PyObject *naught, PyObject *args)
{
  PyObject *py_cp, *py_bc;
  if (!PyArg_ParseTuple(args,"OO",&py_cp, &py_bc)) {
    return NULL;
  }
  struct Beancounter *bc = (struct Beancounter *)py_bc;
  struct CParticle *cp = PyCObject_AsVoidPtr(py_cp);
  if (cp->beancounter) {
    struct Beancounter *obc = cp->beancounter;
    cp->beancounter = NULL;
    obc->count--;
    Py_DECREF(obc);
  }
  Py_INCREF(bc);
  cp->beancounter = bc;
  bc->count++;
  Py_INCREF(Py_None);
  return Py_None;
}

static void CParticle_destructor(void *vpart)
{
  struct CParticle *part = (struct CParticle *)vpart;
  if (part->beancounter) {
    part->beancounter->count--;
    Py_CLEAR(part->beancounter);
  }
  Py_CLEAR(part->type);
  free(part);
}

inline void CParticle_reset_velocities(struct CParticle *part)
{
  double dir = (part->direction*0.017453292519943295)+1.5707963267948966;
  part->xvel = part->speed*cos(dir);
  part->yvel = part->speed*sin(dir);
}

static inline PyObject *GMVar_0(PyObject *a);

static PyObject *CParticle_new(PyObject *naught, PyObject *args)
/* Replaces py_Particle.__init__ */
{
  struct CParticle *part = malloc(sizeof(struct CParticle));
  if (!PyArg_ParseTuple(args,"Off",&part->type,&part->x,&part->y)) {
    free(part);
    return NULL;
  }
  part->type = GMVar_0(part->type);
  ParticleType_object *pt = PyObject_AsParticleType(part->type);
  if (!pt) {
    free(part);
    return NULL;
  }
  Py_INCREF(part->type);
  part->beancounter = NULL;
  part->direction = pt->dir_min+(pt->dir_range*drand48());
  part->speed = pt->speed_min+(pt->speed_range*drand48());
  CParticle_reset_velocities(part);
  part->size = pt->size_min+(pt->size_range*drand48());
  part->ttl = pt->life_min+(pt->life_range*drand48());
  part->startcolor = pt->startcolor;
  part->halfcolor = pt->halfcolor;
  part->endcolor = pt->endcolor;
  // XXX angles should also be randomized.
  part->lifespan = part->ttl;
  part->angle = pt->orientation_min+((pt->orientation_max-pt->orientation_min)*drand48());
  return PyCObject_FromVoidPtr(part, CParticle_destructor);
}

static PyObject *CParticle_new_size_color(PyObject *naught, PyObject *args)
{
  struct CParticle *part = malloc(sizeof(struct CParticle));
  if (!PyArg_ParseTuple(args,"Offfi",&part->type,&part->x,&part->y,&part->size,&part->startcolor)) {
    free(part);
    return NULL;
  }
  part->type = GMVar_0(part->type);
  part->endcolor = part->halfcolor = part->startcolor;// = color_for_format(part->endcolor,g_screen->format);
  ParticleType_object *pt = PyObject_AsParticleType(part->type);
  if (!pt) {
     free(part);
     return NULL;
  }
  part->beancounter = NULL;
  Py_INCREF(part->type);
  part->direction = pt->dir_min+(pt->dir_range*drand48());
  part->speed = pt->speed_min+(pt->speed_range*drand48());
  part->angle = pt->orientation_min+((pt->orientation_max-pt->orientation_min)*drand48());
  CParticle_reset_velocities(part);
  part->ttl = pt->life_min+(pt->life_range*drand48());
  // XXX angles should also be randomized.
  part->lifespan = part->ttl;
  return PyCObject_FromVoidPtr(part, CParticle_destructor);
}

static PyObject *CParticle_dec_ttl(PyObject *naught, PyObject *args)
{
  PyObject *pyp;
  if (!PyArg_ParseTuple(args,"O",&pyp)) {
    return NULL;
  }
  struct CParticle *part = PyCObject_AsVoidPtr(pyp);
  part->ttl --;
  return PyInt_FromLong(part->ttl);
}

/*int max(a,b) {
  if (a>=b) return a;
  return b;
}

int min(a,b) {
  if (a<b) return a;
  return b;
}*/

inline unsigned int DesiredTint_part_gm(struct CParticle *p,float pos)
{
  if (pos>0.5) {
    return interpolate_unformatted(p->startcolor,p->halfcolor,
      (pos-0.5)*2);
  }
  return interpolate_unformatted(
      p->halfcolor,p->endcolor,
      pos*2);
}

void dump(const char *s)
{
  int fd = open("logfile.txt",O_APPEND|O_RDWR|O_CREAT,0644);
  write(fd,s,strlen(s));
  close(fd);
}
SDL_Surface *crasher(SDL_Surface *arg)
{
  dump("crasher called.\n");
  exit(0);
}


#ifdef __MMX__
typedef unsigned int mmx_64bit_t __attribute__ ((vector_size (8)));
typedef unsigned char v8qi_t __attribute__((vector_size (8)));
typedef union {
  mmx_64bit_t m;
  v8qi_t mb;
  unsigned int i[2];
  struct {
    unsigned char a,b,c,d,e,f,g,h;
  } uc;
} subtractor_t;

subtractor_t subtractor_for(unsigned int val)
{
  subtractor_t ret;
  ret.i[0] = ret.i[1] = val;
  return ret;
}

typedef union {
  mmx_64bit_t *m;
  v8qi_t *mb;
  unsigned int *i;
  unsigned char *c;
  unsigned int ui;
} unionptr_t;

unionptr_t startptr_for(void *buf)
{
  unionptr_t ret;
  ret.c = buf;
  return ret;
}

unsigned char *satusub_head(unionptr_t mmxptr, subtractor_t subtractor, int len)
{
  unsigned int maxlen = mmxptr.ui+len;
  while (mmxptr.ui<maxlen) {
    if (subtractor.uc.a<=mmxptr.c[0]) {
      mmxptr.c[0]-=subtractor.uc.a;
    } else {
      mmxptr.c[0] = 0;
    }
    mmxptr.ui ++;
    if (subtractor.uc.b<=mmxptr.c[0]) {
      mmxptr.c[0]-=subtractor.uc.b;
    } else {
      mmxptr.c[0] = 0;
    }
    mmxptr.ui ++;
    if (subtractor.uc.c<=mmxptr.c[0]) {
      mmxptr.c[0]-=subtractor.uc.c;
    } else {
      mmxptr.c[0] = 0;
    }
    mmxptr.ui ++;
    if (subtractor.uc.d<=mmxptr.c[0]) {
      mmxptr.c[0]-=subtractor.uc.d;
    } else {
      mmxptr.c[0] = 0;
    }
    mmxptr.ui ++;
  }
  return mmxptr.c;
}

unsigned char *satusub_body(unionptr_t mmxptr, subtractor_t mmxval, unsigned int len)
{
  unsigned int end = len+mmxptr.ui;
  while (mmxptr.ui<end) {
    //_mm_subs_pi32(*(mmxptr.m),mmxval.m);
    *(mmxptr.m)=__builtin_ia32_psubusb(*(mmxptr.mb),mmxval.mb);
    mmxptr.m ++;
  }
  return mmxptr.c;
}

#define satusub_tail satusub_head
#endif

void satusub(void *buf, int len, unsigned int val)
{
  SDL_imageFilterSubUint(buf,buf,len,val);
  #if 0
  #if __ppc__
  /* My attempt at vector code produced hotspots when satusubbing. */
  
  // XXX what is the correct #if to detect MacOS compiles?
  unsigned char *end = buf+len;
  unsigned char a,b,c,d;
  a = val>>24;
  b = val>>16;
  c = val>>8;
  d = val;
  unsigned char *ptr = buf;
  while (ptr<end) {
    *ptr = (*ptr<=a ? 0 : (*ptr)-a);
    ++ptr;
    *ptr = (*ptr<=b ? 0 : (*ptr)-b);
    ++ptr;
    *ptr = (*ptr<=c ? 0 : (*ptr)-c);
    ++ptr;
    *ptr = (*ptr<=d ? 0 : (*ptr)-d);
    ++ptr;
  }
  return;
  #elif __x86_64
  SDL_imageFilterSubUint(buf,buf,len,val);
  #else
  unsigned int buflo = ((unsigned int)buf)&0x7;
  subtractor_t mmxval = subtractor_for(val);
  unionptr_t mmxptr = startptr_for(buf);
  if (buflo&3) {
    /* Force a crash if not 32-bit aligned. */
    mmxptr.c = NULL;
  }
  unsigned int reallen = len;
  if (buflo &0x4) {
    mmxptr.c = satusub_head(mmxptr,mmxval,4);
    reallen -= 1;
  }
  mmxptr.c = satusub_body(mmxptr,mmxval,reallen&~4);
  if (reallen&0x4) {
    satusub_tail(mmxptr,mmxval,4);
  }
  #endif
  #endif
}

#ifndef __ppc__
typedef union {
    unsigned char bytes[8];
    unsigned int i32[2];
    __m64 m;
} mmxvar;
#endif

void Tint_Surface(SDL_Surface *surf, unsigned int tint)
{
  unsigned int subtractor = (color_for_format(GM_WHITE-tint,surf->format)&~surf->format->Amask);
  #ifdef __ppc__
  void *pixelptr = surf->pixels;
  vector unsigned char mm0;
  vector unsigned char subtractor_v =(vector unsigned char)((vector unsigned int)(subtractor,subtractor,subtractor,subtractor));
  vector unsigned char res;
  void *lastpx = surf->pixels+(surf->pitch*surf->h);
  while (pixelptr<lastpx) {
    mm0 = vec_ld(0,(vector unsigned char *)pixelptr);
    res = vec_subs(mm0,subtractor_v);
    vec_st(res,0,(vector unsigned char *)pixelptr);
    pixelptr += sizeof(vector unsigned char);
  }
  //SDL_imageFilterSubUint(surf->pixels, surf->pixels, surf->pitch*surf->h,subtractor);
  #else
  mmxvar *pixel = surf->pixels;
  mmxvar mmsubtractor;
  mmsubtractor.i32[0] = mmsubtractor.i32[1] = subtractor;
  int surface_size = (surf->h*surf->pitch);
  int real_surface_size = surface_size;
  int tail_size;
  surface_size /= sizeof(__m64);
  if (surface_size*sizeof(__m64)<real_surface_size) {
    tail_size = real_surface_size-(surface_size*sizeof(__m64));
  } else {
    tail_size = 0;
  }
  while (surface_size) {
    pixel->m = _mm_subs_pu8(pixel->m,mmsubtractor.m);
    pixel++;
    surface_size--;
  }
  _mm_empty();
  while (tail_size>0) {
    unsigned char a,b,c,d;
    a = subtractor>>24;
    b = subtractor>>16;
    c = subtractor>>8;
    d = subtractor;
    pixel->bytes[0] = (pixel->bytes[0]>a?pixel->bytes[0]-a:0);
    pixel->bytes[1] = (pixel->bytes[1]>b?pixel->bytes[1]-b:0);
    pixel->bytes[2] = (pixel->bytes[2]>c?pixel->bytes[2]-c:0);
    pixel->bytes[3] = (pixel->bytes[3]>d?pixel->bytes[3]-d:0);
    tail_size-=4;
    pixel = ((void *)pixel)+4;
  }
  #endif
}

inline void particle_render_low(struct CParticle *cparticle, 
  int xoff, int yoff)
{
  PyObject *py_pt = cparticle->type;
  Py_INCREF(py_pt);
  ParticleType_object *pt = PyCObject_AsVoidPtr(py_pt);

  double interp = cparticle->ttl;
  interp /= cparticle->lifespan;
  unsigned int tint = DesiredTint_part_gm(cparticle,interp);
  unsigned int alpha = DesiredAlpha(pt,interp);
  PyObject *shapeobj = pt->shape;
  Py_INCREF(shapeobj);
  SDL_Surface *shapesurf = PySurface_AsSurface(shapeobj);
  // Weakrefs not created before here.

  int w,h;
  int scale = 0;
  if (pt->size_wiggle) {
    w = shapesurf->w*cparticle->size*pt->size_xscale;
    h = shapesurf->h*cparticle->size*pt->size_yscale;
    scale = 1;
  } else {
    w = shapesurf->w*pt->size_xscale*cparticle->size;
    h = shapesurf->h*pt->size_yscale*cparticle->size;
    if ((w!=shapesurf->w) || (h!=shapesurf->h)) {
      scale = 1;
    }
  }
  if (w && h) {
    int iangle = cparticle->angle;
    iangle &= 0xff0;
    SDL_Rect dst;
    dst.x = cparticle->x+xoff;
    dst.y = cparticle->y+yoff;
    if (scale) {
      PyObject *pw = PyInt_FromLong(w);
      PyObject *ph = PyInt_FromLong(h);
      PyObject *ptint = PyInt_FromLong(tint);
      PyObject *palpha = PyInt_FromLong(alpha);
      PyObject *pangle = PyInt_FromLong(iangle);
      PyObject *key = PyTuple_Pack(5,pw,ph,ptint,palpha,pangle);
      Py_DECREF(ph);
      Py_DECREF(pw);
      Py_DECREF(ptint);
      Py_DECREF(palpha);
      Py_DECREF(pangle);
      PyObject *cache = pt->cached_surfaces;
      Py_INCREF(cache);
      PyObject *cachedsurf = PyDict_GetItem(cache,key);
      if (cachedsurf) {
        // Commenting this block out doesn't stop weakref proliferation.
        Py_INCREF(cachedsurf);
        SDL_Surface *sdl_cachedsurf = PySurface_AsSurface(cachedsurf);
        dst.x -= sdl_cachedsurf->w>>1;
        dst.y -= sdl_cachedsurf->h>>1;
        flagged_blit(sdl_cachedsurf,g_screen,&dst,pt->blend); // not the leaking line.
        Py_DECREF(cachedsurf);
      } else {
        if (PyDict_Size(cache)>100) {
          PyDict_Clear(cache);
        }
        double zoomx = w;
        zoomx /= shapesurf->w;
        double zoomy = h;
        zoomy /= shapesurf->h;
        SDL_Surface *surf = zoomSurface(shapesurf, zoomx, zoomy, 0);
        /*
        This is needed to convert 24bit images for proper tinting.
        SDL_Surface *osurf = zoomSurface(shapesurf, zoomx, zoomy, 0);
        SDL_Surface *surf = SDL_ConvertSurface(osurf, g_screen->format,SDL_SRCALPHA);
        SDL_FreeSurface(osurf); */
        if (tint!=GM_WHITE) {
          Tint_Surface(surf,tint); // Removing the tint doesn't cure the leak.
        }
        int additive = pt->blend;
        if (alpha<255) {
          if (additive) {
            unsigned int negalpha = 255-alpha;
            SDL_Surface *tmpsurf = SDL_ConvertSurface(surf, g_screen->format, SDL_SRCALPHA);
            SDL_FreeSurface(surf);
            surf = tmpsurf;
            satusub(surf->pixels,surf->pitch*surf->h,(negalpha<<24)|(negalpha<<16)|(negalpha<<8)|negalpha);
          } else {
            Translucentify(surf,alpha);
          }
        }
        if (iangle) {
          SDL_Surface *rotated = rotozoomSurface(surf, iangle, 1.0, 0);
          SDL_FreeSurface(surf);
          surf = rotated;
        }
        PyObject *newsurf = PySurface_New(surf);
        PyDict_SetItem(cache,key,newsurf);
        Py_DECREF(newsurf);
        dst.x -= surf->w>>1;
        dst.y -= surf->h>>1;
        flagged_blit(
          surf,
          g_screen,
          &dst,
          additive);
      }
      // XXX Do I need to decref each of the ints? //
      Py_DECREF(cache);
      Py_DECREF(key);
    } else if (alpha!=255) { // not scaling
      PyObject *pw = PyInt_FromLong(w);
      PyObject *ph = PyInt_FromLong(h);
      PyObject *ptint = PyInt_FromLong(tint);
      PyObject *palpha = PyInt_FromLong(alpha);
      int iangle = cparticle->angle;
      PyObject *pangle = PyInt_FromLong(iangle);
      PyObject *key = PyTuple_Pack(5,pw,ph,ptint,palpha,pangle);
      Py_DECREF(ph);
      Py_DECREF(pw);
      Py_DECREF(ptint);
      Py_DECREF(palpha);
      Py_DECREF(pangle);
      PyObject *cache = pt->cached_surfaces;
      Py_INCREF(cache);
      PyObject *cachedsurf = PyDict_GetItem(cache,key);
      if (cachedsurf) {
        Py_INCREF(cachedsurf);
        SDL_Surface *sdl_cachedsurf = PySurface_AsSurface(cachedsurf);
        dst.x -= sdl_cachedsurf->w>>1;
        dst.y -= sdl_cachedsurf->h>>1;
        flagged_blit(sdl_cachedsurf,g_screen,&dst,pt->blend);
        Py_DECREF(cachedsurf);
      } else {
        SDL_Surface *surf = SDL_ConvertSurface(shapesurf, g_screen->format,SDL_SRCALPHA);
        if (!surf) {
          fprintf(stderr,"Error reformatting shapesurf: %s\n",SDL_GetError());
        }
        if (tint!=GM_WHITE) {
          Tint_Surface(surf,tint);
        }
        int additive = pt->blend;
        if (additive) {
          unsigned int negalpha = 0xffffffff-((alpha<<24)|(alpha<<16)|(alpha<<8)|alpha);
          satusub(surf->pixels,surf->pitch*surf->h,negalpha);
        } else {
          Translucentify(surf,alpha);
        }
        if (iangle) {
          SDL_Surface *rotated = rotozoomSurface(surf, iangle, 1.0, 0);
          SDL_FreeSurface(surf);
          surf = rotated;
          /* Slightly inefficient - if w/h factor were equivalent then the
             rotozoom could be a single call instead of two. */
        }
        PyObject *newsurf = PySurface_New(surf);
        PyDict_SetItem(cache,key,newsurf);
        dst.x -= surf->w>>1;
        dst.y -= surf->h>>1;
        flagged_blit(
          surf,
          g_screen,
          &dst,
          additive);
      }
      Py_DECREF(cache);
      Py_DECREF(key);
    } else if (tint!=GM_WHITE) {
      PyObject *pw = PyInt_FromLong(w);
      PyObject *ph = PyInt_FromLong(h);
      PyObject *ptint = PyInt_FromLong(tint);
      PyObject *palpha = PyInt_FromLong(alpha);
      PyObject *key = PyTuple_Pack(4,pw,ph,ptint,palpha);
      assert(pw->ob_refcnt==2);
      Py_DECREF(ph);
      Py_DECREF(pw);
      Py_DECREF(ptint);
      Py_DECREF(palpha);
      PyObject *cache = pt->cached_surfaces;
      Py_INCREF(cache);
      PyObject *cachedsurf = PyDict_GetItem(cache,key);
      dst.x -= shapesurf->w>>1;
      dst.y -= shapesurf->h>>1;
      if (cachedsurf) {
        Py_INCREF(cachedsurf);
        flagged_blit(PySurface_AsSurface(cachedsurf),g_screen,&dst,pt->blend);
        Py_DECREF(cachedsurf);
      } else {
        SDL_Surface *surf = RGBA_copy(shapesurf);
        Tint_Surface(surf,tint);
        PyObject *newsurf = PySurface_New(surf);
        flagged_blit(surf,g_screen,&dst,pt->blend);
        PyDict_SetItem(cache,key,newsurf);
        flagged_blit(surf,g_screen,&dst,pt->blend);
      }
      Py_DECREF(cache);
      Py_DECREF(key);
    } else {
      dst.x -= shapesurf->w>>1;
      dst.y -= shapesurf->h>>1;
      flagged_blit(shapesurf,g_screen,&dst,pt->blend);
    }
  }
  Py_DECREF(shapeobj);
  Py_DECREF(py_pt);
}
inline void CParticle_step_low(struct CParticle *part)
{
  part->ttl --;
  part->x += part->xvel;
  part->y += part->yvel;
  ParticleType_object *type = PyCObject_AsVoidPtr(part->type);
  if (type->orientation_wiggle) {
    part->angle += (type->orientation_wiggle*2*drand48())-type->orientation_wiggle;
  }
  if (type->orientation_incr) {
    part->angle += type->orientation_incr;
    if (part->angle<0) {
      part->angle+=360.0;
    } else if (part->angle>360) {
      part->angle -= 360.0;
    }
  }
  if (part->angle>type->orientation_max) {
    part->angle = type->orientation_max;
  } else if (part->angle<type->orientation_min) {
    part->angle = type->orientation_min;
  }
  if (type->size_wiggle||type->size_incr) {
    if (type->size_wiggle) {
      part->size += (type->size_wiggle*2*drand48())-type->size_wiggle;
    }
    if (type->size_incr) {
      part->size += type->size_incr;
    }
    if (part->size<type->size_min) {
      part->size = type->size_min;
    } else if (part->size>type->size_max) {
      part->size = type->size_max;
    }
  }
}

static PyObject *CParticle_step(PyObject *naught, PyObject *args)
{
  PyObject *pyp;
  if (!PyArg_ParseTuple(args,"O",&pyp)) {
    return NULL;
  }
  struct CParticle *part = PyCObject_AsVoidPtr(pyp);
  CParticle_step_low(part);
  return PyInt_FromLong(part->ttl);
}

static PyObject *CParticle_dec_ttl_and_render(PyObject *naught, PyObject *args)
{
  PyObject *pyp;
  int xoff,yoff;
  if (!PyArg_ParseTuple(args,"Oii",&pyp,&xoff,&yoff)) {
    return NULL;
  }
  struct CParticle *part = PyCObject_AsVoidPtr(pyp);
  CParticle_step_low(part);
  if (part->ttl>0) {
    // XXX step has snuck in here.
    particle_render_low(part,xoff,yoff);
  }
  return PyInt_FromLong(part->ttl);
}

/*static PyObject *particle_Render(PyObject *ignore, PyObject *args)
{
  PyObject *screen;
  int xoff,yoff;
  PyObject *py_particle;
  if (!PyArg_ParseTuple(args,"OOii",&py_particle,&screen,&xoff,&yoff)) {
    return NULL;
  }
  float wf,hf;
  struct CParticle *cparticle = PyCObject_AsVoidPtr(py_particle);
  particle_render_low(cparticle,screen,xoff,yoff);
  Py_INCREF(Py_None);
  return Py_None;
}*/

static void ParticleType_destructor(void *vpt)
{
  ParticleType_object *pt = (ParticleType_object *)vpt;
  Py_CLEAR(pt->shape);
  pt->shapesurf = NULL;
  Py_CLEAR(pt->cached_surfaces);
  free(pt);
}

static PyObject *part_type_alpha2(PyObject *ignore, PyObject *args)
{
  float start, end;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Off",&pypt,&start,&end)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->startalpha = start;
  pt->endalpha = end;
  pt->halfalpha = (start+end)/2.0;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_alpha3(PyObject *ignore, PyObject *args)
{
  float start, end, half;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Offf",&pypt,&start,&half,&end)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->startalpha = start;
  pt->endalpha = end;
  pt->halfalpha = half;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_blend(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  float blendf;
  if (!PyArg_ParseTuple(args,"Of",&pypt,&blendf)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->blend = blendf;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_color3(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  unsigned int start, half, end;
  if (!PyArg_ParseTuple(args,"Oiii",&pypt,&start,&half,&end)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->startcolor = start;
  pt->halfcolor = half;
  pt->endcolor = end;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_create(PyObject *ignore1, PyObject *ignore2)
{
  ParticleType_object *pt = calloc(1,sizeof(ParticleType_object));
  pt->life_min = pt->life_max = 30;
  pt->size_xscale = pt->size_yscale = 1.0;
  pt->size_min = pt->size_max = pt->size = 1.0;
  pt->startalpha = pt->halfalpha = pt->endalpha = 1.0;
  pt->startcolor = pt->halfcolor = pt->endcolor = COLOR_WHITE;
  pt->cached_surfaces = Py_BuildValue("{}");
  PyObject *ret = PyCObject_FromVoidPtr(pt, ParticleType_destructor);
  return ret;
}

static PyObject *part_type_direction(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  float min, max, incr, wiggle;
  if (!PyArg_ParseTuple(args,"Offff", &pypt,&min,&max,&incr,&wiggle)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->dir_min = min;
  pt->dir_max = max;
  pt->dir_range = max-min;
  pt->dir_incr = incr;
  pt->dir_wiggle = wiggle;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_gravity(PyObject *naught, PyObject *args)
{
  float amt,dir;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Off",&pypt,&amt,&dir)) return NULL;
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->grav_amount = amt;
  pt->grav_dir = dir;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_life(PyObject *naught, PyObject *args)
{
  float minf, maxf;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Off",&pypt, &minf, &maxf)) return NULL;
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->life_min = minf;
  pt->life_max = maxf;
  pt->life_range = pt->life_max-pt->life_min;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_orientation(PyObject *naught, PyObject *args)
{
  /* XXX note: 'relative' is defined as setting the orientation angles to
        be relative to the direction of motion. */
  float min,max,incr,wiggle,relativef;
  int relative;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Offfff",&pypt,&min,&max,&incr,&wiggle,&relativef)) {
    return NULL;
  }
  relative = relativef;
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->orientation_min = min;
  pt->orientation_max = max;
  pt->orientation_incr = incr;
  pt->orientation_wiggle = wiggle;
  pt->orientation_relative = relative;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_scale(PyObject *naught, PyObject *args)
{
  float xscale, yscale;
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"Off", &pypt, &xscale, &yscale)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->size_xscale = xscale;
  pt->size_yscale = yscale;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_shape(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  PyObject *shape;
  if (!PyArg_ParseTuple(args,"OO",&pypt,&shape)) {
    return NULL;
  }
  if ((PyObject *)shape->ob_type!=pygame_Surface) {
    fprintf(stderr,"part_type_shape: A pygame Surface is required.\n");
    // XXX raise a proper exception.
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  Py_XDECREF(pt->shape);
  pt->shape = shape;
  Py_INCREF(shape);
  pt->shapesurf = PySurface_AsSurface(shape);
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *part_type_size(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  float min, max, incr, wiggle;
  if (!PyArg_ParseTuple(args, "Offff", &pypt, &min, &max, &incr, &wiggle)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  pt->size_min = min;
  pt->size_max = max;
  pt->size_range = max-min;
  pt->size_incr = incr*pt->size_range;
  pt->size_wiggle = wiggle*pt->size_range;
  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *part_type_speed(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  float min, max, incr, wiggle;
  if (!PyArg_ParseTuple(args, "Offff", &pypt, &min, &max, &incr, &wiggle)) {
    return NULL;
  }
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  pt->speed_min = min;
  pt->speed_range = max-min;
  pt->speed_max = max;
  pt->speed_incr = incr;
  pt->speed_wiggle = wiggle;
  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *part_type_sprite(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  PyObject *sprite;
  float animate, stretch, random;
  if (!PyArg_ParseTuple(args,"OOfff",&pypt, &sprite, &animate, &stretch, &random)) {
    return NULL;
  }
  Py_INCREF(sprite);
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  Py_XDECREF(pt->shape);
  pt->shape = sprite;
  if (animate) {
    pt->animate = 1;
    pt->stretch = stretch;
  } else {
    pt->animate = pt->stretch = 0;
    if (random) {
      warning("XXX need to select a random subimage.\n");
    }
  }
  //warning("part_type_sprite incompletely implemented.\n");
  Py_INCREF(Py_None);
  return Py_None;
}


PyObject *set_SDL_screen(PyObject *naught, PyObject *args)
{
  PyObject *screen;
  if (!PyArg_ParseTuple(args,"O",&screen)) {
    return NULL;
  }
  Py_XDECREF(g_screen_pyobj);
  Py_INCREF(screen);
  g_screen_pyobj = screen;
  g_screen = PySurface_AsSurface(screen);
  COLOR_WHITE = g_screen->format->Rmask|g_screen->format->Gmask|g_screen->format->Bmask;
  Py_INCREF(Py_None);
  return Py_None;
}

PyObject *init_subsystems(PyObject *self, PyObject *args)
{
    SDL_InitSubSystem(SDL_INIT_VIDEO);
    Py_INCREF(Py_None);
    return Py_None;
}

/*
 * Begin new magic GM-style variable.
 */

typedef struct {
        PyObject_HEAD
        /* Type-specific fields */
        int magic;
        int arraylen;
        int array2Dlen;
        PyObject **members;
} GMVar_Object;

static PyObject *GMVar_nb_inplace_add(PyObject *a, PyObject *b)
{
  // XXX needs to detect strings and concat them instead.
  //PyObject *ret = PyNumber_Add(PyNumber_Float(a),PyNumber_Float(b));
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  PyObject *ret = PyNumber_InPlaceAdd(aa,bb);
  if (!ret) {
    return NULL;
  }
  if (aa!=a) {
    GMVar_Object *ga = (GMVar_Object *)a;
    PyObject *member = ga->members[0];
    ga->members[0] = NULL;
    Py_DECREF(member);
    //Py_INCREF(ret);
    ga->members[0] = ret;
    Py_INCREF(a);
    return a;
  }
  return ret;
}


static PyObject *GMVar_nb_add(PyObject *a, PyObject *b)
{
  return PyNumber_Add(GMVar_0(a),GMVar_0(b));
}

static PyObject *GMVar_nb_subtract(PyObject *a, PyObject *b)
{
  return PyNumber_Subtract(GMVar_0(a),GMVar_0(b));
}

static PyObject *GMVar_nb_mul(PyObject *a, PyObject *b)
{
  PyObject *fa = PyNumber_Float(GMVar_0(a));
  PyObject *fb = PyNumber_Float(GMVar_0(b));
  PyObject *ret = PyNumber_Multiply(fa,fb);
  Py_DECREF(fa);
  Py_DECREF(fb);
  return ret;
}

static PyObject *GMVar_nb_float(PyObject *o)
{
  PyObject *ret = PyNumber_Float(((GMVar_Object *)o)->members[0]);
  return ret;
}

static int GMVar_nb_nonzero(PyObject *a)
{
  GMVar_Object *o = (GMVar_Object *)a;
  if (o->members && o->members[0]) {
    int ret = PyObject_IsTrue(o->members[0]);
    return ret;
  }
  return 0;
}

static PyObject *GMVar_nb_int(PyObject *o)
{
  PyObject *ret = PyNumber_Int(((GMVar_Object *)o)->members[0]);
  return ret;
}

static PyObject *GMVar_nb_xor(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_Xor(aa,bb);
}

static PyObject *GMVar_nb_and(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_And(aa,bb);
}

static PyObject *GMVar_nb_or(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_Or(aa,bb);
}

static PyObject *GMVar_nb_divide(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_Divide(aa,bb);
}

static int GMVar_nb_coerce(PyObject **a, PyObject **b)
{
  PyObject *aa=GMVar_0(*a);
  PyObject *bb=GMVar_0(*b);
  int ret = PyNumber_Coerce(&aa,&bb);
  if (ret==-1) return -1;
  *a = aa;
  *b = bb;
  return ret;
}

static PyObject *GMVar_nb_remainder(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_Remainder(aa,bb);
}

static PyObject *GMVar_nb_absolute(PyObject *a)
{
  return PyNumber_Absolute(((GMVar_Object *)a)->members[0]);
}

static PyObject *GMVar_nb_negative(PyObject *a)
{
  return PyNumber_Negative(GMVar_0(a));
}

static PyObject *GMVar_nb_positive(PyObject *a)
{
  return PyNumber_Positive(GMVar_0(a));
}

static PyObject *GMVar_nb_floor_divide(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyNumber_FloorDivide(aa,bb);
}

static PyNumberMethods GMVar_numbermethods = {
        /* For numbers without flag bit Py_TPFLAGS_CHECKTYPES set, all
           arguments are guaranteed to be of the object's type (modulo
	   coercion hacks -- i.e. if the type's coercion function
	   returns other types, then these are allowed as well).  Numbers that
	   have the Py_TPFLAGS_CHECKTYPES flag bit set should check *both*
	   arguments for proper type and implement the necessary conversions
	   in the slot functions themselves. */
	GMVar_nb_add, //binaryfunc nb_add;
	GMVar_nb_subtract, //binaryfunc nb_subtract;
	GMVar_nb_mul, //binaryfunc nb_multiply;
	GMVar_nb_divide, //binaryfunc nb_divide;
	GMVar_nb_remainder, //binaryfunc nb_remainder;
	0, //binaryfunc nb_divmod;
	0, //ternaryfunc nb_power;
	GMVar_nb_negative, //unaryfunc nb_negative;
	GMVar_nb_positive, //unaryfunc nb_positive;
	GMVar_nb_absolute, //unaryfunc nb_absolute;
	GMVar_nb_nonzero, //inquiry nb_nonzero;
	0, //unaryfunc nb_invert;
	0, //binaryfunc nb_lshift;
	0, //binaryfunc nb_rshift;
	GMVar_nb_and, //binaryfunc nb_and;
	GMVar_nb_xor, //binaryfunc nb_xor;
	GMVar_nb_or, //binaryfunc nb_or;
	GMVar_nb_coerce, //coercion nb_coerce;
	GMVar_nb_int, //unaryfunc nb_int;
	0, //unaryfunc nb_long;
        GMVar_nb_float, //unaryfunc nb_float;
	0, //unaryfunc nb_oct;
	0, //unaryfunc nb_hex;
	/* Added in release 2.0 */
	GMVar_nb_inplace_add, /*binaryfunc nb_inplace_add;*/
	0, // binaryfunc nb_inplace_subtract;
	0, // binaryfunc nb_inplace_multiply;
	0, // binaryfunc nb_inplace_divide;
	0, // binaryfunc nb_inplace_remainder;
	0, // ternaryfunc nb_inplace_power;
	0, // binaryfunc nb_inplace_lshift;
	0, //binaryfunc nb_inplace_rshift;
	0, //binaryfunc nb_inplace_and;
	0, //binaryfunc nb_inplace_xor;
	0, //binaryfunc nb_inplace_or;*/

	/* Added in release 2.2 */
	/* The following require the Py_TPFLAGS_HAVE_CLASS flag */
	GMVar_nb_floor_divide, /*binaryfunc nb_floor_divide;
	binaryfunc nb_true_divide;
	binaryfunc nb_inplace_floor_divide;
	binaryfunc nb_inplace_true_divide;*/

	/* Added in release 2.5 */
	/*unaryfunc nb_index;*/
};

static int GMVar_init(PyObject *o, PyObject *args, PyObject *kwds)
{
  GMVar_Object *self = (GMVar_Object *)o;
  if (self) {
    if (self->members) free(self->members);
    self->arraylen = 1;
    self->array2Dlen = 1;
    self->members = malloc(sizeof(PyObject *));
    if (self->members) {
      if (PyArg_ParseTuple(args,"O",self->members)) {
        if (self->members[0]->ob_type==self->ob_type) {
          self->members[0] = ((GMVar_Object *)(self->members[0]))->members[0];
        }
        Py_INCREF(self->members[0]);
        return 0;
      }
    }
  }
  return -1;
}

PyObject *GMVar_mp_subscript(PyObject *o, PyObject *key)
{
  GMVar_Object *self = (GMVar_Object *)o;
  if (PyNumber_Check(key)) {
    int i = PyInt_AsLong(key);
    if (i>=0) {
      if (i<self->arraylen) {
        Py_INCREF(self->members[i]);
        return self->members[i];
        /*GMVar_Object *ret = self->ob_type->tp_alloc(self->ob_type,0);
        GMVar_init(ret,self->members[i],0);
        Py_INCREF(ret);
        return (PyObject *)ret;*/
      } else {
        PyObject *ret = PyInt_FromLong(0);
        return ret;
      }
    }
    return NULL;
  } else if (PySequence_Check(key)) {
    PyObject *i = PySequence_GetItem(key,0);
    PyObject *j = PySequence_GetItem(key,1);
    if (PyNumber_Check(i) && PyNumber_Check(j)) {
      int i_, j_;
      i_ = PyInt_AsLong(i);
      if (i_>=0 && i_<self->arraylen) {
        j_ = PyInt_AsLong(j);
        if (j_>=0 && j_<self->array2Dlen) {
          if (!self->members[i_+(j_*self->arraylen)]) {
            self->members[i_+(j_*self->arraylen)] = PyInt_FromLong(0);
          }
          Py_INCREF(self->members[i_+(j_*self->arraylen)]);
          Py_DECREF(i);
          Py_DECREF(j);
          return self->members[i_+(j_*self->arraylen)];
          /*GMVar_Object *ret = self->ob_type->tp_alloc(self->ob_type,0);
          GMVar_init(ret,self->members[i_+(j_*self->arraylen)],0);
          Py_INCREF(ret);
          return (PyObject *)ret;*/
        }
      }
    }
    Py_DECREF(i);
    Py_DECREF(j);
  }
  return PyInt_FromLong(0);
}

static inline int GMVar_stretch(GMVar_Object *self, int i, int j)
{
  int i_ = (i>self->arraylen?i:self->arraylen);
  int j_ = (j>self->array2Dlen?j:self->array2Dlen);
  int bound = i_*j_;
  PyObject **buf = malloc(bound*sizeof(PyObject *));
  if (!buf) return 0;
  int y = 0;
  int z = 0;
  int a = 0;
  while (y<self->array2Dlen) {
    int x = 0;
    while (x<self->arraylen) {
      buf[z++] = self->members[a++];
      x++;
    }
    while (x<i_) {
      buf[z++] = NULL;
      x++;
    }
    y++;
  }
  while (z<bound) {
    buf[z++] = NULL;
  }
  free(self->members);
  self->members = buf;
  self->arraylen = i_;
  self->array2Dlen = j_;
  return 1;
}

static int GMVar_mp_ass_subscript(PyObject *o, PyObject *key, PyObject *newval)
{
  GMVar_Object *self = (GMVar_Object *)o;
  if (!newval) {
    warning("Assigning a blank val\n");
  }
  PyObject *val = GMVar_0(newval);
  if (PyNumber_Check(key)) {
    int i = PyInt_AsLong(key);
    if (i>=0) {
      Py_XINCREF(val);
      if (i>=self->arraylen) {
        if (!GMVar_stretch(self,i+1,self->array2Dlen)) {
          Py_XDECREF(val);
          return -1;
        }
      }
      Py_XDECREF(self->members[i]);
      self->members[i] = val;
      return 0;
    }
    return -1;
  } else if (PySequence_Check(key)) {
    PyObject *i = PySequence_GetItem(key,0);
    PyObject *j = PySequence_GetItem(key,1);
    if (PyNumber_Check(i) && PyNumber_Check(j)) {
      int i_, j_;
      i_ = PyInt_AsLong(i);
      if (i_>=0) {
        j_ = PyInt_AsLong(j);
        if (j_>=0) {
          if (i_>=self->arraylen || j_>=self->array2Dlen) {
            if (!GMVar_stretch(self,i_+1,j_+1)) {
              Py_DECREF(i);
              Py_DECREF(j);
              return -1;
            }
          }
          Py_XINCREF(val);
          Py_XDECREF(self->members[i_+(j_*self->arraylen)]);
          self->members[i_+(j_*self->arraylen)] = val;
          Py_DECREF(i);
          Py_DECREF(j);
          return 0;
        }
      }
    }
  }
  return -1;
}

static Py_ssize_t GMVar_mp_len(PyObject *pyo)
{
  GMVar_Object *o = (GMVar_Object *)pyo;
  if (PySequence_Check(o->members[0])) return PySequence_Length(o->members[0]);
  return o->arraylen*o->array2Dlen;
  //return 0;
}

static PyMappingMethods GMVar_mappingmethods = {
        GMVar_mp_len,
        GMVar_mp_subscript,
        GMVar_mp_ass_subscript,
        };

static PyMethodDef GMVar_methods[] = {
    {NULL}  /* Sentinel */
};

static PyObject *GMVar_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  GMVar_Object *self;
  self = (GMVar_Object *)type->tp_alloc(type,0);
  self->magic = 0xc0ffee55;
  self->members = malloc(sizeof(PyObject *));
  self->arraylen = self->array2Dlen = 1;
  self->members[0] = NULL;
  return (PyObject *)self;
}

static int GMVar_traverse(PyObject *o, visitproc visit, void *arg)
{
  GMVar_Object *self = (GMVar_Object *)o;
  int bound = self->arraylen*self->array2Dlen;
  int ret;
  while (bound) {
    --bound;
    if (self->members[bound]) {
      ret = visit(self->members[bound],arg);
      if (ret) return ret;
    }
  }
  return 0;
}

static int GMVar_clear(PyObject *o)
{
  GMVar_Object *self = (GMVar_Object *)o;
  int bound = self->arraylen*self->array2Dlen;
  PyObject *memb;
  while (bound) {
    --bound;
    if (self->members[bound]) {
      memb = self->members[bound];
      self->members[bound] = NULL;
      Py_DECREF(memb);
    }
  }
  free(self->members);
  //self->members = malloc(sizeof(PyObject *));
  self->members = NULL;
  self->arraylen = self->array2Dlen = 0;
  return 0;
}

static void GMVar_dealloc(PyObject *o)
{
  GMVar_Object *self = (GMVar_Object *)o;
  int bound = self->arraylen*self->array2Dlen;
  PyObject *memb;
  while (bound) {
    bound--;
    if (self->members[bound]) {
      memb = self->members[bound];
      self->members[bound] = NULL;
      Py_XDECREF(memb);
    }
  }
  if (self->members) {
    free(self->members);
    self->members = NULL;
  }
  self->arraylen = self->array2Dlen = 0;
  self->ob_type->tp_free((PyObject *)self);
}

static PyObject *GMVar_getattro(PyObject *o, PyObject *attr)
{
  GMVar_Object *obj = (GMVar_Object *)o;
  return obj->members[0]->ob_type->tp_getattro(obj->members[0],attr);
}

static int GMVar_setattro(PyObject *o, PyObject *attr, PyObject *val)
{
  GMVar_Object *obj = (GMVar_Object *)o;
  return obj->members[0]->ob_type->tp_setattro(obj->members[0],attr,val);
}

static int GMVar_compare(PyObject *a, PyObject *b)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  if (aa->ob_type->tp_compare) {
    int ret = aa->ob_type->tp_compare(aa,bb);
    Py_DECREF(aa); Py_DECREF(bb);
    return ret;
  }
  PyObject *eq = aa->ob_type->tp_richcompare(aa,bb,Py_EQ);
  if (eq) {
    eq = aa->ob_type->tp_richcompare(aa,bb,Py_LT);
    int truth = PyObject_IsTrue(eq);
    Py_DECREF(eq);
    if (truth) {
      Py_DECREF(aa);
      Py_DECREF(bb);
      return -1;
    }
    eq = aa->ob_type->tp_richcompare(aa,bb,Py_GT);
    truth = PyObject_IsTrue(eq);
    Py_DECREF(eq);
    if (truth) {
      Py_DECREF(aa);
      Py_DECREF(bb);
      return 1;
    }
  }
  Py_DECREF(aa);
  Py_DECREF(bb);
  return 0;
}

static PyObject *GMVar_richcompare(PyObject *a, PyObject *b, int op)
{
  PyObject *aa = GMVar_0(a);
  PyObject *bb = GMVar_0(b);
  return PyObject_RichCompare(aa,bb,op);
}

static PyObject *breakable(PyObject *ig1, PyObject *ig2)
{
  debug("breakable\n");
  Py_INCREF(Py_None);
  return Py_None;
}

long GMVar_hash(PyObject *o)
{
  GMVar_Object *go = (GMVar_Object *)o;
  return go->members[0]->ob_type->tp_hash(go->members[0]);
}

PyObject *GMVar_str(PyObject *o)
{
  GMVar_Object *go = (GMVar_Object *)o;
  return go->members[0]->ob_type->tp_str(go->members[0]);
}

PyObject *GMVar_call(PyObject *o, PyObject *args, PyObject *kwargs)
{
  GMVar_Object *go = (GMVar_Object *)o;
  PyObject *memb = go->members[0];
  Py_INCREF(memb);
  PyObject *ret = PyObject_Call(memb,args,kwargs);
  Py_DECREF(memb);
  return ret;
}

static PyTypeObject GMVar_Type={
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "py_fasttint.GMVar",    /*tp_name*/
        sizeof(GMVar_Object), /*tp_basicsize*/
        0,                         /*tp_itemsize*/
        GMVar_dealloc,             /*tp_dealloc*/
        0,                         /*tp_print*/
        0,                         /*tp_getattr*/
        0,                         /*tp_setattr*/
        GMVar_compare,                         /*tp_compare*/
        0,                         /*tp_repr*/
        &GMVar_numbermethods,                         /*tp_as_number*/
        0,                         /*tp_as_sequence*/
        &GMVar_mappingmethods,                         /*tp_as_mapping*/
        GMVar_hash,                         /*tp_hash */
        GMVar_call,                         /*tp_call*/
        GMVar_str,                         /*tp_str*/
        GMVar_getattro,                         /*tp_getattro*/
        GMVar_setattro,                         /*tp_setattro*/
        0,                         /*tp_as_buffer*/
        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_CHECKTYPES,        /*tp_flags*/
        "GM-compatible magical variable",   /* tp_doc */
        GMVar_traverse,                         /* tp_traverse */
        GMVar_clear,                         /* tp_clear */
        GMVar_richcompare,                         /* tp_richcompare */
        0,                         /* tp_weaklistoffset */
        0,                         /* tp_iter */
        0,                         /* tp_iternext */
        GMVar_methods,             /* tp_methods */
        0,//Noddy_members,             /* tp_members */
        0,                         /* tp_getset */
        0,                         /* tp_base */
        0,                         /* tp_dict */
        0,                         /* tp_descr_get */
        0,                         /* tp_descr_set */
        0,                         /* tp_dictoffset */
        (initproc)GMVar_init,      /* tp_init */
        0,                         /* tp_alloc */
        GMVar_new,                 /* tp_new */
};

typedef PyListObject InstanceListObject;

static PyObject *global_instances_dict;
static PyObject *global_objectinstancebase_type;
static PyObject *global_objectinstancezero_type;
static PyObject *global_particlesystem_type;


static PyObject *RegisterInstancesDict(PyObject *ignore, PyObject *args)
{
  Py_XDECREF(global_instances_dict);
  Py_XDECREF(global_objectinstancebase_type);
  Py_XDECREF(global_objectinstancezero_type);
  Py_XDECREF(global_particlesystem_type);
  if (PyArg_ParseTuple(args,"OOOO",&global_instances_dict,&global_objectinstancebase_type,&global_objectinstancezero_type,&global_particlesystem_type)) {
    Py_INCREF(global_instances_dict);
    Py_INCREF(global_objectinstancebase_type);
    Py_INCREF(global_objectinstancezero_type);
    Py_INCREF(global_particlesystem_type);
  }
  gm6_SpriteData_load_images = PyObject_GetAttrString(gm6,"SpriteData_load_images");
  gm6_BackgroundData_load_images = PyObject_GetAttrString(gm6,"BackgroundData_load_images");
  gm6_Room_Compile = PyObject_GetAttrString(gm6, "Room_Compile");
  gm6_Room_Create = PyObject_GetAttrString(gm6, "Room_Create");
  gm6_Room_InitializeBackgrounds = PyObject_GetAttrString(gm6, "Room_InitializeBackgrounds");
  gm6_Room_AddInstance = PyObject_GetAttrString(gm6, "Room_AddInstance");
  gm6_Room_AddDrawnInstance = PyObject_GetAttrString(gm6, "Room_AddDrawnInstance");
  gm6_Room_ResortDrawnObjects = PyObject_GetAttrString(gm6, "Room_ResortDrawnObjects");
  gm6_Room_SetBackgroundIndex = PyObject_GetAttrString(gm6, "Room_SetBackgroundIndex");
  gm6_Room_StepAlarms = PyObject_GetAttrString(gm6, "Room_StepAlarms");
  gm6_Room_MoveInstances = PyObject_GetAttrString(gm6, "Room_MoveInstances");
  gm6_Room_HandleCollisions = PyObject_GetAttrString(gm6, "Room_HandleCollisions");
  gm6_Room_StepEvents = PyObject_GetAttrString(gm6, "Room_StepEvents");
  gm6_Room_DrawBackgrounds = PyObject_GetAttrString(gm6, "Room_DrawBackgrounds");
  gm6_Room_RenderBackground = PyObject_GetAttrString(gm6, "Room_RenderBackground");
  gm6_Room_Leave = PyObject_GetAttrString(gm6, "Room_Leave");
  gm6_Room_MouseButtonDown = PyObject_GetAttrString(gm6, "Room_MouseButtonDown");
  gm6_Room_MouseButtonUp = PyObject_GetAttrString(gm6, "Room_MouseButtonUp");
  gm6_Room_HandleKeyrelease = PyObject_GetAttrString(gm6, "Room_HandleKeyrelease");
  gm6_Room_HandleKeypress = PyObject_GetAttrString(gm6, "Room_HandleKeypress");
  gmapi_draw_sprite_ext = PyObject_GetAttrString(gmapi,"draw_sprite_ext");
  gmapi_py_ParticleSystem = PyObject_GetAttrString(gmapi,"py_ParticleSystem");
  gmapi_py_draw_props = PyObject_GetAttrString(gmapi,"py_draw_props");
  gmapi_py_self_stack = PyObject_GetAttrString(gmapi,"py_self_stack");
  PyObject *pygame = PyObject_GetAttrString(gmapi,"pygame");
  PyObject *pygame_draw = PyObject_GetAttrString(pygame,"draw");
  pygame_draw_rect = PyObject_GetAttrString(pygame_draw,"rect");
  Py_DECREF(pygame_draw);
  pygame_Surface = PyObject_GetAttrString(pygame,"Surface");
  Py_DECREF(pygame);
  //gm6_Room_DrawView = PyObject_GetAttrString(gm6, "Room_DrawView");
  Py_INCREF(Py_None);
  return Py_None;
}

/*
 * Gets the instance for a PyObject.  Returns a new reference.
 */
static PyObject *PyObject_AsInstance(PyObject *o)
{
  PyObject *ret = GMVar_0(o);
  if ((PyObject *)ret->ob_type==global_objectinstancezero_type || (PyObject *)ret->ob_type==global_objectinstancebase_type || (PyObject *)ret->ob_type==global_particlesystem_type) {
    Py_INCREF(ret);
    return ret;
  }
  PyObject *i = PyNumber_Int(ret);
  ret = PyDict_GetItem(global_instances_dict,i);
  Py_DECREF(i);
  return ret;
}

static PyObject *InstanceList_append(InstanceListObject *self, PyObject *args)
{
  PyObject *arg;
  if (!PyArg_ParseTuple(args,"O",&arg)) {
    return NULL;
  }
  PyObject *item = PyObject_AsInstance(arg);
  if (!item) {
    return NULL;
  }
  PyList_Append((PyObject *)self,item);
  Py_DECREF(item);
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *InstanceList_remove(InstanceListObject *self, PyObject *args)
{
  PyObject *arg;
  if (!PyArg_ParseTuple(args,"O",&arg)) {
    return NULL;
  }
  PyObject *item = PyObject_AsInstance(arg);
  int x = 0;
  while (x<self->ob_size) {
    if (self->ob_item[x]==item) {
      Py_DECREF(self->ob_item[x]);
      self->ob_size--;
      int maxx = self->ob_size;
      while (x<maxx) {
        self->ob_item[x] = self->ob_item[x+1];
        x++;
      }
      self->ob_item[x] = NULL;
      Py_DECREF(item);
      //InstanceList_PossiblyCompress(self);
      Py_INCREF(Py_None);
      return Py_None;
    }
    x++;
  }
  Py_DECREF(item);
  return NULL;
}

static Py_ssize_t InstanceList_length(PyObject *o)
{
  return ((InstanceListObject *)o)->ob_size;
}

static int InstanceList_assitem(PyObject *o, Py_ssize_t i, PyObject *v)
{
  InstanceListObject *il = (InstanceListObject *)o;
  if (i>=il->ob_size) {
    return -1;
  }
  PyObject *item = PyObject_AsInstance(v);
  Py_INCREF(item);
  Py_DECREF(il->ob_item[i]);
  il->ob_item[i] = item;
  return 0;
}

static PySequenceMethods InstanceList_tp_seq = {
        InstanceList_length,
        0, // sq_concat
        0, // sq_repeat
        0, //InstanceList_getitem,
        0, // sq_slice
        InstanceList_assitem, // sq_ass_item
        0, //InstanceList_ass_slice, // sq_ass_slice
        0, // sq_contains
        0, // sq_inplace_concat
        0, // sq_inplace_repeat
};

static PyObject *InstanceList_insert(PyObject *o, PyObject *args)
{
  int idx;
  PyObject *item;
  if (!PyArg_ParseTuple(args, "iO", &idx, &item)) {
    return NULL;
  }
  //InstanceListObject *il = (InstanceListObject *)o;
  PyObject *realitem = PyObject_AsInstance(item);
  int ret = PyList_Insert(o, idx, realitem);
  Py_DECREF(item);
  if (ret) return NULL;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyTypeObject InstanceList_Type;
static PyObject *InstanceList_reduce(PyObject *self, PyObject *args)
{
  PyObject *tuple = PySequence_Tuple(self);
  PyObject *ret = Py_BuildValue("O()O",&InstanceList_Type,tuple);
  Py_DECREF(tuple);
  return ret;
}

static PyObject *InstanceList_setstate(PyObject *self, PyObject *args)
{
  PyObject *state = PyTuple_GET_ITEM(args,0);
  PySequence_SetSlice(self,0,PySequence_Size(self),state);
  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef InstanceList_methods[] = {
        {"append", (PyCFunction)InstanceList_append, METH_VARARGS,
            PyDoc_STR("Add an instance")},
        {"remove", (PyCFunction)InstanceList_remove, METH_VARARGS,
            PyDoc_STR("Remove an instance.")},
        {"insert", InstanceList_insert, METH_VARARGS,
            "Insert an instance into the list." },
        {"__reduce__", InstanceList_reduce, METH_VARARGS,
            NULL},
        {"__setstate__", InstanceList_setstate, METH_VARARGS,
            NULL},
        {NULL,NULL},
};


static int InstanceList_traverse(PyObject *o, visitproc visit, void *arg)
{
  InstanceListObject *self = (InstanceListObject *)o;
  int x = self->ob_size;
  int ret;
  while (x) {
    --x;
    ret = visit(self->ob_item[x],arg);
    if (ret) {
      return ret;
    }
  }
  return 0;
}

static int InstanceList_clear(PyObject *o)
{
  return o->ob_type->tp_base->tp_clear(o);
}

static void InstanceList_dealloc(PyObject *o)
{
  o->ob_type->tp_base->tp_dealloc(o);
}

static PyTypeObject InstanceList_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.InstanceList",
        sizeof(InstanceListObject),
    sizeof(PyObject *),                         /*tp_itemsize*/
    InstanceList_dealloc,                         /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,                         /*tp_compare*/
    0,                         /*tp_repr*/
    0,                         /*tp_as_number*/
    &InstanceList_tp_seq,      /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    0,                         /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,        /*tp_flags*/
    "InstanceList",           /* tp_doc */
    InstanceList_traverse,                       /* tp_traverse */
    InstanceList_clear,                       /* tp_clear */
    0,                       /* tp_richcompare */
    0,                       /* tp_weaklistoffset */
    0,                       /* tp_iter */
    0,                       /* tp_iternext */
    InstanceList_methods,          /* tp_methods */
    0,                       /* tp_members */
    0,                       /* tp_getset */
    0,                       /* tp_base */
    0,                       /* tp_dict */
    0,                       /* tp_descr_get */
    0,                       /* tp_descr_set */
    0,                       /* tp_dictoffset */
    0, //(initproc)InstanceList_init,   /* tp_init */
    0,                       /* tp_alloc */
    0,                       /* tp_new */
};

/*
 * GM_Sprite: GameMaker-compatible sprite objects.
 * 
 */
static PyTypeObject GMSprite_Type; 
struct GMSprite {
        PyObject_HEAD
        int w,h;
        int leftbound,rightbound,topbound,bottombound;
        int xorigin,yorigin;
        int transparent;

        int precise_collision_checking;
        int bounding_box_type;

        int smooth_edges, preload_texture;

        int use_video_memory, load_only_on_use;

        PyObject *name;
        PyObject *subimages;
        PyObject *scalecache;
        };

static PyObject *GMSprite_reduce(PyObject *self, PyObject *args)
{
  struct GMSprite *spr = (struct GMSprite *)self;
  PyObject *cache = spr->scalecache;
  spr->scalecache = NULL;
  Py_XDECREF(cache);
  PyObject *state = Py_BuildValue("(" "ii" "iiii" "ii" "i" "ii" "ii" "ii" "Oi)",
    spr->w, spr->h,
    spr->leftbound, spr->rightbound, spr->topbound, spr->bottombound,
    spr->xorigin, spr->yorigin,
    spr->transparent,
    spr->precise_collision_checking, spr->bounding_box_type,
    spr->smooth_edges, spr->preload_texture,
    spr->use_video_memory, spr->load_only_on_use,
    spr->name, PyList_Size(spr->subimages));
  PyObject *ret = Py_BuildValue("(O()O)",&GMSprite_Type,state);
  return ret;
}

static PyObject *GMSprite_setstate(PyObject *self, PyObject *stateo)
{
  PyObject *state = PyTuple_GET_ITEM(stateo,0);
  struct GMSprite *spr = (struct GMSprite *)self;
  spr->w = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,0));
  spr->h = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,1));
  spr->leftbound = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,2));
  spr->rightbound = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,3));
  spr->topbound = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,4));
  spr->bottombound = PyInt_AsLong(PySequence_Fast_GET_ITEM(state,5));
  spr->xorigin = seqi(state,6);
  spr->yorigin = seqi(state,7);
  spr->transparent = seqi(state,8);
  spr->precise_collision_checking = seqi(state,9);
  spr->bounding_box_type = seqi(state,10);
  spr->smooth_edges = seqi(state,11);
  spr->preload_texture = seqi(state,12);
  spr->use_video_memory = seqi(state,13);
  spr->load_only_on_use = seqi(state,14);
  Py_XDECREF(spr->name);
  spr->name = PySequence_GetItem(state,15);
  Py_INCREF(spr->name);
  Py_XDECREF(spr->subimages);
  spr->subimages = PyList_New(0);
  int numsubimages = seqi(state,16);
  Py_XDECREF(spr->scalecache);
  spr->scalecache = PyDict_New();
  PyObject *li_args = Py_BuildValue("OOi",spr,spr->name,numsubimages);
  PyObject *dump = PyObject_CallObject(gm6_SpriteData_load_images, li_args);
  if (dump) {
    Py_DECREF(dump);
  } else {
    fprintf(stderr,"An exception occurred!\n");
    Py_DECREF(li_args);
    return NULL;
  }
  Py_DECREF(li_args);
  if (PyDict_SetItem(gmapi__dict__, spr->name, self)) {
    printf("Retnull\n");
    return NULL;
  }
  Py_INCREF(Py_None);
  return Py_None;
}

static PyMemberDef GMSprite_members[] = {
        {"w", T_INT, offsetof(struct GMSprite, w), 0, "unscaled sprite width."},
        {"h", T_INT, offsetof(struct GMSprite, h), 0, "unscaled sprite height."},
        {"xorigin", T_INT, offsetof(struct GMSprite, xorigin), 0, ""},
        {"yorigin", T_INT, offsetof(struct GMSprite, yorigin), 0, ""},
        {"leftbound", T_INT, offsetof(struct GMSprite, leftbound), 0, ""},
        {"rightbound", T_INT, offsetof(struct GMSprite, rightbound), 0, ""},
        {"topbound", T_INT, offsetof(struct GMSprite, topbound), 0, ""},
        {"bottombound", T_INT, offsetof(struct GMSprite, bottombound), 0, ""},
        {"transparent", T_INT, offsetof(struct GMSprite, transparent), 0, ""},
        {"use_video_memory", T_INT, offsetof(struct GMSprite, use_video_memory), 0, ""},
        {"load_only_on_use", T_INT, offsetof(struct GMSprite, load_only_on_use), 0, ""},
        {"smooth_edges", T_INT, offsetof(struct GMSprite, smooth_edges), 0, ""},
        {"preload_texture", T_INT, offsetof(struct GMSprite, preload_texture),0,""},
        {"bounding_box_type", T_INT, offsetof(struct GMSprite, bounding_box_type), 0, ""},
        {"precise_collision_checking", T_INT, offsetof(struct GMSprite, precise_collision_checking), 0, ""},
        {"name", T_OBJECT_EX, offsetof(struct GMSprite, name), 0, ""},
        {"subimages",T_OBJECT_EX, offsetof(struct GMSprite, subimages),0, ""},
        {"scalecache", T_OBJECT_EX, offsetof(struct GMSprite, scalecache), 0, ""},
        {NULL}
};
static PyMethodDef GMSprite_methods[] = {
        {"__reduce__", GMSprite_reduce, METH_VARARGS, NULL,},
        {"__setstate__", GMSprite_setstate, METH_VARARGS,
            PyDoc_STR("Unpickler")},
        {NULL,NULL},
        };

static PyObject *GMSprite_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  struct GMSprite *ret = (struct GMSprite *)(type->tp_alloc(type,0));
  Py_INCREF(Py_None);
  ret->name = Py_None;
  ret->subimages = PyList_New(0);
  ret->scalecache = PyDict_New();
  return (PyObject *)ret;
}

static int GMSprite_traverse(PyObject *o, visitproc visit, void *arg)
{
  struct GMSprite *self = (struct GMSprite *)o;
  int ret;
  if (self->name) {
    ret = visit(self->name, arg);
    if (ret) return ret;
  }
  if (self->subimages) {
    ret = visit(self->subimages, arg);
    if (ret) return ret;
  }
  if (self->scalecache) {
    ret = visit(self->scalecache, arg);
    if (ret) return ret;
  }
  return 0;
}

static int GMSprite_clear(PyObject *o)
{
  struct GMSprite *self = (struct GMSprite *)o;
  PyObject *tmp = self->name;
  self->name = NULL;
  Py_DECREF(tmp);
  tmp = self->subimages;
  self->subimages = NULL;
  Py_DECREF(tmp);
  tmp = self->scalecache;
  self->scalecache = NULL;
  Py_DECREF(tmp);
  return 0;
}

static void GMSprite_dealloc(PyObject *o)
{
  struct GMSprite *self = (struct GMSprite *)o;
  Py_CLEAR(self->name);
  Py_CLEAR(self->subimages);
  Py_CLEAR(self->scalecache);
  self->ob_type->tp_free(o);
}

static PyObject *GMSprite_str(PyObject *self)
{
  int x = 0;
  PyObject *game = PyObject_GetAttrString(gm6,"game");
  PyObject *sprites = PyObject_GetAttrString(game,"sprites");
  Py_DECREF(game);
  int xmax = PyList_Size(sprites);
  while (x<xmax) {
    if (PySequence_Fast_GET_ITEM(sprites,x)==self) {
      PyObject *ret = PyString_FromFormat("%d",x);
      Py_DECREF(sprites);
      return ret;
    }
    x ++;
  }
  Py_DECREF(sprites);
  return NULL;
}

static PyObject *GMSprite_nb_int(PyObject *self)
{
  return Game_Sequence_Index(self, "sprites");
}

static PyObject *GMSprite_nb_float(PyObject *self)
{
  return Game_Sequence_Index_F(self, "sprites");
}

static PyNumberMethods GMSprite_numbermethods = {
        /* For numbers without flag bit Py_TPFLAGS_CHECKTYPES set, all
           arguments are guaranteed to be of the object's type (modulo
	   coercion hacks -- i.e. if the type's coercion function
	   returns other types, then these are allowed as well).  Numbers that
	   have the Py_TPFLAGS_CHECKTYPES flag bit set should check *both*
	   arguments for proper type and implement the necessary conversions
	   in the slot functions themselves. */
	0, //binaryfunc nb_add;
	0, //binaryfunc nb_subtract;
        0, //binaryfunc nb_multiply;
	0, //binaryfunc nb_divide;
	0, //binaryfunc nb_remainder;
	0, //binaryfunc nb_divmod;
	0, //ternaryfunc nb_power;
	0, //unaryfunc nb_negative;
	0, //unaryfunc nb_positive;
	0, //unaryfunc nb_absolute;
	0, //inquiry nb_nonzero;
	0, //unaryfunc nb_invert;
	0, //binaryfunc nb_lshift;
	0, //binaryfunc nb_rshift;
	0, //binaryfunc nb_and;
	0, //binaryfunc nb_xor;
	0, //binaryfunc nb_or;
	0, //coercion nb_coerce;
	GMSprite_nb_int, //unaryfunc nb_int;
	0, //unaryfunc nb_long;
        GMSprite_nb_float, //unaryfunc nb_float;
};

static PyTypeObject GMSprite_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.GMSprite",
        sizeof(struct GMSprite),
    0,                         /*tp_itemsize*/
    GMSprite_dealloc,                         /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    0,//Object_AsInt_compare,          /*tp_compare*/
    0,                         /*tp_repr*/
    &GMSprite_numbermethods,   /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    GMSprite_str,              /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,
        "GM-compatible sprites.",
        GMSprite_traverse,                         /* tp_traverse */
        GMSprite_clear,                         /* tp_clear */
    Object_AsInt_richcompare,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
        GMSprite_methods, /* tp_methods */
        GMSprite_members,
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
        0, /* tp_init */
        0, /* tp_alloc */
        GMSprite_new, /* tp_new */
        };

static PyObject *GMSprite_from_surface(PyObject *ignore, PyObject *args)
{
  PyObject *pygsurf, *name;
  if (!PyArg_ParseTuple(args,"O|O",&pygsurf, &name)) {
    return NULL;
  }
  PyObject *tmptuple = PyTuple_New(0);
  PyObject *tmpdict = PyDict_New();
  struct GMSprite *self = (struct GMSprite *)GMSprite_Type.tp_new(&GMSprite_Type,tmptuple,tmpdict);
  Py_DECREF(tmptuple);
  Py_DECREF(tmpdict);
  if (name) {
    Py_INCREF(name);
    self->name = name;
  }
  SDL_Surface *sdl_surf = PySurface_AsSurface(pygsurf);
  self->rightbound = self->w = sdl_surf->w;
  self->bottombound = self->h = sdl_surf->h;
  self->leftbound = self->topbound = 0;
  // XXX is this a valid cast?
  if (!PyList_Check(self->subimages)) {
    Py_XDECREF(self->subimages);
    self->subimages = PyList_New(0);
    printf("Subimages initialized\n");
  }
  PyList_Append(self->subimages,PySurface_New(SDL_ConvertSurface(sdl_surf,g_screen->format,0)));
  //XXX does this need to be incref'd? Don't think so, but...
  return (PyObject *)self;
}

static PyObject *BackgroundData_reduce(PyObject *o, PyObject *args);

static PyObject *BackgroundData_setstate(PyObject *self, PyObject *stateo)
{
  PyObject *state = PyTuple_GET_ITEM(stateo,0);
  struct BackgroundData *bg = (struct BackgroundData *)self;
  bg->transparent = seqi(state,1);
  Py_XDECREF(bg->name);
  bg->name = PySequence_GetItem(state,0);
  Py_INCREF(bg->name);
  Py_XDECREF(bg->scaled_data_cache);
  bg->scaled_data_cache = PyDict_New();
  Py_XDECREF(bg->surf);
  Py_INCREF(Py_None);
  bg->surf = Py_None;
  PyObject *li_args = Py_BuildValue("(O)",bg);
  PyObject *dump = PyObject_CallObject(gm6_BackgroundData_load_images, li_args);
  if (dump) {
    Py_DECREF(dump);
  } else {
    fprintf(stderr,"An exception occurred!\n");
    return NULL;
  }
  Py_DECREF(li_args);
  if (PyDict_SetItem(gmapi__dict__, bg->name, self)) {
    printf("Retnull\n");
    return NULL;
  }
  Py_INCREF(Py_None);
  return Py_None;
}


static PyObject *BackgroundData_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  struct BackgroundData *ret = (struct BackgroundData *)(type->tp_alloc(type,0));
  Py_INCREF(Py_None);
  ret->name = Py_None;
  Py_INCREF(Py_None);
  ret->surf = Py_None;
  ret->scaled_data_cache = PyDict_New();
  return (PyObject *)ret;
}

static int BackgroundData_traverse(PyObject *bgo, visitproc visit, void *arg)
{
  int ret;
  struct BackgroundData *bg = (struct BackgroundData *)bgo;
  if (bg->name) {
    ret = visit(bg->name,arg);
    if (ret) return ret;
  }
  if (bg->surf) {
    ret = visit(bg->surf,arg);
    if (ret) return ret;
  }
  if (bg->scaled_data_cache) {
    ret = visit(bg->scaled_data_cache,arg);
    if (ret) return ret;
  }
  return 0;
}

static int BackgroundData_clear(PyObject *bgo)
{
  struct BackgroundData *bg = (struct BackgroundData *)bgo;
  Py_CLEAR(bg->name);
  Py_CLEAR(bg->surf);
  Py_CLEAR(bg->scaled_data_cache);
  return 0;
}

static void BackgroundData_dealloc(PyObject *o)
{
  struct BackgroundData *bg = (struct BackgroundData *)o;
  Py_CLEAR(bg->name);
  Py_CLEAR(bg->surf);
  Py_CLEAR(bg->scaled_data_cache);
  o->ob_type->tp_free(o);
}

static PyMemberDef BackgroundData_members[] = {
        {"name", T_OBJECT_EX, offsetof(struct BackgroundData, name), 0, ""},
        {"surf", T_OBJECT_EX, offsetof(struct BackgroundData, surf), 0, ""},
        {"scaled_data_cache", T_OBJECT_EX, offsetof(struct BackgroundData, scaled_data_cache), 0, ""},
        {"w", T_INT, offsetof(struct BackgroundData, w), 0, ""},
        {"h", T_INT, offsetof(struct BackgroundData, h), 0, ""},
        {"transparent", T_INT, offsetof(struct BackgroundData, transparent), 0, ""},
        };

static PyObject *BackgroundData_nb_int(PyObject *self)
{
  return Game_Sequence_Index(self, "backgrounds");
}


static PyObject *BackgroundData_nb_float(PyObject *self)
{
  return Game_Sequence_Index_F(self, "backgrounds");
}

static PyObject *BackgroundData_str(PyObject *self)
{
  return Game_Sequence_Index_S(self,"backgrounds");
}

static PyNumberMethods BackgroundData_numbermethods = {
        /* For numbers without flag bit Py_TPFLAGS_CHECKTYPES set, all
           arguments are guaranteed to be of the object's type (modulo
	   coercion hacks -- i.e. if the type's coercion function
	   returns other types, then these are allowed as well).  Numbers that
	   have the Py_TPFLAGS_CHECKTYPES flag bit set should check *both*
	   arguments for proper type and implement the necessary conversions
	   in the slot functions themselves. */
	0, //binaryfunc nb_add;
	0, //binaryfunc nb_subtract;
        0, //binaryfunc nb_multiply;
	0, //binaryfunc nb_divide;
	0, //binaryfunc nb_remainder;
	0, //binaryfunc nb_divmod;
	0, //ternaryfunc nb_power;
	0, //unaryfunc nb_negative;
	0, //unaryfunc nb_positive;
	0, //unaryfunc nb_absolute;
	0, //inquiry nb_nonzero;
	0, //unaryfunc nb_invert;
	0, //binaryfunc nb_lshift;
	0, //binaryfunc nb_rshift;
	0, //binaryfunc nb_and;
	0, //binaryfunc nb_xor;
	0, //binaryfunc nb_or;
	0, //coercion nb_coerce;
	BackgroundData_nb_int, //unaryfunc nb_int;
	0, //unaryfunc nb_long;
        BackgroundData_nb_float, //unaryfunc nb_float;
};


static PyMethodDef BackgroundData_methods[] = {
        {"__reduce__", BackgroundData_reduce, METH_VARARGS, NULL,},
        {"__setstate__", BackgroundData_setstate, METH_VARARGS,
            PyDoc_STR("Unpickler")},
        {NULL,NULL},
        };

static PyTypeObject BackgroundData_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.BackgroundData",
        sizeof(struct BackgroundData),
    0,                         /*tp_itemsize*/
    BackgroundData_dealloc,                         /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    Object_AsInt_compare,    /*tp_compare*/
    0,                         /*tp_repr*/
    &BackgroundData_numbermethods,   /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    BackgroundData_str,              /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,
        "GM-compatible backgrounds.",
        BackgroundData_traverse,                         /* tp_traverse */
        BackgroundData_clear,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
        BackgroundData_methods, /* tp_methods */
        BackgroundData_members,
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
        0, /* tp_init */
        0, /* tp_alloc */
        BackgroundData_new, /* tp_new */
        };


static PyObject *BackgroundData_reduce(PyObject *o, PyObject *args)
{
  struct BackgroundData *bg = (struct BackgroundData *)o;
  PyObject *state = Py_BuildValue("(Oi)",bg->name,bg->transparent);
  PyObject *ret = Py_BuildValue("(O()O)",&BackgroundData_Type,state);
  Py_DECREF(state);
  return ret;
}

time_t this_second = -1;
static int current_fps;
static time_t last_second;

static PyObject *Room_Draw(PyObject *ignore, PyObject *args)
{
  PyObject *room, *py_surface;
  if (!PyArg_ParseTuple(args,"OO",&room, &py_surface)) {
    return NULL;
  }
  time_t now = time(NULL);
  if (now!=this_second) {
    PyObject *fpsint = PyInt_FromLong(current_fps);
    int rv = PyObject_SetAttrString(gmapi, "fps", fpsint);
    Py_DECREF(fpsint);
    if (rv==-1) {
      return NULL;
    }
    last_second = this_second;
    this_second = now;
    current_fps = 0;
    //debug_fps_delay_intended = 0
  }
  current_fps++;
  
  PyObject *ve = PyObject_GetAttrString(room,"view_enabled");
  if (!ve) {
    return NULL;
  }
  int views = PyObject_IsTrue(ve);
  Py_DECREF(ve);
  if (views==-1) return NULL;
  PyObject *drawn_objects = PyObject_GetAttrString(room,"drawn_objects");
  if (!drawn_objects) {
    return NULL;
  }
  PyObject *drawn = PySequence_Tuple(drawn_objects);
  Py_DECREF(drawn_objects);
  if (!drawn) {
    fprintf(stderr,"Error making tuple from drawn_objects\n");
    return NULL;
  }
  Py_ssize_t numdrawn = PyTuple_Size(drawn);
  int depthsorted_bytelen = sizeof(PyObject *)*numdrawn;
  PyObject **depthsorted = malloc(depthsorted_bytelen);
  if (!depthsorted) return NULL;
  memcpy(depthsorted, ((PyTupleObject *)drawn)->ob_item, depthsorted_bytelen);
  SDL_Surface *sdl_surf = PySurface_AsSurface(py_surface);
  if (!views) {
    g_view_xscale = g_view_yscale = 1.0;
    PyObject *db = PyObject_GetAttrString(room,"DrawBackgrounds");
    if (db) {
      // PyTuple_SetItem consumes a reference.
      PyObject *dbargs = PyTuple_New(3);
      Py_INCREF(py_surface);
      PyTuple_SetItem(dbargs, 0, py_surface);
      PyObject *zero = PyInt_FromLong(0);
      Py_INCREF(zero);
      PyTuple_SetItem(dbargs, 1, zero);
      PyTuple_SetItem(dbargs, 2, zero);
      PyObject *draw_backgrounds_ret = PyObject_Call(db,dbargs,NULL);
      if (!draw_backgrounds_ret) {
        fprintf(stderr,"Drawing backgrounds failed.\n");
        return NULL;
      }
      Py_DECREF(draw_backgrounds_ret);
      Py_DECREF(dbargs);
      Py_DECREF(db);
    }
    if (PyErr_Occurred()) {
      return NULL;
    }
    /* # XXX assigning to objects_depthsorted means that removing an object
     # while a room
     # is being drawn won't prevent it from being drawn during this step,
     # but will prevent it from preventing another object from being drawn
     # during this step.  I don't know how it behaves in GM proper, though.*/
    PyObject *tiles = PyObject_GetAttrString(room,"tiles");
    if (!tiles) {
      free(depthsorted);
      return NULL;
    }
    int numtiles = PySequence_Size(tiles);
    if (numtiles) {
      SDL_Rect srcrect, dstrect;
      PyObject *game = PyObject_GetAttrString(gmapi,"game");
      PyObject *game_backgrounds = PyObject_GetAttrString(game,"backgrounds");
      Py_DECREF(game);
      int x = 0;
      while (x<numtiles) {
        struct RoomTileData *tile = (struct RoomTileData *)PyList_GetItem(tiles,x);
        /* Returns a borrowed reference */
        if (!tile) {
          free(depthsorted);
          Py_DECREF(game_backgrounds);
          Py_DECREF(tiles);
          return NULL;
        }
        struct BackgroundData *bg = (struct BackgroundData *)PySequence_GetItem(game_backgrounds,tile->background_idx);
        dstrect.x = tile->x;
        dstrect.y = tile->y;
        srcrect.x = tile->tilex;
        srcrect.y = tile->tiley;
        srcrect.w = tile->w;
        srcrect.h = tile->h;
        if (SDL_BlitSurface(PySurface_AsSurface(bg->surf), &srcrect, sdl_surf, &dstrect)) {
          free(depthsorted);
          Py_DECREF(game_backgrounds);
          Py_DECREF(tiles);
          return NULL;
        }
        x++;
      }
      Py_DECREF(game_backgrounds);
    }
    Py_DECREF(tiles);
    //room.view_current = 0
    PyObject_SetAttrString(room,"view_current",PyInt_FromLong(0));
    int x = 0;
    PyObject *draw_args;
    while (x<numdrawn) {
      PyObject *Draw = PyObject_GetAttrString(depthsorted[x], "Draw");
      if (Draw) {
        draw_args = Py_BuildValue("()");//O)",depthsorted[x]);
        PyObject *drawret = PyObject_CallObject(Draw,draw_args);
        Py_DECREF(Draw);
        Py_DECREF(draw_args);
        if (!drawret) {
          fprintf(stderr,"Error!\n");
          return NULL;
        }
      } else {
        fprintf(stderr,"ERror!\n");
        return NULL;
      }
      x++;
    }
  } else {
    PyObject *view_visible = PyObject_GetAttrString(room,"view_visible");
    if (!view_visible) {
      free(depthsorted);
      return NULL;
    }
    //PyObject *DrawView = PyObject_GetAttrString(room,"DrawView");
    int x = 0;
    while (x<8) {
      PyObject *vv = PySequence_GetItem(view_visible,x);
      if (vv) {
        int rv = PyObject_IsTrue(vv);
        Py_DECREF(vv);
        if (rv==1) {
          //PyObject *xo = PyInt_FromLong(x);
          if (!Room_DrawView((struct GMRoom *)room,py_surface,x,drawn)) {
            free(depthsorted);
            Py_DECREF(view_visible);
            Py_DECREF(drawn);
            return NULL;
          }
          //PyObject *dvr = invoke_4(gm6_Room_DrawView, room, py_surface, xo, drawn);
          //Py_DECREF(xo);
          /*if (!dvr) {
            //Py_DECREF(DrawView);
            Py_DECREF(view_visible);
            Py_DECREF(drawn);
            free(depthsorted);
            return NULL;
          }*/
          //Py_DECREF(dvr);
        } else if (rv==-1) {
          Py_DECREF(view_visible);
          Py_DECREF(drawn);
          free(depthsorted);
          return NULL;
        }
      } else {
        x = 8;
      }
      x++;
    }
    //Py_DECREF(DrawView);
    Py_DECREF(view_visible);
  }
  free(depthsorted);
  Py_DECREF(drawn);
  Py_INCREF(Py_None);
  return Py_None;
}

static inline PyObject *GMVar_0(PyObject *a)
{
  if (a->ob_type==&GMVar_Type) return ((GMVar_Object *)a)->members[0];
  return a;
}

static ParticleType_object *PyObject_AsParticleType(PyObject *o)
{
  if (o->ob_type==&GMVar_Type) {
    if (((GMVar_Object *)o)->members[0]->ob_type!=g_cobject_type_pointer) return NULL;
    return PyCObject_AsVoidPtr(((GMVar_Object *)o)->members[0]);
  }
  if (o->ob_type==g_cobject_type_pointer) return PyCObject_AsVoidPtr(o);
  warning("Returning NULL!\n");
  return NULL;
}

static void gmvar_modinit(PyObject *module)
{
  GMVar_Type.tp_new = GMVar_new;
  if (PyType_Ready(&GMVar_Type)<0) return;
  Py_INCREF(&GMVar_Type);
  PyModule_AddObject(module, "GMVar", (PyObject *)&GMVar_Type);
  if (PyType_Ready(&GMSprite_Type)<0) {
    fprintf(stderr,"GMSprite_Type not ready.\n");
    return;
  }
  PyModule_AddObject(module, "GMSprite", (PyObject *)&GMSprite_Type);
  Py_INCREF(&GMSprite_Type);
  if (PyType_Ready(&BackgroundData_Type)<0) {
    fprintf(stderr, "BackgroundData_Type not ready.\n");
    return;
  }
  PyModule_AddObject(module, "BackgroundData", (PyObject *)&BackgroundData_Type);
  Py_INCREF(&BackgroundData_Type);
  if (PyType_Ready(&RoomTileData_Type)<0) {
    fprintf(stderr, "RoomTileData_Type not ready.\n");
    return;
  }
  PyModule_AddObject(module, "RoomTileData", (PyObject *)&RoomTileData_Type);
  Py_INCREF(&RoomTileData_Type);
  Py_INCREF(&PyList_Type);
  InstanceList_Type.tp_base = &PyList_Type;
  //InstanceList_Type.tp_new = PyList_Type.tp_new;
  //InstanceList_Type.tp_alloc = PyList_Type.tp_alloc;
  if (PyType_Ready(&InstanceList_Type)<0) return;
  Py_INCREF(&InstanceList_Type);
  PyModule_AddObject(module, "InstanceList", (PyObject *)&InstanceList_Type);
  if (PyType_Ready(&GMRoom_Type)<0) {
    fprintf(stderr,"GMRoom_Type not ready.\n");
    return;
  }
  Py_INCREF(&GMRoom_Type);
  PyModule_AddObject(module,"GMRoom", (PyObject *)&GMRoom_Type);
  if (PyType_Ready(&Floatlist8_Type)<0) {
    fprintf(stderr, "Floatlist8_Type not ready.\n");
    return;
  }
  Py_INCREF(&Floatlist8_Type);
  PyModule_AddObject(module, "Floatlist8", (PyObject *)&Floatlist8_Type);
  if (PyType_Ready(&Beancounter_Type)<0) {
    fprintf(stderr,"Beancounter_Type not ready.\n");
    return;
  }
  Py_INCREF(&Beancounter_Type);
  PyModule_AddObject(module, "Beancounter", (PyObject *)&Beancounter_Type);
}

PyObject *sprite_scalecache_add(struct GMSprite *spr, PyObject *key,
  int subimg, int w, int h, int clockwise, PyObject *coloro, int colori, 
  int alpha, int flatten)
{
  /* subimg is validated by the caller. */
  PyObject *srcimg = PySequence_GetItem(spr->subimages,subimg);
  if (!srcimg) {
    fprintf(stderr,"sequence error.\n");
    return NULL;
  }
  PyObject *osrcimg = srcimg;
  if (h<0) {
    if (w<0) {
      SDL_Surface *flipped = flip_vh(PySurface_AsSurface(srcimg));
      Py_DECREF(srcimg);
      srcimg = PySurface_New(flipped);
      w = -w;
      //xscale = -xscale;
    } else {
      /* Only flipping vertically */
      SDL_Surface *flipped = flip_v(PySurface_AsSurface(srcimg));
      Py_DECREF(srcimg);
      srcimg = PySurface_New(flipped);
    }
    h = -h;
  } else if (w<0) {
    SDL_Surface *flipped = flip_h(PySurface_AsSurface(srcimg));
    Py_DECREF(srcimg);
    srcimg = PySurface_New(flipped);
    w = -w;
  }
  SDL_Surface *srcimg_sdl = PySurface_AsSurface(srcimg);
  if (clockwise) {
    if (w!=srcimg_sdl->w || h!=srcimg_sdl->h) {
      float wfac = w;
      wfac /= srcimg_sdl->w;
      float hfac = h;
      hfac /= srcimg_sdl->h;
      if (wfac!=hfac) {
        // XXX should honour smoothing directive?  XXX is this supposed to be
        // clockwise or counterclockwise?
        SDL_Surface *scaledsurf = zoomSurface(srcimg_sdl, wfac, hfac, 0);
        SDL_Surface *rotatedsurf = rotozoomSurface(scaledsurf, clockwise, 1.0, 0);
        SDL_FreeSurface(scaledsurf);
        Py_DECREF(srcimg);
        srcimg = PySurface_New(rotatedsurf);
        srcimg_sdl = rotatedsurf;
      } else if (w!=srcimg_sdl->w) {
        /* Is this clockwise or counterclockwise? */
        SDL_Surface *rotated = rotozoomSurface(srcimg_sdl, clockwise, wfac, 0);
        Py_DECREF(srcimg);
        srcimg = PySurface_New(rotated);
        srcimg_sdl = rotated;
      }
    } else {
      /* Not scaled, just rotated */
      SDL_Surface *rotated = rotozoomSurface(srcimg_sdl, clockwise, 1.0, 0);
      Py_DECREF(srcimg);
      srcimg = PySurface_New(rotated);
      srcimg_sdl = rotated;
    }
  } else if (w!=srcimg_sdl->w || h!=srcimg_sdl->h) {
    float wfac = w;
    wfac /= srcimg_sdl->w;
    float hfac = h;
    hfac /= srcimg_sdl->h;
    SDL_Surface *zoomed = zoomSurface(srcimg_sdl, wfac, hfac, 0);
    Py_DECREF(srcimg);
    srcimg = PySurface_New(zoomed);
    srcimg_sdl = zoomed;
  }
  if (colori!=COLOR_WHITE) {
    if (srcimg==osrcimg) {
      PyObject *copy = PyObject_GetAttrString(srcimg,"copy");
      PyObject *args = Py_BuildValue("()");
      PyObject *dupe = PyObject_CallObject(copy,args);
      Py_DECREF(args);
      Py_DECREF(copy);
      Py_DECREF(srcimg);
      srcimg = dupe;
    }
    PyObject *fill = PyObject_GetAttrString(srcimg,"fill");
    PyObject *args = Py_BuildValue("(i)",colori);
    PyObject *res = PyObject_CallObject(fill,args);
    Py_DECREF(args);
    Py_DECREF(res);
  }
  if (flatten) {
    SDL_Surface *srcimg_sdl = PySurface_AsSurface(srcimg);
    SDL_Surface *newsurf_sdl = SDL_CreateRGBSurface(
      SDL_SWSURFACE,
      srcimg_sdl->w,
      srcimg_sdl->h,
      32,
      srcimg_sdl->format->Rmask, srcimg_sdl->format->Gmask, 
      srcimg_sdl->format->Bmask, srcimg_sdl->format->Amask);
    SDL_FillRect(newsurf_sdl, NULL, SDL_MapRGBA(newsurf_sdl->format,0,0,0,0));
    SDL_BlitSurface(srcimg_sdl, NULL, newsurf_sdl, NULL);
    Py_DECREF(srcimg);
    if (alpha!=255) {
      SDL_SetAlpha(newsurf_sdl,SDL_SRCALPHA,alpha);
      SDL_Surface *newsurf2 = SDL_CreateRGBSurface(SDL_SWSURFACE,
        srcimg_sdl->w,
        srcimg_sdl->h, 32, g_screen->format->Rmask, g_screen->format->Gmask, g_screen->format->Bmask, g_screen->format->Amask);
      SDL_BlitSurface(newsurf_sdl,NULL,newsurf2,NULL);
      SDL_FreeSurface(newsurf_sdl);
      newsurf_sdl = newsurf2;
      //SDL_imageFilterSubByte(newsurf_sdl->pixels,newsurf_sdl->pixels,newsurf_sdl->pitch*newsurf_sdl->h,255-alpha);
    }
    srcimg = PySurface_New(newsurf_sdl);
  } else if (alpha!=255) {
    if (srcimg==osrcimg) {
      PyObject *copy = PyObject_GetAttrString(srcimg,"copy");
      PyObject *args = Py_BuildValue("()");
      PyObject *dupe = PyObject_CallObject(copy,args);
      Py_DECREF(args);
      Py_DECREF(copy);
      Py_DECREF(srcimg);
      srcimg = dupe;
    }
    SDL_Surface *srcimg_sdl = PySurface_AsSurface(srcimg);
    SDL_LockSurface(srcimg_sdl);
    SDL_SetAlpha(srcimg_sdl,SDL_SWSURFACE|SDL_SRCALPHA,alpha);
    Translucentify(srcimg_sdl,alpha);
    SDL_UnlockSurface(srcimg_sdl);
  }
  if (!srcimg) {
    fprintf(stderr,"No source?\n");
    return NULL;
  }
  if (PyDict_SetItem(spr->scalecache,key,srcimg)) {
    fprintf(stderr,"Error storing item in scalecache.\n");
    return NULL;
  }
  return srcimg;
}

static PyObject *draw_sprite_ext(PyObject *ignore, PyObject *args)
{
  fprintf(stderr,"This does not behave properly on Mac, and does not provide the desired speed boost on Linux.\n");
  return NULL;
  PyObject *spr_arg;
  int subimg, x, y;
  float xscale, yscale;
  int clockwise;
  PyObject *color;
  float alpha_f;
  if (!PyArg_ParseTuple(args,"OiiiffiOf", &spr_arg, &subimg, &x, &y,
        &xscale, &yscale, &clockwise, &color, &alpha_f)) return NULL;
  int alpha = alpha_f * 255;
  if (alpha>255) {
    alpha=255;
  } else if (alpha<=0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  struct GMSprite *spr = (struct GMSprite *)GMVar_0(spr_arg);
  PyObject *spri_o = PyNumber_Int((PyObject *)spr);
  int spri = PyInt_AsLong(spri_o);
  Py_DECREF(spri_o);
  if (spri==-1) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  int numsubimages = PySequence_Size(spr->subimages);
  if (subimg>=numsubimages) {
    subimg = subimg % numsubimages;
  } else if (subimg==-1) {
    PyObject *self = PyObject_GetAttrString(gmapi,"self");
    PyObject *image_index = PyObject_GetAttrString(self,"image_index");
    Py_DECREF(self);
    subimg = PyInt_AsLong(image_index);
    Py_DECREF(image_index);
  }
  color = GMVar_0(color);
  struct GMRoom *room = (struct GMRoom *)PyObject_GetAttrString(gmapi,"room");
  int colori = color_as_int(color);
  PyObject *cachedsurf;
  PyObject *dp = PyObject_GetAttrString(gmapi,"py_draw_props");
  PyObject *blend = PyObject_GetAttrString(dp,"blend_mode");
  int blend_mode = PyInt_AsLong(GMVar_0(blend));
  Py_DECREF(blend);
  Py_DECREF(dp);
  int flatten = blend_mode_flatten(blend_mode);
  if (xscale*spr->w*g_view_xscale!=spr->w || yscale*spr->h*g_view_yscale!=spr->h || colori!=COLOR_WHITE || alpha!=255 || clockwise || flatten) {
    PyObject *key = Py_BuildValue("iiiiiiii",subimg, xscale*spr->w*g_view_xscale, yscale*spr->h*g_view_yscale, clockwise, colori, alpha, flatten);
    cachedsurf = PyDict_GetItem(spr->scalecache, key);
    if (cachedsurf) {
      Py_INCREF(cachedsurf);
    } else {
      cachedsurf = sprite_scalecache_add(spr, key, subimg, xscale*spr->w, yscale*spr->h, clockwise, color, colori, alpha, flatten);
      if (!cachedsurf) {
        Py_DECREF(room);
        Py_DECREF(key);
        return NULL;
      }
    }
    Py_DECREF(key);
  } else {
    cachedsurf = PySequence_GetItem(spr->subimages, subimg);
  }
  //XXX does PySurface_AsSurface hold on to a reference?
  x -= room->view_xview->f[room->view_current];
  y -= room->view_yview->f[room->view_current];
  int xorig = spr->xorigin;
  int yorig = spr->yorigin;
  if (clockwise) {
    PyObject *orig_surf_py = PySequence_GetItem(spr->subimages,subimg);
    SDL_Surface *orig_surf_sdl = PySurface_AsSurface(orig_surf_py);
    int vx = orig_surf_sdl->w - xorig;
    int vy = orig_surf_sdl->h - yorig;
    float mag = hypot(vx,vy);
    float vx_over_vy = vx;
    vx_over_vy /= vy;
    int angle = atan(vx_over_vy);
    angle += clockwise;
    if (angle>=360) angle-=360;
    xorig += mag*cos(angle);
    yorig += mag*sin(angle);
    Py_DECREF(orig_surf_py);
    // XXX test in work to see if offsets are right or quandrantwise stiff is needed.
    //offset_dist = point_distance(xorig,yorig,srcimg.get_width(),srcimg.get_height())
    //offset_dire = point_direction(xorig,yorig,srcimg.get_width(),srcimg.get_height())
    // # Combined size:
    // # aggregate height = new height + (oldheight-rotatedheighty)
    //    # aggregate width = new width + (oldwidth-rotatedwhx)
    /*
        rot = euclid.Matrix3.new_rotate(math.radians(clockwise))
        hdiff = srcimg.get_height()-(rot*euclid.Vector2(0,srcimg.get_height())).y
        aggh = surf.get_height()+hdiff-srcimg.get_height()
        if clockwise<=90:
            yorig += aggh
        elif clockwise<=180:
            yorig += surf.get_height()
            xorig -= (rot*euclid.Vector2(srcimg.get_width())).x
        elif clockwise<=270:
            #yorig += srcimg.get_height()
            xorig += surf.get_width()
        else:
            yorig -= (rot*euclid.Vector2(srcimg.get_height())).y
            pass
     */
  }
  SDL_Rect dest;
  dest.x = x-xorig;
  dest.y = y-yorig;
  SDL_Surface *cachedsurf_tmp = PySurface_AsSurface(cachedsurf);
  // XXX can optimize by making draw_props C routines.
  flagged_blit(cachedsurf_tmp,g_screen,&dest,blend_mode);
  Py_DECREF(cachedsurf);
  Py_INCREF(Py_None);
  return Py_None;
  //__blended_blit(surf, (round(x+room.view_xport[room.view_current]-xorig), round(y+room.view_yport[room.view_current]-yorig)), alpha);
}

static PyObject *draw_rectangle(PyObject *naught, PyObject *args)
{
  float x1,y1,x2,y2,outline=0;
  if (!PyArg_ParseTuple(args,"ffff|f",&x1,&y1,&x2,&y2,&outline)) {
    return NULL;
  }
  PyObject *py_room = PyObject_GetAttrString(gmapi,"room");
  struct GMRoom *room = (struct GMRoom *)py_room;
  x1 = round((x1-room->view_xview->f[room->view_current])*g_view_xscale)+room->view_xport->f[room->view_current];
  x2 = round((x2-room->view_xview->f[room->view_current])*g_view_xscale)+room->view_xport->f[room->view_current];
  y1 = round((y1-room->view_yview->f[room->view_current])*g_view_yscale)+room->view_yport->f[room->view_current];
  y2 = round((y2-room->view_yview->f[room->view_current])*g_view_yscale)+room->view_yport->f[room->view_current];
  PyObject *py_alpha = PyObject_GetAttrString(gmapi_py_draw_props,"alpha");
  float alpha = PyFloat_AsDouble(py_alpha);
  Py_DECREF(py_alpha);
  Py_DECREF(room);
  if (alpha!=1.0) {
    if (outline>0.5 || outline<=-0.5) {
      fprintf(stderr,"Can't do alphablending on outline rectangles yet.\n");
      // XXX raise error properly.
      return NULL;
    }
    PyObject *__color_to_int = PyObject_GetAttrString(gmapi,"__color_to_int");
    PyObject *draw_props_color = PyObject_GetAttrString(gmapi_py_draw_props,"color");
    PyObject *io = invoke(__color_to_int,draw_props_color);
    Py_DECREF(draw_props_color);
    Py_DECREF(__color_to_int);
    unsigned int color_as_int = PyInt_AsLong(io);
    Py_DECREF(io);
    fill_translucent_low(g_screen,x1,y1,x2,y2, color_as_int, 255*alpha);
  } else if (outline>=0.5 || outline<=-0.5) {
    PyObject *draw_props_color = PyObject_GetAttrString(gmapi_py_draw_props,"color");
    PyObject *rectargs = Py_BuildValue("(OO(ffff)i)",g_screen_pyobj,draw_props_color,x1,y1,x2-x1,y2-y1,1);
    Py_DECREF(draw_props_color);
    PyObject *rectret = PyObject_CallObject(pygame_draw_rect,rectargs);
    Py_DECREF(rectargs);
    if (!rectret) {
      fprintf(stderr,"An error occurred calling pygame.draw.rect(screen,color,%f,%f,%f,%f,1).\n",x1,y1,x2-x1,y2-y1);
      return NULL;
    }
    Py_DECREF(rectret);
  } else {
    PyObject *fill = PyObject_GetAttrString(g_screen_pyobj,"fill");
    PyObject *draw_props_color = PyObject_GetAttrString(gmapi_py_draw_props,"color");
    PyObject *fillargs = Py_BuildValue("(O(ffff))",draw_props_color,x1,y1,x2-x1,y2-y1);
    Py_DECREF(draw_props_color);
    PyObject *ret = PyObject_CallObject(fill,fillargs);
    Py_DECREF(fill);
    Py_DECREF(fillargs);
    if (!ret) {
      fprintf(stderr,"An error occurred filling surface.\n");
      return NULL;
    }
    Py_DECREF(ret);
  }
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *CParticle_type_cache_clear(PyObject *naught, PyObject *args)
{
  PyObject *pypt;
  if (!PyArg_ParseTuple(args,"O",&pypt)) return NULL;
  ParticleType_object *pt = PyObject_AsParticleType(pypt);
  if (!pt) return NULL;
  PyDict_Clear(pt->cached_surfaces);
  Py_INCREF(Py_None);
  return Py_None;
}

/*
 * End new magic GM-style variable.
 */

static PyMethodDef FastTintDefs[] = {
        {"breakable", breakable, METH_VARARGS, 
                "A hook for GDB.", },
        {"fast_draw_gradient_v",fast_draw_gradient_v,METH_VARARGS,
                "Draw a vertical gradient, with alpha blending.", },
        {"fast_draw_gradients_vc",fast_draw_gradients_vc,METH_VARARGS,
                "Draw a vertical translucent gradient, then fill translucently."},
        {"fast_draw_gradients_vv",fast_draw_gradients_vv,METH_VARARGS,
                "Draw two vertical translucent gradients."},
        {"fast_draw_gradients_vcv",fast_draw_gradients_vcv,METH_VARARGS,
                "Draw a vertical gradient, a constant, and another vgrad, each alphablent.", },
        {"fast_draw_gradients_vcvvc",fast_draw_gradients_vcvvc, METH_VARARGS,
                "More complex gradients.", },
        {"fast_draw_rectangle_color",fast_draw_rectangle_color, METH_VARARGS,
                "C core for draw_rectangle_color." },
        /*{"fill_opaque",fill_opaque,METH_VARARGS,
                "Fill a rectangle, opaquely.", }, - Was slow.*/
        {"draw_rectangle",draw_rectangle,METH_VARARGS,
                "Draw a rectangle.", },
        {"fill_translucent",fill_translucent,METH_VARARGS,
                "Draw a rectangle, possibly with alpha blending.", },
        /*{"particle_Render",particle_Render,METH_VARARGS,
                "C replacement for py_Particle.Render.", },*/
        {"init_subsystems", init_subsystems, METH_VARARGS,
                "Initialize the subsystem variables."},
        {"CParticle_new", CParticle_new, METH_VARARGS,
                "Create a new particle.", },
        {"CParticle_new_size_color", CParticle_new_size_color, METH_VARARGS,
                "Create a new particle of a given size and color.", },
	{"CParticle_step", CParticle_step, METH_VARARGS, 
		"Per-particle step function."},
        {"CParticle_dec_ttl", CParticle_dec_ttl, METH_VARARGS,
                "Decrement the particle TTL and return the new value." },
        {"CParticle_dec_ttl_and_render", CParticle_dec_ttl_and_render, METH_VARARGS,
                "Decrement the particle TTL and render if the particle still should exist." },
        {"CParticle_attach_beancounter", CParticle_attach_beancounter, METH_VARARGS, "Attach a beancounter to the particle.  The beancounter will be increased.  Destroying the particle will decrease it.  If there is a preexisting beancounter, it will be decreased." },

        {"CParticle_type_cache_clear", CParticle_type_cache_clear, METH_VARARGS,
                "Remove cache of prerendered particle surfaces for this type."},
        {"flip_vh", py_flip_vh, METH_VARARGS, "Flip a surface."},
        {"flip_v", py_flip_v, METH_VARARGS, "Flip a surface."},
        {"flip_h", py_flip_h, METH_VARARGS, "Flip a surface."},

        {"part_type_alpha2", part_type_alpha2, METH_VARARGS,
                "Set start and end alphas, linear interpolation."},
        {"part_type_alpha3", part_type_alpha3, METH_VARARGS,
                "Set start/half/end alphas."},
        {"part_type_blend", part_type_blend, METH_VARARGS,
                "Set particle blend mode flag."},
        {"part_type_color3", part_type_color3, METH_VARARGS,
                "Lowlevel color3 implementation.  Expects ints."},
        {"part_type_create", part_type_create, METH_VARARGS,
                "C implementation of part_type_create." },
        {"part_type_direction", part_type_direction, METH_VARARGS,
                "Set the direction min/max/incr/wiggle."},
        {"part_type_gravity", part_type_gravity, METH_VARARGS,
                "Set the gravity strength and angle."},
        {"part_type_life", part_type_life, METH_VARARGS,
                "Set the particle minimum and maximum lifespan in frames."},
        {"part_type_orientation", part_type_orientation, METH_VARARGS,
                "Set the orientation min/max/incr/wiggle/relativity for this particle type."},
        {"part_type_scale", part_type_scale, METH_VARARGS,
                "Set the x and y scale factors for the particle type."},
        {"part_type_shape", part_type_shape, METH_VARARGS,
                "Set the shape of a particle type."},
        {"part_type_size", part_type_size, METH_VARARGS,
                "Set min/max/incr/wiggle for particle size."},
        {"part_type_speed", part_type_speed, METH_VARARGS,
                "Set min/max/incr/wiggle por particle speed."},
        {"part_type_sprite", part_type_sprite, METH_VARARGS,
                "Set particle type to use a sprite/animated/stretched/random"},
        {"rotoscale", rotoscale, METH_VARARGS,
                "Rotate and scale a Pygame surface; return a new Pygame surface."},
        {"scale", scale, METH_VARARGS,
                "Scale a Pygame surface." },
        {"set_SDL_screen", set_SDL_screen, METH_VARARGS,
                "Store the SDL screen in a C variable." },
        {"RegisterInstancesDict", RegisterInstancesDict, METH_VARARGS,
                "Store data for InstanceList." },
        {"GMSprite_from_surface", GMSprite_from_surface, METH_VARARGS,
                "Create a new GMSprite from a Pygame surface." },
        {"Room_Draw", Room_Draw, METH_VARARGS,
                "Draw the current room." },
        {"draw_sprite_ext",draw_sprite_ext,METH_VARARGS,
                "Draw a sprite."},
        {NULL, NULL, 0, NULL}
};

inline PyObject *PySurface_FromSurface(SDL_Surface *surf)
{
  return PySurface_New(surf);
}

PyMODINIT_FUNC
initfasttint(void)
{
    PyObject *fasttint = Py_InitModule("fasttint", FastTintDefs);
    gmvar_modinit(fasttint);
    import_pygame_surface();
    fasttint_init_rooms();
    /* I wonder if this works on Macs? */
    long int seed;
    int urandom = open("/dev/urandom",O_RDONLY);
    if (urandom>=0) {
        read(urandom,&seed,sizeof(seed));
        close(urandom);
    }
    PyObject *tmp = part_type_create(NULL,NULL);
    g_cobject_type_pointer = tmp->ob_type;
    Py_INCREF(g_cobject_type_pointer);
    Py_DECREF(tmp);
    seed ^= getpid();
    seed ^= (getppid()<<16);
    srand48(seed);
    gm6 = PyImport_AddModule("gm6");
    Py_INCREF(gm6);
    gmapi = PyImport_AddModule("gmapi");
    gmapi__dict__ = PyObject_GetAttrString(gmapi,"__dict__");
    Py_INCREF(gmapi);
    /* This may break rollback in Ren'Py-embedded GM code.  XXX */
}
