#include <pygame/pygame.h>
#include <SDL.h>
#include <SDL_rotozoom.h>
#include "common.h"

inline SDL_Surface *clone_surface(SDL_Surface *surf)
{
  return SDL_CreateRGBSurface(surf->flags, surf->w, surf->h,
    surf->format->BitsPerPixel, surf->format->Rmask, surf->format->Gmask,
    surf->format->Bmask, surf->format->Amask);
}

SDL_Surface *flip_vh(SDL_Surface *src)
{
  /* This is a slow operation.  Could be sped using MMX; might also be sped
     by AltiVec. */
  SDL_Surface *ret = clone_surface(src);
  int dsty = 0;
  int srcy = src->h;
  if (src->format->BytesPerPixel==4) {
    while (srcy) {
      unsigned int *srcpx = &src->pixels[srcy*src->pitch];
      unsigned int *dstpx = &ret->pixels[dsty*ret->pitch];
      int dstx = 0;
      int srcx = src->w;
      while (srcx) {
        srcx--;
        dstpx[dstx] = srcpx[srcx];
        dstx++;
      }
      dsty++;
      srcy--;
    }
    return ret;
  } else {
    fprintf(stderr,"Error: non-32bit surfaces not yet flippable.\n");
    return NULL;
  }
}

SDL_Surface *flip_h(SDL_Surface *src)
{
  SDL_Surface *ret = clone_surface(src);
  int y = 0;
  if (src->format->BytesPerPixel==4) {
    while (y<src->h) {
      int srcx = src->w;
      int dstx = 0;
      unsigned int *dstpixels = &ret->pixels[y*ret->pitch];
      unsigned int *srcpixels = &src->pixels[y*src->pitch];
      while (dstx<src->w) {
        srcx--;
        dstpixels[dstx] = srcpixels[srcx];
        dstx++;
      }
      y++;
    }
    return ret;
  } else {
    fprintf(stderr,"Cannot horizontally flip non-32-bit surfaces.\n");
    return NULL;
  }
}

SDL_Surface *flip_v(SDL_Surface *src)
{
  SDL_Surface *ret = clone_surface(src);
  int dst = 0;
  int srci = src->h*src->pitch;
  int pitch = (ret->pitch>src->pitch)?src->pitch:ret->pitch;
  while (srci) {
    srci -= src->pitch;
    memcpy(&ret->pixels[dst],&src->pixels[srci],pitch);
    dst += ret->pitch;
  }
  return ret;
}

PyObject *scale(PyObject *naught, PyObject *args)
{
  PyObject *py_surf;
  float xscale,yscale;
  if (!PyArg_ParseTuple(args,"Off",&py_surf,&xscale,&yscale)) return NULL;
  SDL_Surface *surf = PySurface_AsSurface(py_surf);
  SDL_Surface *ret = zoomSurface(surf,xscale,yscale,1);
  if (!ret) return NULL;
  if (surf->flags&SDL_SRCCOLORKEY) {
    if (SDL_SetColorKey(ret,SDL_SRCCOLORKEY,surf->format->colorkey)) {
      SDL_FreeSurface(ret);
      return NULL;
    }
  }
  PyObject *py_ret = PySurface_FromSurface(ret);
  return py_ret;
}

PyObject *rotoscale(PyObject *naught, PyObject *args)
{
  PyObject *py_src;
  float xscale, yscale, clockwise;
  if (!PyArg_ParseTuple(args,"Offf",&py_src,&xscale,&yscale,&clockwise)) {
    return NULL;
  }
  int iangle = clockwise;
  SDL_Surface *rotated;
  if (xscale!=yscale) {
    rotated = rotozoomSurfaceXY(
      PySurface_AsSurface(py_src),iangle,xscale,yscale,0);
  } else {
    rotated = rotozoomSurface(PySurface_AsSurface(py_src),iangle,xscale,0);
  }
  PyObject *ret = PySurface_FromSurface(rotated);
  return ret;
}

PyObject *py_flip_vh(PyObject *naught, PyObject *args)
{
  PyObject *surf;
  if (!PyArg_ParseTuple(args,"O",&surf)) return NULL;
  return PySurface_FromSurface(flip_vh(PySurface_AsSurface(surf)));
}
PyObject *py_flip_v(PyObject *naught, PyObject *args)
{
  PyObject *surf;
  if (!PyArg_ParseTuple(args,"O",&surf)) return NULL;
  return PySurface_FromSurface(flip_v(PySurface_AsSurface(surf)));
}
PyObject *py_flip_h(PyObject *naught, PyObject *args)
{
  PyObject *surf;
  if (!PyArg_ParseTuple(args,"O",&surf)) return NULL;
  return PySurface_FromSurface(flip_h(PySurface_AsSurface(surf)));
}
