/*
 * rooms.c copyright 2010 Spiky Caterpillar.  All rights reserved.
 * This file contains the room-specific routines for fasttint, including
 * replacements for the Python RoomData and RoomTileData classes in gm6.py
 */

#include <pygame/pygame.h>
#include <structmember.h>
#include "common.h"

double g_view_xscale, g_view_yscale;

static PyObject *RoomTileData_reduce(PyObject *o, PyObject *args)
{
  struct RoomTileData *tile = (struct RoomTileData *)o;
  PyObject *state = Py_BuildValue("(iiiiiiiiii)",tile->x, tile->y, 
    tile->background_idx, tile->tilex, tile->tiley,
    tile->w, tile->h, tile->layer_depth, tile->id, tile->locked);
  PyObject *ret = Py_BuildValue("(O()O)",&RoomTileData_Type,state);
  Py_DECREF(state);
  return ret;
}

#define membroom(n,t) { #n , t, offsetof(struct GMRoom, n ), 0, NULL}

static PyMemberDef GMRoom_members[] = {
        membroom(id,T_INT),
        membroom(name,T_OBJECT),
        membroom(room_width,T_INT),
        membroom(room_height, T_INT),
        membroom(snapx, T_INT),
        membroom(snapy,T_INT),
        membroom(isogrid, T_INT),
        membroom(room_speed, T_INT),
        membroom(room_persistent, T_INT),
        membroom(bgcolor, T_INT),
        membroom(draw_bgcolor, T_INT),
        membroom(room_caption, T_OBJECT),
        membroom(creation_code, T_OBJECT),
        membroom(bg0, T_OBJECT),
        membroom(bg1, T_OBJECT),
        membroom(bg2, T_OBJECT),
        membroom(bg3, T_OBJECT),
        membroom(bg4, T_OBJECT),
        membroom(bg5, T_OBJECT),
        membroom(bg6, T_OBJECT),
        membroom(bg7, T_OBJECT),
        membroom(view_enabled, T_INT),
        membroom(numviews, T_INT),
        membroom(view_xview, T_OBJECT),
        membroom(view_yview, T_OBJECT),
        membroom(view_object, T_OBJECT),
        membroom(view_hborder, T_OBJECT),
        membroom(view_vborder, T_OBJECT),
        membroom(view_wview, T_OBJECT),
        membroom(view_hview, T_OBJECT),
        membroom(view_xport, T_OBJECT),
        membroom(view_yport, T_OBJECT),
        membroom(view_wport, T_OBJECT),
        membroom(view_hport, T_OBJECT),
        membroom(background_index, T_OBJECT),
        membroom(instances, T_OBJECT),
        membroom(tiles, T_OBJECT),
        membroom(tile_w, T_INT),
        membroom(tile_h, T_INT),
        membroom(tile_hsep, T_INT),
        membroom(tile_vsep, T_INT),
        membroom(tile_hoffset, T_INT),
        membroom(tile_voffset, T_INT),
        membroom(created, T_INT),
        membroom(all_objects, T_OBJECT),
        membroom(drawn_objects, T_OBJECT),
        membroom(deactivated_objects, T_OBJECT),
        membroom(keypressed_objects, T_OBJECT),
        membroom(keyreleased_objects, T_OBJECT),
        membroom(mouse_leftpressed_objects, T_OBJECT),
        membroom(mouse_rightpressed_objects, T_OBJECT),
        membroom(collisioned_objects, T_OBJECT),
        membroom(stepped_objects, T_OBJECT),
        membroom(moving_objects, T_OBJECT),
        membroom(alarmed_objects, T_OBJECT),
        membroom(animated_objects, T_OBJECT),
        membroom(key_objects, T_OBJECT),
        membroom(mouse_objects, T_OBJECT),
        membroom(cached_bg, T_OBJECT),
        membroom(cached_bg_scaled, T_OBJECT),
        membroom(frame_duration, T_FLOAT),
        membroom(creation_code_compiled, T_OBJECT),
        membroom(background_x, T_OBJECT),
        membroom(background_y, T_OBJECT),
        membroom(background_width, T_OBJECT),
        membroom(background_height, T_OBJECT),
        membroom(background_xscale, T_OBJECT),
        membroom(background_yscale, T_OBJECT),
        membroom(background_hspeed, T_OBJECT),
        membroom(background_vspeed, T_OBJECT),
        membroom(background_blend, T_OBJECT),
        membroom(background_alpha, T_OBJECT),
        membroom(background_htiled, T_OBJECT),
        membroom(background_vtiled, T_OBJECT),
        membroom(background_visible, T_OBJECT),
        membroom(view_visible, T_OBJECT),
        membroom(view_current, T_INT),
        membroom(initial_view_data, T_OBJECT),
        {NULL},
};

static int GMRoom_traverse(PyObject *o, visitproc visit, void *arg)
{
  struct GMRoom *room = (struct GMRoom *)o;
  Py_VISIT(room->name);
  Py_VISIT(room->room_caption);
  Py_VISIT(room->creation_code);
  Py_VISIT(room->view_xview);
  Py_VISIT(room->view_yview);
  Py_VISIT(room->view_object);
  Py_VISIT(room->view_hborder);
  Py_VISIT(room->view_vborder);
  Py_VISIT(room->view_wview);
  Py_VISIT(room->view_hview);
  Py_VISIT(room->view_xport);
  Py_VISIT(room->view_yport);
  Py_VISIT(room->view_wport);
  Py_VISIT(room->view_hport);
  Py_VISIT(room->background_index);
  Py_VISIT(room->instances);
  Py_VISIT(room->tiles);
  Py_VISIT(room->all_objects);
  Py_VISIT(room->all_objects);
  Py_VISIT(room->drawn_objects);
  Py_VISIT(room->deactivated_objects);
  Py_VISIT(room->keypressed_objects);
  Py_VISIT(room->keyreleased_objects);
  Py_VISIT(room->mouse_leftpressed_objects);
  Py_VISIT(room->mouse_rightpressed_objects);
  Py_VISIT(room->collisioned_objects);
  Py_VISIT(room->stepped_objects);
  Py_VISIT(room->moving_objects);
  Py_VISIT(room->alarmed_objects);
  Py_VISIT(room->animated_objects);
  Py_VISIT(room->key_objects);
  Py_VISIT(room->mouse_objects);
  Py_VISIT(room->cached_bg);
  Py_VISIT(room->cached_bg_scaled);
  // XXX I don't need to visit floatlist8s, right?
  return 0;
}

static int GMRoom_clear(PyObject *o)
{
  struct GMRoom *room = (struct GMRoom *)o;
  Py_CLEAR(room->name);
  Py_CLEAR(room->room_caption);
  Py_CLEAR(room->creation_code);
  Py_CLEAR(room->bg0);
  Py_CLEAR(room->bg1);
  Py_CLEAR(room->bg2);
  Py_CLEAR(room->bg3);
  Py_CLEAR(room->bg4);
  Py_CLEAR(room->bg5);
  Py_CLEAR(room->bg6);
  Py_CLEAR(room->bg7);
  Py_CLEAR(room->view_xview);
  Py_CLEAR(room->view_yview);
  Py_CLEAR(room->view_object);
  Py_CLEAR(room->view_hborder);
  Py_CLEAR(room->view_vborder);
  Py_CLEAR(room->view_wview);
  Py_CLEAR(room->view_hview);
  Py_CLEAR(room->view_object);
  Py_CLEAR(room->view_hborder);
  Py_CLEAR(room->view_vborder);
  Py_CLEAR(room->view_wview);
  Py_CLEAR(room->view_hview);
  Py_CLEAR(room->view_xport);
  Py_CLEAR(room->view_yport);
  Py_CLEAR(room->view_wport);
  Py_CLEAR(room->view_hport);
  Py_CLEAR(room->background_index);
  Py_CLEAR(room->instances);
  Py_CLEAR(room->tiles);
  Py_CLEAR(room->all_objects);
  Py_CLEAR(room->drawn_objects);
  Py_CLEAR(room->deactivated_objects);
  Py_CLEAR(room->keypressed_objects);
  Py_CLEAR(room->keyreleased_objects);
  Py_CLEAR(room->mouse_leftpressed_objects);
  Py_CLEAR(room->mouse_rightpressed_objects);
  Py_CLEAR(room->collisioned_objects);
  Py_CLEAR(room->stepped_objects);
  Py_CLEAR(room->moving_objects);
  Py_CLEAR(room->alarmed_objects);
  Py_CLEAR(room->animated_objects);
  Py_CLEAR(room->key_objects);
  Py_CLEAR(room->mouse_objects);
  Py_CLEAR(room->cached_bg);
  Py_CLEAR(room->cached_bg_scaled);
  Py_CLEAR(room->background_x);
  Py_CLEAR(room->background_y);
  Py_CLEAR(room->background_visible);
  Py_CLEAR(room->background_xscale);
  Py_CLEAR(room->background_yscale);
  Py_CLEAR(room->background_hspeed);
  Py_CLEAR(room->background_vspeed);
  Py_CLEAR(room->background_blend);
  Py_CLEAR(room->background_alpha);
  Py_CLEAR(room->background_htiled);
  Py_CLEAR(room->background_vtiled);
  Py_CLEAR(room->background_width);
  Py_CLEAR(room->background_height);
  // XXX is this the right macro?  I want it to assign the variable to a temp
  // var, null the original, and XDECREF it.  Not to unconditionally clear it.
  return 0;
}

static void GMRoom_dealloc(PyObject *o)
{
  GMRoom_clear(o);
  o->ob_type->tp_free(o);
}

static PyObject *GMRoom_int(PyObject *o)
{
  return Game_Sequence_Index(o, "rooms");
}

static PyObject *GMRoom_float(PyObject *o)
{
  return Game_Sequence_Index_F(o, "rooms");
}

static PyNumberMethods GMRoom_numbermethods = {
        0, /* add */
        0, /* sub */
        0, /* mul */
        0, /* divide */
        0, /* remainder */
        0, /* divmod */
        0, /* power*/
        0, /* negative */
        0, /* positive */
        0, /* absolute */
        0, /* nonzero */
        0, /* invert */
        0, /* lshift */
        0, /* rshift */
        0, /* and */
        0, /* xor */
        0, /* or */
        0, /* coerce */
	GMRoom_int, //unaryfunc nb_int;
	0, //unaryfunc nb_long;
        GMRoom_float, //unaryfunc nb_float;
        };

static PyObject *GMRoom_str(PyObject *o)
{
  return Game_Sequence_Index_S(o,"rooms");//PyString_FromFormat("%d",((struct GMRoom *)o)->id);
}

static PyObject *GMRoom_Compile(PyObject *self, PyObject *args)
{
  PyObject *val = Py_BuildValue("(O)",self);
  PyObject *ret = PyObject_CallObject(gm6_Room_Compile,val);
  //struct GMRoom *room = (struct GMRoom *)self;
  //((struct GMRoom *)self)->id = Game_Sequence_Index(self,"rooms");
  Py_DECREF(val);
  return ret;
}

static PyObject *GMRoom_Create(PyObject *self, PyObject *args)
{
  PyObject *val = Py_BuildValue("(O)",self);
  PyObject *ret = PyObject_CallObject(gm6_Room_Create,val);
  Py_DECREF(val);
  return ret;
}

inline PyObject *invoke(PyObject *func, PyObject *self)
{
  PyObject *val = Py_BuildValue("(O)",self);
  PyObject *ret = PyObject_CallObject(func, val);
  Py_DECREF(val);
  return ret;
}

static inline PyObject *invoke_2(PyObject *func, PyObject *self, PyObject *arg2)
{
  PyObject *val = Py_BuildValue("(OO)",self,arg2);
  PyObject *ret = PyObject_CallObject(func, val);
  Py_DECREF(val);
  return ret;
}

static PyObject *invoke_3(PyObject *func, PyObject *self, PyObject *arg2, PyObject *arg3)
{
  PyObject *val = Py_BuildValue("(OOO)",self,arg2,arg3);
  PyObject *ret = PyObject_CallObject(func, val);
  Py_DECREF(val);
  return ret;
}

static PyObject *GMRoom_InitializeBackgrounds(PyObject *self, PyObject *args)
{
  return invoke(gm6_Room_InitializeBackgrounds, self);
}

static PyObject *GMRoom_AddInstance(PyObject *self, PyObject *args)
{
  return invoke_2(gm6_Room_AddInstance,self,PyTuple_GET_ITEM(args,0));
}

static PyObject *GMRoom_AddDrawnInstance(PyObject *self, PyObject *args)
{
  return invoke_2(gm6_Room_AddDrawnInstance,self,PyTuple_GET_ITEM(args,0));
}

static PyObject *GMRoom_ResortDrawnObjects(PyObject *self, PyObject *args)
{
  return invoke(gm6_Room_ResortDrawnObjects,self);
}

static PyObject *GMRoom_SetBackgroundIndex(PyObject *self, PyObject *args)
{
  PyObject *num, *bg;
  if (!PyArg_ParseTuple(args, "OO", &num, &bg)) return NULL;
  PyObject *ret = invoke_3(gm6_Room_SetBackgroundIndex,self,num,bg);
  return ret;
}

static PyObject *GMRoom_StepAlarms(PyObject *self, PyObject *ignore)
{
  return invoke(gm6_Room_StepAlarms, self);
}

static PyObject *GMRoom_MoveInstances(PyObject *self, PyObject *nothing)
{
  return invoke(gm6_Room_MoveInstances,self);
}

static PyObject *GMRoom_HandleCollisions(PyObject *self, PyObject *nothing)
{
  return invoke(gm6_Room_HandleCollisions, self);
}

static PyObject *GMRoom_StepEvents(PyObject *self, PyObject *nothing)
{
  return invoke(gm6_Room_StepEvents, self);
}

static PyObject *GMRoom_DrawBackgrounds(PyObject *self, PyObject *args)
{
  PyObject *surf, *xoff, *yoff;
  if (!PyArg_ParseTuple(args,"OOO",&surf, &xoff, &yoff)) return NULL;
  return invoke_4(gm6_Room_DrawBackgrounds,self,surf,xoff,yoff);
}

static PyObject *GMRoom_RenderBackground(PyObject *self, PyObject *ignore)
{
  return invoke(gm6_Room_RenderBackground,self);
}

static PyObject *GMRoom_Leave(PyObject *self, PyObject *ignore)
{
  return invoke(gm6_Room_Leave,self);
}

static PyObject *GMRoom_MouseButtonDown(PyObject *self, PyObject *args)
{
  PyObject *ev;
  if (!PyArg_ParseTuple(args,"O",&ev)) return NULL;
  return invoke_2(gm6_Room_MouseButtonDown, self, ev);
}

static PyObject *GMRoom_MouseButtonUp(PyObject *self, PyObject *args)
{
  PyObject *ev;
  if (!PyArg_ParseTuple(args,"O",&ev)) return NULL;
  return invoke_2(gm6_Room_MouseButtonUp, self, ev);
}

static PyObject *GMRoom_HandleKeyrelease(PyObject *self, PyObject *args)
{
  PyObject *ev;
  if (!PyArg_ParseTuple(args,"O",&ev)) return NULL;
  return invoke_2(gm6_Room_HandleKeyrelease, self, ev);
}

static PyObject *GMRoom_HandleKeypress(PyObject *self, PyObject *args)
{
  PyObject *ev;
  if (!PyArg_ParseTuple(args,"O",&ev)) return NULL;
  return invoke_2(gm6_Room_HandleKeypress, self, ev);
}

static PyMethodDef GMRoom_methods[] = {
        {"__reduce__", membertuple, METH_VARARGS, NULL,},
        {"__setstate__",load_membertuple, METH_VARARGS, NULL,},
        {"Compile", GMRoom_Compile, METH_VARARGS, NULL,},
        {"Create", GMRoom_Create, METH_VARARGS, NULL,},
        {"InitializeBackgrounds", GMRoom_InitializeBackgrounds, METH_VARARGS, NULL,},
        {"AddInstance", GMRoom_AddInstance, METH_VARARGS, NULL, },
        {"AddDrawnInstance", GMRoom_AddDrawnInstance, METH_VARARGS, NULL, },
        {"ResortDrawnObjects", GMRoom_ResortDrawnObjects, METH_VARARGS, NULL, },
        {"SetBackgroundIndex", GMRoom_SetBackgroundIndex, METH_VARARGS, NULL, },
        {"StepAlarms", GMRoom_StepAlarms, METH_NOARGS, NULL, },
        {"MoveInstances", GMRoom_MoveInstances, METH_NOARGS, NULL, },
        {"HandleCollisions", GMRoom_HandleCollisions, METH_NOARGS, NULL, },
        {"StepAlarms", GMRoom_StepAlarms, METH_NOARGS, NULL, },
        {"StepEvents", GMRoom_StepEvents, METH_NOARGS, NULL, },
        {"DrawBackgrounds", GMRoom_DrawBackgrounds, METH_VARARGS, NULL, },
        {"RenderBackground", GMRoom_RenderBackground, METH_NOARGS, NULL, },
        {"Leave", GMRoom_Leave, METH_NOARGS, NULL, },
        {"MouseButtonDown", GMRoom_MouseButtonDown, METH_VARARGS, NULL, },
        {"MouseButtonUp", GMRoom_MouseButtonUp, METH_VARARGS, NULL, },
        {"HandleKeyrelease", GMRoom_HandleKeyrelease, METH_VARARGS, NULL, },
        {"HandleKeypress", GMRoom_HandleKeypress, METH_VARARGS, NULL, },
        {NULL},
};

PyTypeObject GMRoom_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.GMRoom",
        sizeof(struct GMRoom),
    0,                         /*tp_itemsize*/
    GMRoom_dealloc,                         /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    Object_AsInt_compare,    /*tp_compare*/
    0,                         /*tp_repr*/
    &GMRoom_numbermethods,   /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    GMRoom_str,              /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,
        "GM-compatible backgrounds.",
        GMRoom_traverse,                         /* tp_traverse */
        GMRoom_clear,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
        GMRoom_methods, /* tp_methods */
        GMRoom_members,
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
        0, /* tp_init */
        0, /* tp_alloc */
        PyType_GenericNew, /* tp_new */
        };

static PyMemberDef RoomTileData_members[] = {
        {"x", T_INT, offsetof(struct RoomTileData, x), 0, ""},
        {"y", T_INT, offsetof(struct RoomTileData, y), 0, ""},
        {"id", T_INT, offsetof(struct RoomTileData, id), 0, ""},
        {"background_idx", T_INT, offsetof(struct RoomTileData, background_idx), 0, ""},
        {"tilex", T_INT, offsetof(struct RoomTileData, tilex), 0, ""},
        {"tiley", T_INT, offsetof(struct RoomTileData, tiley), 0, ""},
        {"w", T_INT, offsetof(struct RoomTileData, w), 0, ""},
        {"h", T_INT, offsetof(struct RoomTileData, h), 0, ""},
        {"layer_depth", T_INT, offsetof(struct RoomTileData, layer_depth), 0, ""},
        //{"id", T_INT, offsetof(struct RoomTileData, id), 0, ""},
        {"locked", T_INT, offsetof(struct RoomTileData, locked), 0, ""},
        {NULL},
        };

static PyObject *RoomTileData_str(PyObject *self)
{
  return PyString_FromFormat("%d",((struct RoomTileData *)self)->id);
}

static PyObject *RoomTileData_reduce(PyObject *o,PyObject *args);

static PyObject *RoomTileData_nb_int(PyObject *self)
{
  return PyInt_FromLong(((struct RoomTileData *)self)->id);
}

static PyObject *RoomTileData_nb_float(PyObject *self)
{
  return PyFloat_FromDouble(((struct RoomTileData *)self)->id);
}

static PyNumberMethods RoomTileData_numbermethods = {
        /* For numbers without flag bit Py_TPFLAGS_CHECKTYPES set, all
           arguments are guaranteed to be of the object's type (modulo
	   coercion hacks -- i.e. if the type's coercion function
	   returns other types, then these are allowed as well).  Numbers that
	   have the Py_TPFLAGS_CHECKTYPES flag bit set should check *both*
	   arguments for proper type and implement the necessary conversions
	   in the slot functions themselves. */
	0, //binaryfunc nb_add;
	0, //binaryfunc nb_subtract;
        0, //binaryfunc nb_multiply;
	0, //binaryfunc nb_divide;
	0, //binaryfunc nb_remainder;
	0, //binaryfunc nb_divmod;
	0, //ternaryfunc nb_power;
	0, //unaryfunc nb_negative;
	0, //unaryfunc nb_positive;
	0, //unaryfunc nb_absolute;
	0, //inquiry nb_nonzero;
	0, //unaryfunc nb_invert;
	0, //binaryfunc nb_lshift;
	0, //binaryfunc nb_rshift;
	0, //binaryfunc nb_and;
	0, //binaryfunc nb_xor;
	0, //binaryfunc nb_or;
	0, //coercion nb_coerce;
	RoomTileData_nb_int, //unaryfunc nb_int;
	0, //unaryfunc nb_long;
        RoomTileData_nb_float, //unaryfunc nb_float;
};


static PyObject *RoomTileData_setstate(PyObject *self, PyObject *stateo)
{
  PyObject *state = PyTuple_GET_ITEM(stateo,0);
  struct RoomTileData *tile = (struct RoomTileData *)self;
  tile->x = seqi(state,0);
  tile->y = seqi(state,1);
  tile->background_idx = seqi(state,2);
  tile->tilex = seqi(state,3);
  tile->tiley = seqi(state,4);
  tile->w = seqi(state,5);
  tile->h = seqi(state,6);
  tile->layer_depth = seqi(state,7);
  tile->id = seqi(state, 8);
  tile->locked = seqi(state, 9);
  Py_INCREF(Py_None);
  return Py_None;
}

static inline int view_draw_particlesystem(PyObject *o, PyObject *ps_draw_args)
{
  int ret = 1;
  int do_autodraw = 0;
  PyObject *autodraw = PyObject_GetAttrString(o,"py_automatic_draw");
  if (autodraw) {
    do_autodraw = PyObject_IsTrue(autodraw);
    if (do_autodraw==-1) {
      do_autodraw = 0;
      ret = 0;
    }
    Py_DECREF(autodraw);
  }
  if (!do_autodraw) return ret;
  PyObject *py_RealDraw = PyObject_GetAttrString(o,"py_RealDraw");
  if (!py_RealDraw) return 0;
  PyObject *dump = PyObject_CallObject(py_RealDraw,ps_draw_args);
  if (dump) {
    Py_DECREF(dump);
  } else {
    ret = 0;
  }
  Py_DECREF(py_RealDraw);
  return ret;
}

static inline int init_instance_drawfunc(PyObject *o, PyObject *blanktuple)
{
  int ret=1;
  PyObject *object_index = PyObject_GetAttrString(o,"object_index");
  PyObject *py_objectdata = PyObject_GetAttrString(object_index,"py_objectdata");
  Py_DECREF(object_index);
  PyObject *events = PyObject_GetAttrString(py_objectdata,"events");
  PyObject *ev8 = NULL;
  if (events) {
    Py_DECREF(py_objectdata);
    if (PySequence_Size(events)<=8) {
      fprintf(stderr,"Events too small!\n");
      // Arguably should free stuff.  TODO
      ret = 0;
    } else {
      ev8 = PySequence_GetItem(events,8);
    } 
    Py_DECREF(events);
  }
  if (events && (PySequence_Size(ev8)>0)) {
    PyObject *code = PySequence_GetItem(ev8,0);
    PyObject *python = PyObject_GetAttrString(code,"python");
    Py_DECREF(code);
    PyObject *tmpname = PyObject_GetAttrString(py_objectdata,"name");
    Py_DECREF(tmpname);
    if (PyObject_SetAttrString(o,"py_Draw",python)==-1) {
      fprintf(stderr,"Error setting attribute string to code.\n");
      ret = 0;
    }
    PushSelf(o);
    PyObject *evalret = PyEval_EvalCode(python,gmapi__dict__,gmapi__dict__);
    if (evalret) {
      Py_DECREF(evalret);
    } else {
      fprintf(stderr,"Code eval failed\n");
      ret = 0;
    }
    PopSelf();
    Py_DECREF(python);
  } else {
    PyObject *new = PyInt_FromLong(1);
    if (PyObject_SetAttrString(o,"py_Draw",new)==-1) {
      fprintf(stderr,"Error setting attribute string to number.\n");
      ret = 0;
    }
    Py_DECREF(new);
  }
  Py_XDECREF(ev8);
  return ret;
}

static inline int view_draw_instance(PyObject *o, PyObject *blanktuple)
{
  int ret = 1;
  PyObject *py_Draw = PyObject_GetAttrString(o, "py_Draw");
  if (py_Draw) {
    if (py_Draw->ob_type==&PyInt_Type) {
      if (PyInt_AsLong(py_Draw)==0) {
        Py_DECREF(py_Draw);
        return init_instance_drawfunc(o,blanktuple);
      }
      PyObject *sprite_index = PyObject_GetAttrString(o,"sprite_index");
      PyObject *image_index = PyObject_GetAttrString(o,"image_index");
      PyObject *o_x = PyObject_GetAttrString(o, "x");
      PyObject *o_y = PyObject_GetAttrString(o, "y");
      PyObject *o_xscale = PyObject_GetAttrString(o, "image_xscale");
      PyObject *o_yscale = PyObject_GetAttrString(o, "image_yscale");
      PyObject *image_angle = PyObject_GetAttrString(o,"image_angle");
      PyObject *image_blend = PyObject_GetAttrString(o,"image_blend");
      PyObject *image_alpha = PyObject_GetAttrString(o,"image_alpha");
      PyObject *args = Py_BuildValue("(OOOOOOOOO)",
          sprite_index, image_index,
          o_x, o_y, o_xscale, o_yscale,
          image_angle, image_blend, image_alpha);
      PyObject *dump = PyObject_CallObject(gmapi_draw_sprite_ext, args);
      if (dump) {
        Py_DECREF(dump);
      } else {
        ret = 0;
      } 
      Py_DECREF(args);
      Py_DECREF(sprite_index);
      Py_DECREF(image_index);
      Py_DECREF(o_x); Py_DECREF(o_y);
      Py_DECREF(o_xscale); Py_DECREF(o_yscale);
      Py_DECREF(image_angle); Py_DECREF(image_blend);
      Py_DECREF(image_alpha);
    } else {
      PushSelf(o);
      PyObject *dump = PyEval_EvalCode(py_Draw,gmapi__dict__,gmapi__dict__);
      if (dump) {
        Py_DECREF(dump);
      } else {
        ret = 0;
      }
      PopSelf();
    }
    Py_DECREF(py_Draw);
  } else {
    return init_instance_drawfunc(o,blanktuple);
  }
  return ret;
}

int Room_DrawView(struct GMRoom *room, PyObject *surface_py, int x, PyObject *depthsorted)
{
    int ret = 1;
    SDL_Surface *surface_sdl = PySurface_AsSurface(surface_py);
    int following_num = room->view_object->f[x];
    double vx = room->view_xview->f[x];
    double vy = room->view_yview->f[x];
    if (following_num>=0) {
      PyObject *game = PyObject_GetAttrString(gm6,"game");
      PyObject *objects = PyObject_GetAttrString(game,"objects");
      PyObject *following_obj = PySequence_GetItem(objects,following_num);
      PyObject *following_instance=NULL;
      if (following_obj) {
          following_instance = GetFirstInstance(following_obj);
          Py_DECREF(following_obj);
      }
      Py_DECREF(objects);
      Py_DECREF(game);
      if (following_instance) {
        PyObject *following_x = PyObject_GetAttrString(following_instance,"x");
        PyObject *following_y = PyObject_GetAttrString(following_instance,"y");
        vx = PyFloat_AsDouble(following_x)-room->view_hborder->f[x];
        vy = PyFloat_AsDouble(following_y)-room->view_vborder->f[x];
        if (vx<0) {
          vx = 0.0;
        } else if (vx+room->view_wview->f[x]>room->room_width) {
          vx = room->room_width-room->view_wview->f[x];
        }
        if (vy<0) {
          vy = 0.0;
        } else if (vy+room->view_hview->f[x]>room->room_height) {
          vy = room->room_height-room->view_hview->f[x];
        }
        Py_DECREF(following_x);
        Py_DECREF(following_y);
        if (vx!=room->view_xview->f[x] || vy!=room->view_yview->f[x]) {
            room->view_xview->f[x] = vx;
            room->view_yview->f[x] = vy;
        }
      }
    }
    /*
    following = room.view_object[x]
    if type(following)==ObjectData:
        following = following.instances[0]
    elif type(following) in (int,float):
        lowid = 2000000000
        real_following = None
        for o in game.objects[int(following)]:
            if o.id<lowid:
                real_following = o
                lowid = o.id
        following = real_following
    */
    // If it's an instance, this will work, I think 
    room->view_current = x;
    if (PyErr_Occurred()) fprintf(stderr,"An error has occurred 615\n");

    //global view_xscale,view_yscale

    SDL_Surface *destsurf;
    if (room->view_xport->f[x]==0 && room->view_yport->f[x]==0 && room->view_wview->f[x]==surface_sdl->w && room->view_hview->f[x]==surface_sdl->h) {
        destsurf = surface_sdl;
        PyObject *view_xscale_o = PyInt_FromLong(1);
        PyObject_SetAttrString(gm6,"view_xscale",view_xscale_o);
        PyObject_SetAttrString(gm6,"view_yscale",view_xscale_o);
        g_view_xscale = g_view_yscale = 1.0;
        Py_DECREF(view_xscale_o);
    } else {
        destsurf = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, 
            room->view_wport->f[x], room->view_hport->f[x],
            32, // XXX use actual screen depth
            g_screen->format->Rmask, g_screen->format->Gmask, g_screen->format->Bmask, g_screen->format->Amask);
        g_view_xscale = room->view_wport->f[x]/room->view_wview->f[x];
        PyObject *view_xscale_o = PyFloat_FromDouble(g_view_xscale);
        PyObject_SetAttrString(gm6,"view_xscale",view_xscale_o);
        Py_DECREF(view_xscale_o);
        g_view_yscale = room->view_hport->f[x]/room->view_hview->f[x];
        PyObject *view_yscale_o = PyFloat_FromDouble(g_view_yscale);
        PyObject_SetAttrString(gm6,"view_yscale", view_yscale_o);
        Py_DECREF(view_yscale_o);
    }
    if (PyErr_Occurred()) fprintf(stderr,"An error has occurred 638\n");
        // begin narbleriffic
    if (destsurf==surface_sdl) { //XXX need to fix and reenable this!
        PyObject *tmpxoff, *tmpyoff;
        PyObject *dump = invoke_4(gm6_Room_DrawBackgrounds,room,surface_py,tmpxoff=PyFloat_FromDouble(room->view_xview->f[x]),tmpyoff=PyFloat_FromDouble(room->view_yview->f[x]));
        Py_DECREF(tmpxoff);
        Py_DECREF(tmpyoff);
        Py_DECREF(dump);
        //room.DrawBackgrounds(destsurf, int(room.view_xview[x]), int(room.view_yview[x]))
    } else {
        PyObject *destsurf_py = PySurface_New(destsurf);
        PyObject *tmpxoff, *tmpyoff;
        PyObject *dump = invoke_4(gm6_Room_DrawBackgrounds,(PyObject *)room,destsurf_py,tmpxoff=PyFloat_FromDouble(room->view_xview->f[x]),tmpyoff=PyFloat_FromDouble(room->view_yview->f[x]));
        Py_DECREF(tmpxoff);
        Py_DECREF(tmpyoff);
        Py_DECREF(dump);
        SDL_Rect rect;
        rect.x = room->view_xport->f[x];
        rect.y = room->view_yport->f[x];
        SDL_BlitSurface(destsurf, NULL, surface_sdl, &rect);
        Py_DECREF(destsurf_py);
        //SDL_FreeSurface(destsurf);
    }
        // end narbleriffic
    //PyObject *xoff = PyFloat_FromDouble(-room->view_xview->f[x]);
    //PyObject *yoff = PyFloat_FromDouble(-room->view_yview->f[x]);
    if (PyErr_Occurred()) fprintf(stderr,"660\n");
    int omax = PySequence_Size(depthsorted);
    if (PyErr_Occurred()) fprintf(stderr,"660 a\n");
    int onum = 0;
    PyObject *blanktuple = PyTuple_New(0);
    if (PyErr_Occurred()) fprintf(stderr,"An error has occurred 661\n");
    int vxi = -vx;
    int vyi = -vy;
    PyObject *offset_args = Py_BuildValue("(ii)",vxi,vyi);
    while (onum<omax) {
        PyObject *o = PySequence_GetItem(depthsorted,onum);
        if (PyErr_Occurred()) fprintf(stderr,"Error after GI %d\n",onum);
        if (o->ob_type==(void *)gmapi_py_ParticleSystem) {
          if (x==0) {
            ret &= view_draw_particlesystem(o,offset_args);
            if (PyErr_Occurred()) fprintf(stderr,"Error drawing PS\n");
          }
        } else {
          ret &= view_draw_instance(o,blanktuple);
          if (PyErr_Occurred()) {
            fprintf(stderr,"Error drawing instance\n");
          }
        }
        onum++;
    }
    Py_DECREF(blanktuple);
    //Py_DECREF(xoff);
    //Py_DECREF(yoff);
    Py_DECREF(offset_args);
    if (PyErr_Occurred()) fprintf(stderr,"An error has occurred\n");
    return ret;
}


static PyMethodDef RoomTileData_methods[] = {
        {"__reduce__", RoomTileData_reduce, METH_VARARGS, NULL,},
        {"__setstate__", RoomTileData_setstate, METH_VARARGS, NULL,},
        {NULL,NULL},
        };

PyTypeObject RoomTileData_Type = {
        PyObject_HEAD_INIT(NULL)
        0,      /*ob_size*/
        "fasttint.RoomTileData",
        sizeof(struct RoomTileData),
    0,                         /*tp_itemsize*/
    0,                         /*tp_dealloc*/
    0,                         /*tp_print*/
    0,                         /*tp_getattr*/
    0,                         /*tp_setattr*/
    Object_AsInt_compare,    /*tp_compare*/
    0,                         /*tp_repr*/
    &RoomTileData_numbermethods,   /*tp_as_number*/
    0,                         /*tp_as_sequence*/
    0,                         /*tp_as_mapping*/
    0,                         /*tp_hash */
    0,                         /*tp_call*/
    RoomTileData_str,              /*tp_str*/
    0,                         /*tp_getattro*/
    0,                         /*tp_setattro*/
    0,                         /*tp_as_buffer*/
    Py_TPFLAGS_DEFAULT,
        "GM-compatible room tile entries.",
        0,                         /* tp_traverse */
        0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
        RoomTileData_methods, /* tp_methods */
        RoomTileData_members,
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
        0, /* tp_init */
        0, /* tp_alloc */
        PyType_GenericNew, /* tp_new */
        };

void fasttint_init_rooms(void)
{
  import_pygame_surface();
}
