import struct
from struct import unpack
import string
import pygame
#import pygame.surfarray YYY XXX not in Ren'Py
import zlib
import random
import gmapi
import time
import sys
import traceback
import math
import supersound_compat
import os
import cPickle
try:
    import renpy.exports as renpy
    use_renpy = 1
except:
    use_renpy = 0

GMVar = gmapi.py_fasttint.GMVar
GMSprite = gmapi.py_fasttint.GMSprite

g_mungedict = dict()

replace_scripts = dict()
replace_scripts_gml = dict()
instances = dict()

def slurpdata(f):
    l = f.read(4)
    datalen = unpack('<l',l)[0]
    if datalen:
        return f.read(datalen)
    return ''

def slurpdata_v(f):
    l = f.read(4)
    datalen = unpack('<l',l)[0]
    print 'datalen',datalen,l
    if datalen:
        return f.read(datalen)
    return ''
def slurpint(f):
    return unpack('<l',f.read(4))[0]

def slurpdouble(f):
    return unpack('<d',f.read(8))[0]

def dump_file(fn,data):
    f = file(fn,'wb')
    f.write(data)
    f.close()

class SoundData(object):
    py_soundobject = None
    def __init__(self,f):
        self.name = slurpdata(f)
        self.ver = slurpint(f)
        self.kind = slurpint(f)
        # Interpreted differently depending on version.
        # 600:
        # 0 = Normal, 1=Background, 2=3D, 3=Multimedia
        # 'Background' appears to be 'midi'
        # 440:
        # -1=None, 0=Wave, 1=Midi, 2=Mp3, 10=Unknown
        self.filetype = slurpdata(f)
        self.musicdata = ''
        midi = False
        if self.ver >= 600:
            if self.kind==1:
                midi = True
            self.filename = slurpdata(f)
            if slurpint(f):
                self.musicdata = slurpdata(f)
            effects = slurpint(f)
            if effects:
                print "Warning: effects on sounds not yet handled.",self.name
            self.chorus = effects & 1
            self.echo = effects & 2
            self.flanger = effects & 4
            self.gargle = effects & 8
            self.reverb = effects & 16
            self.volume = slurpdouble(f)
            self.pan = slurpdouble(f)
            self.preload = slurpint(f)
        elif self.ver==440:
            if self.kind!=-1:
                self.musicdata = slurpdata(f)
                if self.kind==1:
                    midi = True
            self.allow_for_sound_effects = slurpint(f)
            self.buffers = slurpint(f)
            self.load_only_on_use = slurpint(f)
        self.filename = 'out/'+self.name
        print "Sound",self.filename,self.ver,self.kind
        musicdata = zlib.decompress(self.musicdata)
        # TODO Some space savings could probably be had by oggification of
        # sounds.
        of = file(self.filename,'w')
        of.write(musicdata)
        of.close()
        if midi:
            print "A midi",os.spawnvp(os.P_WAIT, 'timidity', ('timidity',self.filename,'-o',self.filename+'.ogg','-Ov'))
            os.unlink(self.filename)
            self.filename += '.ogg'
        del self.musicdata
        gmapi.__dict__[self.name] = self
    def Stop(snd):
        if use_renpy:
            for channel in supersound_compat.channels:
                if (channel in supersound_compat.used_channels) and supersound_compat.used_channels[channel]==snd.filename:
                    renpy.music.stop(channel=channel)
        elif snd.py_soundobject:
            snd.py_soundobject.stop()
    def Play(snd):
        if snd.py_soundobject is None:
            if not use_renpy:
                snd.py_soundobject = pygame.mixer.Sound(snd.filename)
        if use_renpy:
            supersound_compat.playonchan(snd.filename,supersound_compat.available_channel())
            #renpy.sound.play(snd.filename,channel=supersound_compat.available_channel(),loop=False)
        else:
            snd.py_soundobject.play()
    def Loop(snd):
        if snd.py_soundobject is None:
            if not use_renpy:
                snd.py_soundobject = pygame.mixer.music.load(snd.filename)
        if use_renpy:
            renpy.sound.play(snd.filename,channel=supersound_compat.available_channel(),loop=True)
        else:
            pygame.mixer.music.play(-1)
def slurpsound(f):
    if slurpint(f):
        return SoundData(f)
    return None

def SpriteData_load_images(self,name,numsubimages):
    while len(self.subimages):
        self.subimages.pop()
    x = 0
    while x<numsubimages:
        fn = 'out/'+name+'.'+str(len(self.subimages))
        frame = gmapi.renpy_glue.readsurf(fn+'.png',transparent=self.transparent)
        if not frame:
            frame = gmapi.renpy_glue.readsurf(fn+'.bmp',transparent=self.transparent)
        self.subimages.append(frame)
        if self.transparent:
            for frame in self.subimages:
                # Docs say bottom left is used for backgrounds.  Not
                # sure if it's also used for sprites.
                frame.set_colorkey(frame.get_at((0,frame.get_height()-1)))
        x+=1

def SpriteData(f,name=None):
    if type(f) is pygame.Surface:
        raise "GMSprite_from_surface corrupts lists."
        return gmapi.py_fasttint.GMSprite_from_surface(f)
    self = GMSprite()
    self.name = slurpdata(f)
    ver = slurpint(f)
    #l = unpack("<7l",f.read(28))
    #self.w,self.h = l[0:2]
    #self.leftbound = l[2]
    #print 'lr'
    #self.rightbound = l[3]
    #self.bottombound, self.topbound = l[4:6]
    #self.transparent = l[6]
    self.w,self.h,self.leftbound,self.rightbound,self.bottombound,self.topbound,self.transparent = unpack("<7l",f.read(28))
    if ver>=542:
        self.smooth_edges,self.preload_texture = unpack("<2l",f.read(8))
    self.bounding_box_type = slurpint(f)
    # 0 = automatic, 1=full image, 2=manual
    self.precise_collision_checking = slurpint(f)
    if ver==400:
        self.use_video_memory = slurpint(f)
        self.load_only_on_use = slurpint(f)
    self.xorigin,self.yorigin,numsubimages = unpack("<3l",f.read(12))
    #self.subimages = []
    while numsubimages>=1:
        if f.read(4)=='\012\0\0\0':
            filename = 'out/'+self.name+'.'+str(len(self.subimages))+'.bmp'
            f2 = file(filename,'w+')
            f2.writelines(zlib.decompress(slurpdata(f)))
            f2.close()
            frame = pygame.image.load(filename)
            try:
                pygame.image.save(frame,'out/'+self.name+'.'+str(len(self.subimages))+'.png')
                os.unlink(filename)
            except:
                pass
            self.subimages.append(frame)
        else:
            self.subimages.append(None)
        numsubimages -= 1
    if self.transparent:
        for frame in self.subimages:
            frame.lock()
            # Docs say bottom left is used for backgrounds.  Not
            # sure if it's also used for sprites.
            frame.set_colorkey(frame.get_at((0,frame.get_height()-1)))
            frame.unlock()
    self.scalecache = dict()
    gmapi.__dict__[self.name] = self
    return self

def slurpsprite(f):
    if slurpint(f):
        return SpriteData(f)
    return None

fasttint_BackgroundData = gmapi.py_fasttint.BackgroundData

def BackgroundData(f):
    self = fasttint_BackgroundData()
    if f is None:
        return self
    self.name = slurpdata(f)
    ver = slurpint(f)
    self.w,self.h,self.transparent = unpack('<3l',f.read(12))
    if ver==400:
        use_video_memory, load_only_on_use = unpack('<2l',f.read(8))
    else:
        smooth_edges, preload_texture, use_as_tile_set, tile_width, tile_height, hoff, voff, hsep, vsep = unpack('<9l',f.read(36))
    image_exists = f.read(4)!='\0\0\0\0'
    if f.read(4)!='\377\377\377\377':
        dump_file('out/'+self.name,zlib.decompress(slurpdata(f)))
        self.surf = pygame.image.load('out/'+self.name)
        try:
            pygame.image.save(self.surf,'out/'+self.name+'.png')
            os.unlink('out/'+self.name)
        except:
            print "Background dump error:", sys.exc_type,sys.exc_value,sys.exc_traceback
            pass
        if self.transparent:
            self.surf.set_colorkey(self.surf.get_at((0,self.surf.get_height()-1)))
        self.scaled_data_cache = dict()
    else:
        self.surf = None
    gmapi.__dict__[self.name] = self
    return self

def BackgroundData_load_images(self):
    if use_renpy:
        try:
            fn = renpy.exports.config.searchpath[0]+'/out/'+self.name+'.png'
            self.surf = pygame.image.load(fn)
        except:
            self.surf = pygame.image.load(renpy.exports.config.searchpath[0]+'/out/'+self.name)
    else:
        try:
            self.surf = pygame.image.load('out/'+self.name+'.png')
        except:
            self.surf = pygame.image.load('out/'+self.name)
    if self.transparent:
        self.surf.set_colorkey(self.surf.get_at((0,self.surf.get_height()-1)))

def Background_SetFromFile(self,filename):
    if type(filename) is GMVar:
        filename = filename[0]
    self.surf = gmapi.renpy_glue.readsurf(filename)
    self.w,self.h = self.surf.get_size()
    return True

class BackgroundData_OLD(object):
    def __eq__(self,other):
        if type(other) is GMVar:
            return self is other[0]
        return self is other
    def __ne__(self,other):
        if type(other) is GMVar:
            return self is not other[0]
        return self is not other
    def __getstate__(self):
        ret = dict(self.__dict__)
        del ret['image_data']
        return ret
    def __setstate__(self,state):
        self.__dict__ = state
        if use_renpy:
            try:
                fn = renpy.exports.config.searchpath[0]+'/out/'+self.name+'.png'
                self.image_data = pygame.image.load(fn)
            except:
                self.image_data = pygame.image.load(renpy.exports.config.searchpath[0]+'/out/'+self.name)
        else:
            try:
                self.image_data = pygame.image.load('out/'+self.name+'.png')
            except:
                self.image_data = pygame.image.load('out/'+self.name)
        if self.transparent:
            self.image_data.set_colorkey(self.image_data.get_at((0,self.image_data.get_height()-1)))
    def __init__(self,f=None):
        if f is None:
            return
        self.name = slurpdata(f)
        self.ver = slurpint(f)
        self.w,self.h,self.transparent = unpack('<3l',f.read(12))
        if self.ver==400:
            self.use_video_memory,self.load_only_on_use = unpack('<2l',f.read(8))
        else:
            self.smooth_edges, self.preload_texture, self.use_as_tile_set, self.tile_width, self.tile_height, self.hoff, self.voff, self.hsep, self.vsep = unpack('<9l',f.read(36))
        self.image_exists = f.read(4)!='\0\0\0\0'
        if f.read(4)!='\377\377\377\377':
            dump_file('out/'+self.name,zlib.decompress(slurpdata(f)))
            self.image_data = pygame.image.load('out/'+self.name)
            try:
                pygame.image.save(self.image_data,'out/'+self.name+'.png')
                os.unlink('out/'+self.name)
            except:
                pass
            if self.transparent:
                self.image_data.set_colorkey(self.image_data.get_at((0,self.image_data.get_height()-1)))
            self.scaled_data_cache = dict()
        else:
            self.image_data = None
        gmapi.__dict__[self.name] = self
    def SetFromFile(self,filename):
        if type(filename) is GMVar:
            filename = filename[0]
        self.image_data = gmapi.renpy_glue.readsurf(filename)
        self.w,self.h = self.image_data.get_size()
        return True
    def ScaledData(self, xscale, yscale):
        key = (xscale,yscale)
        if self.scaled_data_cache.has_key(key):
            return self.scaled_data_cache[key]
        self.scaled_data_cache[key] = gmapi.py_fasttint.scale(self.image_data,xscale,yscale)
        return self.scaled_data_cache[key]
gmapi.py_BackgroundData = BackgroundData

def slurpbackground(f):
    if slurpint(f):
        return BackgroundData(f)
    return None

#SYNT_EXPECTING_STATEMENT = 0
#SYNT_EXPECTING_NAMELIST = 1
#SYNT_EXPECTING_LVALUE = 2
#SYNT_GOT_LVALUE = 3
#class SyntaxTreeNode_OLD(object):
#    def __init__(self, depth=0, parent=None):
#        self.state = SYNT_EXPECTING_STATEMENT
#        self.tokname = ''
#        self.depth = depth
#        self.parent = parent

"""
Things to read:
/* */ comments
// comments
declaration 
for (init statement, continue-while-true-condition, incrementor statement) block

"""

NODE_BLOCK      = 1000
NODE_VAR        = 1001
NODE_IF         = 1002
NODE_CALL       = 1003
NODE_VARNAME    = 1004  # For 'if x' and the like
NODE_ASSIGN     = 1005
NODE_REAL       = 1006
NODE_RETURN     = 1007
NODE_OPERATOR   = 1008
NODE_SWITCH     = 1009 # Main switch block.
NODE_CASE       = 1010
NODE_BREAK      = 1011
NODE_FOR        = 1012
NODE_ADD        = 1013
NODE_SUB        = 1014
NODE_MULTIPLY   = 1015
NODE_DIVIDE     = 1016
NODE_STRING     = 1017
NODE_ARRAYREF   = 1018
NODE_MEMBER     = 1019
NODE_UNARY_MINUS= 1020
NODE_EXIT       = 1021
NODE_WITH       = 1022
NODE_WHILE      = 1023
NODE_REPEAT     = 1024
NODE_DO         = 1025

class SyntaxNode(object):
    def __init__(node,type):
        node.type = type
        if type==NODE_BLOCK:
            node.statements = []
        elif type==NODE_VAR:
            node.vars = []
        elif type==NODE_IF:
            node.conditions = []
            node.thenblock = None
            node.elseblock = None
        elif type==NODE_CALL:
            node.funcname = None
            node.args = []
        elif type==NODE_VARNAME:
            node.name = None
        elif type==NODE_ASSIGN:
            node.target = None
            node.value = None
        elif type==NODE_REAL:
            node.val = None
        elif type==NODE_RETURN: # What _is_ a blank return?
            node.val = None
        elif type==NODE_OPERATOR:
            node.operator = None
            return
        elif type==NODE_SWITCH:
            node.condition = None
            node.cases = []
        elif type==NODE_CASE:
            node.condition = None
            node.block = None
        elif type==NODE_BREAK:
            return
        elif type==NODE_FOR:
            node.setupblock = None
            node.condition = None
            node.incrementblock = None
            node.block = None
        elif type==NODE_ADD:
            node.target = None
            node.value = None
        elif type==NODE_SUB:
            node.target = None
            node.value = None
        elif type==NODE_MULTIPLY:
            node.target = None
            node.value = None
        elif type==NODE_DIVIDE:
            node.target = None
            node.value = None
        elif type==NODE_STRING:
            node.val = None
        elif type==NODE_ARRAYREF:
            node.name = None
            node.idx = None
        elif type==NODE_MEMBER:
            node.obj = None
            node.member = None
        elif type==NODE_UNARY_MINUS:
            node.value = None
        elif type==NODE_EXIT:
            return
        elif type==NODE_WITH:
            node.obj = None
            node.block = None
        elif type==NODE_WHILE:
            node.condition = None
            node.block = None
        elif type==NODE_REPEAT:
            node.times = None
            node.block = None
        elif type==NODE_DO:
            node.block = None
            node.condition = None
        else:
            raise "Invalid node type."
    def __repr__(node):
        if node.type==NODE_BLOCK:
            return '<SyntaxNode: {'+str(node.statements)+'}>'
        elif node.type==NODE_VAR:
            return '<SyntaxNode: var '+str(node.vars)+'>'
        elif node.type==NODE_IF:
            return '<SyntaxNode: if ('+str(node.conditions)+') then '+str(node.thenblock)+' else '+str(node.elseblock)+'>'
        elif node.type==NODE_CALL:
            return '<SyntaxNode: call '+node.funcname+'('+str(node.args)+')>'
        elif node.type==NODE_REAL:
            return '<SyntaxNode: '+str(node.val)+'>'
        elif node.type==NODE_VARNAME:
            return '<SyntaxNode: name '+str(node.name)+'>'
        elif node.type==NODE_ASSIGN:
            return '<SyntaxNode: mov '+str(node.target)+', '+str(node.value)+'>'
        elif node.type==NODE_OPERATOR:
            return '<SyntaxNode: operator '+node.operator+'>'
        elif node.type==NODE_STRING:
            return '<SyntaxNode: "'+node.val+'">'
        elif node.type==NODE_SWITCH:
            return '<SyntaxNode: switch '+str(node.condition)+'{'+str(node.cases)+'}>'
        elif node.type==NODE_ARRAYREF:
            return '<SyntaxNode: '+node.name+'['+str(node.idx)+']>'
        elif node.type==NODE_MEMBER:
            return '<SyntaxNode: member '+str(node.obj)+node.member+'>'
        else:
            return '<SyntaxNode: Unknown: '+str(node.type)+'>'

def safe_varname(s):
    x = 1
    if not s:
        return False
    if not s[0] in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz':
        return False
    while x<len(s):
        if not s[x] in '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz.':
            return False
        x+=1
    return True

def safe_real(s):
    x = 0
    point = False
    while x<len(s):
        if s[x]=='.':
            if point:
                return False
            point = True
        elif not s[x] in '0123456789':
            return False
        x+=1
    return True

BooleanOperators = ('=','&&','&','||','|','>','<','>=','<=','!=',
        '!','~',
        '+','-','*','/',
        '^','mod', 'not', 'div',
        'or', 'and',)

def slurp_call_args(funcname,l,x):
    callnode = SyntaxNode(NODE_CALL)
    callnode.funcname = funcname
    while l[x]==';':
        x += 1
    if l[x]==')':
        return x+1,callnode
    x, subcond = slurp_conditions(l, x)
    callnode.args.append(subcond)
    while l[x]==';':
        x += 1
    while l[x]==',':
        x += 1
        while l[x]==';':
            x += 1
        x, subcond = slurp_conditions(l, x)
        callnode.args.append(subcond)
        while l[x]==';':
            x += 1
    if l[x]==')':
        x += 1
    else:
        print 404,l,x,l[x:]
        raise ") required."
    return x, callnode

def slurp_arrayref(arrname,l,x):
    anode = SyntaxNode(NODE_ARRAYREF)
    anode.name = arrname
    global g_nest_depth
    g_nest_depth += 1
    global g_nest_stack
    g_nest_stack.append('[')
    x,anode.idx = slurp_conditions(l,x+1)
    if l[x]==',':
        x, idx_2 = slurp_conditions(l,x+1)
        anode.idx = (anode.idx, idx_2)
    if l[x]!=']':
        raise '] expected'
    del g_nest_stack[-1]
    g_nest_depth -= 1
    x += 1
    return x, anode

def slurp_conditions(l,x):
    conditions = []
    global g_nest_depth
    while x<len(l):
        if l[x]=='-' and safe_real(l[x+1]):
            ret = SyntaxNode(NODE_REAL)
            ret.val = -float(l[x+1])
            x += 2
            conditions.append(ret)
        elif l[x] in BooleanOperators:
            opnode = SyntaxNode(NODE_OPERATOR)
            conditions.append(opnode)
            opnode.operator = l[x]
            x += 1
        elif safe_varname(l[x]):
            if conditions and not l[x-1] in BooleanOperators:
                return x,conditions
            varname = l[x]
            if x>=len(l)-1:
                conditions.append(SyntaxNode(NODE_VARNAME))
                conditions[-1].name = varname
                return x,conditions
            x += 1
            if l[x]=='(':
                x,callnode = slurp_call_args(varname,l,x+1)
                conditions.append(callnode)
            elif l[x]=='[':
                x, anode = slurp_arrayref(varname,l,x)
                conditions.append(anode)
            else:
                conditions.append(SyntaxNode(NODE_VARNAME))
                conditions[-1].name = varname
            if x<len(l) and l[x]==';' and g_nest_depth and g_nest_stack[-1]=='(':
                x += 1
            elif x<len(l) and not l[x] in BooleanOperators:
                return x, conditions
        elif l[x]=='(':
            g_nest_stack.append('(')
            g_nest_depth += 1
            x, subcond = slurp_conditions(l,x+1)
            conditions.append(subcond)
            x += 1
            del g_nest_stack[-1]
            g_nest_depth -= 1
            while g_nest_depth and g_nest_stack[-1]=='(' and l[x]==';':
                x += 1
            if l[x].startswith('.'):
                x,node = slurp_memberref(conditions[-1],l,x)
                conditions[-1] = node
            elif not l[x] in BooleanOperators:
                return x, conditions
        elif l[x]==')':
            return x,conditions
        elif l[x]==']':
            return x,conditions
        elif safe_real(l[x]):
            ret = SyntaxNode(NODE_REAL)
            try:
                ret.val = float(l[x])
            except:
                raise "float err on"+str(l[x])
            conditions.append(ret)
            x += 1
        elif l[x][0]=='"':
            snode = SyntaxNode(NODE_STRING)
            snode.val = l[x][1:]
            conditions.append(snode)
            x += 1
        elif l[x][0]=='$':
            ret = SyntaxNode(NODE_REAL)
            ret.val = int(l[x][1:],16)
            conditions.append(ret)
            x += 1
        elif l[x]==';' and g_nest_stack and g_nest_stack[-1]=='(':
            x += 1
        else: # not l[x] in BooleanOperators:
            return x,conditions
    return x, conditions

def slurp_memberref(parent,l,x):
    ret = SyntaxNode(NODE_MEMBER)
    ret.obj = parent
    ret.member = l[x]
    return x+1,ret

def slurp_switch(l,x):
    ret = SyntaxNode(NODE_SWITCH)
    x, ret.condition = slurp_conditions(l,x)
    while l[x]==';':
        x+=1
    if l[x]!='{':
        raise "Switch expects {block}, got"+str(l[x:])
    x += 1
    while l[x]!='}':
        while l[x]==';':
            x += 1
        if l[x]=='case':
            case_cond = SyntaxNode(NODE_CASE)
            x,case_cond.condition = slurp_conditions(l,x+1)
            ret.cases.append(case_cond)
        elif l[x]=='default':
            case_cond = SyntaxNode(NODE_CASE)
            case_cond.condition = None
            ret.cases.append(case_cond)
            x += 1
        else:
            raise "Switch block expects cases, got "+str(l[x:])
        if l[x]!=':':
            raise "case expects : after value, got "+str(l[x:])
        x, case_cond.block = slurp_block(l,x+1,('case','default'))
    return x+1, ret

def slurp_block(l,x,extra_endstr=()):
    ret = SyntaxNode(NODE_BLOCK)
    while not l[x] in ('}',)+extra_endstr:
        x,statement = slurp_statement(l,x)
        ret.statements.append(statement)
    if not extra_endstr:
        x += 1
    return x, ret

def slurp_op_assign(nodetype, target, l, x):
    ret = SyntaxNode(nodetype)
    ret.target = target
    x, ret.value = slurp_conditions(l,x)
    return x, ret

def slurp_statement(l,x):
    if l[x]=='break':
        ret = SyntaxNode(NODE_BREAK)
        x += 1
    elif l[x]=='do':
        ret = SyntaxNode(NODE_DO)
        x += 1
        while x<len(l) and l[x]==';':
            x += 1;
        x,ret.block = slurp_statement(l,x)
        while l[x]==';':
            x += 1;
        if l[x]!='until':
            raise "do expects until, got ("+str(x)+")\n"+str(l[x:])+"\n"+str(l)+"\n"
        x,ret.condition = slurp_conditions(l,x+1)
    elif l[x]=='exit':
        ret = SyntaxNode(NODE_EXIT)
        x += 1
    elif l[x]=='for':
        ret = SyntaxNode(NODE_FOR)
        if l[x+1]!='(':
            raise 'for expects ('
        x, ret.setupblock = slurp_statement(l,x+2)
        if l[x]!=';':
            raise 'for expects (;;)'
        x, ret.condition = slurp_conditions(l,x+1)
        if l[x]!=';':
            raise 'for expects (;;)'
        x, ret.incrementblock = slurp_block(l, x+1, (')',))
        while l[x]==';':
            x += 1
        if l[x]==')':
            x += 1
        else:
            raise 'for expects (;;)'
        while l[x]==';':
            x+=1
        if l[x]!='{':
            x,ret.block = slurp_statement(l,x)
        else:
            x,ret.block = slurp_block(l,x+1)
    elif l[x]=='if':
        ret = SyntaxNode(NODE_IF)
        x += 1
        x,ret.conditions = slurp_conditions(l,x) # Perhaps an ignore-; flag?
        #... or maybe a number specifying the syntax nest depth at which to
        #honor a semicolon?  Which requires keeping track of depths, but.
        while l[x]==';':
            x += 1
            while l[x] in BooleanOperators:
                raise '499,CONTINUATION!'
                x, continuation = slurp_conditions(l,x)
                ret.conditions.extend(continuation)
        if l[x]=='then':
            x += 1
        x,ret.thenblock = slurp_statement(l,x)
        while x<len(l) and l[x] == ';':
            x += 1
        if x<len(l) and l[x]=='else':
            x += 1
            while x<len(l) and l[x] == ';':
                x += 1
            x, ret.elseblock = slurp_statement(l,x)
    elif l[x]=='repeat':
        ret = SyntaxNode(NODE_REPEAT)
        x, ret.times = slurp_conditions(l,x+1)
        while x<len(l) and l[x]==';':
            x += 1
        x, ret.block = slurp_statement(l,x)
    elif l[x]=='return':
        ret = SyntaxNode(NODE_RETURN)
        x, ret.val = slurp_conditions(l,x+1)
    elif l[x]=='switch':
        x, ret = slurp_switch(l,x+1)
    elif l[x]=='var':
        x += 1
        ret = SyntaxNode(NODE_VAR)
        while l[x]!=';':
            if safe_varname(l[x]):
                ret.vars.append(l[x])
            else:
                raise "Unsafe variable name.",l[x]
            x+=1
            if l[x]==',':
                x+=1
            elif l[x]!=';':
                raise '; expected.'
    elif l[x]=='while':
        ret = SyntaxNode(NODE_WHILE)
        x,ret.condition = slurp_conditions(l,x+1)
        while l[x]==';':
            x+=1
        x,ret.block = slurp_statement(l,x)
    elif l[x]=='with':
        ret = SyntaxNode(NODE_WITH)
        x, ret.obj = slurp_conditions(l,x+1)
        while l[x]==';':
            x+=1
        x, ret.block = slurp_statement(l,x)
    elif l[x]==';':
        x += 1
        ret = None
    elif l[x]=='{':
        x,ret = slurp_block(l,x+1)
    elif safe_varname(l[x]):
        sx = x+1
        while sx<len(l)-1 and l[sx]==';':
            sx += 1
        if sx<len(l) and l[sx]=='(':
            x,ret = slurp_call_args(l[x],l,sx+1)
            return x,ret
        if sx<len(l) and l[sx]=='[':
            x, target = slurp_arrayref(l[x],l,sx)
            if l[x][0]=='.':
                x, target = slurp_memberref(target,l,x)
        else:
            target = l[x]
            x += 1
        if x>=len(l):
            ret = SyntaxNode(NODE_VARNAME)
            ret.name = target
            return x,ret
        elif l[x]=='=':
            ret = SyntaxNode(NODE_ASSIGN)
            ret.target = target
            # Might need to slurp only one condition.
            x, ret.value = slurp_conditions(l,x+1)
        elif l[x]=='-=':
            x, ret = slurp_op_assign(NODE_SUB, target, l, x+1)
        elif l[x]=='+=':
            x, ret = slurp_op_assign(NODE_ADD, target, l, x+1)
        elif l[x]=='*=':
            x, ret = slurp_op_assign(NODE_MULTIPLY, target, l, x+1)
        elif l[x]=='/=':
            x,ret = slurp_op_assign(NODE_DIVIDE, target, l, x+1)
        elif l[x]=='|=':
            x,ret = slurp_op_assign(NODE_BITWISEOR, target, l, x+1)
        elif l[x]=='&=':
            x,ret = slurp_op_assign(NODE_BITWISEAND, target, l, x+1)
        elif l[x]=='^=':
            x, ret = slurp_op_assign(NODE_BITWISEXOR, target, l, x+1)
        else:
            raise "412: Unexpected keyword or varname "+str(l[x:])+str(l)
    elif safe_real(l[x]):
        ret = SyntaxNode(NODE_REAL)
        ret.val = float(l[x])
        x += 1
    else:
        print "ERROR token",l,x,l[x:]
        x += 1
    return x,ret

def group_tokens(l):
    x = 0
    rootnode = SyntaxNode(NODE_BLOCK)
    while x<len(l):
        x,statement = slurp_statement(l,x)
        rootnode.statements.append(statement)
    return rootnode

reserved_varnames = dict()
class GlobalObject(object):
    pass
class Magic_Arrint_OLD(int):
    def __init__(self,val):
        int.__init__(self,val)
        self.list = []
        self.list.append(val)
    def __getitem__(ma,idx):
        while idx>=len(ma.list):
            ma.list.append(Magic_Arrint(0))
        return ma.list[idx]
    def __setitem__(ma,idx,val):
        while idx>=len(ma.list):
            ma.list.append(Magic_Arrint(0))
        ma.list[idx] = val
    def __len__(ma):
        return len(ma.list)
class GlobalObjectAutoZero(object):
    def __getattr__(obj, attr):
        if attr in obj.__dict__:
            print attr
            return object.__getattr__(obj,attr)
        ret = obj.__dict__[attr] = GMVar(0);
        return ret#Magic_Arrint(0)
reserved_varnames['global'] = GlobalObject()
gmapi.py_reserved_varnames = reserved_varnames
mp_grid_instances = list()
class MpGrid(object):
    def __init__(grid,left,top,hcells,vcells,cellw,cellh):
        grid.left = left
        grid.top = top
        grid.cells = []
        while len(grid.cells)<vcells:
            row = []
            while len(row)<hcells:
                row.append(0)
            grid.cells.append(row)
        grid.cellw = cellw
        grid.cellh = cellh
        grid.hcells = hcells
        grid.vcells = vcells
        mp_grid_instances.append(grid)
gmapi.mp_grid_create = MpGrid
def mp_grid_destroy(grid):
    print 772,'mp_grid_destroy'
    mp_grid_instances.remove(grid)
gmapi.mp_grid_destroy = mp_grid_destroy


def inttorgb(i):
    return (i%256,(i>>8)%256,i>>16)
def interpolate_bytes(a,b):
    return (a+b)>>1
def interpolate_rgb(start, end):
    return ((start[0]+end[0])>>1,
        (start[1]+end[1])>>1,
        (start[2]+end[2])>>1,
        )

def call_to_python(statement,vars):
    if statement.funcname in verbose_calls:
        ret = 'py_verbose_call('+statement.funcname+','
    #elif statement.funcname=='string_char_at':
    #    return value_to_python(statement.args[0],vars)+'[int('+value_to_python(statement.args[1],vars)+')-1]'
    else:
        ret = statement.funcname+'('
    y = 0
    while y<len(statement.args)-1:
        ret += value_to_python(statement.args[y],vars)
        ret += ','
        y += 1
    if y<len(statement.args):
        ret += value_to_python(statement.args[y],vars)
    return ret + ')'

room_object_vars = (
    'background_color', 'background_showcolor',
    'background_visible', 'background_foreground',
    'background_index', 'background_x','background_y',
    'background_width', 'background_height', # Readonly
    'background_htiled', 'background_vtiled',
    'background_xscale', 'background_yscale',
    'background_hspeed', 'background_vspeed',
    'background_blend', # Default is c_white
    'background_alpha', # NOTE registered only : 0.0 - 1.0
    'room_width', 'room_height', 'room_caption', 
    # XXX is room_caption supposed to change the window title when changed?
    'room_persistent', 'room_speed',
    'view_enabled', 'view_current', 'view_visible',
    'view_xview', 'view_yview', 'view_wview', 'view_hview',
    'view_xport', 'view_yport', 'view_wport', 'view_hport',
    'view_angle', 'view_hborder', 'view_vborder',
    'view_hspeed', 'view_vspeed',
    'view_object',)

force_arrays = (
        'background_alpha', 'background_blend',
        'background_foreground', 
        'background_height',
        'background_hspeed',
        'background_htiled',
        'background_index',
        'background_visible', 'background_vspeed', 'background_vtiled',
        'background_width',
        'background_x', 'background_xscale',
        'background_y', 'background_yscale',
        'view_xview', 'view_yview',
        )
reserved_varname_translations = dict()

def resolve_reserved_varname(nodename):
    if nodename in reserved_varname_translations:
        return reserved_varname_translations[nodename]
    global event_handler_counter
    event_handler_counter += 1
    newvarname = 'py_reserved_varname_'+str(event_handler_counter)
    reserved_varname_translations[nodename] = newvarname
    gmapi.__dict__[newvarname] = gmapi.py_reserved_varnames[nodename]
    return newvarname
    #return 'py_reserved_varnames["'+nodename+'"]'

def prefix_varname(nodename,vars):
    if nodename in reserved_varnames:
        return resolve_reserved_varname(nodename)
    elif nodename in room_object_vars:
        return 'room.'+nodename
    return 'self.'+nodename

def varname_to_python(nodename,vars):
    if type(nodename)==list:
        return value_to_python(nodename,vars)
    elif type(nodename)==SyntaxNode:
        return value_to_python([nodename,],vars)
    if '.' in nodename:
        fields = string.split(nodename,'.')
        ret = varname_to_python(fields[0],vars)+'.'
        for field in fields[1:-1]:
            ret += field+'.'
        return ret + fields[-1]
    elif nodename in reserved_varnames:
        return resolve_reserved_varname(nodename)
    elif nodename in room_object_vars:
        return 'room.'+nodename
    elif nodename.startswith('argument') and nodename[8] in '01234567' and len(nodename)==9:
        return 'args['+nodename[8]+']'
    elif nodename=='false':
        return 'False'
    elif nodename=='true':
        return 'True'
    elif nodename in gmapi.__dict__:
        return nodename
    elif not nodename in vars:
        return 'self.'+nodename
    else:
        return nodename

def array_indices_to_python(idx, vars):
    #ret = '[int('
    if type(idx)!=tuple:
        return '['+value_to_python_int(idx,vars)+']'
    else:
        return '['+value_to_python_int([idx[0],],vars)+','+value_to_python_int([idx[1],],vars)+']'

def value_to_python_int(cond,vars):
    if len(cond)==1 and type(cond[0])==SyntaxNode and cond[0].type==NODE_REAL:
        return str(int(cond[0].val))
    return 'int('+value_to_python(cond,vars)+')'

def value_to_python(cond,vars):
    ret = ''
    if type(cond)==SyntaxNode:
        cond = (cond,)
    for node in cond:
        if type(node)==list:
            ret += '('
            ret += value_to_python(node,vars)
            ret += ')'
        elif type(node)==str:
            print 825,"String node",node
            return node
        elif type(node)==tuple:
            print 828,"tuple node",node
        elif node.type==NODE_VARNAME:
            if node.name in force_arrays and len(cond)>1:
                ret += prefix_varname(node.name,vars)+'[0]'
            else:
                ret += varname_to_python(node.name,vars)
        elif node.type==NODE_OPERATOR:
            if node.operator=='!':
                ret += ' not '
            elif node.operator=='=':
                ret += '=='
            elif node.operator=='&&':
                ret += ' and '
            elif node.operator=='||':
                ret += ' or '
            elif node.operator=='mod':
                ret += '%'
            elif node.operator=='not':
                ret += ' not ' # XXX are there different precedence orders for not/!?
            elif node.operator=='div':
                ret += '//'
            elif node.operator=='or':
                ret += ' or '
            elif node.operator=='and':
                ret += ' and '
            else:
                ret += node.operator
        elif node.type==NODE_STRING:
            #XXX escape characters where needed.
            ret += '"'
            for char in node.val:
                if char in "\\\"":
                    ret += "\\"+char
                elif char not in "\r\n":
                    ret += char
            ret += '"'
        elif node.type==NODE_MEMBER:
            ret += value_to_python([node.obj,],vars)+node.member
        elif node.type==NODE_CALL:
            ret += call_to_python(node,vars)
        elif node.type==NODE_REAL:
            ret += str(node.val)
        elif node.type==NODE_ARRAYREF:
            ret += varname_to_python(node.name,vars)+array_indices_to_python(node.idx,vars)
        else:
            raise "Unexpected node type in value_to_python "+str(node)+':'+str(node.type)
    return ret

class AddableList(gmapi.py_PythonAutovivifyingArray):
    def __add__(l,a):
        return l[0]+a
    def __sub__(l,a):
        return l[0]-a
    def __iadd__(l,a):
        l[0]+=a
        return l
    def __int__(l):
        return int(l[0])

def is_watched(target):
    return target in watched_variables

def idx_to_python(node, vars):
    if type(node) is SyntaxNode:
        if node.type==NODE_REAL:
            return str(int(node.val))
    if type(node) is list and len(node)==1:
        return idx_to_python(node[0],vars)
    elif type(node) is tuple:
        return idx_to_python(node[0],vars)+','+idx_to_python(node[1],vars)
    return value_to_python(node,vars)

magic_assignors_array = dict(background_index='room.SetBackgroundIndex')

def py_assign_array(node,vars,depth):
    if node.target.name in magic_assignors_array:
        return (' '*depth)+magic_assignors_array[node.target.name]+'(int('+value_to_python(node.target.idx,vars)+'),'+value_to_python(node.value,vars)+')\n'
    return (' '*depth)+varname_to_python(node.target.name,vars)+'['+idx_to_python(node.target.idx,vars)+']='+value_to_python(node.value,vars)

def py_assign_array_OLD(node, vars, depth):
    varname = varname_to_python(node.target.name,vars)
    splitname = varname.rsplit('.',1)
    if splitname[-1] == 'background_index':
        return (' '*depth)+'room.SetBackgroundIndex(int('+value_to_python(node.target.idx,vars)+'),'+value_to_python(node.value,vars)+')\n'
    if game.default_undef_is_zero:
        arrinit = 'py_PythonAutovivifyingZeroArray'
    else:
        arrinit = 'py_PythonAutovivifyingArray'
    if type(node.target.idx)==tuple:
        arrinit += '2D()'
    else:
        arrinit += '()'
    valpython = value_to_python(node.value,vars)
    ret = (
            (' '*depth)+'try:\n'+
            (' '*depth)+' len('+varname+')\n'+
            (' '*depth)+' '+value_to_python([node.target,],vars)+'='+valpython+'\n'+
            (' '*depth)+'except:\n')
    if len(splitname)==2:
        ret += (
            (' '*depth)+' if type('+splitname[0]+')==gm6.GM_Object:\n'+
            (' '*depth)+'  py_vivify_array_val='+valpython+'\n'+
            (' '*depth)+'  for py_vivify_array_obj in tuple('+splitname[0]+'):\n'+
            (' '*depth)+'   py_vivify_array_obj.'+splitname[1]+'='+arrinit+'\n'+
            (' '*depth)+'   py_vivify_array_obj.'+splitname[1]+array_indices_to_python(node.target.idx,vars)+'=py_vivify_array_val\n'+
            (' '*depth)+'   print py_vivify_array_obj,py_vivify_array_obj.'+splitname[1]+'\n'+
            (' '*depth)+' else:\n'+
            (' '*depth)+'  '+varname+'='+arrinit+"\n"+
            (' '*depth)+'  '+value_to_python([node.target,],vars)+'='+valpython+'\n'+
            '')
    else:
        ret += (
            (' '*depth)+' '+varname+'='+arrinit+'\n')
    return ret
            #ret = py_vivify_array(statement.target,vars,depth=depth)+ret
            #ret += value_to_python([statement.target,],vars)+'='
            #ret += value_to_python(statement.value,vars)

def py_vivify_array_OLD(node, vars,depth):
    varname = varname_to_python(node.name,vars)
    splitname = varname.rsplit('.',1)
    if game.default_undef_is_zero:
        arrinit = 'py_PythonAutovivifyingZeroArray'
    else:
        arrinit += 'py_PythonAutovivifyingArray'
    if type(node.idx)==tuple:
        arrinit += '2D()'
    else:
        arrinit += '()'
    ret = (
            (' '*depth)+'try:\n'+
            (' '*depth)+' len('+varname+')\n'+
            (' '*depth)+'except:\n')
    if len(splitname)==2:
        ret += (
            #(' '*depth)+' py_vivify_array_val =
            (' '*depth)+' if type('+splitname[0]+')==gm6.GM_Object:\n'+
            (' '*depth)+'  for py_vivify_array_obj in tuple('+splitname[0]+'):\n'+
            (' '*depth)+'   py_vivify_array_obj.'+splitname[1]+'='+arrinit+'\n'+
            (' '*depth)+' else:\n'+
            (' '*depth)+'  '+varname+'='+arrinit+"\n"
            )
    else:
        ret += (
            (' '*depth)+varname+'='+arrinit+'\n')
    return ret

magic_assignors_scalar = dict(x='SetX',y='SetY',depth='SetDepth',sprite_index='SetSpriteIndex',speed='SetSpeed',
        hspeed='SetHSpeed',
        vspeed='SetVSpeed',
        visible='SetVisible',
        direction='SetDirection',
        gravity='SetGravity',
        image_speed='SetImageSpeed',
        image_single='SetImageSingle',
        path_speed='SetPathSpeed'
        )

def magic_assignment_seq(target,value,vars,depth):
    tail = ''
    if target[-1]=='background_index':
        ret = 'room.SetBackgroundIndex(0,'
        tail = ')'
    elif target[-1] in magic_assignors_scalar:
        if len(target)>1:
            ret = varname_to_python(string.join(target[:-1],'.'),vars)+'.'+magic_assignors_scalar[target[-1]]+'('
        else:
            ret = 'self.'+magic_assignors_scalar[target[-1]]+'('
        tail = ')'
    else:
        target = string.join(target,'.')
        ret = varname_to_python(target,vars)
        if target in force_arrays:
            ret += '[0]'
        ret += '='
    if is_watched(target):
        ret += "py_assignment_debug('"+varname_to_python(target,vars)+"',"
    ret += 'py_GMVar('+value_to_python(value,vars)+')'
    if is_watched(target):
        ret += ')'
    return ret+tail

def magic_assignment_txt(target,value,vars,depth):
    real_target = string.split(target,'.')
    return magic_assignment_seq(real_target,value,vars,depth)

magic_assignors_opid = dict()
magic_assignors_opid['+'] = '_add'
magic_assignors_opid['-'] = '_sub'
magic_assignors_opid['*'] = '_mul'

def opass_to_python(statement,vars,op,depth=0):
    if type(statement.target) is SyntaxNode and statement.target.type==NODE_ARRAYREF:
        return varname_to_python(statement.target,vars)+op+'='+value_to_python(statement.value,vars)
    target = statement.target.rsplit('.',2)
    if target[-1] in magic_assignors_scalar:
        if len(target)>1:
            return varname_to_python(target[0],vars)+'.'+magic_assignors_scalar[target[-1]]+magic_assignors_opid[op]+'('+value_to_python(statement.value,vars)+')'
        else:
            return 'self.'+magic_assignors_scalar[target[0]]+magic_assignors_opid[op]+'('+value_to_python(statement.value,vars)+')'
    return varname_to_python(statement.target,vars)+op+'='+value_to_python(statement.value,vars)

def regroup_conditions_muldiv(l):
    # I don't think I need to regroup these, do I?
    if len(l)==1:
        return l[0]
    return l

def regroup_conditions_addsub(l):
    ret = [[]]
    for cond in l:
        if type(cond) is SyntaxNode and cond.type==NODE_OPERATOR and cond.operator in '+-':
            ret.append(cond)
            ret.append([])
        else:
            ret[-1].append(cond)
    ret2 = []
    for grp in ret:
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        else:
            ret2.append(regroup_conditions_muldiv(grp))
    if len(ret2)==1:
        return ret2[0]
    return ret2


def regroup_conditions_bitshift(l):
    ret = [[]]
    for cond in l:
        if type(cond) is SyntaxNode and cond.type==NODE_OPERATOR and cond.operator in ('<<', '>>'):
            ret.append(cond)
            ret.append([])
        else:
            ret[-1].append(cond)
    ret2 = []
    for grp in ret:
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        else:
            ret2.append(regroup_conditions_addsub(grp))
    if len(ret2)==1:
        return ret2[0]
    return ret2

def regroup_conditions_bitlog(l):
    if len(l)==1:
        return l[0]
    ret = [[]]
    for cond in l:
        if type(cond) is SyntaxNode and cond.type==NODE_OPERATOR and cond.operator in '|&^':
            ret.append(cond)
            ret.append([])
        else:
            ret[-1].append(cond)
    ret2 = []
    for grp in ret:
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        elif grp:
            ret2.append(regroup_conditions_bitshift(grp))
    if len(ret2)==1:
        return ret2[0]
    return ret2

def regroup_conditions_cmp(l):
    ret = [[]]
    for cond in l:
        if type(cond) is SyntaxNode and cond.type==NODE_OPERATOR and cond.operator in ('<','<=', '==', '=', '!=','>','>='):
            ret.append(cond)
            ret.append([])
        else:
            ret[-1].append(cond)
    ret2 = []
    for grp in ret:
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        elif grp:
            ret2.append(regroup_conditions_bitlog(grp))
    if len(ret2)==1:
        return ret2[0]
    return ret2

def regroup_conditions_or(l):
    ret = [[]]
    l2 = list(l)
    print "Grouping or",l2
    while l2:
        cond = l2[0]
        del l2[0]
        if type(cond)==list:
            ret.append(regroup_conditions(cond))
        elif cond.type==NODE_OPERATOR and cond.operator in ('or','||'):
            node = SyntaxNode(NODE_CALL)
            node.funcname = 'py_or'
            node.args = [ret[-1],regroup_conditions(l2)]
            l2 = []
            ret[-1] = node
            ret.append([])
        else:
            ret[-1].append(cond)
    ret2 = []
    for grp in ret:
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        elif grp:
            ret2.append(regroup_conditions_cmp(grp))
    if len(ret2)==1:
        return ret2[0]
    return ret2

def regroup_conditions_OLD(l):
    ret = [[]]
    for cond in l:
        if type(cond)==list:
            if ret[-1]:
                ret.append(regroup_conditions(cond))
            else:
                ret[-1] = regroup_conditions(cond)
            print "+l",ret,cond
        elif cond.type==NODE_OPERATOR and cond.operator in ('&&','^^','and','xor'):
            ret.append(cond)
            ret.append([])
        else:
            ret[-1].append(cond)
        print 1216,ret
    ret2 = []
    for grp in ret:
        print 1219,grp
        if type(grp) is SyntaxNode:
            ret2.append(grp)
        elif grp:
            ret2.append(regroup_conditions_or(grp))
    return ret2

breakers = (
        ('&&','and','^^','xor'),
        ('||','or'),
        ('not','!'),
        ('<','<=','==','=','!=','>','>=',),
        ('|','&','^'),
        ('<<','>>'),
        ('+','-'),
        ('*','/', 'div','mod'),
        )

def regroup_conditions(l,breakerlvl=0):
    ret = []
    acc = []
    if len(l)==1:
        if type(l[0])==list:
            return regroup_conditions(l[0])
        return l[0]
    if breakerlvl>=len(breakers):
        return l
    for cond in l:
        if type(cond)==list:
            #if acc:
            #    ret.append(regroup_conditions(acc,breakerlvl=breakerlvl+1))
            #    acc = []
            acc.append(regroup_conditions(cond))
        elif cond.type==NODE_OPERATOR and cond.operator in breakers[breakerlvl]:
            if acc:
                ret.append(regroup_conditions(acc,breakerlvl=breakerlvl+1))
                acc = []
            ret.append(cond)
        else:
            acc.append(cond)
    if acc:
        ret.append(regroup_conditions(acc,breakerlvl=breakerlvl+1))
    if len(ret)==1:
        return ret[0]
    return ret

def statement_to_python(statement,vars,depth=0,loop_stack=-1):
    if statement is None:
        return ''
    if statement.type!=NODE_BLOCK:
        ret = ' '*depth
    if statement.type==NODE_BLOCK:
        ret = tree_to_python(statement,vars,depth=depth,loop_stack=loop_stack)
    elif statement.type==NODE_VAR:
        if game.default_undef_is_zero:
            ret = ''
        for var in statement.vars:
            if game.default_undef_is_zero:
                ret += (' '*depth)+var+'=0\n'
            vars[var] = var
    elif statement.type==NODE_IF:
        conditions = regroup_conditions(statement.conditions)
        ret += 'if '+value_to_python(conditions,vars)+":\n"
        thenblock = tree_to_python(statement.thenblock,vars=vars,depth=depth+1,loop_stack=loop_stack)
        if thenblock:
            ret += thenblock
        else:
            ret += ' '*(depth+1)+'pass\n'
        if statement.elseblock is not None:
            ret += ' '*depth
            ret += "else:\n"
            ret += tree_to_python(statement.elseblock,vars=vars,depth=depth+1,loop_stack=loop_stack)
    elif statement.type==NODE_CALL:
        ret += call_to_python(statement,vars)
    elif statement.type==NODE_VARNAME:
        ret += varname_to_python(statement.name,vars)
    elif statement.type==NODE_ASSIGN:
        if type(statement.target)!=SyntaxNode:
            ret += magic_assignment_txt(statement.target,statement.value,vars,depth)
        elif statement.target.type==NODE_ARRAYREF:
            ret = py_assign_array(statement,vars,depth)
            #ret = py_vivify_array(statement.target,vars,depth=depth)+ret
            #ret += value_to_python([statement.target,],vars)+'='
            #ret += value_to_python(statement.value,vars)
        elif statement.target.type==NODE_VARNAME and statement.target.name in watched_variables:
            print "1066 Assigning to",statement.target,'from',statement.value
            ret += 'py_value='+value_to_python([statement.target,],vars)+"\n"
            ret += (' '*depth)+'print "Changing '+statement.target.name+' to",py_value\n'
            ret += (' '*depth)+value_to_python([statement.target,],vars)+'[0]=GMVar('+value_to_python(statement.value,vars)+')'
        else:
            ret += value_to_python([statement.target,],vars)+'[0]='+value_to_python(statement.value,vars)
            #raise "An assignment to unknown type "+str(statement.target.type)
    elif statement.type==NODE_ADD:
        ret += opass_to_python(statement,vars,'+',depth=depth)
        #ret += varname_to_python(statement.target,vars)+'+='
        #ret += value_to_python(statement.value,vars)
    elif statement.type==NODE_MULTIPLY:
        ret += opass_to_python(statement,vars,'*',depth=depth)
        #ret += varname_to_python(statement.target,vars)+'*='+value_to_python(statement.value,vars)
    elif statement.type==NODE_SUB:
        ret += opass_to_python(statement,vars,'-',depth=depth)
        #ret += varname_to_python(statement.target,vars)+'-='+value_to_python(statement.value,vars)
    elif statement.type==NODE_EXIT:
        # exit is not C exit(), but rather closer to return.
        # game_end() is C exit.
        if loop_stack and loop_stack[-1]=='switch':
            ret += 'switchctx.returned=True\n'+(' '*depth)
        ret += "return"
    elif statement.type==NODE_RETURN:
        #if loop_stack and loop_stack[-1]=='switch':
        #    ret += 'switchctx.returned=True\n'+(' '*depth)
        if statement.val is not None:
            ret += 'return '+value_to_python(statement.val,vars)
        else:
            ret += "return"
    elif statement.type==NODE_SWITCH:
        ret += 'switchctx=py_switch_context()\n'
        ret += (' '*depth)+'switchctx.val='+value_to_python(statement.condition,vars)+"\n"
        casenum = 0
        while casenum<len(statement.cases):
            if statement.cases[casenum].condition is None:
                ret += (' '*depth)+'if (not switchctx.case_met) or (not switchctx.broke):\n'
            else:
                ret += (' '*depth)+'if (switchctx.val=='+value_to_python(statement.cases[casenum].condition,vars)+") or not switchctx.broke:\n"
            ret += (' '*(depth+1))+'switchctx.broke=False\n'
            ret += tree_to_python(statement.cases[casenum].block,vars,depth=depth+1,loop_stack=loop_stack+['switch'])
            #ret += (' '*depth)+' switchctx.retval = case'+str(casenum)+"()\n"
            #ret += (' '*depth)+' if switchctx.returned:\n'
            #ret += (' '*depth)+'  return switchctx.retval\n'
            casenum += 1
    elif statement.type==NODE_BREAK:
        if loop_stack:
            if loop_stack[-1]=='switch':
                ret += 'switchctx.broke = True'
            elif loop_stack[-1] in ('for','while','repeat'):
                ret += 'break'
            else:
                # Docs say 'with' can be exited using break.
                # Docs do not mention 'do' being exited using break.
                print "WARNING: Not sure what to do with break here",loop_stack
                ret += 'return'
        else:
            ret += 'return'
    elif statement.type==NODE_FOR:
        ret = tree_to_python(statement.setupblock,vars,depth=depth,loop_stack=loop_stack+['for'])
        ret += (' '*depth)+'while '+value_to_python(statement.condition,vars)+':\n'
        ret += tree_to_python(statement.block,vars,depth=depth+1,loop_stack=loop_stack+['for'])
        ret += tree_to_python(statement.incrementblock,vars,depth=depth+1,loop_stack=loop_stack+['for'])
        #setupblock, condition,incrementblock, block
    # XXX by the way, I suspect that the parser loses the parens in a-b-(c+d)
    elif statement.type==NODE_WITH:
        with_obj = value_to_python(statement.obj,vars)
        ret += 'py_self_stack.append(self)\n'
        ret += (' '*depth)+'py_other_stack.append(other)\n'
        ret += (' '*depth)+'py_objlist = '+with_obj+'\n'
        ret += (' '*depth)+'other = self\n'
        ret += (' '*depth)+'if type(py_objlist) is py_GMVar:\n'
        ret += (' '*depth)+' py_objlist=py_objlist[0]\n'
        ret += (' '*depth)+'if object_exists(py_objlist):\n'
        #ret += (' '*depth)+' if type(py_objlist) in (int,float):\n'
        #ret += (' '*depth)+'  py_objlist=gm6.game.objects[int(py_objlist)]\n'
        ret += (' '*depth)+' for self in tuple(py_objlist):\n'
        #ret += (' '*depth)+'  if type(self)==int:\n'
        #ret += (' '*depth)+'   self = gm6.game.objects[self]\n'
        #ret += (' '*depth)+'   print 997,"meta",self\n'
        ret += tree_to_python(statement.block,vars,depth=depth+2,loop_stack=loop_stack+['with'])
        ret += (' '*depth)+'elif instance_exists(py_objlist):\n'
        ret += (' '*depth)+' py_loopcounter = 1\n'
        ret += (' '*depth)+' self = py_objlist\n'
        ret += (' '*depth)+' if type(self)==int:\n'
        ret += (' '*depth)+'  self = gm6.instances[self]\n'
        ret += (' '*depth)+' while py_loopcounter:\n'
        ret += (' '*depth)+'  py_loopcounter=0\n'
        ret += tree_to_python(statement.block,vars,depth=depth+2,loop_stack=loop_stack+['with'])
        ret += (' '*depth)+'other = py_other_stack.pop()\n'
        ret += (' '*depth)+'self = py_self_stack.pop()'
    elif statement.type==NODE_WHILE:
        ret += 'while '+value_to_python(statement.condition,vars)+":\n"
        ret += statement_to_python(statement.block,vars,depth=depth+1,loop_stack=loop_stack+['while'])
    elif statement.type==NODE_REPEAT:
        ret += 'py_repeat_stack.append('+value_to_python(statement.times,vars)+')\n'
        ret += (' '*depth)+'while py_repeat_stack[-1]>0:\n'
        ret += statement_to_python(statement.block,vars,loop_stack=loop_stack+['repeat'],depth=depth+1)
        ret += (' '*depth)+' py_repeat_stack[-1]-=1\n'
        ret += (' '*depth)+'py_repeat_stack.pop()'
    elif statement.type==NODE_DO:
        ret += 'while 1:\n'
        ret += statement_to_python(statement.block,vars,loop_stack=loop_stack+['do'],depth=depth+1)
        ret += (' '*depth)+' if '+value_to_python(statement.condition,vars)+":\n"
        ret += (' '*depth)+'  break\n'
    elif statement is not None:
        print "Unexpected type for statement",statement.type
        ret += '#'+str(statement)
    if statement is not None:
        ret += '\n'
    return ret

def get_vivifiers(root,vars):
    if root is None:
        return
    elif type(root) in (list,tuple):
        for statement in root:
            get_vivifiers(statement,vars)
        return
    elif type(root) is type(""):
        return
    t = root.type
    if t==NODE_BLOCK:
        for statement in root.statements:
            get_vivifiers(statement,vars)
    elif t==NODE_VAR:
        return
    elif t==NODE_IF:
        get_vivifiers(root.conditions,vars)
        get_vivifiers(root.thenblock,vars)
        get_vivifiers(root.elseblock,vars)
    elif t == NODE_CALL:
        get_vivifiers(root.args,vars)
    elif t==NODE_VARNAME:
        return
    elif t==NODE_ASSIGN:
        get_vivifiers(root.target,vars)
        get_vivifiers(root.value,vars)
    elif t==NODE_REAL:
        return
    elif t==NODE_RETURN:
        get_vivifiers(root.val,vars)
    elif t==NODE_OPERATOR:
        return
    elif t==NODE_SWITCH:
        get_vivifiers(root.condition,vars)
        get_vivifiers(root.cases,vars)
    elif t==NODE_CASE:
        get_vivifiers(root.condition,vars)
        get_vivifiers(root.block,vars)
    elif t==NODE_BREAK:
        return
    elif t==NODE_FOR:
        get_vivifiers(root.setupblock,vars)
        get_vivifiers(root.condition,vars)
        get_vivifiers(root.incrementblock,vars)
        get_vivifiers(root.block,vars)
    elif t in (NODE_ADD, NODE_SUB, NODE_MULTIPLY, NODE_DIVIDE):
        get_vivifiers(root.target,vars)
        get_vivifiers(root.value,vars)
    elif t==NODE_STRING:
        return
    elif t==NODE_ARRAYREF:
        get_vivifiers(root.idx,vars)
        if type(root.idx)==tuple:
            vars.add((root.name,2))
        else:
            vars.add((root.name,1))
    elif t==NODE_MEMBER:
        get_vivifiers(root.obj,vars)
    elif t==NODE_UNARY_MINUS:
        get_vivifiers(root.value)
    elif t==NODE_EXIT:
        return
    elif t==NODE_WITH:
        get_vivifiers(root.obj,vars)
        get_vivifiers(root.block,vars)
    elif t==NODE_WHILE:
        get_vivifiers(root.condition,vars)
        get_vivifiers(root.block,vars)
    elif t==NODE_REPEAT:
        get_vivifiers(root.times,vars)
        get_vivifiers(root.block,vars)
    elif t==NODE_DO:
        get_vivifiers(root.block,vars)
        get_vivifiers(root.condition,vars)
    else:
        raise "Unexpected node "+str(root)+" in get_vivifiers ("+str(t)+")"

def tree_to_vivifiers(root,depth=0,vars=None,pythonvars=None):
    if root is None:
        return ''
    if vars is None:
        vars = set()
    get_vivifiers(root,vars)
    ret = ''
    if game.default_undef_is_zero:
        arrinit = 'py_PythonAutovivifyingZeroArray'
    else:
        arrinit = 'py_PythonAutovivifyingArray'
    for k in vars:
        k0 = varname_to_python(k[0],pythonvars)
        if k[1]==1:
            if (k0,2) in vars:
                next
            ret += (' '*depth)+'try:\n'
            ret += (' '*depth)+' len('+k0+')\n'
            ret += (' '*depth)+'except:\n'
            ret += (' '*depth)+' print "vivifying '+k0+'"\n'
            ret += (' '*depth)+' try:\n'
            #ret += (' '*depth)+'  '+k[0]+'\n'
            ret += (' '*depth)+'  '+k0+'='+arrinit+'('+k0+')\n'
            ret += (' '*depth)+' except:\n'
            ret += (' '*depth)+'  '+k0+'='+arrinit+'()\n'
        else:
            # XXX will this actually do for vivifying a 2D array?
            ret += (' '*depth)+'try:\n'
            ret += (' '*depth)+' len('+k0+')\n'
            ret += (' '*depth)+' try:\n'
            ret += (' '*depth)+'  len('+k0[0]+')\n'
            ret += (' '*depth)+' except:\n'
            ret += (' '*depth)+'  print "vivifying 2D/a '+k0+'"\n'
            ret += (' '*depth)+'  '+k0+'[0]='+arrinit+'()\n'
            ret += (' '*depth)+'except:\n'
            ret += (' '*depth)+' print "vivifying 2D '+k0+'"\n'
            #ret += (' '*depth)+'  '+k0+'='+arrinit+'2D('+k0+')\n'
            #ret += (' '*depth)+' except:\n'
            ret += (' '*depth)+' '+k0+'='+arrinit+'2D()\n'
        ret += '\n'
    return ret
        

def tree_to_python(root,vars=None,depth=0,loop_stack=None):
    x = 0
    ret = ''
    if root is None:
        return ''
    if vars is None:
        vars = dict()
    if loop_stack is None:
        loop_stack = []
    if root.type!=NODE_BLOCK:
        ret = statement_to_python(root,vars,depth=depth,loop_stack=loop_stack)
        return ret
    while x<len(root.statements):
        r1 = statement_to_python(root.statements[x],vars,depth=depth,loop_stack=loop_stack)
        ret += r1
        x += 1
    return ret

def tokstream_to_tree(l):
    global g_mungedict
    x = 0
    ret = ''
    SYNT_EXPECT_STATEMENT = 0
    state = SYNT_EXPECT_STATEMENT
    l2 = []
    i = 0
    for v in l:
        try:
            v = g_mungedict.get(v,v)
        except:
            print "Failure:",v,l
        if v:
            if v=='begin':
                l2.append('{')
            elif v=='end':
                l2.append('}')
            else:
                l2.append(v)
    l = l2
    rootnode = group_tokens(l)
    return rootnode

def gml2tokstream(s):
    global g_nest_depth
    g_nest_depth = 0
    global g_nest_stack
    g_nest_stack = []
    x = 0
    tokens = []
    token = ''
    if type(s)==GMVar:#Magic_Arrint:
        return [str(s),]
    while x<len(s):
        if s[x] in '\015\012;':
            tokens.append(token)
            tokens.append(';')
            token = ''
        elif s[x] in '\t ':
            if token:
                tokens.append(token)
                token = ''
        elif s[x]=='"':
            if token:
                tokens.append(token)
                token = ''
            s2 = ''
            x+=1
            while s[x]!='"':
                if s[x]!='\\':
                    s2 += s[x]
                else:
                    x += 1
                    s2 += s[x]
                x += 1
            tokens.append('"'+s2)
        elif s[x]=='&':
            if token:
                tokens.append(token)
                token = ''
            if x<len(s)-1 and s[x+1]=='&':
                tokens.append('&&')
                x+= 1
            else:
                tokens.append('&')
        elif s[x]=="'":
            if token:
                tokens.append(token)
                token = ''
            s2 = ''
            x+=1
            while s[x]!="'":
                s2 += s[x]
                x += 1
            tokens.append('"'+s2)
        elif s[x]==',':
            if token:
                tokens.append(token)
                token = ''
            tokens.append(',')
        elif s[x]=='^' and x<len(s)-1 and s[x+1]=='^':
            if token:
                tokens.append(token)
                token = ''
            tokens.append('^^')
            x += 1
        elif s[x] in '+-*^!':
            if token:
                tokens.append(token)
                token = ''
            if s[x+1]=='=':
                tokens.append(s[x]+'=')
                x += 1
            else:
                tokens.append(s[x])
        elif s[x]=='/':
            if token:
                tokens.append(token)
                token = ''
            if x<len(s)-1:
                if s[x+1]=='*':
                    x += 2
                    while s[x:x+2]!='*/' and x<len(s)-2:
                        x+=1
                    x += 1
                elif s[x+1]=='/':
                    tokens.append(';')
                    x += 2
                    while x<len(s) and not (s[x] in '\015\012'):
                        x+=1
                elif s[x+1]=='=':
                    tokens.append('/=')
                    x += 1
                else:
                    tokens.append('/')
        elif s[x] == ':':
            if token:
                tokens.append(token)
                token = ''
            if s[x+1]=='=':
                tokens.append('=')
                x += 1
            else:
                tokens.append(':')
        elif s[x] in '<>':
            if token:
                tokens.append(token)
                token = ''
            tokens.append(s[x])
            if x<len(s)-1:
                if s[x+1]=='=':
                    tokens[-1]+='='
                    x += 1
                elif s[x+1]==s[x]:
                    tokens[-1]+=s[x]
                    x += 1
        elif s[x]=='=':
            if token:
                tokens.append(token)
                token = ''
            tokens.append('=')
            if x<len(s)-1:
                if s[x+1]=='=':
                    x+=1
        elif s[x]=='|':
            if token:
                tokens.append(token)
                token = ''
            if x<len(s)-1 and s[x+1]=='|':
                tokens.append('||')
                x+= 1
            elif x<len(s)-1 and s[x+1]=='=':
                tokens.append('|=')
                x += 1
            else:
                tokens.append('|')
        elif s[x] in '!+-()[]{}':
            if token:
                tokens.append(token)
                token = ''
            tokens.append(s[x])
        else:
            token += s[x]
        x+=1
    tokens.append(token)
    return tokens
def gml2syntaxtree(s):
    return tokstream_to_tree(gml2tokstream(s))

debug_list_all_gml = 0

class ScriptData(object):
    def __init__(self,f):
        self.name = slurpdata(f)
        self.ver = slurpint(f)
        gml = slurpdata(f)
        if debug_list_all_gml:
            print "//",self.name,"(version",self.ver,'))'
            print gml
        if replace_scripts.has_key(self.name):
            if debug_list_all_script_content or debug_list_all_gml:
                print "// overridden with python script",replace_scripts[self.name]
            gmapi.__dict__[self.name] = replace_scripts[self.name]
            self.syntax_tree = None
            return
        elif replace_scripts_gml.has_key(self.name):
            if debug_list_all_script_content or debug_list_all_gml:
                print "// overridden with GML script"
                print replace_scripts_gml[self.name]
            self.syntax_tree = gml2syntaxtree(replace_scripts_gml[self.name])
            return
        self.syntax_tree = gml2syntaxtree(gml)
        gmapi.__dict__[self.name] = self
    def python(self,ctx):
        python = 'def '+self.name+'(*args):\n global self\n global other\n global room\n global keyboard_string\n global score\n global lives\n'
        if ctx.default_undef_is_zero:
            python += ' args = py_PythonAutovivifyingArray(args)\n'
        pythonvars = dict()
        python += tree_to_python(self.syntax_tree,depth=1,vars=pythonvars)+"\n"
        # XXX old, bad vivifier.
        #tail =tree_to_python(self.syntax_tree,depth=1,vars=pythonvars)+"\n"
        #python += tree_to_vivifiers(self.syntax_tree,depth=1,pythonvars=pythonvars)+'\n'
        #python += tail
        return python
    def compile(self,ctx):
        if self.syntax_tree is None:
            return
        locals = dict()
        try:
            exec self.python(ctx) in gmapi.__dict__
        except:
            print "Failure compiling script",self.name,self.python(ctx),']END]'
        if debug_list_all_script_content or (self.name in debug_list_script_content):
            print self.python(ctx)

def slurpscript(f):
    if f.read(4)!='\0\0\0\0':
        return ScriptData(f)
    return None

class PathData(object):
    def __init__(self,f=None):
        if type(f) is PathData:
            self.name = ''
            self.kind = f.kind
            self.closed = f.closed
            self.precision = f.precision
            self.room_index = f.room_index
            self.snapx = f.snapx
            self.snapy = f.snapy
            self.points = []
            return
        self.name = slurpdata(f)
        self.ver = slurpint(f)
        self.kind = slurpint(f) # 0=straight lines, 1=smooth curve
        self.closed = slurpint(f)
        self.precision = slurpint(f) # 1-8, default is 4
        self.room_index = slurpint(f) # -1 = None
        self.snapx,self.snapy,numpoints = unpack('<3l',f.read(12))
        self.points = []
        while numpoints>len(self.points):
            self.points.append(unpack('<3d',f.read(24)))
        gmapi.__dict__[self.name] = self
    def ChildPathOffset(path,xoff,yoff):
        ret = PathData(path)
        for point in path.points:
            ret.points.append((xoff+point[0],yoff+point[1]))
        # XXX orientation not yet handled.
        # XXX scaling not yet handled.
        return ret
        

def slurppath(f):
    if f.read(4)!='\0\0\0\0':
        return PathData(f)
    return None

debugger_font = None

class FontData(object):
    def __init__(self,f):
        self.name = slurpdata(f)
        global debugger_font
        debugger_font = self
        ver = slurpint(f)
        if ver!=540:
            print "Warning: non-540 fonts not yet implimented."
        self.fontname = slurpdata(f)
        # XXX check to see if the font actually exists.
        # XXX Gegi reports that fonts seem a little too small (on 1.2).
        # Trying 1.3*
        self.size = slurpint(f)
        self.bold = slurpint(f)
        if not self.bold:
            self.size = int(self.size*1.3)
        else:
            self.size = int(self.size*1.1)
        self.italic = slurpint(f)
        self.character_range_min, self.character_range_max = unpack('<2l',f.read(8))
        gmapi.__dict__[self.name] = self

def slurpfont(f):
    if f.read(4)!='\0\0\0\0':
        return FontData(f)
    return None

actions_that_may_apply_to_nothing = (
        211, # Probably play sound
        222, # room_goto_next, also takes a transition.
        331, # Restart 
        332, # Game end
        404, # if the number of instances is...
        405, # Dice
        421, # else ?  I think.  XXX
        422,423,424, # {, repeat, }
        425, # Exit this event - I think!
        524, # Set the drawing color
        526, # Set font - XXX unimplimented
        605, # A comment
        701, # Set score
        709, # Show Highscores
        711, # Set lives
        714,  # draw_life_images
        721, # Set health
        731, # Window caption info
        )
actions_that_may_apply_to_specific = (
        202, # Change instance into object
        203, # Destroy the instance
        603, # A piece of code.
        )

def assignment(var,val,relative):
    if relative:
        return [var,'+=',]+gml2tokstream(val)+[';',]
    return [var,'=',]+gml2tokstream(val)+[';',]

def generic_action(action):
    ret = [action.funcname,'(']
    for arg in action.argvals:
        ret.extend(gml2tokstream(arg))
        ret.append(',')
    ret[-1] = ')'
    ret.append(';')
    return ret

class EventAction(object):
    def __init__(self,f):
        ver = slurpint(f)
        self.libid = slurpint(f)
        self.actid = slurpint(f)
        self.actkind = slurpint(f)
        self.may_be_relative = slurpint(f)
        self.is_question = slurpint(f)
        self.applies_to_something = slurpint(f)
        self.type = slurpint(f)
        self.funcname = slurpdata(f)
        self.code = slurpdata(f)
        self.argcount,self.howmanyargs = unpack('<2l',f.read(8))
        self.python = None
        if self.howmanyargs!=8:
            print "Warning: howmanyargs=",self.howmanyargs,'0x%x'%self.howmanyargs
        self.argkinds = []
        while len(self.argkinds)<self.howmanyargs:
            self.argkinds.append(slurpint(f))
        self.applies_to_object_index = slurpint(f)
        # -1 for Self, -2 for Other
        self.relative, howmanyargs = unpack('<2l',f.read(8))
        if howmanyargs!=8:
            print "Warning: Not an octarg."
        self.argvals = []
        while len(self.argvals)<howmanyargs:
            self.argvals.append(slurpdata(f))
        self.negated = slurpint(f)
    def __repr__(self):
        return '<EventActionData:libid='+str(self.libid)+':actid'+str(self.actid)+':'+self.argvals[0]+'>'
    def TokenStream(action):
        if not action.applies_to_something:
            if not action.actid in actions_that_may_apply_to_nothing:
                print action,action.argvals
                raise "Not sure what to do with an action that applies to nothing."+str(action)+action.funcname
        if action.applies_to_object_index==-2:
            if not action.actid in actions_that_may_apply_to_specific:
                raise "Not sure what to do with an action that applies to Other."
        elif action.applies_to_object_index!=-1:
            if not action.actid in actions_that_may_apply_to_specific:
                print action,action.argvals,action.applies_to_object_index
                raise "Not sure what to do with an action that does not apply to self."
        if action.actid==101:
            if action.funcname!='action_move':
                print "Warning: 101 that is not action_move."
            return generic_action(action)
        elif action.actid==102:
            if action.relative:
                return ['motion_add','(',]+gml2tokstream(action.argvals[0])+[',',]+gml2tokstream(action.argvals[1])+[')',';']
            return ['{',]+assignment('direction',action.argvals[0],0)+assignment('speed',action.argvals[1],0)+['}',]
        elif action.actid==103:
            if action.funcname!='action_set_hspeed':
                raise "103, but funcname is "+str(action.funcname)
            if not action.relative:
                return ('hspeed','=',gml2tokstream(action.argvals[0]),';')
            else:
                print "WARNING: Not sure what the correct behavior for relative speed is."
                return ('hspeed','+=',gml2tokstream(action.argvals[0]),';')
        elif action.actid==104:
            # set vertical speed
            if action.funcname!='action_set_vspeed':
                raise "104, but funcname is "+str(action.funcname)
            if not action.relative:
                return ('vspeed','=',gml2tokstream(action.argvals[0]),';')
            else:
                print "WARNING: Not sure what the correct behavior for relative speed is."
                return ('vspeed','+=',gml2tokstream(action.argvals[0]),';')
        elif action.actid==105:
            if action.funcname!='action_move_point':
                raise "105, but not action_move_point"
            ret = action.ApplyWrapperStart()+['move_towards_point','(']
            ret.extend(gml2tokstream(action.argvals[0]))
            if action.relative:
                ret.extend(('+','x',',','y','+'))
            else:
                ret.append(',')
            ret.extend(gml2tokstream(action.argvals[1]))
            ret.append(',')
            ret.extend(gml2tokstream(action.argvals[2]))
            ret.extend((')',';'))
            return ret
        elif action.actid==108:
            if action.funcname!='action_set_friction':
                raise '108, but not frictive.'
            if not action.relative:
                return ['friction', '=']+gml2tokstream(action.argvals[0])+[';']
            else:
                return ['friction', '+=']+gml2tokstream(action.argvals[0])+[';']
        elif action.actid==109:
            if action.funcname!='action_move_to':
                raise "109, but not moveto"
            if not action.relative:
                return ('{',
                    'x', '=', )+tuple(gml2tokstream(action.argvals[0]))+(';',
                    'y', '=',)+ tuple(gml2tokstream(action.argvals[1]))+(';','}',)
            else:
                return ('{', 'x', '+=', )+tuple(gml2tokstream(action.argvals[0]))+(';',
                    'y', '+=',)+tuple(gml2tokstream(action.argvals[1]))+(';','}')
        elif action.actid==111:
            return ['action_move_random','(']+gml2tokstream(action.argvals[0])+[
                ',']+gml2tokstream(action.argvals[1])+[')',';']
        elif action.actid==115:
            if action.argvals[1]=='0':
                return ('move_bounce_solid','(',action.argvals[0],')',';')
            elif action.argvals[1]=='1':
                return ('move_bounce_all','(',action.argvals[0],')',';')
            raise "Bounce"+str(action.argvals)
        elif action.actid==201:
            ret = action.ApplyWrapperStart()+['instance_create','(']
            ret.extend(gml2tokstream(action.argvals[1]))
            if action.relative:
                ret.extend(('+','x',',','y','+'))
            else:
                ret.append(',')
            ret.extend(gml2tokstream(action.argvals[2]))
            ret.append(',')
            ret.append(game.objects[int(action.argvals[0])].name)
            ret.extend((')',';'))
            return ret+action.ApplyWrapperEnd()
        elif action.actid==202:
            raise "XXX instance changing not yet implimented."
        elif action.actid==203:
            return action.ApplyWrapperStart()+['instance_destroy','(',')',';']+action.ApplyWrapperEnd()
        elif action.actid==211:
            if action.argvals[1]=='0':
                return ['sound_play','(']+gml2tokstream(action.argvals[0])+[
                    ')',';']
            return gml2tokstream('sound_loop('+action.argvals[0]+');')
        elif action.actid==222:
            if action.funcname!='action_next_room':
                raise "Expected action_next_room, didn't get it."
            print "XXX transitions unimplimented.",action.argvals
            return ('room_goto_next','(',')',';')
        elif action.actid==331:
            return 'game_restart','(',')',';'
        elif action.actid==332:
            return 'game_end','(',')',';'
        elif action.actid==404:
            if action.argvals[2]=='0':
                return ['if','(','instance_number','(',game.objects[int(action.argvals[0])].name,')','=',]+gml2tokstream(action.argvals[1])+[')']
            else:
                raise "Haven't checked the other math-ops."
        elif action.actid==405:
            ret = ['if', '(']
            if action.negated:
                ret.append('not')
            ret.extend(('py_random.randint','(','1',','))
            ret.extend(gml2tokstream(action.argvals[0]))
            ret.extend((')','=','1',')'))
            return ret
        elif action.actid==408:
            # if an expression is true
            if action.negated:
                return ['if','(','not',]+gml2tokstream(action.argvals[0])+[')',]
            return ['if','(',]+gml2tokstream(action.argvals[0])+[')',]
        elif action.actid==421:
            return ('else',)
        elif action.actid==422:
            return ('{',)
        elif action.actid==423:
            # XXX is this repeat?
            ret = ['py_print','(','"Repeat?',')',';', 
                'repeat','(']+gml2tokstream(action.argvals[0])+[')' ]
            print ret
            return ret
        elif action.actid==424:
            # XXX just guessing that this is an end-block.
            return ('}',)
        elif action.actid==425:
            return ('return',)
        elif action.actid==501:
            ret = gml2tokstream('draw_sprite('+game.sprites[int(action.argvals[0])].name+','+action.argvals[3]+','+action.argvals[1])
            if action.relative:
                ret.extend(('+','x',','))
                ret.extend(gml2tokstream(action.argvals[2]+'+y);'))
                return ret
            return ret+gml2tokstream(","+action.argvals[1]+');')
        elif action.actid==514:
            ret = ['draw_text','(',]
            ret.extend(gml2tokstream(action.argvals[1]))
            if action.relative:
                ret.extend(('+','x',',','y','+'))
            else:
                ret.append(',')
            ret.extend(gml2tokstream(action.argvals[2]))
            ret.append(',')
            if action.argvals[0][0] in ('"',"'"):
                ret.extend(gml2tokstream(action.argvals[0]))
            else:
                ret.append('"'+action.argvals[0])
            ret.extend((')',';'))
            return ret
        elif action.actid==524:
            # XXX does this actually work as intended?
            return 'draw_set_color','(','__to_rgb','(',action.argvals[0],')',')',';'
        elif action.actid==526:
            return ['{','draw_set_font','(','gm6.game.fonts','[']+gml2tokstream(action.argvals[0])+[']',')',';','draw_set_halign','(']+gml2tokstream(action.argvals[1])+[')',';','}']
        elif action.actid==541:
            return ['{', 'sprite_index','=']+gml2tokstream(action.argvals[0])+[';',
                'image_index','=']+gml2tokstream(action.argvals[1])+[';',
                'image_speed','=']+gml2tokstream(action.argvals[2])+[';','}']
        elif action.actid==542:
            if action.funcname!='action_sprite_transform':
                raise "Not a transform."
            ret = ['{','image_xscale','=']
            if int(action.argvals[3]):
                ret.append('-')
            ret.extend(gml2tokstream(action.argvals[0]))
            ret.extend((';','image_yscale','='))
            if int(action.argvals[3]):
                ret.append('-')
            ret.extend(gml2tokstream(action.argvals[1]))
            ret.extend((';','image_angle','='))
            ret.extend(gml2tokstream(action.argvals[2]))
            ret.extend((';','}'))
            return ret
        elif action.actid==601:
            if action.funcname!='action_execute_script':
                raise "Not sure what to do with 601:"+action.funcname
            ret = [game.scripts[int(action.argvals[0])].name,'(',]
            for arg in action.argvals[1:6]:
                ret.extend(gml2tokstream(arg))
                #ret.append('"'+arg) XXX
                ret.append(',')
            del ret[-1]
            ret.extend((')',';'))
            return ret
        elif action.actid==603:
            return action.ApplyWrapperStart()+['{',]+gml2tokstream(action.argvals[0])+['}',]+action.ApplyWrapperEnd()
        elif action.actid==605:
            return ()
        elif action.actid==611:
            if action.relative:
                return gml2tokstream(action.argvals[0]+' += '+action.argvals[1])+[';']
            else:
                return gml2tokstream(action.argvals[0]+' = '+action.argvals[1])+[';']
        elif action.actid==612:
            if action.funcname!='action_if_variable':
                raise "612, but funcname "+str(action.funcname)
            gml = 'if '+action.argvals[0]+' '
            op = int(action.argvals[2])
            if op==0:
                gml += '= '
            elif op==2:
                gml += '> '
            else:
                print "If",action,op,action.argvals[1]
                raise 'if: unexpected operator'
            gml += action.argvals[1]
            return gml2tokstream(gml)
        elif action.actid==701:
            if action.relative:
                return 'score','+=',action.argvals[0],';'
            return 'score','=',action.argvals[0],';'
        elif action.actid==709:
            print "Highscore table arguments",action.argvals
            font = action.argvals[4].split(',')
            ret = ['{',
                # Arg 0 - background
                'highscore_set_background','(',action.argvals[0],')',';',
                # Arg 1 - border?
                'highscore_set_border','(',action.argvals[1],')',';',
                # 2 - new score Red
                # 3 - other scores Yellow
                'highscore_set_colors','(','0',',',action.argvals[2],
                ',',action.argvals[3],')',';',
                'highscore_set_font','(',font[0][:-1],',',action.argvals[5],
                ',',font[2],')',';',
                # XXX what is the rest of the fontstring?  It looks like
                # size, ???, bold?, ???, ???
                # 4 - font string: ex. "Times New Roman",12,0,1,0,0,0
                # 5 - font size?
                # XXX font
                # XXX border
                # XXX colors
                'highscore_show','(','score',')',';',
                '}'
                ]
            return ret
        elif action.actid==711:
            ret = ['action_set_life','(']
            ret.extend(gml2tokstream(action.argvals[0]))
            ret.append(',')
            if action.relative:
                ret.append('1')
            else:
                ret.append('0')
            ret.extend((')',';'))
            return ret
            #print action.funcname,2304
            #if action.relative:
            #    return gml2tokstream('lives += '+action.argvals[0]+';')
            #return gml2tokstream('lives= '+action.argvals[0]+';')
        elif action.actid==714:
            # Draw the lives as an image
            if action.relative:
                return gml2tokstream('action_draw_life_images(x+'+action.argvals[0]+',y+'+action.argvals[1]+','+game.sprites[int(action.argvals[2])].name+');')
            return gml2tokstream('action_draw_life_images('+action.argvals[0]+','+action.argvals[1]+','+game.sprites[int(action.argvals[2])].name+');')
        elif action.actid==721:
            if action.relative:
                return 'health','+=',action.argvals[0],';'
            return 'health','=',action.argvals[0],';'
        elif action.actid==731:
            print "XXX window caption untested"
            # 0, score, 0, lives, 0, health
            return ('pygame.display.set_caption','(','"'+action.argvals[0],',',
                '"'+action.argvals[0],')')
        elif action.funcname:
            print "gm6.py:2306 Warning",action,action.funcname
            ret = [action.funcname,'(']
            for arg in action.argvals:
                ret.extend(gml2tokstream(arg))
                ret.append(',')
            ret[-1] = ')'
            ret.append(';')
            return ret
            #action.argvals[0],',',action.argvals[1],',',action.argvals[2],',',action.argvals[3],',',action.argvals[4],',',action.argvals[5],',',action.argvals[6],',',action.argvals[7],')',';'
        else:
            raise "Unknown actid for TokenStream "+str(action.actid)+'///'+str(action.argvals)+'/'+action.funcname
    def ApplyWrapperStart(action):
        if action.applies_to_object_index == -1:
            return []
        elif action.applies_to_object_index == -2:
            return ['with','(','other',')','{']
            #return ['py_oldself','=','self', ';', 'self','=','other',';','other','=','py_oldself',';']
            return ['py_oldself','=','self', ';', 'self','=','other',';']
        elif action.applies_to_object_index >= 0:
            return ['with', '(', game.objects[action.applies_to_object_index].name,')', '{', ]
        raise "Not sure how to apply-wrap an arbitrary object."
    def ApplyWrapperEnd(action):
        if action.applies_to_object_index == -1:
            return []
        elif action.applies_to_object_index == -2:
            return ['}',]
            #return ['self','=','py_oldself',';',]
        elif action.applies_to_object_index>=0:
            return ['}',]
        raise "Not sure how to apply-wrap an arbitrary object."

event_handler_counter = 0

class EventData(object):
    def __init__(self, numb, f):
        self.numb = numb
        ver,numactions = unpack('<2l',f.read(8))
        self.actions = []
        while numactions>len(self.actions):
            self.actions.append(EventAction(f))
        self.dynamic_actions = [] # Events added at runtime with object_event_add
        self.python = None
        self.counter = None
    def RunActionsOn(self, instance):
        raise "RunActionsOn"
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        if not len(gmapi.py_other_stack):
            gmapi.other = instance
        exec self.python in gmapi.__dict__
        gmapi.self = gmapi.py_self_stack.pop()
    def __getstate__(self):
        ret = dict(self.__dict__)
        ret['python'] = self.Python()
        return ret
    def __setstate__(self,state):
        exec state['python'][1] in gmapi.__dict__
        self.python = compile(
            '__py_603_tmpfunc_'+str(state['python'][0])+'()',
            ':event:'+state['python'][2]+'#'+str(state['numb']),'exec')
        del state['python']
        for k in state:
            self.__dict__[k] = state[k]
    def Python(self,verbose=False,name=''):
        tokstream = []
        for action in self.actions:
            tokstream.extend(action.TokenStream())
        if self.counter is None:
            global event_handler_counter
            event_handler_counter += 1
            self.counter = event_handler_counter
        python = 'def __py_603_tmpfunc_'+str(self.counter)+'():\n global self\n global other\n global keyboard_string\n global score\n global lives\n'+tree_to_python(tokstream_to_tree(tokstream),depth=1)
        for action in self.dynamic_actions:
            python += '\n'
            python += action
            python += '\n'
        if verbose:
            print '(2298)',python
        return self.counter,python,name
    def Compile(self,verbose=False,name=''):
        event_handler_counter, python, dump = self.Python(verbose=verbose,name=name)
        try:
            exec python in gmapi.__dict__
            self.python = compile(
                '__py_603_tmpfunc_'+str(event_handler_counter)+'()',':event:'+name+'#'+str(self.numb),'exec')
        except:
            print "Error compiling",self.actions
            raise sys.exc_type,sys.exc_value,sys.exc_traceback
    def __repr__(self):
        return '<EventData numb='+str(self.numb)+str(self.actions)+'>'

class DynamicEventData(EventData):
    # Used for events created by object_event_add and __object_event_add_python
    def __init__(self, numb):
        self.numb = numb
        self.numactions = 0
        self.actions = ()
        self.dynamic_actions = []
        self.python = None
        self.counter = None

py_last_target = None # Used so 'target' will work in execute_script()

def RunScriptOn(script,instance,*args):
    gmapi.py_self_stack.append(gmapi.self)
    gmapi.self = instance
    # XXX should other be changed too?
    #try:
    if 1:
        exec script.name+"(*args)" in gmapi.__dict__,dict(args=args)
    gmapi.self = gmapi.py_self_stack.pop

def gml2python(s):
    return tree_to_python(gml2syntaxtree(s))

def RunActionGroup(action):
    try:
        exec action.python in gmapi.__dict__
    except gmapi.py_GameEndException:
        raise sys.exc_type,sys.exc_value,sys.exc_traceback
    except:
        print "Warning: RunActionGroup failed (python was '",action.python,"')",gmapi.blank_item
        import traceback
        print traceback.extract_tb(sys.exc_traceback)
        raise sys.exc_type,sys.exc_value,sys.exc_traceback

def RunActionGroupOn(action,instance):
    # It appears to run SLOWER (0.00037/call vs. 0.00026/call) if 
    # I use a called function object in lieu of exec.
    # ...  and 0.00028/call if I take out the try.
    gmapi.py_self_stack.append(gmapi.self)
    gmapi.self = instance
    try:
        exec action.python in gmapi.__dict__
    except gmapi.py_GameEndException:
        raise sys.exc_type,sys.exc_value,sys.exc_traceback
    except:
        print "Warning: RunActionGroupOn failed (python was '",action.python,"')",instance,instance.sprite_index
        print sys.exc_type,sys.exc_value,sys.exc_traceback
        import traceback
        print traceback.extract_tb(sys.exc_traceback)
    gmapi.self = gmapi.py_self_stack.pop()

def list_del(l,i):
    x = len(l)
    while x:
        x -= 1
        if l[x] is i:
            del l[x]

class ObjectInstanceBase(object):
    slots = ['x','y',
        'bbox_top','bbox_bottom','bbox_left','bbox_right',
        'py_Draw',
        ]
    depth = 0
    image_xscale = 1
    image_yscale = 1
    #GlobLeftButtonActionGroup = None
    def __float__(self):
        return float(self.id)
    def __eq__(self,other):
        # Not the cause of Awk's spellcasting lag.
        if type(other) is GMVar:
            other = other[0]
        if other is self:
            return True
        if type(other) in (float,int,InstanceID):
            return other==self.id
        return False
    def __init__(self, parent_object, id=None):
        if type(parent_object)==GM_Object:
            self.object_index = parent_object
            parent_object = parent_object.__dict__['py_objectdata']
        else:
            raise "1561..."+str(parent_object)
        parent_object.instances.append(self)
        self.__dict__['visible'] = parent_object.visible
        self.alarm = [-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1]
        # XXX YYY Caller will need to depthsort after adding.
        self.depth = parent_object.depth
        if id is None:
            global highid
            highid += 1
            self.id = InstanceID(highid)
        else:
            self.id = id
        instances[self.id] = self
        self.persistent = parent_object.persistent
        self.__dict__['x'] = 0
        self.__dict__['y'] = 0
        self.__dict__['speed'] = self.__dict__['path_speed'] = self.__dict__['gravity'] = self.__dict__['friction'] = 0
        self.__dict__['image_speed'] = 1
        self.__dict__['sprite_index'] = parent_object.sprite_index
        self.__dict__['direction'] = 0
        self.__dict__['image_angle'] = 0
        self.__dict__['path_index'] = None
        self.__dict__['image_index'] = 0
        self.__dict__['image_single'] = 0
        self.__dict__['hspeed'] = self.__dict__['vspeed'] = 0
        self.py_path_next_waypoint = 0
        self.points = []
        self.bbox_left = 0
        self.bbox_right = 0
        self.bbox_top = 0
        self.bbox_bottom = 0
        self.image_blend = (255,255,255)
        self.image_alpha = 1
        self.image_xscale = 1
        self.image_yscale = 1
        self.py_last_image_speed = 0
        self.py_Draw = 0
        self.ResolveSprite()
    def ResolveSprite(self):
        if type(self.sprite_index) is int or type(self.sprite_index) is float:
            if self.sprite_index!=-1:
                self.SetSpriteIndex(game.sprites[self.sprite_index])
    def RunAlarm(instance,x):
        for grp in instance.object_index.py_objectdata.events[2]:
            if grp.numb==x:
                RunActionGroupOn(grp,instance)
                return
    def RunCreateEvent(self):
        x = len(self.object_index.py_objectdata.events[0])
        while x>0:
            x -= 1
            for action in self.object_index.py_objectdata.events[0]:
                 RunActionGroupOn(action, self)
    def __repr__(self):
        return '<ObjectInstanceBase: '+self.object_index.py_objectdata.name+'>'
    def HandleCollisions(instance, ev):
        bb_l = instance.bbox_left
        bb_t = instance.bbox_top
        bb_r = instance.bbox_right
        bb_b = instance.bbox_bottom
        gmapi.py_other_stack.append(gmapi.other)
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        for i2 in tuple(game.objects[ev.numb]):
            # XXX formerly if i2.IsUnderBox(box).  Optimizing it out cuts
            # HandleCollisions' duration nearly in half.
            if i2.bbox_left<bb_r and i2.bbox_right>=bb_l and i2.bbox_top<bb_b and i2.bbox_bottom>=bb_t:
                if i2.object_index.py_objectdata.solid:
                    instance.x = instance.xprevious
                    instance.y = instance.yprevious
                    instance.SetBoundingBox()
                gmapi.other = i2
                if gmapi.py_other_stack[-1] is None:
                    gmapi.py_other_stack[-1] = i2
                RunActionGroupOn(ev,instance)
                bb_l = instance.bbox_left
                bb_t = instance.bbox_top
                bb_r = instance.bbox_right
                bb_b = instance.bbox_bottom
                if gmapi.__dict__['__new_room']!=gmapi.room:
                    gmapi.self = gmapi.py_self_stack.pop()
                    gmapi.other = gmapi.py_other_stack.pop()
                    return True
        gmapi.self = gmapi.py_self_stack.pop()
        gmapi.other = gmapi.py_other_stack.pop()
    def RoomStart(instance):
        if instance.object_index.py_objectdata.py_RoomStartEvent:
            raise instance.object_index.py_objectdata.name+' instance has events '+str(instance.object_index.py_objectdata.events[7])
    def RoomEnd(instance):
        if instance.object_index.py_objectdata.py_RoomEndEvent:
            raise instance.object_index.py_objectdata.name+' instance has events '+str(instance.object_index.py_objectdata.events[7])
    def StepAnimation(instance):
        if instance.sprite_index!=-1:
            instance.image_index += instance.image_speed
            # XXX how is no sprite represented?
            if instance.image_index>=len(instance.sprite_index.subimages):
                instance.image_index = 0
                if instance.object_index.py_objectdata.py_AnimationEndEvent:
                    RunActionGroupOn(instance.object_index.py_objectdata.py_AnimationEndEvent, instance)
        elif instance.object_index.py_objectdata.py_AnimationEndEvent:
            print "Sprite_index is -1, ending animation for",instance
            RunActionGroupOn(instance.object_index.py_objectdata.py_AnimationEndEvent, instance)
    def MoveInstance(instance):
        instance.xprevious = instance.x
        instance.yprevious = instance.y
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        if instance.path_index:
            path = instance.path_index
            dist = instance.path_speed
            next_waypoint = instance.py_path_next_waypoint
            if next_waypoint>=len(instance.path_index.points):
                dist = 0
            while dist>0:
                nextx, nexty = path.points[next_waypoint]
                waypoint_dist = math.hypot(instance.x-nextx,instance.y-nexty)
                if waypoint_dist>dist:
                    gmapi.move_towards_point(nextx,nexty,dist)
                    instance.py_path_position_pixels += dist
                    break
                else:
                    instance.x = nextx
                    instance.y = nexty
                    instance.SetBoundingBox()
                    dist -= waypoint_dist
                    instance.py_path_position_pixels += waypoint_dist
                    next_waypoint += 1
                    if next_waypoint>=len(path.points):
                        if instance.path_endaction==0:
                            instance.path_index = None
                            dist = 0
                            instance.x, instance.y = path.points[-1]
                            instance.SetPathSpeed(0)
                            instance.SetSpeed(0)
                            instance.SetDirection(0)
                            instance.path_position = 1.0
                        elif instance.path_endaction==1:
                            if not path.closed:
                                instance.x, instance.y = path.points[0]
                                instance.py_path_position_pixels = 0
                            else:
                                next_waypoint = 0
                                nextx,nexty = path.points[0]
                        elif instance.path_endaction==2:
                            # XXX this may fail.
                            instance.path_index = path.ChildPathOffset(instance.x,instance.y)
                            next_waypoint = 0
                        elif instance.path_endaction==3:
                            raise "Reversing paths not yet handled."
                            # XXX I'm guessing that they run from 1 to 0
            instance.py_path_next_waypoint = next_waypoint
            if instance.path_index:
                instance.path_position = instance.py_path_position_pixels*path.multiplier
        else:
            # Apply speed, gravity
            instance.x += instance.hspeed
            instance.y += instance.vspeed
            if instance.gravity:
                instance.__dict__['vspeed'] += instance.gravity*math.cos(math.radians(instance.gravity_direction+90))
                instance.hspeed += instance.gravity*math.sin(math.radians(instance.gravity_direction+90))
                instance.RecalculateAngularSpeed() # XXX not sure if this is always needed, or if it will be faster with it dumped in amongst friction/gravity.
        # XXX should friction happen begore or after movement?
        if instance.friction:
            if instance.hspeed>0:
                instance.hspeed -= instance.friction
                if instance.hspeed<0:
                    instance.hspeed = 0
            elif instance.hspeed<0:
                instance.hspeed += instance.friction
                if instance.hspeed>0:
                    instance.hspeed = 0
            if instance.vspeed>0:
                instance.vspeed -= instance.friction
                if instance.vspeed<0:
                    instance.vspeed = 0
            elif instance.vspeed<0:
                instance.vspeed += instance.friction
                if instance.vspeed>0:
                    instance.vspeed = 0
            instance.RecalculateAngularSpeed() # XXX not sure if this is always needed, or if it will be faster with it dumped in amongst friction/gravity.
        instance.SetBoundingBox()
        if instance.object_index.py_objectdata.py_IntersectBoundaryEvent:
            if (instance.bbox_left<=0 and instance.bbox_right>=0) or (instance.bbox_top<=0 and instance.bbox_bottom>=0) or (instance.bbox_left<=gmapi.room.room_width and instance.bbox_right>=gmapi.room.room_width) or (instance.bbox_top<=gmapi.room.room_height and instance.bbox_bottom>=gmapi.room.room_height):
                RunActionGroupOn(instance.object_index.py_objectdata.py_IntersectBoundaryEvent, instance)
        if instance.object_index.py_objectdata.py_OutsideRoomEvent:
            if instance.bbox_right<0 or instance.bbox_bottom<0 or instance.bbox_top>gmapi.room.room_height or instance.bbox_left>gmapi.room.room_width:
                RunActionGroupOn(instance.object_index.py_objectdata.py_OutsideRoomEvent, instance)
        # XXX Outside event should be here, too - if this is the right point for an IntersectBoundaryEvent handler.
        gmapi.self = gmapi.py_self_stack.pop()
    def StepKey(instance):
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        for actiongroup in instance.object_index.py_objectdata.events[5]:
            if gmapi.keyboard_check(actiongroup.numb):
                RunActionGroupOn(actiongroup,instance)
        gmapi.self = gmapi.py_self_stack.pop()
    def StepMouse(instance):
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        po = instance.object_index.py_objectdata
        if gmapi.mouse_button & gmapi.mb_left:
            if po.py_LocalLeftButtonActionGroup and instance.IsUnderPos(gmapi.mouse_x, gmapi.mouse_y):
                RunActionGroupOn(po.py_LocalLeftButtonActionGroup, instance)
            if po.py_GlobLeftButtonActionGroup:
                RunActionGroupOn(po.py_GlobLeftButtonActionGroup, instance)
        if gmapi.mouse_button & gmapi.mb_right:
            if po.py_LocalRightButtonActionGroup and instance.IsUnderPos(gmapi.mouse_x, gmapi.mouse_y):
                RunActionGroupOn(po.py_LocalRightButtonActionGroup, instance)
            if po.py_GlobRightButtonActionGroup:
                RunActionGroupOn(po.py_GlobRightButtonActionGroup, instance)
        gmapi.self = gmapi.py_self_stack.pop()
    def StepKeyMouse_OLD(instance):
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        for actiongroup in instance.object_index.py_objectdata.events[5]:
            if gmapi.keyboard_check(actiongroup.numb):
                RunActionGroupOn(actiongroup,instance)
        
        po = instance.object_index.py_objectdata
        if gmapi.mouse_button & gmapi.mb_left:
            if po.py_LocalLeftButtonActionGroup and instance.IsUnderPos(gmapi.mouse_x, gmapi.mouse_y):
                RunActionGroupOn(po.py_LocalLeftButtonActionGroup, instance)
            if po.py_GlobLeftButtonActionGroup:
                RunActionGroupOn(po.py_GlobLeftButtonActionGroup, instance)
        if gmapi.mouse_button & gmapi.mb_right:
            if po.py_LocalRightButtonActionGroup and instance.IsUnderPos(gmapi.mouse_x, gmapi.mouse_y):
                RunActionGroupOn(po.py_LocalRightButtonActionGroup, instance)
            if po.py_GlobRightButtonActionGroup:
                RunActionGroupOn(po.py_GlobRightButtonActionGroup, instance)
        gmapi.self = gmapi.py_self_stack.pop()
    def AttachPath(instance, path, speed, endaction, absolute):
        instance.SetSpeed(0)
        instance.path_position = 0.000001
        if len(path.points)==0:
            instance.SetPathSpeed(0)
            instance.path_index = None
            return
        if absolute:
            if speed>=0:
                instance.path_index = path
            else:
                instance.path_index = path.ChildPathReversed()
        else:
            raise "Relative paths not implimented."
            instance.path_index = path.ChildPathOffset(instance.x,instance.y)
            if speed<0:
                instance.path_index.points.reverse()
        instance.SetPathSpeed(speed)
        instance.path_endaction = endaction
        instance.path_orientation = 0
        instance.path_positionprevious = 0
        # XXX If I recall correctly, changing path_position should teleport
        # you along the path.
        instance.path_position = 0
        instance.py_path_next_waypoint = 0
        instance.py_path_position_pixels = 0
    def Draw_fast_ev(instance, **kwargs):
        gmapi.py_self_stack.append(gmapi.self)
        gmapi.self = instance
        try:
            exec instance.object_index.py_objectdata.events[8][0].python in gmapi.__dict__
        except:
            print "Warning: RunActionGroupOn failed (python was '",instance.object_index.py_objectdata.events[8][0].python,"')",instance,instance.sprite_index
            raise sys.exc_type,sys.exc_value,sys.exc_traceback
        gmapi.self = gmapi.py_self_stack.pop()
    def Draw(instance, xoff=0, yoff=0):
        e8 = instance.object_index.py_objectdata.events[8]
        if e8:
            instance.Draw = instance.Draw_fast_ev
            gmapi.py_self_stack.append(gmapi.self)
            gmapi.self = instance
            try:
                exec e8[0].python in gmapi.__dict__
            except:
                print "Warning: RunActionGroupOn failed (python was '",e8[0].python,"')",instance,instance.sprite_index
                raise sys.exc_type,sys.exc_value,sys.exc_traceback
            gmapi.self = gmapi.py_self_stack.pop()
            return
        gmapi.draw_sprite_ext(instance.sprite_index,instance.image_index,
            instance.x,instance.y,instance.image_xscale,instance.image_yscale,
            instance.image_angle, instance.image_blend, instance.image_alpha)
    def Remove(instance,kill=False):
        if kill:
            if instance.py_inactive == 2:
                return
            if instance.id in instances:
                del instances[instance.id]
                instance.id = None
            else:
                print instance.id,'not in instances'
            instance.py_inactive = 2
        elif instance.py_inactive:
            return
        try:
            gmapi.room.all_objects.remove(instance)
        except:
            print "WARNING: Attempted to remove an object not in the object list:",instance
        pyo = instance.object_index.py_objectdata
        if instance.visible:
            gmapi.room.drawn_objects.remove(instance)
        #if gmapi.room.deactivated_objects:
        #    print "Debug: deactivated object count",len(gmapi.room.deactivated_objects)
        #list_del(gmapi.room.deactivated_objects,instance)
        if pyo.events[9]:
            gmapi.room.keypressed_objects.remove(instance)
        if pyo.events[10]:
            gmapi.room.keyreleased_objects.remove(instance)
        if pyo.py_LocalLeftPressedActionGroup or pyo.py_GlobLeftPressedActionGroup:
            gmapi.room.mouse_leftpressed_objects.remove(instance)
        if pyo.py_GlobRightPressedActionGroup:
            gmapi.room.mouse_rightpressed_objects.remove(instance)
        if pyo.events[4]:
            gmapi.room.collisioned_objects.remove(instance)
        if pyo.events[5]: # 5 are keyboard.
            gmapi.room.key_objects.remove(instance)
        if pyo.py_LocalLeftButtonActionGroup or pyo.py_GlobLeftButtonActionGroup or pyo.py_LocalRightButtonActionGroup or pyo.py_GlobRightButtonActionGroup:
            gmapi.room.mouse_objects.remove(instance)
        if instance.image_speed:
            gmapi.room.animated_objects.remove(instance)
        if instance.hspeed or instance.vspeed or instance.gravity or instance.path_speed:
            try:
                gmapi.room.moving_objects.remove(instance)
            except:
                print "WARNING:",instance,"isn't in moving objects list."
        if instance.object_index.py_objectdata.events[2]:
            gmapi.room.alarmed_objects.remove(instance)
        if instance.object_index.py_objectdata.events[3]:
            gmapi.room.stepped_objects.remove(instance)
        try:
            instance.object_index.py_objectdata.instances.remove(instance)
        except:
            print "Exception removing",instance
    def Destroy(instance):
        if instance.object_index.py_objectdata.events[1]:
            if len(instance.object_index.py_objectdata.events[1])>1:
                raise "Too many destroy events?"
            RunActionGroupOn(
                instance.object_index.py_objectdata.events[1][0],instance)
        if instance in gmapi.room.deactivated_objects:
            print "Instance is deactivated?"
        instance.Remove(kill=True)
    def HandleKeypress(self, keynum):
        # Keynum is a GM keysym.
        #    #events[5] trigger as long as the key is held down.
        for actiongroup in self.object_index.py_objectdata.events[9]:
            # Events[9] are triggered on keypress.
            if actiongroup.numb==keynum:
                RunActionGroupOn(actiongroup,self)
        # Keyrelease events handled in the room object.
    def __getattr__(self, attr):
        if attr in self.__dict__:
            return self.__dict__[attr]
        #elif item in self.object_index.__dict__:
        #    return self.object_index.__dict__[item]
        else:
            print "Error getting attribute",attr,'on instance',self
            raise "Boom!"+attr
    def SetGravity(instance, gravity):
        if gravity:
            if (not instance.gravity) and (not instance.speed) and (not instance.path_speed):
                gmapi.room.moving_objects.append(instance)
        elif instance.gravity and (not instance.speed) and (not instance.path_speed):
            gmapi.room.moving_objects.remove(instance)
        instance.__dict__['gravity'] = gravity
    def SetHSpeed(instance, hspeed):
        if hspeed:
            if (not instance.speed) and (not instance.gravity) and (not instance.path_speed):
                gmapi.room.moving_objects.append(instance)
        elif instance.hspeed and (not instance.gravity) and (not instance.path_speed) and (not instance.vspeed):
            gmapi.room.moving_objects.remove(instance)
        instance.__dict__['hspeed'] = hspeed
        instance.RecalculateAngularSpeed()
    def SetVSpeed(instance, vspeed):
        if vspeed:
            if (not instance.speed) and (not instance.gravity) and (not instance.path_speed):
                gmapi.room.moving_objects.append(instance)
        elif instance.vspeed and (not instance.hspeed) and (not instance.path_speed) and (not instance.gravity):
            gmapi.room.moving_objects.remove(instance)
        ovspeed = instance.vspeed
        instance.__dict__['vspeed'] = vspeed
        instance.RecalculateAngularSpeed()
    def SetDirection(instance, angle):
        instance.__dict__['direction'] = angle
        instance.RecalculateVectorSpeed()
    def SetSpeed_add(instance, speed):
        instance.SetSpeed(instance.speed+speed)
    def SetSpeed(instance,speed):
        #if instance.path_index is not None:
        #    return # XXX should possibly check for path_index -1 as well?
        if instance.path_index:
            return
        if speed:
            if (not instance.speed) and (not instance.path_speed) and (not instance.gravity):
                gmapi.room.moving_objects.append(instance)
        elif instance.speed and (not instance.path_speed) and (not instance.gravity):
            gmapi.room.moving_objects.remove(instance)
        instance.__dict__['speed'] = speed
        instance.RecalculateVectorSpeed()
    def SetPathSpeed(instance, speed):
        if speed:
            if (not instance.path_speed) and (not instance.speed) and (not instance.gravity):
                gmapi.room.moving_objects.append(instance)
        elif instance.path_speed and (not instance.speed) and (not instance.gravity):
            gmapi.room.moving_objects.remove(instance)
        instance.path_speed = speed
    def RecalculateAngularSpeed(instance):
        instance.__dict__['speed'] = math.hypot(instance.hspeed,instance.vspeed)
        if instance.hspeed:
            instance.__dict__['direction'] = math.degrees(math.atan(instance.vspeed/instance.hspeed))
        elif instance.vspeed<0:
            instance.__dict__['direction'] = 90 # XXX?
        elif instance.vspeed>0:
            instance.__dict__['direction'] = 270
    def RecalculateVectorSpeed(instance):
        instance.__dict__['vspeed'] = instance.speed*math.cos(math.radians(instance.direction+90))
        instance.__dict__['hspeed'] = instance.speed*math.sin(math.radians(instance.direction+90))
    def SetDepth(instance,depth):
        if depth==instance.depth:
            return
        instance.__dict__['depth'] = depth
        gmapi.room.drawn_objects.sort(cmp=depthsort)
    def SetImageSpeed(instance,speed):
        if speed:
            if not instance.image_speed:
                gmapi.room.animated_objects.append(instance)
            instance.__dict__['py_last_image_speed'] = speed
            instance.__dict__['image_single'] = -1
        elif instance.image_speed:
            gmapi.room.animated_objects.remove(instance)
        instance.__dict__['image_speed'] = speed
    def SetImageSingle(instance, single):
        if single<0:
            if not instance.image_speed:
                instance.SetImageSpeed(instance.py_last_image_speed)
            #instance.__dict__['image_index'] = 0
            instance.__dict__['image_single'] = -1
        else:
            instance.__dict__['image_index'] = single
            instance.__dict__['image_single'] = single
            instance.SetImageSpeed(0)
    def SetSpriteIndex(self, idx):
        if type(idx) is GMVar:
            idx = idx[0]
        if (type(idx) is int or type(idx) is float) and idx!=-1:
            self.__dict__['sprite_index'] = game.sprites[int(idx)]
        else:
            self.__dict__['sprite_index'] = idx
        if self.sprite_index>=0:
            # XXX are these changed by sprite scaling?  Are they faster
            # or slower than get_width()/get_height and sprite derefs?
            if self.sprite_index.subimages:
                self.sprite_width = self.sprite_index.subimages[0].get_width()
                self.sprite_height = self.sprite_index.subimages[0].get_height()
            else:
                self.sprite_width = 1
                self.sprite_height = 1
            self.sprite_xoffset = self.sprite_index.xorigin
            self.sprite_yoffset = self.sprite_index.yorigin
            self.image_number = len(self.sprite_index.subimages)
            if self.image_index>=self.image_number:
                self.image_index = 0
                if self.image_single>0:
                    self.__dict__['image_single'] = 0
        self.SetBoundingBox()
    def SetX(self, x):
        if type(x) is GMVar:
            x = x[0]
        self.__dict__['x'] = x
        # XXX can optimize by only setting left and right bounds.
        self.SetBoundingBox()
    def SetX_add(self,x):
        if type(x) is GMVar:
            x = x[0]
        self.__dict__['x'] += x
        # XXX can optimize by only setting left and right bounds.
        self.SetBoundingBox()
    def SetY(self, y):
        if type(y) is GMVar:
            y = y[0]
        self.__dict__['y'] = y
        # XXX can optimize by only setting top and bottom bounds.
        self.SetBoundingBox()
    def SetY_add(self,y):
        if type(y) is GMVar:
            y = y[0]
        self.__dict__['y'] += y
        # XXX can optimize by only setting top and bottom bounds
        self.SetBoundingBox()
    def __cmp__(self,other):
        if type(other) in (int, float, InstanceID):
            return cmp(self.id,other)
        elif type(other)==gmapi.py_ParticleSystem:
            return 1
        elif other is None:
            return 1
        return cmp(self.id,other.id)
    def SetVisible(instance, val):
        # Visible can only be 1 or 0 now.
        if instance.visible:
            if not val:
                instance.__dict__['visible'] = 0
                gmapi.room.drawn_objects.remove(instance)
        elif val:
            instance.__dict__['visible'] = 1
            gmapi.room.AddDrawnInstance(instance)
    __set_trapped_dict_OLD = dict(
        hspeed=SetHSpeed,
        vspeed=SetVSpeed,
        visible=SetVisible,
        direction=SetDirection,
        #speed=SetSpeed,
        #depth=SetDepth,
        gravity=SetGravity,
        image_speed=SetImageSpeed,
        image_single=SetImageSingle,
        #sprite_index=SetSpriteIndex,
        path_speed=SetPathSpeed,
        #x=SetX,
        #y=SetY,
        #image_index=SetImageIndex,
        )
    def BoundingBox_At(self,x,y):
        self.ResolveSprite()
        spr = self.sprite_index
        if spr<0:
            print (spr<0),spr,'<0'
            raise "Cannot specify bounding box for spriteless."
        if self.image_xscale==1 and self.image_yscale==1:
            if spr.bounding_box_type==1:
                bbox_left = x-spr.xorigin
                bbox_top = y-spr.yorigin 
                return(bbox_left,bbox_top,bbox_left+spr.subimages[0].get_width(),bbox_top+spr.subimages[0].get_height())
            else:
                return (x+spr.leftbound-spr.xorigin,
                    y+spr.topbound-spr.yorigin,
                    x+spr.rightbound-spr.xorigin,
                    y+spr.bottombound-spr.yorigin)
        else:
            if spr.bounding_box_type==1:
                bbox_left = x-(spr.xorigin*self.image_xscale)
                bbox_top = y-(spr.yorigin *self.image_yscale)
                return (bbox_left, bbox_top,
                    bbox_left+(spr.subimages[0].get_width()*self.image_xscale),
                    bbox_top+(spr.subimages[0].get_height()*self.image_yscale))
            else:
                return (
                    self.x+((spr.leftbound-spr.xorigin)*self.image_xscale),
                    self.y+((spr.topbound-spr.yorigin)*self.image_yscale),
                    self.x+((spr.rightbound-spr.xorigin)*self.image_xscale),
                    self.y+((spr.bottombound-spr.yorigin)*self.image_yscale))
        raise "Boom!"
    def SetBoundingBox(self):
        self.ResolveSprite()
        spr = self.sprite_index
        if spr<0:
            return
        if self.image_xscale==1 and self.image_yscale==1:
            if spr.bounding_box_type==1:
                self.bbox_left = self.x-spr.xorigin
                self.bbox_top = self.y-spr.yorigin 
                self.bbox_right = self.bbox_left+spr.subimages[0].get_width()
                self.bbox_bottom = self.bbox_top+spr.subimages[0].get_height()
            else:
                self.bbox_left = self.x+spr.leftbound-spr.xorigin
                self.bbox_top = self.y+spr.topbound-spr.yorigin
                self.bbox_right = self.x+spr.rightbound-spr.xorigin
                self.bbox_bottom = self.y+spr.bottombound-spr.yorigin
        else:
            if spr.bounding_box_type==1:
                self.bbox_left = self.x-(spr.xorigin*self.image_xscale)
                self.bbox_top = self.y-(spr.yorigin *self.image_yscale)
                self.bbox_right = self.bbox_left+(spr.subimages[0].get_width()*self.image_xscale)
                self.bbox_bottom = self.bbox_top+(spr.subimages[0].get_height()*self.image_yscale)
            else:
                self.bbox_left = self.x+((spr.leftbound-spr.xorigin)*self.image_xscale)
                self.bbox_top = self.y+((spr.topbound-spr.yorigin)*self.image_yscale)
                self.bbox_right = self.x+((spr.rightbound-spr.xorigin)*self.image_xscale)
                self.bbox_bottom = self.y+((spr.bottombound-spr.yorigin)*self.image_yscale)
    def BoundingBox(self,x=None,y=None):
        if x is None:
            return self.bbox_left, self.bbox_top, self.bbox_right, self.bbox_bottom
        return self.BoundingBox_OLD(x=x,y=y)
    def BoundingBox_OLD(self,x=None,y=None):
        if self.image_xscale!=1 or self.image_yscale!=1:
            raise "Sprite scaling not yet handled."
        self.ResolveSprite()
        spr = self.sprite_index
        if x is None:
            x = self.x
            y = self.y
        if spr.bounding_box_type==1:
            return (x-spr.xorigin, y-spr.yorigin, 
                x+self.subimages[0].get_width()-spr.xorigin,
                y+self.subimages[0].get_height()-spr.yorigin)
        elif spr.bounding_box_type==0:
            if spr.subimages:
                return (x-spr.xorigin, y-spr.yorigin, 
                    x+spr.subimages[0].get_width()-spr.xorigin,
                    y+spr.subimages[0].get_height()-spr.yorigin)
        return (x+spr.leftbound-spr.xorigin,
            y+spr.topbound-spr.yorigin,
            x+spr.rightbound-spr.xorigin,
            y+spr.bottombound-spr.yorigin)
    def IsUnderPos(self,x,y):
        # XXX precise collision checking not implemented.
        if x<self.bbox_left:
            return False
        elif x>self.bbox_right:
            return False
        elif y<self.bbox_top:
            return False
        elif y>self.bbox_bottom:
            return False
        return True
    def IsUnderPos_OLD(self,x,y):
        if gmapi.room.view_enabled:
            x -= gmapi.room.view_xview[0]
            y -= gmapi.room.view_yview[0]
        self.ResolveSprite()
        spr = self.sprite_index
        if type(spr)!=SpriteData:
            if spr==-1.0:
                return False
            spr = game.sprites[spr]
        if spr.precise_collision_checking:
            print "WARNING: IsUnderPos might need to take precise_collision_checking into account for",self.name
        if spr.bounding_box_type==1:
            if x<self.x-spr.xorigin:
                return False
            if y<self.y-spr.yorigin:
                return False
            img = spr.subimages[int(self.image_index)%len(spr.subimages)]
            if x>=self.x+img.get_width()-spr.xorigin:
                return False
            if y>=self.y+img.get_height()-spr.yorigin:
                return False
            return True
        if x<self.x+spr.leftbound-spr.xorigin:
            return False
        if x>=self.x+spr.rightbound-spr.xorigin:
            return False
        if y<self.y+spr.topbound-spr.yorigin:
            return False
        if y>=self.y+spr.bottombound-spr.yorigin:
            return False
        return True
    def IsUnderBox(self,box):
        if self.bbox_left>=box[2]:
            return False
        elif self.bbox_right<box[0]:
            return False
        elif self.bbox_top>=box[3]:
            return False
        elif self.bbox_bottom<box[1]:
            return False
        return True

class VisibleFPSIndicator(ObjectInstanceBase):
    depth = -10000000
    def __init__(self,dbg=0):
        self.dbg = dbg
        pass
    def Draw(self, **kwargs):
        if gmapi.room.view_current:
            return
        gmapi.draw_text(gmapi.room.view_xview[0],gmapi.room.view_yview[0],gmapi.fps)
        if self.dbg:
            gmapi.draw_text(gmapi.room.view_xview[0],gmapi.room.view_yview[0]+30,
                'Idle:'+str(debug_fps_delay_intended)+'\nDrop:'+str(debug_fps_skippedframes))

class ObjectInstanceZero(ObjectInstanceBase):
    def __getattr__(self,item):
        if item in self.__dict__:
            return self.__dict__[item]
        elif item in self.object_index.__dict__:
            # XXX does this deal properly with inheritance?
            return self.object_index.__dict__[item]
        ret = GMVar(0)
        self.__dict__[item] = ret
        return ret
ObjectInstance = ObjectInstanceBase
gmapi.__ObjectInstance = ObjectInstanceBase
gmapi.__BackgroundData = BackgroundData

def ObjectInstanceGenerator(o):
    for instance in o.instances:
        yield instance
    for child in o.children:
        for thing in child:
            yield thing

def GM_ObjectInstanceGenerator(o):
    o = o.__dict__['py_objectdata']
    for instance in o.instances:
        yield instance
    for child in o.children:
        for thing in child:
            yield thing

def ObjectChildrenDataGenerator(o):
    j = 0
    for child in o.children:
        yield child
        j += 1
        for sc in ObjectChildrenDataGenerator(child):
            yield sc
            j += 1
            if j>1000:
                print "1888!!"

class GM_Object(object):
    def __init__(self,parent):
        self.__dict__['py_objectdata'] = parent
    def __iter__(self):
        return GM_ObjectInstanceGenerator(self)
    def SetSpriteIndex(self,idx):
        for child in self:
            child.SetSpriteIndex(idx)
    def SetX(self,x):
        if type(x) is GMVar:
            x = x[0]
        for o in self:
            o.SetX(x)
    def SetY(self,y):
        if type(y) is GMVar:
            y = y[0]
        for o in self:
            o.SetY(y)
    def SetVisible(self,val):
        if type(val) is GMVar:
            val = val[0]
        for child in self:
            child.SetVisible(val)
    def SetSpeed(self,val):
        if type(val) is GMVar:
            val = val[0]
        for child in self:
            child.SetSpeed(val)
    def __getattr__(self,attr):
        for child in self:
            ret = child.__getattr__(attr)
            return ret
        if attr in self.__dict__:
            return attr
        if game.default_undef_is_zero:
            return 0
        print "Error getting attribute",attr,"on",self
        raise AttributeError
    def __length_hint__(self):
        return len(self.py_objectdata.instances)
    def __setattr__(self, item, val):
        for instance in self:
            instance.__setattr__(item, val)
    #(accessor functions for all objectinstance slots)
    def __getx(self):
        for child in self:
            return child.x
        if game.default_undef_is_zero:
            return 0
        # XXX possibly not the right exception to raise for 'there are no
        # children'.
        raise LookupError
    def __setx(self,x):
        for child in self:
            child.x = x
    x = property(__getx,__setx)
    def __gety(self):
        for child in self:
            return child.y
        if game.default_undef_is_zero:
            return 0
        raise LookupError
    def __sety(self,y):
        for child in self:
            child.y = y
    y = property(__gety,__sety)
    def __getbbox_top(self):
        for child in self:
            return child.bbox_top
        if game.default_undef_is_zero:
            return 0
        raise LookupError
    # XXX bbox_* are readonly variables in GM.  What does this do if I try to
    # set them?
    bbox_top = property(__getbbox_top)
    def __getbbox_bottom(self):
        for child in self:
            return child.bbox_bottom
        if game.default_undef_is_zero:
            return 0
        raise LookupError
    bbox_bottom = property(__getbbox_bottom)
    def __getbbox_left(self):
        for child in self:
            return child.bbox_left
        if game.default_undef_is_zero:
            return 0
        raise LookupError
    bbox_left = property(__getbbox_left)
    def __getbbox_right(self):
        for child in self:
            return child.bbox_right
        if game.default_undef_is_zero:
            return 0
        raise LookupError
    bbox_right = property(__getbbox_right)
           

class ObjectData(object):
    __slots__ = ('name',
        'sprite_index',
        'solid',
        'visible',
        'depth',
        'persistent',
        'parent_object_index',
        'mask_sprite_index',
        'instances',
        'children',
        'gmobject',
        'events',
        'py_UserDefinedEvents',
        'py_AnimationEndEvent',
        'py_IntersectBoundaryEvent',
        'py_RoomStartEvent',
        'py_RoomEndEvent',
        'py_LocalLeftButtonActionGroup', 'py_GlobLeftButtonActionGroup',
        'py_LocalRightButtonActionGroup', 'py_GlobRightButtonActionGroup',
        'py_LocalLeftPressedActionGroup', 'py_GlobLeftPressedActionGroup',
        'py_LocalRightPressedActionGroup', 'py_GlobRightPressedActionGroup',
        'py_LocalMiddlePressedActionGroup', 'py_GlobMiddlePressedActionGroup',
        'py_NoMoreLivesEvent',
        'py_IntersectBoundaryEvent', 'py_OutsideRoomEvent',
        'py_GameStartEvent',
        'nearby_grid','nearby_grid_room'
        )
    def __getstate__(self):
        ret = (self.name, self.sprite_index, self.solid, self.visible,
            self.depth, self.persistent, self.parent_object_index,
            self.mask_sprite_index, # self.instances, 
            self.children,
            #self.gmobject, 
            self.events,
            self.py_UserDefinedEvents, self.py_AnimationEndEvent,
            self.py_IntersectBoundaryEvent,
            self.py_RoomStartEvent, self.py_RoomEndEvent,
            self.py_LocalLeftButtonActionGroup, 
            self.py_GlobLeftButtonActionGroup,
            self.py_LocalRightButtonActionGroup,
            self.py_GlobRightButtonActionGroup,
            self.py_LocalLeftPressedActionGroup, 
            self.py_GlobLeftPressedActionGroup, 
            self.py_LocalRightPressedActionGroup,
            self.py_GlobRightPressedActionGroup,
            self.py_LocalMiddlePressedActionGroup,
            self.py_GlobMiddlePressedActionGroup,
            self.py_NoMoreLivesEvent,
            self.py_IntersectBoundaryEvent,
            self.py_OutsideRoomEvent,
            self.py_GameStartEvent, )
            # nearby_grid and nearby_grid_room are also dynamic.
        # If any more slots are added, they should be stored in a different
        # dict entry.
        return dict(d=ret)
    def __setstate__(self,state):
        (self.name, self.sprite_index, self.solid, self.visible,
            self.depth, self.persistent, self.parent_object_index,
            self.mask_sprite_index, # self.instances, 
            self.children,
            #self.gmobject, # gmobject is dynamic.
            self.events,
            self.py_UserDefinedEvents, self.py_AnimationEndEvent,
            self.py_IntersectBoundaryEvent,
            self.py_RoomStartEvent, self.py_RoomEndEvent,
            self.py_LocalLeftButtonActionGroup, 
            self.py_GlobLeftButtonActionGroup,
            self.py_LocalRightButtonActionGroup,
            self.py_GlobRightButtonActionGroup,
            self.py_LocalLeftPressedActionGroup, 
            self.py_GlobLeftPressedActionGroup, 
            self.py_LocalRightPressedActionGroup,
            self.py_GlobRightPressedActionGroup,
            self.py_LocalMiddlePressedActionGroup,
            self.py_GlobMiddlePressedActionGroup,
            self.py_NoMoreLivesEvent,
            self.py_IntersectBoundaryEvent,
            self.py_OutsideRoomEvent,
            self.py_GameStartEvent) = state['d']
        self.instances = []
        self.nearby_grid = None
        self.nearby_grid_room = -1
        gmapi.__dict__[self.name] = self.gmobject = GM_Object(self)
    def __iter__(self):
        return ObjectInstanceGenerator(self)
    def __init__(self,f):
        self.name = slurpdata(f)
        gmver, self.sprite_index, self.solid, self.visible, self.depth, self.persistent, self.parent_object_index, self.mask_sprite_index, reserved = unpack('<9l',f.read(36))
        # Sprite index -1 = None
        # Parent object index -100 = None
        # Mask sprite index -1 = None
        if reserved!=10:
            print "Warning: reserved",reserved,"!=10?"
        self.events = [[],]
        totalevents = 0
        while totalevents<11:
            numb = slurpint(f)
            if numb!=-1:
                ev = EventData(numb,f)
                self.events[-1].insert(0,ev)
            else:
                totalevents+=1
                self.events.append([])
        # Events:
        # 0 Create, 1 Destroy, 2 Alarm, 3 Step, 4 Collision, 5 Keyboard,
        # 6 Mouse, 7 Other, 8 Draw, 9 Keypress, 10 Keyrelease
        if len(self.events[3])>1:
            begin_steps = []
            middle_steps = []
            end_steps = []
            for step in self.events[3]:
                if step.numb==0:
                    middle_steps.append(step)
                elif step.numb==1:
                    begin_steps.append(step)
                elif step.numb==2:
                    end_steps.append(step)
                else:
                    raise "Unexpected step number "+str(step.numb)+" on "+self.name+'['+str(self.events[3])+']'
            self.events[3] = begin_steps + middle_steps + end_steps
        self.py_UserDefinedEvents = [
            None, None, None, None, None, None, None, None,
            None, None, None, None, None, None, None, None,
            ]
        self.py_AnimationEndEvent = None
        self.py_IntersectBoundaryEvent = None
        self.py_RoomStartEvent = None
        self.py_RoomEndEvent = None
        gmapi.__dict__[self.name]=self
        self.instances = []
        self.children = []
        self.py_LocalLeftButtonActionGroup = None
        self.py_GlobLeftButtonActionGroup = None
        self.py_LocalRightButtonActionGroup = None
        self.py_GlobRightButtonActionGroup = None
        self.py_LocalLeftPressedActionGroup = None
        self.py_GlobLeftPressedActionGroup = None
        self.py_LocalRightPressedActionGroup = None
        self.py_GlobRightPressedActionGroup = None
        self.py_LocalMiddlePressedActionGroup = None
        self.py_GlobMiddlePressedActionGroup = None
        self.py_IntersectBoundaryEvent = None
        self.py_OutsideRoomEvent = None
        self.py_NoMoreLivesEvent = None
        self.py_GameStartEvent = None
        gmapi.__dict__[self.name] = self.gmobject = GM_Object(self)
        self.nearby_grid = None
        self.nearby_grid_room = -1
    def NearbyObjects(self,x,y,range):
        if self.nearby_grid_room!=gmapi.room:
            self.nearby_grid = grid = []
            for obj in self:
                xmin = int(obj.bbox_left/32)
                xmax = math.ceil(obj.bbox_right/32)
                ymin = int(obj.bbox_top/32)
                ymax = math.ceil(obj.bbox_bottom/32)
                while ymax>=len(grid):
                    grid.append([])
                yi = ymin
                while yi<=ymax:
                    while xmax >= len(grid[yi]):
                        grid[yi].append([])
                    xi = xmin
                    while xi<=xmax:
                        grid[yi][xi].append(obj)
                        xi += 1
                    yi += 1
            self.nearby_grid_room = gmapi.room
        x = int(x)>>5
        y = int(y)>>5
        ret = set()
        xmin = max(x-range,0)
        ymin = max(y-range,0)
        xmax = x+range
        ymax = y+range
        while ymin<ymax:
            if ymax>=len(self.nearby_grid):
                return ret
            x = xmin
            xb = min(xmax,len(self.nearby_grid[ymin])-1)
            while x<=xb:
                for o in self.nearby_grid[ymin][x]:
                    ret.add(o)
                x += 1
            ymin += 1
        return ret
    def InheritEvents(self):
        parent = self.parent_object_index
        if type(parent) in (int,float):
            if parent>=0:
                parent = game.objects[parent]
            else:
                parent = None
        if parent:
            parent.children.append(self)
        for i in (0,1,3,4,5,7,8,9,10,11):
            parent = self.parent_object_index
            if type(parent) in (int,float):
                if parent>=0:
                    parent = game.objects[parent]
                else:
                    parent = None
            elif type(parent) is GM_Object:
                parent = parent.py_objectdata
            while type(parent) is ObjectData and (not self.events[i]):
                if type(parent) in (int,float):
                    if parent>=0:
                        parent = game.objects[parent]
                    else:
                        parent = None
                elif type(parent) is GM_Object:
                    parent = parent.py_objectdata
                if parent is not None:
                    self.events[i] = parent.events[i]
                    parent = parent.parent_object_index
        parent = self.parent_object_index
        if type(parent) in (int, float):
            if parent>=0:
                parent = game.objects[parent]
            else:
                parent = None
        elif type(parent) is GM_Object:
            parent = parent.py_objectdata
        while parent is not None:
            if not self.py_LocalLeftButtonActionGroup:
                self.py_LocalLeftButtonActionGroup = parent.py_LocalLeftButtonActionGroup
            if not self.py_LocalLeftPressedActionGroup:
                self.py_LocalLeftPressedActionGroup = parent.py_LocalLeftPressedActionGroup
            if not self.py_GlobLeftButtonActionGroup:
                self.py_GlobLeftButtonActionGroup = parent.py_GlobLeftButtonActionGroup
            if not self.py_GlobLeftPressedActionGroup:
                self.py_GlobLeftPressedActionGroup = parent.py_GlobLeftPressedActionGroup
            if not self.py_GlobRightPressedActionGroup:
                self.py_GlobRightPressedActionGroup = parent.py_GlobRightPressedActionGroup
            if not self.py_LocalMiddlePressedActionGroup:
                self.py_LocalMiddlePressedActionGroup = parent.py_LocalMiddlePressedActionGroup
            if not self.py_GlobMiddlePressedActionGroup:
                self.py_GlobMiddlePressedActionGroup = parent.py_GlobMiddlePressedActionGroup
            self.InheritUniqueEventNumbs(self.events[2],parent.events[2])
            self.InheritUniqueEventNumbs(self.events[7],parent.events[7])
            parent = parent.parent_object_index
            if type(parent) in (int, float):
                if parent>=0:
                    parent = game.objects[parent]
                else:
                    parent = None
            elif type(parent) is GM_Object:
                parent = parent.py_objectdata
        for ev in self.events[7]:
            if ev.numb==0:
                if self.py_OutsideRoomEvent:
                    raise "Error: multiple OutsideRoom events."
                self.py_OutsideRoomEvent = ev
            elif ev.numb==1:
                if self.py_IntersectBoundaryEvent:
                    raise "Error: Multiple IntersectBoundary events."
                self.py_IntersectBoundaryEvent = ev
            elif ev.numb==2:
                if self.py_GameStartEvent:
                    raise "Error: Multiple GameStartEvents"
                self.py_GameStartEvent = ev
            elif ev.numb==6:
                if self.py_NoMoreLivesEvent:
                    raise "Error: Multiple NoMoreLives events."
                self.py_NoMoreLivesEvent = ev
            elif ev.numb==7:
                if self.py_AnimationEndEvent:
                    raise "Error: Multiple AnimationEndEvents."
                self.py_AnimationEndEvent = ev
            elif ev.numb>=10 and ev.numb<=25:
                if self.py_UserDefinedEvents[ev.numb-10]:
                    raise "Error: User Defined Event already exists."
                self.py_UserDefinedEvents[ev.numb-10] = ev
            else:
                raise "Unexpected event "+str(ev)+', '+self.name+', '+str(ev.numb)
    def InheritUniqueEventNumbs(self, selfevs, parentevs):
        # XXX might need to bring this back for correct keyboard event hierarchy.
        for ev in parentevs:
            inheritme = True
            for eself in selfevs:
                if eself.numb==ev.numb:
                    inheritme = False
                    break
            if inheritme:
               selfevs.append(ev)
    def Compile(self):
        global g_mungedict
        g_mungedict = mungedicts.get(self.name,dict())
        self.InheritEvents()
        verbose = debug_list_all_script_content or (self.name in debug_list_script_content)
        if verbose:
            print "Compiling object",self.name
        for event in self.events[0:4]+self.events[5:]:
            for actiongroup in event:
                for action in actiongroup.actions:
                    if action.applies_to_object_index == -2:
                        action.applies_to_object_index = -1
        for eventnum in (0,1,2,3,4,5,6,7,8,9,10,11):
            #self.events[0:6]+self.events[7:]:
            for actiongroup in self.events[eventnum]:
                if verbose:
                    print actiongroup
                actiongroup.Compile(verbose,name=self.name+str(eventnum))
        for actiongroup in self.events[6]:
            actiongroup.Compile(verbose)
            if actiongroup.numb == 0:
                self.py_LocalLeftButtonActionGroup = actiongroup
            elif actiongroup.numb == 4:
                self.py_LocalLeftPressedActionGroup = actiongroup
            elif actiongroup.numb == 50:
                self.py_GlobLeftButtonActionGroup = actiongroup
            elif actiongroup.numb == 53:
                self.py_GlobLeftPressedActionGroup = actiongroup
            elif actiongroup.numb == 54:
                self.py_GlobRightPressedActionGroup = actiongroup
            # Should set middle button action groups.
            else:
                print "Unhandled action group numb",actiongroup
                raise "Kaboom."
            ## Guessing from the Order of Things
            #if actiongroup.numb in (0,1,2,3):
            #    # Left/Right/Middle/No
            #    self.py_LocalMouseButtonEvents.append(actiongroup)
            #elif actiongroup.numb in (4,5,6):
            #    # Left/Right/Middle Pressed
            #    self.py_LocalMousePressedEvents.append(actiongroup)
            #elif actiongroup.numb in (7,8,9):
            #    # Left/Right/Middle Released
            #    self.py_LocalMouseReleasedEvents.append(actiongroup)
            #elif actiongroup.numb == 10:
            #    #Mouse Enter
            #    self.py_MouseEnterEvent = actiongroup
            #elif actiongroup.numb==11:
            #    
            #Mouse Leave
            # Wheel Up/Down
        self.events[6] = None
        if verbose:
            print "...  done compiling."
        #    Global LRM
        #    Global LRM pressed
        #    Global LRM released
        #    JS1 LRUD, JS1 Butt1-8
        #    JS2 LRUD, JS2 Butt1-8
        g_mungedict = dict()
    def AddEvent(obj, ev, numb, python):
        egroup = None
        if ev!=6:
            for e in obj.events[ev]:
                if e.numb==numb:
                    egroup = e
            if egroup is None:
                egroup = DynamicEventData(numb)
                obj.events[ev].append(egroup)
        else:
            if numb==53:
                if not obj.py_GlobLeftPressedActionGroup:
                    obj.py_GlobLeftPressedActionGroup = DynamicEventData(numb)
                egroup = obj.py_GlobLeftPressedActionGroup
        egroup.dynamic_actions.append(python)
        egroup.Compile()
        
gmapi.py_ObjectData = ObjectData

def slurpobject(f):
    if f.read(4)!='\0\0\0\0':
        return ObjectData(f)
    return None

class RoomBackgroundData(object):
    def __init__(self,f=None):
        if f is None:
            return
        self.visible_when_room_starts, self.fg_image, self.bg_image, self.x, self.y, self.tile_h, self.tile_v, self.h_speed, self.v_speed, self.stretch = unpack('<10l',f.read(40))

class RoomViewData(object):
    def __init__(self,ver,f):
        self.visible = slurpint(f)
        if ver==520:
            self.left,self.top,self.width,self.height,self.x,self.y = unpack('<6l',f.read(24))
        elif ver>=541:
            self.view_x, self.view_y, self.view_w, self.view_h, self.port_x, self.port_y, self.port_w, self.port_h = unpack('<8l',f.read(32))
        self.hbor, self.vbor, self.hsp, self.vsp, self.objfollowing = unpack('<5l',f.read(20))

class InstanceID(int):
    def __setattr__(i,attr,val):
        if int(i) in instances:
            instances[int(i)].__setattr__(attr,val)
        else:
            print "DEBUG: InstanceID",int(i),"does not exist setting",attr,val,"?"
    def __getattr__(i,attr):
        return instances[int(i)].__getattr__(attr)

highid = InstanceID()

class RoomInstanceData(object):
    def __init__(self,f):
        self.x,self.y,self.object_idx,self.id = unpack('<4l',f.read(16))
        self.creation_code = slurpdata(f)
        global highid
        if self.id>highid:
            highid = self.id
        self.locked = slurpint(f)

class RoomTileData_OLD(object):
    def __init__(self,f):
        self.x, self.y, self.background_idx, self.tilex, self.tiley, self.w, self.h, self.layer_depth, self.id, self.locked = unpack('<10l',f.read(40))
fasttint_RoomTileData = gmapi.py_fasttint.RoomTileData
def RoomTileData(f):
    self = fasttint_RoomTileData()
    self.x, self.y, self.background_idx, self.tilex, self.tiley, self.w, self.h, self.layer_depth, self.id, self.locked = unpack('<10l',f.read(40))
    return self

def depthsort(b,a):
    if a.depth<b.depth:
        return -1
    elif a.depth>b.depth:
        return 1
    return 0

this_second = 0
last_second = -1

def debug(*args):
    print 'debug:',args

def tile_fill(dest, src, startx, starty, htile, vtile):
    if htile:
        maxx = dest.get_width()
    else:
        maxx = startx+1
    if vtile:
        maxy = dest.get_height()
    else:
        maxy = starty+1
    srcw,srch = src.get_size()
    y = starty
    while y<maxy:
        x = startx
        while x<maxx:
            dest.blit(src,(x,y))
            x += srcw
        y += srch

debug_print_draw = False
debug_print_fps = False
debug_fps_delay_intended = 0
debug_fps_skippedframes = 0
current_fps = 0

InstanceList = gmapi.py_fasttint.InstanceList
Floatlist8 = gmapi.py_fasttint.Floatlist8

def InstanceListFromSeq(t):
    ret = InstanceList()
    for i in t:
        ret.append(i)
    return ret

def Room_Compile(room):
    if debug_list_all_script_content:
        print "Compiling room",room.name
    room.creation_code_compiled = tree_to_python(gml2syntaxtree(room.creation_code))
    if debug_list_all_script_content:
        print room.creation_code_compiled,'//Done'
def RoomData(f):
    self = gmapi.py_fasttint.GMRoom()
    self.name = slurpdata(f)
    ver = slurpint(f)
    self.room_caption = slurpdata(f)
    self.room_width,self.room_height,self.snapx,self.snapy,self.isogrid,self.room_speed,self.room_persistent,self.bgcolor,self.draw_bgcolor = unpack('<9l',f.read(36))
    self.room_caption = ''
    self.frame_duration = 1.0/self.room_speed
    self.creation_code = slurpdata(f)
    numbgs = slurpint(f)
    if numbgs!=8:
        print "Warning: Not 8 bgs?"
    self.bg0 = RoomBackgroundData(f)
    self.bg1 = RoomBackgroundData(f)
    self.bg2 = RoomBackgroundData(f)
    self.bg3 = RoomBackgroundData(f)
    self.bg4 = RoomBackgroundData(f)
    self.bg5 = RoomBackgroundData(f)
    self.bg6 = RoomBackgroundData(f)
    self.bg7 = RoomBackgroundData(f)
    self.view_enabled = slurpint(f)
    numviews = slurpint(f)
    if numviews!=8:
        print "Warning: Not 8 views."
    self.view_xview = gmapi.py_fasttint.Floatlist8()
    self.view_yview = gmapi.py_fasttint.Floatlist8()
    self.view_object = gmapi.py_fasttint.Floatlist8()
    self.view_hborder = gmapi.py_fasttint.Floatlist8()
    self.view_vborder = gmapi.py_fasttint.Floatlist8()
    self.view_wview = Floatlist8()
    self.view_hview = Floatlist8()
    self.view_xport = Floatlist8()
    self.view_yport = Floatlist8()
    self.view_wport = Floatlist8()
    self.view_hport = Floatlist8()
    self.initial_view_data = []
    self.background_index = GMVar(-1)
    self.view_visible = Floatlist8()
    x = 0;
    #while len(self.views)<numviews:
    while x<numviews:
        vd = RoomViewData(ver,f)
        #self.views.append(vd)
        self.view_xview[x] = vd.view_x
        self.view_yview[x] = vd.view_y
        self.view_object[x] = vd.objfollowing
        self.view_hborder[x] = vd.hbor
        self.view_vborder[x] = vd.vbor
        self.view_hview[x] = vd.view_h
        self.view_wview[x] = vd.view_w
        self.view_xport[x] = vd.port_x
        self.view_yport[x] = vd.port_y
        self.view_wport[x] = vd.port_w
        self.view_hport[x] = vd.port_h
        self.view_visible[x] = vd.visible
        self.initial_view_data.append(vd)
        x+=1
    numinstances = slurpint(f)
    self.instances = []
    while len(self.instances)<numinstances:
        self.instances.append(RoomInstanceData(f))
    numtiles = slurpint(f)
    self.tiles = []
    while len(self.tiles)<numtiles:
        self.tiles.append(RoomTileData(f))
    f.read(44)
    if ver==520:
        self.tile_w, self.tile_h, self.tile_hsep, self.tile_vsep, self.tile_hoffset, self.tile_voffset = unpack('<6l',f.read(24))
    f.read(12)
    self.all_objects = InstanceList()
    self.drawn_objects = gmapi.py_fasttint.InstanceList()
    gmapi.__dict__[self.name] = self
    self.created = False
    self.deactivated_objects = gmapi.py_fasttint.InstanceList()
    self.keypressed_objects = gmapi.py_fasttint.InstanceList()
    self.keyreleased_objects = gmapi.py_fasttint.InstanceList()
    self.mouse_leftpressed_objects = gmapi.py_fasttint.InstanceList()
    self.mouse_rightpressed_objects = gmapi.py_fasttint.InstanceList()
    self.collisioned_objects = gmapi.py_fasttint.InstanceList()
    self.stepped_objects = gmapi.py_fasttint.InstanceList()
    self.moving_objects = gmapi.py_fasttint.InstanceList()
    self.alarmed_objects = gmapi.py_fasttint.InstanceList()
    self.cached_bg = None
    self.cached_bg_scaled = dict()
    self.animated_objects = gmapi.py_fasttint.InstanceList()
    self.key_objects = InstanceList()
    self.mouse_objects = InstanceList()
    print 3832,self.key_objects
    return self

def Room_Create(room):
    global view_xscale,view_yscale
    view_xscale=1
    view_yscale=1
    exec room.creation_code_compiled in gmapi.__dict__
    if room.created:
        print "RETURNING"
        return
    room.InitializeBackgrounds()
    tocreate = []
    #room.view_visible = []
    #for view in room.views:
    #    room.view_visible.append(view.visible)
    for instance in room.instances:
        if not (gmapi.instance_exists(instance.id) and game.objects[instance.object_idx].persistent):
            instanceo = ObjectInstance(game.objects[instance.object_idx].gmobject,id=instance.id)
            instanceo.x = instance.x
            instanceo.y = instance.y
            instanceo.xstart = instance.x
            instanceo.ystart = instance.y
            room.AddInstance(instanceo)
            tocreate.append(instanceo)
    for instanceo in tocreate:
        instanceo.SetBoundingBox()
        instanceo.RunCreateEvent()
    room.created = True

def Room_InitializeBackgrounds(self):
    self.background_index = GMVar(-1)
    self.background_x = Floatlist8()
    self.background_y = Floatlist8()
    self.background_visible = Floatlist8()
    self.background_width = Floatlist8()
    self.background_height = Floatlist8()
    self.background_xscale = Floatlist8()
    self.background_yscale = Floatlist8()
    self.background_hspeed = Floatlist8()
    self.background_vspeed = Floatlist8()
    self.background_blend = Floatlist8()
    self.background_alpha = Floatlist8()
    self.background_htiled = Floatlist8()
    self.background_vtiled = Floatlist8()
    i = 0;
    for bg in (self.bg0,self.bg1,self.bg2,self.bg3,self.bg4,self.bg5,self.bg6,self.bg7):
        self.background_index[i] = bg.bg_image
        self.background_x[i] = bg.x
        self.background_y[i] = bg.y
        self.background_visible[i] = bg.visible_when_room_starts
        if game.backgrounds[bg.bg_image] is not None:
            self.background_width[i] = game.backgrounds[bg.bg_image].w
            self.background_height[i] = game.backgrounds[bg.bg_image].h
        else:
            self.background_height[i] = 0
            self.background_width[i] = 0
        # XXX should use 'stretch'?
        self.background_xscale[i] = 1 #bg.xscale)
        self.background_yscale[i] = 1 #bg.yscale)
        self.background_hspeed[i] = bg.h_speed
        self.background_vspeed[i] = bg.v_speed
        # XXX possibly should be an int?
        self.background_blend[i] = gmapi.__color_to_int(gmapi.c_white)
        self.background_alpha[i] = 1.0
        self.background_htiled[i] = bg.tile_h
        self.background_vtiled[i] = bg.tile_v
        i += 1
def Room_AddInstance(room, instance):
    instance.py_inactive = 0
    i = 0
    inserted = False
    room.all_objects.append(instance)
    pyo = instance.object_index.py_objectdata
    if instance.visible:
        room.AddDrawnInstance(instance)
    if instance.speed or instance.gravity or instance.path_speed:
        room.moving_objects.append(instance)
    if type(instance)==gmapi.py_ParticleSystem:
        # XXX should be stepped, when stepped events are implimented.
        # XXX possibly should be drawn, when drawn events are implimented,
        # XXX depending on how they are.
        return
    #room.all_objects.append(instance)
    if pyo.events[2]:
        room.alarmed_objects.append(instance)
    if pyo.events[3]:
        room.stepped_objects.append(instance)
    if pyo.events[4]:
        room.collisioned_objects.append(instance)
    if pyo.events[5]:
        room.key_objects.append(instance)
    if pyo.py_LocalLeftButtonActionGroup or pyo.py_GlobLeftButtonActionGroup or pyo.py_LocalRightButtonActionGroup or pyo.py_GlobRightButtonActionGroup:
        room.mouse_objects.append(instance)
    #if instance.object_index.events[5]:
    #    room.keyboarded_objects.append(instance)
    #if instance.object_index.events[6]:
    #    room.moused_objects.append(instance)
    if pyo.py_LocalLeftPressedActionGroup or pyo.py_GlobLeftPressedActionGroup:
        room.mouse_leftpressed_objects.append(instance)
    if pyo.py_GlobRightPressedActionGroup:
        room.mouse_rightpressed_objects.append(instance)
    # 7 - other
    #if instance.object_index.events[8]:
    #    room.drawn_objects.append(instance)
    if pyo.events[9]:
        room.keypressed_objects.append(instance)
    if pyo.events[10]:
        room.keyreleased_objects.append(instance)
    #XXX room should have a list of objects that handle keyboard events
    #XXX room should have a list of objects that handle mouse events
    if instance.image_speed:
        room.animated_objects.append(instance)
    if instance.sprite_index!=-1:
        instance.SetBoundingBox()

def Room_ResortDrawnObjects(room):
    room.drawn_objects.sort(cmp=depthsort)

def Room_AddDrawnInstance(room, instance):
    i = 0
    while i<len(room.drawn_objects):
        if instance.depth>room.drawn_objects[i].depth:
            room.drawn_objects.insert(i,instance)
            return
        i += 1
    room.drawn_objects.append(instance)

def Room_SetBackgroundIndex(room,num,bg):
    if type(bg) is GMVar:
        bg = bg[0]
    x = bg
    room.background_index[num] = bg
    room.cached_bg = None
    room.cached_bg_scaled = dict()

def Room_StepAlarms(room):
    for instance in room.alarmed_objects:
        x = 0
        while x<12:
            if instance.alarm[x]>-1:
                instance.alarm[x] -= 1
                if instance.alarm[x]<=0:
                    instance.alarm[x] = -1
                    instance.RunAlarm(x)
            x += 1

def Room_StepKey(room):
    for instance in room.key_objects:
        instance.StepKey()
def Room_StepMouse(room):
    for instance in room.mouse_objects:
        instance.StepMouse()
def Room_MoveInstances(room):
    for instance in room.moving_objects:
        instance.MoveInstance()
def Room_HandleCollisions(room):
    if gmapi.__dict__['__new_room']!=gmapi.room:
        return
    for instance in room.collisioned_objects:
        for ev in instance.object_index.py_objectdata.events[4]:
            if instance.HandleCollisions(ev):
                return
def Room_MouseButtonDown(room,event):
    butt = event.button
    if butt>=3:
        if butt>3:
            return
        butt = 4
    #gmapi.mouse_lastbutton = gmapi.mouse_button
    if butt==1:
        left = tuple(room.mouse_leftpressed_objects)
        for obj in left:
            if obj.object_index.py_objectdata.py_GlobLeftPressedActionGroup:
                RunActionGroupOn(obj.object_index.py_objectdata.py_GlobLeftPressedActionGroup, obj)
            if obj.object_index.py_objectdata.py_LocalLeftPressedActionGroup and obj.IsUnderPos(gmapi.mouse_x,gmapi.mouse_y):
                RunActionGroupOn(obj.object_index.py_objectdata.py_LocalLeftPressedActionGroup, obj)
    elif butt==2:
        # XXX should handle middle button.
        return
        for obj in room.all_objects:
            print obj.object_index.py_objectdata,obj,obj.object_index
            if obj.object_index.py_objectdata.py_LocalMiddlePressedActionGroup and obj.IsUnderPos(gmapi.mouse_x,gmapi.mouse_y):
                RunActionGroupOn(obj.object_index.py_objectdata.py_LocalMiddlePressedActionGroup, obj)
            if obj.object_index.py_objectdata.py_GlobMiddlePressedActionGroup:
                RunActionGroupOn(obj.object_index.py_objectdata.py_GlobMiddlePressedActionGroup, obj)
    elif butt==4:
        right = tuple(room.mouse_rightpressed_objects)
        x = len(right)
        while x:
            x -= 1
            if (x<len(right)):
                obj = right[x]
                if obj.object_index.py_objectdata.py_LocalRightPressedActionGroup and obj.IsUnderPos(gmapi.mouse_x,gmapi.mouse_y):
                    RunActionGroupOn(obj.object_index.py_objectdata.py_LocalRightPressedActionGroup, obj)
                if obj.object_index.py_objectdata.py_GlobRightPressedActionGroup:
                    RunActionGroupOn(obj.object_index.py_objectdata.py_GlobRightPressedActionGroup, obj)
        
    else:
        raise "Unexpected mouse button pressed."
def Room_MouseButtonUp(room,event):
    butt = event.button
    if butt>=3:
        if butt==3:
            butt = 4
        else:
            return
    gmapi.mouse_button &= ~butt
    # TODO: run button-up events?

def Room_HandleKeypress(room, key):
    for obj in room.keypressed_objects:
        obj.HandleKeypress(key)
def Room_HandleKeyrelease(room, key):
    #if gmapi.py_translate_keys.has_key(key):
    #    key = gmapi.py_translate_keys[key]
    #gmapi.py_keys_pressed[key] = 0
    for obj in room.keyreleased_objects:
        for actiongroup in obj.events[10]:
            if actiongroup.numb==key:
                RunActionGroupOn(obj,actiongroup)

def Room_StepEvents(room):
    for instance in room.animated_objects:
        instance.StepAnimation()
    for instance in room.stepped_objects:
        for actiongroup in instance.object_index.py_objectdata.events[3]:
            RunActionGroupOn(actiongroup,instance)

def Room_DrawBackgrounds(room, surface, xoff, yoff):
    vc = room.view_current
    if view_xscale!=1 or view_yscale!=1:
        key = (view_xscale,view_yscale)
        if not room.cached_bg_scaled.has_key(key):
            room.cached_bg_scaled[key] = gmapi.py_fasttint.scale(room.cached_bg,view_xscale,view_yscale)
        surface.blit(room.cached_bg_scaled[key], (-(xoff*view_xscale), -(yoff*view_yscale)))
        return
    if room.cached_bg is None:
        room.RenderBackground()
    surface.blit(room.cached_bg,(-xoff,-yoff))

def Room_RenderBackground(room):
    surface = pygame.surface.Surface((room.room_width,room.room_height))
    if room.draw_bgcolor:
        surface.fill(room.bgcolor)
    x = 0
    while x<len(room.background_index):
        if room.background_index[x]!=-1 and room.background_visible[x]:
            bg = room.background_index[x]
            if type(bg) in (int,float):
                bg = room.background_index[x] = game.backgrounds[bg]
            if bg is not None and bg.surf is not None:
                if room.background_xscale[x]==1 and room.background_yscale[x]==1:
                    bgsurf = bg.surf
                else:
                    bgsurf = gmapi.py_fasttint.scale(bg.surf,room.background_xscale,room.background_yscale)
                tile_fill(surface, bgsurf,
                    room.background_x[x], room.background_y[x], 
                    room.background_htiled[x], room.background_vtiled[x]
                    )
        x += 1
    for tile in room.tiles:
        surface.blit(
            game.backgrounds[tile.background_idx].surf,
            (tile.x,tile.y),
            (tile.tilex,tile.tiley,tile.w,tile.h))
    room.cached_bg = surface

def Room_Leave(room):
    for instance in room.all_objects:
        # XXX this may well be Bad.
        try:
            instance.object_index.py_objectdata.instances.remove(instance)
        except:
            print "Removal failed",instance,instance.object_index.py_objectdata.persistent
    room.deactivated_objects = gmapi.py_fasttint.InstanceList()
    room.keypressed_objects = gmapi.py_fasttint.InstanceList()
    room.keyreleased_objects = gmapi.py_fasttint.InstanceList()
    room.mouse_leftpressed_objects = gmapi.py_fasttint.InstanceList()
    room.mouse_rightpressed_objects = gmapi.py_fasttint.InstanceList()
    room.created = False
    room.all_objects = InstanceList()
    room.drawn_objects = InstanceList()
    room.collisioned_objects = InstanceList()
    room.stepped_objects = InstanceList()
    room.moving_objects = InstanceList()
    room.alarmed_objects = InstanceList()
    room.animated_objects = InstanceList()
    room.key_objects = InstanceList()
    room.mouse_objects = InstanceList()
    room.cached_bg = None
    room.cached_bg_scaled = dict()

def slurproom(f):
    if f.read(4)!='\0\0\0\0':
        return RoomData(f)
    return None

class GM6_File(object):
    def __getstate__(self):
        self.highid = highid
        self.event_handler_counter = event_handler_counter
        self.reserved_varname_translations = reserved_varname_translations
        return self.__dict__
    def __setstate__(self,state):
        global reserved_varname_translations
        global reserved_varnames
        if state['default_undef_is_zero']:
            global ObjectInstance
            ObjectInstance = ObjectInstanceZero
            gmapi.py_ObjectInstance = ObjectInstanceZero
            gmapi.__dict__['py_PythonAutovivifyingArray'] = gmapi.__dict__['py_PythonAutovivifyingZeroArray']
            gmapi.py_reserved_varname_1 = reserved_varnames['global'] = GlobalObjectAutoZero()
        else:
            gmapi.py_ObjectInstance = ObjectInstanceBase
        global highid
        highid = state['highid']
        del state['highid']
        global event_handler_counter
        event_handler_counter = state['event_handler_counter']
        del state['event_handler_counter']
        reserved_varname_translations = state['reserved_varname_translations']
        del state['reserved_varname_translations']
        self.__dict__ = state
    def __init__(self,s,destdir):
        f = file(s,'rb')
        if f.read(4)!=struct.pack('<l',1234321):
            print "Warning: Not a GM file?\n"
        self.version = struct.unpack("<l",f.read(4))[0]
        if self.version==530:
            if f.read(4)!='\0\0\0\0':
                print "Warning: Not a GM530 file?\n"
        self.game_id = struct.unpack('<l16x',f.read(20))[0]
        version = struct.unpack('<l',f.read(4))[0]
        if version!=self.version:
            print "Version incoherency."
        self.fullscreen = struct.unpack('<l',f.read(4))
        if version>=600:
            self.interpolate_colors_between_pixels = unpack('<l',f.read(4))[0]
        self.dont_draw_border,self.display_the_cursor = unpack('<2l',f.read(8))
        if version==530:
            self.scale_percent_windowed,self.scale_percent_fullscreen,self.scale_hw_only = struct.unpack('3<l',f.read(12))
        if version>=542:
            b = f.read(16)
            self.scaling,self.allow_resize,self.always_on_top,self.color_outside_room = struct.unpack('<4l',b)
        self.set_resolution_of_screen = struct.unpack('<l',f.read(4))
        if version == 530:
            self.color_depth,self.exclusive_mode,self.resolution,self.frequency,self.wait_for_vblank,self.display_caption_in_fullscreen = struct.unpack('6<l',f.read(24))
        if version>=542:
            self.color_depth,self.resolution,self.frequency = struct.unpack('<3l',f.read(12))
        self.hide_caption_buttons = struct.unpack('<l',f.read(4))
        if version>=542:
            self.use_vsync = unpack('<l',f.read(4))
        b = f.read(20)
        self.f4_switches_screen_modes,self.f1_shows_game_info,self.esc_quits,self.f5_saves_f6_loads,self.process_priority = unpack('<5l',b)
        if version==530:
            f.read(8)
        if version>=702:
            self.f9_is_screenshot,self.close_is_esc == unpack('<2l',f.read(8))
            raise "GM7 not supported."
        self.pause_on_focus_loss, self.loading_progress_bar = unpack('<2l',f.read(8))
        if self.loading_progress_bar==2:
            if f.read(4)!='\377\377\377\377':
                self.back_image_data = slurpdata(f)
            if f.read(4)!='\377\377\377\377':
                self.front_image_data = slurpdata(f)
        self.show_own_image_while_loading = f.read(4)!='\0\0\0\0'
        if self.show_own_image_while_loading:
            if f.read(4)!='\377\377\377\377':
                self.front_image_data_2 = slurpdata(f)
        self.make_image_partially_transparent = f.read(4)!='\0\0\0\0'
        self.make_translucent_with_alpha = unpack('<l',f.read(4))[0]
        self.scale_progress_bar = f.read(4)!='\0\0\0\0'
        self.icon_image_data = slurpdata(f)
        try:
            os.mkdir(destdir)
        except:
            pass
        if self.icon_image_data:
            if os.uname()[0]=='Darwin':
                pygame.display.set_icon(gmapi.renpy_glue.readsurf('icon.png'))
            else:
                dump_file(destdir+'/icon.ico',self.icon_image_data)
                pygame.display.set_icon(pygame.image.load(destdir+'/icon.ico'))
            del self.icon_image_data
        self.display_error_messages, self.write_errors_to_log, self.abort_on_errors, self.default_undef_is_zero = unpack('<4l',f.read(16))
        global ObjectInstance
        if self.default_undef_is_zero:
            ObjectInstance = ObjectInstanceZero
            gmapi.py_ObjectInstance = ObjectInstanceZero
        else:
            ObjectIndex = ObjectInstanceBase
            gmapi.py_ObjectInstance = ObjectInstanceBase
        self.author = slurpdata(f)
        if version<702:
            f.read(4)
        else:
            slurpdata(f)
            raise "GM7 not supported."
        self.last_changed = slurpdouble(f)
        self.information = slurpdata(f)
        self.constants = dict()
        numconstants = slurpint(f)
        while numconstants>0:
            cname = slurpdata(f)
            cdata = slurpdata(f)
            self.constants[cname] = cdata
            numconstants = numconstants-1
        if version>=702:
            self.major, self.minor, self.release, self.build = unpack('<4l',f.read(16))
            self.company = slurpdata(f)
            self.product = slurpdata(f)
            self.description = slurpdata(f)
            raise "GM7 not supported."
        elif version>=542:
            numincludes = slurpint(f)
            self.includes = []
            while numincludes:
                self.includes.append(slurpdata(f))
                numincludes = numincludes-1
            self.includes_in_tmp,self.includes_overwrite,self.includes_delete_at_game_end = unpack('<3l',f.read(12))
        soundblockversion = slurpint(f)
        numsounds = slurpint(f)
        self.sounds = []
        while numsounds>0:
            self.sounds.append(slurpsound(f))
            numsounds -= 1
        spriteblockversion = slurpint(f)
        if spriteblockversion>702:
            print "Warning: sprite block version",spriteblockversion
            raise "GM7 not supported."
        numsprites = slurpint(f)
        self.sprites = []
        while numsprites>0:
            self.sprites.append(slurpsprite(f))
            numsprites -= 1
        bgblockversion = slurpint(f)
        numbgblocks = slurpint(f)
        self.backgrounds = []
        while numbgblocks>0:
            self.backgrounds.append(slurpbackground(f))
            numbgblocks -= 1
        pathblockver = slurpint(f)
        numpaths = slurpint(f)
        self.paths = []
        while numpaths>0:
            self.paths.append(slurppath(f))
            # XXX unimplimented
            numpaths -= 1
        scriptblockver = slurpint(f)
        numscripts = slurpint(f)
        self.scripts = []
        while numscripts>len(self.scripts):
            self.scripts.append(slurpscript(f))
        datafilesversion = slurpint(f)
        if datafilesversion!=540:
            print "Warning: Version 440 data files unimplimented."
        else:
            numfonts = slurpint(f)
            self.fonts = []
            while len(self.fonts)<numfonts:
                self.fonts.append(slurpfont(f))
        timelinesversion = slurpint(f)
        numtimelines = slurpint(f)
        self.timelines = []
        while numtimelines<len(self.timelines):
            self.timelines.append(slurptimeline(f))
        objectver = slurpint(f)
        numobjects = slurpint(f)
        self.objects = []
        while len(self.objects)<numobjects:
            self.objects.append(slurpobject(f))
        self.rooms = []
        roomversion,roomcount = unpack('<2l',f.read(8))
        while len(self.rooms)<roomcount:
            self.rooms.append(slurproom(f))
        self.last_instance_placed, self.last_tile_placed = unpack('<2l',f.read(8))
        if self.version==701:
            # ??? Not 702?
            includesver,numincludes = unpack('<2l',f.read(8))
            self.includes = []
            while len(self.includes)<numincludes:
                self.includes.append(slurpinclude(f))
            packagesver, numpackages = unpack('<2l',f.read(8))
            self.packages = []
            while len(self.packages)<numpackages:
                self.packages.append(slurpdata(f))
            raise "GM7 not supported."
        infover, self.infobgcolor, self.infomimicsmainwindow = unpack('<3l',f.read(12))
        if infover>=600:
            self.infowindowcaption = slurpdata(f)
            self.infowindow_left, self.infowindow_top, self.infowindow_w, self.infowindow_h, self.infowindow_show_border_and_caption, self.infowindow_resize, self.infowindow_ontop, self.infowindow_pauses = unpack('<8l',f.read(32))
        self.info_rtf = slurpdata(f)
        libver = slurpint(f)
        numlibs = slurpint(f)
        self.lib_creation_code = []
        while len(self.lib_creation_code)<numlibs:
            self.lib_creation_code.append(slurpdata(f))
        execroomsver = slurpint(f)
        numexecrooms = slurpint(f)
        self.execrooms = []
        while len(self.execrooms)<numexecrooms:
            self.execrooms.append(slurpint(f))
        gmapi.room = self.rooms[self.execrooms[0]]
        x = 0
        if self.default_undef_is_zero:
            gmapi.__dict__['py_PythonAutovivifyingArray'] = gmapi.__dict__['py_PythonAutovivifyingZeroArray']
            reserved_varnames['global'] = GlobalObjectAutoZero()
        global game
        game = self
        while x<len(self.scripts):
            if self.scripts[x]:
                self.scripts[x].compile(self)
            x += 1
        x = 0
        while x<len(self.rooms):
            if self.rooms[x]:
                self.rooms[x].Compile()
            x += 1
        x = 0
        while x<len(self.objects):
            if self.objects[x]:
                self.objects[x].Compile()
            x += 1
    def OptimizeTextures(game):
        x = 0
        while x<len(game.sprites):
            if type(game.sprites[x])==SpriteData:
                i = 0
                while i<len(game.sprites[x].subimages):
                    game.sprites[x].subimages[i] = game.sprites[x].subimages[i].convert_alpha()
                    i += 1
            x += 1
    def PickleTo(game,f):
        import pickle
        try:
            pickle.dump(game,f,protocol=cPickle.HIGHEST_PROTOCOL)
        except:
            for k in game.__dict__:
                print "Trying",k
                try:
                    pickle.dumps(game.__dict__[k])
                except:
                    print type(game.__dict__[k])
                    if type(game.__dict__[k])==list:
                        for i in game.__dict__[k]:
                            print 'Failed to pickle',i
                            pickle.dumps(i)
                    raise "Pickling error."

def GM6_File_FromPickle(fn,loading_bg=None):
    if use_renpy:
        fn = renpy.exports.config.searchpath[0]+'/'+fn
    f = file(fn,'r')
    if pygame.display.get_surface() is None:
        pygame.display.set_mode((640,480),0)
    ret = cPickle.load(f)
    f.close()
    gmapi.room = ret.rooms[ret.execrooms[0]]
    global game
    game = ret
    for sprite in ret.sprites:
        if sprite:
            gmapi.__dict__[sprite.name] = sprite
    for sound in ret.sounds:
        if sound:
            gmapi.__dict__[sound.name] = sound
    for background in ret.backgrounds:
        if background:
            gmapi.__dict__[background.name] = background
    for path in ret.paths:
        if path:
            gmapi.__dict__[path.name] = path
    for script in ret.scripts:
        if script:
            # Don't compile yet - we need the objects loaded into the
            # namespace too.
            gmapi.__dict__[script.name] = script
    global debugger_font
    for font in ret.fonts:
        if font:
            debugger_font = font
            gmapi.__dict__[font.name] = font
    # TODO timelines should be in the namespace.
    for obj in ret.objects:
        if obj:
            # Don't compile yet - compilation needs to take place only after
            # the namespace is fully populated.
            gmapi.__dict__[obj.name] = obj.gmobject
    for room in ret.rooms:
        if room:
            gmapi.__dict__[room.name] = room

    # Now we can compile.
    for script in ret.scripts:
        if script:
            script.compile(ret)
    for script in replace_scripts_gml:
        raise boom#self.syntax_tree = gml2syntaxtree(replace_scripts_gml[script])
    for script in replace_scripts:
        gmapi.__dict__[script] = replace_scripts[script]
    for obj in ret.objects:
        if obj:
            for evg in obj.events:
                for ag in evg or ():
                    ag.Compile()
    return ret

def set_pygame_screen(screen):
    global pygame_screen
    pygame_screen = screen
    gmapi.__pygame_screen = screen
    gmapi.py_fasttint.set_SDL_screen(screen)

gmapi.py_fasttint.RegisterInstancesDict(instances, ObjectInstanceZero, ObjectInstanceBase, gmapi.py_ParticleSystem)
