import pygame
import random as py_random
import sys
import math
#import gm6
import string as py_string
import os
import re
import stat
import time
import glob as py_glob
try:
    import renpy
    #os.chdir(renpy.config.searchpath[0])
except:
    pass
import renpy_glue
py_uname = os.uname()
import platform
import imp
def py_import_fasttint():
    if 'fasttint' in sys.modules:
        return sys.modules['fasttint']
    uname = platform.uname()
    if uname[0]=='Linux':
        machine = platform.machine()
        filename = 'fasttint_i586.so'
        try:
            imp.load_dynamic('libSDL_gfx','libSDL_gfx.so.13')
        except:
            pass
        if machine=='x86_64':
            filename = 'fasttint_x86_64.so'
        elif (machine not in ('i586','i686')):
            print "Debug: Unidentified machine type",machine,"- defaulting to i586"
    elif uname[0]=='Darwin':
        # It LOOKS like loading a lib
        # like this will import all the symbols, then spew an irrelevant
        # error about initlibSDL_gfx not being defined, enabling fasttint
        # imported from within Python on an intel mac to complain about
        # pygame.surflock next, and I got a report that this actually ran on
        # an Intel.
        try:
            imp.load_dynamic('libSDL_gfx','libSDL_gfx.13.5.2.dylib')
        except:
            pass
        if uname[5] in ('powerpc','powerpc64','ppc64','ppc'):
            # XXX 'powerpc' observed in the wild.  I don't know what ppc64 will
            # return, or if anything will return 'ppc'
            filename = 'fasttint_macos_ppc.so'
        else:
            filename = 'fasttint_macos_i386.so'
    else:
        print "Debug: Unidentified machine type",uname,"- defaulting to Linux/i586"
        filename = 'fasttint_i586.so'
    try:
        f = file(filename,'r')
        return imp.load_module('fasttint',f,filename,('.so','r',imp.C_EXTENSION))
    except:
        if uname[0]=='Linux':
            filename = 'fasttint_i586.so'
            f = file(filename,'r')
            return imp.load_module('fasttint',f,filename,('.so','r',imp.C_EXTENSION))
        else:
            print "Failed to load fasttint module"
            raise sys.exc_type,sys.exc_value,sys.exc_traceback
py_fasttint = py_import_fasttint()

py_GMVar = py_fasttint.GMVar
py_SpriteData = py_fasttint.GMSprite
import euclid
py_mouse_pressed = 0
py_mouse_released = 0
py_TRAP = 0
py_game_restart = False
#import Numeric
#import numpy
#import pills as py_pills

# This should present the namespace as seen from inside a GM program.

# Everything prefixed 'py_' is part of the compatibility API, used to
# let python code access things that aren't exposed in GM (or possibly just
# aren't exposed in a documented fashion)

# Used to determine how 'break' is handled - if the last entry is 'switch',
# 'break' becomes the code sequences used to break out of a pseudoswitch.
# Otherwise, it will be 'break'.  XXX 'with(single)' is unimplimented but
# should cancel a 'with'.
py_repeat_stack = []

# Used for room_goto.
class py_GameEndException:
    pass

# Used to ensure that GM 'with' doesn't clobber self/other in the
# outside context.
py_self_stack = []
py_other_stack = []

# Various small internal variables
__screen_redraw_in_progress = False

# __pygame_screen is used so functions can actually get at the drawing
# surface.
__pygame_screen = None

__new_room = None

health = None
lives = None
noone = None # -4 in GM
def py_AllObjectsGenerator():
    for obj in gm6.game.objects:
        if obj is not None:
            for instance in obj.gmobject:
                yield instance
class py_AllObjects(object):
    def __iter__(self):
        return py_AllObjectsGenerator()
all = py_AllObjects()
other = None
room = None
score = 0
self = None
keyboard_string = u''

# fps counter.
fps = 30


# GM arrays are different from Python arrays.  They are created 
# automatically by attempts to reference array members.  Uninitialized members
# may default to zero.  Arrays accessed by name *without* an index will
# return index 0.
class py_PythonAutovivifyingArray(list):
    def __add__(l,a):
        return l[0]+a
    def __setitem__(l,idx,val):
        while len(l)<=idx:
            l.append(None)
        list.__setitem__(l,idx,val)

def __to_rgb(i):
    if type(i)==py_GMVar:
        i = i[0]
    if type(i)==tuple:
        return i
    i = int(i)
    #return (i>>16,(i>>8)&0xff,i&0xff)
    ret = i&0xff,(i>>8)&0xff,i>>16
    return ret

class py_PythonAutovivifyingZeroArray(py_PythonAutovivifyingArray):
    def __init__(l,*args):
        list.__init__(l,*args)
    def __getitem__(l,idx):
        if idx>=len(l):
            return 0
        return list.__getitem__(l,idx)
class py_PythonAutovivifyingArray2D(list):
    def __getitem__(l,idx):
        while len(l)<=idx:
            l.append(py_PythonAutovivifyingArray())
        return list.__getitem__(l,idx)
class py_PythonAutovivifyingZeroArray2D(list):
    def __getitem__(l,idx):
        while len(l)<=idx:
            l.append(py_PythonAutovivifyingZeroArray())
        return list.__getitem__(l,idx)

py_keys_pressed = py_PythonAutovivifyingZeroArray()

class py_ParticleEmitter(object):
    def __init__(em):
        # XXX set from screen?
        em.xmin = None
        em.xmax = None
        em.ymin = None
        em.ymax = None
        em.shape = ps_shape_rectangle
        em.distribution = ps_distr_linear
        em.part_type = None
        em.number = 0
        # XXX what is the default?
# Number is number to generate per step.  If <0, 1/(-number) odds of
        # particle creation.
    def SpawnParticle(em):
        if em.shape!=ps_shape_rectangle:
            raise "Error generating particles: unexpected shape."
        if em.distribution==ps_distr_gaussian:
            raise "Gaussian particle distribution unimplimented."
        elif em.distribution!=ps_distr_linear:
            raise "Unexpected distribution."
        if em.part_type is None:
            print "Error: No particle type attached to emitter?",em.__dict__
            return None
        if abs(em.xmin-(em.xmax-1))<=1:
            x = em.xmin
        else:
            x = py_random.randint(em.xmin, em.xmax-1)
        if abs(em.ymin-(em.ymax-1))<=1:
            y = em.ymin
        else:
            y = py_random.randint(em.ymin, em.ymax-1)
        ret = py_Particle(em.part_type, x, y)
        return ret
    def py_Step(em):
        if em.number<0:
            if py_random.random()<1.0/-em.number:
                return [em.SpawnParticle(),]
            return
        i = 0
        ret = []
        while i<em.number:
            ret.append(em.SpawnParticle())
            i += 1
        return ret

class py_ParticleSystem(object):
    __slots__ = ('persistent','visible', 'py_objectdata', 'alarm', 
        'depth','x','y','emitters','py_automatic_update', 'speed',
        'py_automatic_draw','py_particles','object_index', 'py_inactive')
    persistent = True
    visible = 1
    class py_objectdata(object):
        __slots__ = ('persistent', 'name', 'solid', 'py_GameStartEvent')
        persistent = True
        name = 'particle system'
        py_GameStartEvent = None
        solid = 0
        class instances_class(object):
            def remove(o,i):
                pass
            def append(o,i):
                pass
        instances = instances_class()
    alarm = (-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1)
    def IsUnderBox(self,box):
        return False
    def __init__(ps):
        ps.depth = 0
        ps.x = None
        ps.y = None
        ps.emitters = []
        ps.py_automatic_update = 1
        ps.speed = 1
        # Is particlesystem speed actually used?
        ps.py_automatic_draw = 1
        ps.py_particles = []
        ps.object_index = py_ParticleSystem
        room.AddInstance(ps)
        ps.py_inactive = 0
    def IsUnderPos(ps,x,y):
        return 0
    def RoomStart(ps):
        pass
    def Destroy(ps):
        ps.py_particles = []
        ps.emitters = []
        room.all_objects.remove(ps)
        room.drawn_objects.remove(ps)
    def Remove(ps):
        room.all_objects.remove(ps)
        room.drawn_objects.remove(ps)
    def EmitterCreate(ps):
        ps.emitters.append(py_ParticleEmitter())
        return ps.emitters[-1]
    def py_RealStep(ps):
        for em in ps.emitters:
            ext = em.py_Step()
            if ext:
                ps.py_particles.extend(ext)
    def MoveInstance(ps):
        if ps.py_automatic_update:
            ps.py_RealStep()
    def StepKeyMouse(ps):
        pass
    def Draw(ps,xoff=0,yoff=0):
        if ps.py_automatic_draw:
            ps.py_RealDraw(xoff,yoff)
    def py_RealDraw(ps,xoff,yoff):
        plen = len(ps.py_particles)
        if plen:
            x = 0
            xoff = int(xoff)
            yoff = int(yoff)
            particles = ps.py_particles
            render = py_fasttint.CParticle_dec_ttl_and_render
            while x<plen:
                if render(particles[x],xoff,yoff)>0:
                    x += 1
                else:
                    del particles[x]
                    plen -= 1
    def RoomEnd(self):
        pass

class py_MinMaxIncWiggle(object):
    __slots__ = ('min','max','incr','wiggle')
    def __init__(mmiw, min=0, max=0, incr=0, wiggle=0):
        mmiw.min, mmiw.max, mmiw.incr, mmiw.wiggle = min,max,incr,wiggle

py_Particle = py_fasttint.CParticle_new
#py_Particle.Render = py_fasttint.particle_Render
class py_ParticleType(object):
    shape = None
    animate = False
    stretch = False
    random = False
    size = 1
    size_min = 1
    size_max = 1
    size_incr = 0
    size_wiggle = 0
    size_xscale = 1
    size_yscale = 1
    startcolor = (255,255,255)
    halfcolor = (255,255,255)
    endcolor = (255,255,255)
    startalpha = 1
    halfalpha = 1
    endalpha = 1
    additive = 0
    angle_min = 0
    angle_max = 0
    angle_incr = 0
    angle_wiggle = 0
    angle_relative = 0
    speed_min = 0
    speed_max = 0
    speed_incr = 0
    speed_wiggle = 0
    dir = py_MinMaxIncWiggle()
    life_min = 100
    life_max = 100
    # XXX how does death behave?  Can an arbitrary number of particle types
    # be emitted when one particle of death_type dies, or does one replace
    # another?
    grav_amount = 0
    grav_dir = None
    def __init__(pt):
        pt.cached_surfaces = dict()
    def py_SpawnAt(pt,x,y):
        newpart = py_Particle(pt,x,y)
        return newpart
    def py_DesiredTint(pt,pos):
        # XXX I'm not sure if this is the desired tint or exactly wrong for
        # the desired tint.
        if pos>0.5:
            return merge_color(pt.startcolor,pt.halfcolor,(pos-0.5)*2)
        else:
            return merge_color(pt.halfcolor,pt.endcolor,pos*2)
    def py_DesiredAlpha(pt,pos):
        if pos>0.5:
            pos = (pos-0.5)*2
            pos1 = 1-pos
            return int(((pt.startalpha*pos)+(pt.halfalpha*pos1))*255)
        else:
            pos = pos*2
            pos1 = 1 - pos
            return int(((pt.halfalpha*pos)+pt.endalpha*pos1)*255)
    def Regenerate(pt): #XXX kill?
        pass

class py_switch_context(object):
    # Used to enable a switch statement to work.
    val = None
    broke = True
    returned = False
    case_met = False

def py_verbose_call(func, *args):
    print "Calling",func,args
    ret = func(*args)
    print func,"Returned",ret
    return ret

def py_assignment_debug(varname,value):
    print "DEBUG: Set",varname,"to",value,". (self=",self,')'
    return value

def py_print(*args):
    # Just here for debugging.
    print args

def __color_to_int(rgba):
    if type(rgba) is py_GMVar:
        rgba = rgba[0]
    # XXX this may need to be endianness-specific
    #ret = (int(rgba[0])<<16)+(int(rgba[1])<<8)+int(rgba[2])
    ret = int(rgba[0])+(int(rgba[1])<<8)+(int(rgba[2])<<16)
    if len(rgba)==3:
        return ret
    return ret+(int(rgba[3])<<24)

__gm5_file = None # GM5 routines only support one open file at a time?
# Only one ini file can be open at a time.
py_current_ini_file = None
class __IniFile(object):
    def __init__(self,filename):
        self.f = renpy_glue.open_rw(filename)
        self.sections = dict()
        if self.f:
            lines = self.f.readlines()
            cursection = dict()
            for line in lines:
                if line:
                    while line.endswith("\n") or line.endswith("\r"):
                        line = line[:-1]
                    if not line:
                        continue
                    if line[0]=='[' and line[-1]==']':
                        cursection = dict()
                        self.sections[line[1:-1]] = cursection
                    else:
                        eq = line.index('=')
                        cursection[line[:eq]] = line[eq+1:]
        global py_current_ini_file
        if py_current_ini_file is not None:
            py_current_ini_file.Close()
        py_current_ini_file = self
    def WriteOut(ini):
        ini.f.seek(0)
        lines = []
        for section in ini.sections:
            lines.append('['+section+"]\n")
            for var in ini.sections[section]:
                lines.append(var+'='+str(ini.sections[section][var]+"\n"))
        ini.f.writelines(lines)
        ini.f.truncate()
    def Close(ini):
        ini.WriteOut()
        ini.f.close()
        global py_current_ini_file
        py_current_ini_file = None

# Blend mode constants.
bm_normal = 0
bm_add = 128            # pygame.BLEND_RGB_ADD
bm_subtract = 129       # pygame.BLEND_SUB
bm_max = 130            # pygame.BLEND_MAX

# Color constants.  In GM, these were ints; for my convenience, I'm using
# RGB tuples.
c_white = (255,255,255)
c_aqua=(0,255,255)
c_black=(0,0,0)
c_blue=(0,0,255)
c_dkgray = (64,64,64)
c_fuchsia = (255,0,255)
c_gray = (128,128,128)
c_green =(0,128,0)
c_lime=(0,255,0)
c_ltgray=(192,192,192)
c_maroon=(128,0,0)
c_navy=(0,0,128)
c_olive=(128,128,0)
c_purple=(128,0,128)
c_red=(255,0,0)
c_silver=(192,192,192)
c_teal=(0,128,128)
c_yellow=(255,255,0)

# ef_* constants set at the end.
py_effect_ptypes = dict()

ev_create = 0
ev_destroy = 1
ev_step = 2
ev_alarm = 3
ev_keyboard = 4
ev_mouse = 5
ev_collision = 6
ev_other = 7
ev_draw = 8
ev_keypress = 9
ev_keyrelease = 10

# XXX Mouse events unfinished
ev_left_press = 0
ev_right_press = None
ev_middle_button = None
ev_no_button = None
ev_left_press = 4
ev_right_press = None
ev_middle_press = None
ev_left_release = None
ev_right_release = None
ev_middle_release = None
ev_mouse_enter = None
ev_mouse_leave = None
ev_mouse_wheel_up = None
ev_mouse_wheel_down = None
ev_global_left_button = 50
ev_global_right_button = None
ev_global_left_press = 53
ev_global_right_press = 54
ev_global_middle_press = None
ev_global_left_release = None
ev_global_right_release = None
ev_global_middle_release = None
ev_joystick1_left = None
ev_joystick1_right = None
ev_joystick1_up = None
ev_joystick1_down = None
ev_joystick1_button1 = None
ev_joystick1_button2 = None
ev_joystick1_button3 = None
ev_joystick1_button4 = None
ev_joystick1_button5 = None
ev_joystick1_button6 = None
ev_joystick1_button7 = None
ev_joystick1_button8 = None
ev_joystick2_left = None
ev_joystick2_right = None
ev_joystick2_up = None
ev_joystick2_down = None
ev_joystick2_button1 = None
ev_joystick2_button2 = None
ev_joystick2_button3 = None
ev_joystick2_button4 = None
ev_joystick2_button5 = None
ev_joystick2_button6 = None
ev_joystick2_button7 = None
ev_joystick2_button8 = None

# XXX other events unfinished.
ev_user0  =  10
ev_user1  =  11
ev_user2  =  12
ev_user3  =  13
ev_user4  =  14
ev_user5  =  15
ev_user6  =  16
ev_user7  =  17
ev_user8  =  18
ev_user9  =  19
ev_user10 =  20
ev_user11 =  21
ev_user12 =  22
ev_user13 =  23
ev_user14 =  24
ev_user15 =  25

# fa_* are Font Alignment constants for draw_text-related aligns.
fa_left = 0
fa_center = 1 # Horizontal center
fa_right = 2
fa_top = 0
fa_middle = 1 # vertical center
fa_bottom = 2

mb_none = 0
mb_left = 1
mb_middle = 2
mb_right = 4
mb_any = 7
# Not sure if these should really _default_ to 0.
mouse_x=0
mouse_y=0
# XXX I'm not really sure how this should be handled.  An or, possibly?
mouse_button = mb_none
mouse_lastbutton = mb_none 

# XXX primitive-related constants
#pr_pointlist = 0
#pr_linelist = 1
#pr_linestrip = 2
#pr_trianglelist = 3
#pr_trianglestrip = 4
#pr_trianglefan = 5

# XXX I don't know what the values of these were in GM.  It may matter. 
# Somewhere.  They're particle-related constants.
ps_distr_linear = 0
ps_distr_gaussian = 1

ps_shape_rectangle = 0
ps_shape_ellipse = 1
ps_shape_diamond = 2
ps_shape_line = 3

# XXX impliment default particle types
pt_shape_pixel = None
pt_shape_disk = None
pt_shape_square = None
pt_shape_line = None
pt_shape_star = pygame.image.load('rt/star.png')
pt_shape_circle = pygame.image.load('rt/circle.png')
pt_shape_ring = None
pt_shape_sphere = None
pt_shape_flare = None
pt_shape_spark = pygame.image.load('rt/spark.png')
pt_shape_explosion = None
# Does scale2x leak a weakref in Ren'Py?
pt_shape_cloud = pygame.transform.scale2x(pygame.image.load('rt/cloud.png'))
pt_shape_smoke = None
pt_shape_snow = pygame.image.load('rt/snowflake.png')

# Keyboard constants.
py_translate_keys = dict()
py_translate_keys[pygame.K_LEFT] = vk_left = 37
py_translate_keys[pygame.K_UP] = vk_up = 38
py_translate_keys[pygame.K_RIGHT] = vk_right = 39
py_translate_keys[pygame.K_DOWN] = vk_down = 40
py_translate_keys[pygame.K_KP_ENTER] = vk_enter = pygame.K_RETURN
vk_escape = pygame.K_ESCAPE
vk_space = pygame.K_SPACE
# XXX shift, control, alt
vk_backspace = pygame.K_BACKSPACE
vk_tab = pygame.K_TAB
vk_home = pygame.K_HOME
vk_end = pygame.K_END
vk_delete = pygame.K_DELETE
vk_insert = pygame.K_INSERT
vk_pageup = pygame.K_PAGEUP
vk_pagedown = pygame.K_PAGEDOWN
vk_pause = pygame.K_PAUSE
vk_printscreen = pygame.K_PRINT
py_translate_keys[pygame.K_F1] = vk_f1 = 112
py_translate_keys[pygame.K_F2] = vk_f2 = 113
py_translate_keys[pygame.K_F3] = vk_f3 = 114
py_translate_keys[pygame.K_F4] = vk_f4 = 115
py_translate_keys[pygame.K_F5] = vk_f5 = 116
py_translate_keys[pygame.K_F6] = vk_f6 = 117
py_translate_keys[pygame.K_F7] = vk_f7 = 118
py_translate_keys[pygame.K_F8] = vk_f8 = 119
py_translate_keys[pygame.K_F9] = vk_f9 = 120
py_translate_keys[pygame.K_F10] = vk_f10 = 121
py_translate_keys[pygame.K_F11] = vk_f11 = 122
py_translate_keys[pygame.K_F12] = vk_f12 = 123
vk_numpad0 = pygame.K_KP0
vk_numpad1 = pygame.K_KP1
vk_numpad2 = pygame.K_KP2
vk_numpad3 = pygame.K_KP3
vk_numpad4 = pygame.K_KP4
vk_numpad5 = pygame.K_KP5
vk_numpad6 = pygame.K_KP6
vk_numpad7 = pygame.K_KP7
vk_numpad8 = pygame.K_KP8
vk_numpad9 = pygame.K_KP9
vk_multiply = pygame.K_KP_MULTIPLY
vk_divide = pygame.K_KP_DIVIDE
py_translate_keys[pygame.K_KP_PLUS] = vk_add = 107
py_translate_keys[pygame.K_KP_MINUS] = vk_subtract = 109
vk_decimal = pygame.K_KP_PERIOD
# Not sure how I should represent vk_nokey and vk_anykey
def __set_translate_letters():
    x = 65
    while x<91:
        py_translate_keys[x+32] = x
        x += 1
__set_translate_letters()

# Now we have the properties.  They're used by various API functions so all
# the text styling doesn't have to be passed in every single call.

class __DrawProperties(object):
    color = (255,255,255)
    font = None
    halign = None
    valign = None
    blend_mode = 0
    alpha = 1.0
py_draw_props = __DrawProperties()

class py_FontCacheEntry_OLD(object):
    def __init__(self,font):
        self.font = font
        self.cache = dict()
        self.cachesize = 100
    def render(self,line,antialias,color):
        if not self.cache.has_key((line,color)):
            if len(self.cache)>self.cachesize:
                self.cache = dict()
            self.cache[(line,color)] = self.font.render(line,antialias,color)
        return self.cache[(line,color)]
    def set_bold(font, bold):
        font.font.set_bold(bold)
    def set_italic(font, italic):
        font.font.set_italic(italic)
    def size(font,sub):
        return font.font.size(sub)
class __FontCacheDict(dict):
    def __getitem__(cache, f):
        key = (f.fontname, f.size, 0, f.italic)
        if not cache.has_key(key):
            if gm6.use_renpy:
                fn = renpy.exports.config.searchpath[0]+'/'+f.fontname
            else:
                fn = f.fontname
            try:
                cache[key] = font = pygame.font.Font(fn, f.size)
            except:
                print "Failed to read",f.fontname
            #font.set_bold(f.bold)
            font.set_italic(f.italic)
        return dict.__getitem__(cache,key)
        
__fontcache = __FontCacheDict()

class __Fontspec(object):
    def __init__(self,*args):
        self.name = args[0]
        self.size = int(args[1])
        self.color = args[2]
        flags = int(args[3])
        self.bold = flags&1
        self.italic = flags&2
    def Colored(self,color):
        flags = 0
        if self.bold:
            flags = 1
        if self.italic:
            flags |= 2
        return __Fontspec(self.name,self.size,color,flags)
class __MessageProperties(object):
    def __init__(self):
        self.background = None
        self.text_font = None
        self.button_font = None
        self.input_font = None
        self.button_font_hover_color = None
        self.w = 320
        self.h = 200
        self.button_spr = None
__message_props = __MessageProperties()

def __apply_text_valign(y,h):
    if py_draw_props.valign:
        if py_draw_props.valign==fa_middle:
            y -= int(h)>>1
        else:
            y -= h
    return int(y)
def __apply_text_halign(x,w):
    if py_draw_props.halign:
        if py_draw_props.halign==fa_center:
            x -= w>>1
        else:
            x -= w
    return int(x)
def __apply_text_alignment(x,y,w,h):
    return __apply_text_halign(x,w), __apply_text_valign(y,h)

# __string_unescape converts #\r\n in strings into linebreaks.
def __string_unescape(s):
    ret = []
    if type(s) is py_GMVar:
        s = s[0]
    s2 = ''
    x = 0
    while x<len(s):
        if s[x] in '#\015\012':
            ret.append(s2)
            s2 = ''
        elif s[x]=='\\' and x<len(s)-1 and s[x+1]=='#':
            s2 += '#'
        else:
            s2 += s[x]
        x += 1
    if s2 or ret:
        ret.append(s2)
    return ret

# Action API stuff
def action_draw_life_images(x,y,spr):
    x2 = x
    global lives
    i = lives
    if type(spr) in (int, float):
        spr = gm6.game.sprites[int(spr)]
    while i>0:
        draw_sprite(spr,-1,x2,y)
        x2 += spr.subimages[0].get_width()
        i-=1
    return

def action_move(direction,vel,*args):
    if type(direction) is py_GMVar:
        direction = direction[0]
    if type(vel) is py_GMVar:
        vel = vel[0]
    if args[0]:
        print "WARNING: Relative motion not yet implemented."
    global self
    if direction==100000:
        # Seems to be left
        self.SetHSpeed(-vel)
        self.SetVSpeed(0)
    elif direction==1000:
        # Seems to be right
        self.SetHSpeed(vel)
        self.SetVSpeed(0)
    else:
        print "action_move: Unhandled direction",direction,vel,args

def action_move_random(xsnap,ysnap):
    global self
    while 1:
        # Bad, really should make a grid instead.
        newx = int(py_random.random()*room.room_width)
        if xsnap:
            newx -= newx%xsnap
        newy = int(py_random.random()*room.room_height)
        if ysnap:
            newy -= newy%ysnap
        if place_free(newx,newy):
            self.x = newx
            self.y = newy
            self.SetBoundingBox()
            return

def action_set_alarm(frames,alarm,relative,*args):
    global self
    if type(frames) is py_GMVar:
        frames = frames[0]
    if relative:
        self.alarm[int(alarm)]+=frames
    else:
        self.alarm[int(alarm)]=frames

def action_set_life(newlives,relative):
    global lives
    hadlives = (lives>0)
    if relative:
        lives += newlives
    else:
        lives = newlives
    if hadlives and not lives:
        global room
        for instance in room.all_objects:
            pyod = instance.object_index.py_objectdata
            try:
                nml = pyod.py_NoMoreLivesEvent
            except:
                nml = None
            if nml:
                try:
                    gm6.RunActionGroupOn(nml,instance)
                except:
                    print "Debug: An exception occurred in action_set_life."
                    print sys.exc_type,sys.exc_value,sys.exc_traceback
                    import traceback
                    print traceback.extract_tb(sys.exc_traceback)

def action_sprite_color(*args):
    global self
    self.image_blend = __to_rgb(args[0])
    self.image_alpha = args[1]
    if args[2]!=1.0:
        print "Debug: action_sprite_color argument 2 != 1;",args[2],self

# Okay, now the actual GM api
def background_add(filename,transparent,smooth,preload):
    if type(filename) is py_GMVar:
        filename = filename[0]
    ret = py_BackgroundData(None)
    gm6.game.backgrounds.append(ret) #- XXX disabled because CKK leaks with this
    # enabled!  It may be a CKK bug, though.
    # XXX XXX where is this leaking?
    #ret.visible_when_game_starts = True XXX?
    if gm6.Background_SetFromFile(ret,filename) is None:
        return -1
    # XXX transparent not yet implemented.
    # XXX implement preload?
    return ret
def background_delete(bg):
    if type(bg) is py_GMVar:
        bg = bg[0]
    x = 0
    maxx = len(gm6.game.backgrounds)
    while x<maxx:
        if gm6.game.backgrounds[x] is bg:
            gm6.game.backgrounds[x] = None
        x += 1
    while gm6.game.backgrounds and gm6.game.backgrounds[-1] is None:
        gm6.game.backgrounds.pop()
    bg.surf = None
def background_exists(name):
    if type(name) is py_GMVar:
        name = name[0]
    if type(name)==py_fasttint.BackgroundData:
        return True
    return False
# XXX background_get_texture - in Advanced Drawing Functions
def background_replace(bg, filename, transparent, smooth, preload):
    # XXX args
    bg.SetFromFile(filename)
    return True
def background_set_alpha_from_background(target,mask):
    target.surf = __surf_apply_alpha(target.surf,mask.surf)
        #renpy.display.im.cache.get(
        #renpy.display.im.AlphaMask(
        #    renpy_glue.SurfImage(target.image_data), 
        #    renpy_glue.SurfImage(mask.image_data)))

def background_set_alpha_from_background_SURFARRAY(target,mask):
    target.surf = target.surf.convert_alpha()
    target_alpha = pygame.surfarray.pixels_alpha(target.surf)
    mask_vals = pygame.surfarray.pixels3d(mask.surf)
    a = 0
    bbound = len(mask_vals[0])
    abound = len(mask_vals)
    while a<abound:
        b = 0
        while b<bbound:
            target_alpha[a][b] = mask_vals[a][b][0]
            b += 1
        a += 1

# Floor and ceil are the same in Python and GM
ceil = math.ceil

# From http://www.bryceboe.com/2006/10/23/line-segment-intersection-algorithm/
class py_cp(object):
    __slots__ = ['x','y']
    def __init__(p,x,y):
        p.x = x
        p.y = y
def py_ccw(A,B,C):
    return (C.y-A.y)*(B.x-A.x) > (B.y-A.y)*(C.x-A.x)
def py_intersect(A,B,C,D):
    return py_ccw(A,C,D) != py_ccw(B,C,D) and py_ccw(A,B,C) != py_ccw(A,B,D)
# End linesegment intersection

def collision_line(x1,y1,x2,y2,obj,prec,notme):
    A = py_cp(x1,y1)
    B = py_cp(x2,y2)
    if prec:
        raise "Precise line collision not yet implemented."
    for instance in obj:
        t = instance.bbox_top
        l = instance.bbox_left
        tl = py_cp(l,t)
        r = instance.bbox_right
        tr = py_cp(r,t)
        if py_intersect(A,B,tl,tr):
            if instance is notme:
                next
            return instance
        b = instance.bbox_bottom
        bl = py_cp(b,l)
        if py_intersect(A,B,tl,bl):
            if instance is notme:
                next
            return instance
        br = py_cp(b,r)
        if py_intersect(A,B,tr,br):
            if instance is notme:
                next
            return instance
        elif py_intersect(A,B,br,bl):
            if instance is notme:
                next
            return instance
        elif x1>=l and x1<=r and y1>=t and y1<=b:
            if instance is not notme:
                return instance
    return False

def color_get_blue(c):
    return c[2]
def color_get_green(c):
    return c[1]
# color_get_hue XXX HSV
def color_get_red(c):
    return c[0]
# color_get_saturation XXX HSV
# color_get_value XXX HSV

def date_current_datetime():
    raise "Days since 12/30/1899 is a truly insane measurement."

def date_current_date():
    # January 1st, 1970
    # There were 17 leap days between 12/30/1899 and the Epoch.
    # (365*70)+17+2 (Dec 30-31, 1899)
    now = time.time()
    if time.localtime(now)[8]:
        now -= time.altzone
    else:
        now -= time.timezone
    ret = 25569 + (now/86400)
    return int(ret)

def date_current_time():
    t = time.localtime()
    year,mon,mday,hour,min,sec,wday,julianday,isdst = t
    return (float(hour)/24)+(float(min)/(24*60))+(float(sec)/86400)

def date_get_day(datetime):
    return time.gmtime((datetime-25569)*86400)[2]
def date_get_hour(datetime):
    try:
        return time.gmtime((datetime-25569)*86400)[3]
    except:
        # Exception handlers for 32-bit systems
        return time.gmtime(datetime*86400)[3]
def date_get_minute(datetime):
    try:
        return time.gmtime((datetime-25569)*86400)[4]
    except:
        # Exception handlers for 32-bit systems
        return time.gmtime(datetime*86400)[4]
def date_get_month(datetime):
    return time.gmtime((datetime-25569)*86400)[1]
def date_get_second(datetime):
    return time.gmtime((datetime-25569)*86400)[5]

def date_get_year(datetime):
    return time.gmtime((datetime-25569)*86400)[0]

def directory_create(dir):
    try:
        os.mkdir(dir)
    except:
        pass

def directory_exists(dirname):
    try:
        st = os.stat(dirname)
    except:
        # XXX which doesn't do what you intended if the dir exists but you
        # don't have perms.
        return False
    return stat.S_ISDIR(st.st_mode)

def display_get_frequency():
    #print "WARNING: display_get_frequency unimplimented."
    return 60
def display_get_height():
    #print "WARNING: display_get_height unimplimented."
    return 600
def display_get_width():
    #print "WARNING: display_get_width unimplimented."
    return 800
def display_set_all(w,h,freq,depth):
    #print "WARNING: display_set_all unwritten.",w,h,freq,depth
    return True

def distance_to_object(obj):
    x = self.x
    y = self.y
    nearest = None
    nearestdist = 1000000000
    for o in obj:
        h = math.hypot(x-o.x,y-o.y)
        if h<nearestdist:
            nearestdist = h
    if nearestdist<1000000000:
        return nearestdist
    else:
        return None
# draw functions
# XXX draw_arrow (x1,y1,x2,y2, size)
def draw_background(bg,x,y):
    if bg.surf is not None:
        __pygame_screen.blit(bg.surf,(x,y))
    # Don't bother drawing backgrounds with no surfaces.
# XXX draw_background_part(bg,left,top,width,height,x,y)
# XXX draw_background_stretched(bg,x,y,w,h)
# XXX draw_background_tiled(bg,x,y)
# XXX draw_button unimplimented
def draw_circle(x,y,r,outline):
    # XXX test outline.
    if outline:
        width=1
    else:
        width=0
    pygame.draw.circle(__pygame_screen,py_draw_props.color,(x,y),r,width=width)
def draw_circle_color(x,y,r,col1,col2,outline):
    raise "Colored circles not yet implimented."
def draw_clear(color):
    __pygame_screen.fill(py_draw_props.color)
def draw_clear_alpha(color,alpha):
    __pygame_screen.fill(py_draw_props.color+(int(alpha*255),))
def draw_ellipse(x1,y1,x2,y2,outline):
    if outline:
        width=1
    else:
        width=0
    pygame.draw.ellipse(__pygame_screen, py_draw_props.color,
        (x1,y1,x2-x1,y2-y1),width=width)
# XXX draw_ellipse_color not yet implimented.
def draw_get_alpha():
    return py_draw_props.alpha
def draw_get_color():
    return py_draw_props.color
def draw_getpixel(x,y):
    return __pygame_screen.get_at(x,y)
# XXX draw_healthbar unimplimented
def draw_line(x1,y1,x2,y2):
    pygame.draw.aaline(__pygame_screen,py_draw_props.color,(x1,y1),(x2,y2))
# XXX draw_line_color unimplimented.  Args are draw_line's + col1, col2;
# result is an interpolation between col1 and col2 over the length of the
# line.
def draw_point(x,y):
    __pygame_screen.set_at((x,y),py_draw_props.color)
def draw_point_color(x,y,color):
    __pygame_screen.set_at((x,y),color)
def draw_primitive_begin(kind):
    py_draw_props.primitive = __ComplexPrimitive(kind)
    py_draw_props.primitive.color = py_draw_props.color
    py_draw_props.primitive.alpha = py_draw_props.alpha
# draw_primitive_begin_texture - in Advanced Drawing Functions
def draw_primitive_end():
    raise "Should draw the Complex Primitive."
def draw_rectangle(x1,y1,x2,y2,outline):
    x1 = round(((x1-room.view_xview[room.view_current])*gm6.view_xscale)+room.view_xport[room.view_current])
    x2 = round(((x2-room.view_xview[room.view_current])*gm6.view_xscale)+room.view_xport[room.view_current])
    y1 = round(((y1-room.view_yview[room.view_current])*gm6.view_yscale)+room.view_yport[room.view_current])
    y2 = round(((y2-room.view_yview[room.view_current])*gm6.view_yscale)+room.view_yport[room.view_current])
    if py_draw_props.alpha!=1:
        if outline:
            raise "Can't do alpha'd outline rects yet."
        #surf = pygame.Surface((x2-x1,y2-y1))
        #surf.fill(py_draw_props.color)
        #surf.set_alpha(int(py_draw_props.alpha*255))
        #__pygame_screen.blit(surf,(x1,y1))
        # XXX Fasttint is currently a 5% speed savings over creating a new surface.
        py_fasttint.fill_translucent(__pygame_screen,x1,y1,x2,y2,__color_to_int(py_draw_props.color),int(py_draw_props.alpha*255))
    elif outline:
        pygame.draw.rect(__pygame_screen, py_draw_props.color, (x1,y1,x2-x1,y2-y1),1)
    else:
        # fill_opaque gives 0.82 secs for 1K iterations, while screen.fill()
        # gives 0.75.
        #py_fasttint.fill_opaque(__pygame_screen,x1,y1,x2,y2,__color_to_int(py_draw_props.color))
        __pygame_screen.fill(py_draw_props.color, (x1,y1,x2-x1,y2-y1))
draw_rectangle = py_fasttint.draw_rectangle

def draw_rectangle_color(x1,y1,x2,y2,col_tl,col_tr,col_br,col_bl,outline):
    col_tl = __color_to_int(__to_rgb(col_tl))
    col_tr = __color_to_int(__to_rgb(col_tr))
    col_br = __color_to_int(__to_rgb(col_br))
    col_bl = __color_to_int(__to_rgb(col_bl))
    y1 = int(y1-room.view_yview[room.view_current])
    y2 = int(y2-room.view_yview[room.view_current])
    x1 = int(x1-room.view_xview[room.view_current])
    x2 = int(x2-room.view_xview[room.view_current])
    py_fasttint.fast_draw_rectangle_color(x1,y1,x2,y2,col_tl,col_tr,col_br,col_bl,py_draw_props.alpha,int(outline))
    return

def __draw_rectangle_color_to(surf,x1,y1,x2,y2,col_tl,col_tr,col_br,col_bl,outline):
    # XXX this actually is drawing vertical opaque gradients backwards,
    # *I THINK* but
    # since it's only used internally this doesn't matter.  Still - don't
    # call it from new code.
    # Caller applies all view-related tranforms to the target coordinates
    if col_tl==col_tr and col_tr!=col_br and col_br==col_bl:
        rect_stride = 1.0/float(y2-y1)
        rect_pos = 0.0
        y = y1
        if outline:
            y += 1
            ymax = y2-1
            while y<=ymax:
                color = merge_color(col_tr, col_bl,rect_pos)
                surf.set_at((x1,y),color)
                surf.set_at((x2,y),color)
                y += 1
                rect_pos += rect_stride
            pygame.draw.line(surf, col_tr, (x1,y1), (x2,y1))
            pygame.draw.line(surf, col_br, (x1,y2), (x2,y2))
            return
        while y<=y2:
            pygame.draw.line(surf, merge_color(col_bl, col_tr, rect_pos), (x1,y), (x2,y))
            rect_pos += rect_stride
            y += 1
        return
    elif col_tl==col_bl and col_bl!=col_tr and col_tr==col_br:
        print 1035
        if outline: # XXX
            print "ERROR: hgradient outline not yet handled."
        x = x1
        rect_length = float(x2-x1)
        while x<=x2:
            linecolor = merge_color(col_tr,col_tl, (x2-x)/rect_length)
            pygame.draw.line(surf, linecolor, (x,y1),(x,y2))
            x += 1
        return
    print 1045
    if col_tl!=col_tr or col_tr!=col_br or col_br!=col_bl:
        raise "Warning: should interpolate!\n"+str((col_tl, col_tr, col_br, col_bl))
    # XXX need to draw.  Should probably use draw_line_color for the outlines.
    if outline:
        pygame.draw.rect(surf, col_tl, (x1,y1,x2-x1,y2-y1))
    else:
        surf.fill(col_tl, (x1,y1,x2-x1,y2-y1))

def draw_roundrect(x1,y1,x2,y2,outline=False):
    # Providing a default for outline enables both gm5 and gm6 code to run.
    x1 = int(x1)
    x2 = int(x2)
    y1 = int(y1)
    y2 = int(y2)
    __pygame_screen.set_at((x1+2,y1+1),py_draw_props.color)
    __pygame_screen.set_at((x1+1,y1+2),py_draw_props.color)
    __pygame_screen.set_at((x2-1,y1+2),py_draw_props.color)
    __pygame_screen.set_at((x2-2,y1+1),py_draw_props.color)
    __pygame_screen.set_at((x1+2,y2-1),py_draw_props.color)
    __pygame_screen.set_at((x1+1,y2-2),py_draw_props.color)
    __pygame_screen.set_at((x2-2,y2-1),py_draw_props.color)
    __pygame_screen.set_at((x2-1,y2-2),py_draw_props.color)
    if x2-x1>2 and y2-y1>2 and not outline:
        __pygame_screen.fill(py_draw_props.color,(x1+1,y1+1,(x2-x1)-2,(y2-y1)-2))
        __pygame_screen.set_at((x1+2,y1+2),py_draw_props.color)
        __pygame_screen.set_at((x2-2,y1+2),py_draw_props.color)
        __pygame_screen.set_at((x1+2,y2-2),py_draw_props.color)
        __pygame_screen.set_at((x2-2,y2-2),py_draw_props.color)
    pygame.draw.line(__pygame_screen,py_draw_props.color,(x1+3,y1),(x2-3,y1))
    pygame.draw.line(__pygame_screen,py_draw_props.color,(x1+3,y2),(x2-3,y2))
    pygame.draw.line(__pygame_screen,py_draw_props.color,(x1,y1+3),(x1,y2-3))
    pygame.draw.line(__pygame_screen,py_draw_props.color,(x2,y1+3),(x2,y2-3))
   
# XXX draw_roundrect unimplimented.  Need to test some roundrects in GM
# to determine the algorythm used.
# XXX draw_roundrect_color unimplimented.  col1, col2 = middle, boundary

def draw_set_alpha(alpha):
    # 0.0 - 1.0
    py_draw_props.alpha = alpha
def draw_set_blend_mode(mode):
    if type(mode) is py_GMVar:
        mode = mode[0]
    py_draw_props.blend_mode = mode
# XXX draw_set_blend_mode_ext - in Advanced Drawing Functions
def draw_set_color(rgb):
    py_draw_props.color = __to_rgb(rgb)
def draw_set_halign(halign):
    py_draw_props.halign = halign
def draw_set_font(font):
    py_draw_props.font = font
def draw_set_valign(valign):
    py_draw_props.valign = valign

def draw_sprite(sprite, subimg, x, y):
    if self:
        draw_sprite_ext(sprite, subimg, x, y, self.image_xscale, self.image_yscale, self.image_angle, self.image_blend, self.image_alpha)
    else:
        draw_sprite_ext(sprite, subimg, x, y, 1, 1, 0, (255,255,255), 1.0)

def __blended_blit(surf, xy, alpha):
    # was 1.3422592435722465e-06/call local; making it so that there is only
    # one 'if alpha!=1' brings it down to 1.1922e-06/call
    if not py_draw_props.blend_mode:
        if alpha!=1:
            oalpha = surf.get_alpha()
            surf.set_alpha(int(alpha*255))
            __pygame_screen.blit(surf,xy)
            surf.set_alpha(oalpha)
        else:
            __pygame_screen.blit(surf,xy)
    elif py_draw_props.blend_mode == bm_add:
        # Workaround for add and alpha not playing nice together
        black = pygame.Surface(surf.get_size())
        black.blit(surf,(0,0))
        if alpha!=1:
            black.set_alpha(int(alpha*255))
            black2 = pygame.Surface(surf.get_size())
            black2.blit(black,(0,0))
            __pygame_screen.blit(black2, xy, None, pygame.BLEND_RGB_ADD)
            return
        __pygame_screen.blit(black, xy, None, pygame.BLEND_RGB_ADD)
    elif py_draw_props.blend_mode == bm_subtract:
        print "Warning: GM subtract is actually invert-and-multiply.  Your game will run faster if you pre-invert the image and use code munging to replace bm_subtract with pygame.BLEND_RGB_MULT."
        
        __pygame_screen.blit(py_pills.invert(surf), xy, None, pygame.BLEND_RGB_MULT)
    elif py_draw_props.blend_mode == bm_max:
        print "XXX bm_max untested."
        __pygame_screen.blit(surf, xy, None, pygame.BLEND_RGB_MAX)
    else:
        __pygame_screen.blit(surf, xy, None, py_draw_props.blend_mode)

def draw_sprite_ext(spr, subimg, x, y, xscale, yscale, clockwise, color, alpha):
    if type(spr) is py_GMVar:
        spr = spr[0]
    if spr==-1:
        return
    if not alpha:
        return
    clockwise = abs(clockwise) % 360
    x = x-room.view_xview[room.view_current]
    y = y-room.view_yview[room.view_current]
    # May not work well if draw_sprite_ext somehow winds up being called with
    # a number
    #if type(spr)==float or type(spr)==int:
    #    if spr==-1:
    #        return
    #    print "XXX sprite referenced by number",spr
    #    spr = gm6.game.sprites[spr]
    xorig = spr.xorigin
    yorig = spr.yorigin
    if subimg != -1:
        if subimg<len(spr.subimages):
            srcimg = spr.subimages[int(subimg)]
        elif not spr.subimages:
            print "Warning: empty sprite",subimg,spr
            return
        else:
            print "Warning: invalid subimage",subimage,spr.subimages
            srcimg = spr.subimages[int(subimg)%len(spr.subimages)]
    elif self.image_index>=len(spr.subimages):
        srcimg = spr.subimages[int(self.image_index)%len(spr.subimages)]
    else:
        srcimg = spr.subimages[int(self.image_index)]
    if yscale<0:
        if xscale<0:
            srcimg = py_fasttint.flip_vh(srcimg)
            xscale = -xscale
        else:
            srcimg = py_fasttint.flip_v(srcimg)
        yscale = -yscale
    elif xscale<0:
        srcimg = py_fasttint.flip_h(srcimg)
        xscale = -xscale
    if clockwise:
        surf = py_fasttint.rotoscale(srcimg,xscale*gm6.view_xscale,yscale*gm6.view_yscale,clockwise)
        if spr.transparent:
            surf.set_colorkey(srcimg.get_at((0,srcimg.get_height()-1)))
        clockwise_r = math.radians(clockwise)
        vectx = (srcimg.get_width())-xorig
        vecty = (srcimg.get_height())-yorig
        if clockwise<=90:
            # Left ---- Top
            # Bottom--- Right
            # If anchor is 0,0
            #   rotx should be 0,0
            #   roty should range from 0 to width
            # If anchor is 0,24
            #   rotx should range from 0,0 to 0,24
            #   roty should range from 0 to width
            # Rotx SHOULD be fairly close to 0 in all of these
            #rotx = ((vectx*math.cos(clockwise_r))+(vecty*math.sin(clockwise_r)))
            #roty = ((vecty*math.cos(clockwise_r))+(vectx*math.sin(clockwise_r)))
            rotx = (xorig*math.cos(clockwise_r))+((yorig*math.sin(clockwise_r)))
            roty = (
                (vectx*math.sin(clockwise_r))+
                (yorig*math.cos(clockwise_r))
                )
        elif clockwise<=180:
            rotx = -((vectx*math.cos(clockwise_r))-(yorig*math.sin(clockwise_r)))
            roty = -((yorig*math.cos(clockwise_r))+-(vectx*math.sin(clockwise_r)))
        elif clockwise<=270:
            rotx = -((vectx*math.cos(clockwise_r))+(yorig*math.sin(clockwise_r)))
            roty = -((yorig*math.cos(clockwise_r))+(xorig*math.sin(clockwise_r)))
        else:
            rotx = ((xorig*math.cos(clockwise_r))-(yorig*math.sin(clockwise_r)))
            roty = ((yorig*math.cos(clockwise_r))-(xorig*math.sin(clockwise_r)))
        #print (xorig,yorig),'/_',clockwise,'->',(rotx,roty),surf.get_size()
        xorig,yorig=rotx,roty
    elif xscale!=1 or yscale!=1:
        surf = py_fasttint.scale(srcimg,xscale*gm6.view_xscale,yscale*gm6.view_yscale)
        x *= gm6.view_xscale
        y *= gm6.view_yscale
        xorig *= xscale
        yorig *= yscale
    else:
        surf = srcimg
    if color!=c_white:
        surf = surf.copy()
        if type(color) is py_GMVar:
            color = color[0]
        if type(color) is tuple:
            if len(color)==3:
                colalpha = 255
            else:
                colalpha = color[3]
            surf.fill(pygame.Color(int(color[0]),int(color[1]),int(color[2]),colalpha),None,pygame.BLEND_RGBA_MULT)
        else:
            try:
                surf.fill(pygame.Color(int(color),None,pygame.BLEND_RGBA_MULT))
            except:
                pass
    if gm6.view_xscale!=1 or gm6.view_yscale!=1:
        surf = py_fasttint.scale(surf,gm6.view_xscale,gm6.view_yscale)
        x *= gm6.view_xscale
        y *= gm6.view_yscale
        xorig *= gm6.view_xscale
        yorig *= gm6.view_yscale
    __blended_blit(surf, (round(x+room.view_xport[room.view_current]-xorig), round(y+room.view_yport[room.view_current]-yorig)), alpha)

def draw_sprite_ext_OLD(spr, subimg, x, y, xscale, yscale, clockwise, color, alpha):
    if type(spr) is py_GMVar:
        spr = spr[0]
    if spr==-1:
        return
    if not alpha:
        return
    clockwise = abs(clockwise) % 360
    x = x-room.view_xview[room.view_current]
    y = y-room.view_yview[room.view_current]
    # May not work well if draw_sprite_ext somehow winds up being called with
    # a number
    #if type(spr)==float or type(spr)==int:
    #    if spr==-1:
    #        return
    #    print "XXX sprite referenced by number",spr
    #    spr = gm6.game.sprites[spr]
    xorig = spr.xorigin
    yorig = spr.yorigin
    if subimg != -1:
        if subimg<len(spr.subimages):
            srcimg = spr.subimages[int(subimg)]
        elif not spr.subimages:
            print "Warning: empty sprite",subimg,spr
            return
        else:
            print "Warning: invalid subimage",subimage,spr.subimages
            srcimg = spr.subimages[int(subimg)%len(spr.subimages)]
    elif self.image_index>=len(spr.subimages):
        srcimg = spr.subimages[int(self.image_index)%len(spr.subimages)]
    else:
        srcimg = spr.subimages[int(self.image_index)]
    if yscale<0:
        if xscale<0:
            srcimg = py_fasttint.flip_vh(srcimg)
            xscale = -xscale
        else:
            srcimg = py_fasttint.flip_v(srcimg)
        yscale = -yscale
    elif xscale<0:
        srcimg = py_fasttint.flip_h(srcimg)
        xscale = -xscale
    if clockwise:
        #srcimg = th
        orig_w,orig_h = srcimg.get_size()
        # XXX should honour sprite smoothing directive
        surf = py_fasttint.rotoscale(srcimg,xscale*gm6.view_xscale,yscale*gm6.view_yscale,clockwise)
        offset_dist = point_distance(xorig,yorig,srcimg.get_width(),srcimg.get_height())
        offset_dire = point_direction(xorig,yorig,srcimg.get_width(),srcimg.get_height())
        # Combined size:
        # aggregate height = new height + (oldheight-rotatedheighty)
        # aggregate width = new width + (oldwidth-rotatedwhx)
        rot = euclid.Matrix3.new_rotate(math.radians(clockwise))
        hdiff = srcimg.get_height()-(rot*euclid.Vector2(0,srcimg.get_height())).y
        aggh = surf.get_height()+hdiff-srcimg.get_height()
        if clockwise<=90:
            yorig += aggh
        elif clockwise<=180:
            yorig += surf.get_height()
            xorig -= (rot*euclid.Vector2(srcimg.get_width())).x
        elif clockwise<=270:
            #yorig += srcimg.get_height()
            xorig += surf.get_width()
        else:
            yorig -= (rot*euclid.Vector2(srcimg.get_height())).y
            pass
            # Ignore yorig for >270 degrees.
        #xorig -= v.x
    elif xscale!=1 or yscale!=1:
        surf = py_fasttint.scale(srcimg,xscale*gm6.view_xscale,yscale*gm6.view_yscale)
        x *= gm6.view_xscale
        y *= gm6.view_yscale
        xorig *= xscale
        yorig *= yscale
    elif color!=c_white:
        surf = srcimg.copy()
        surf.fill(color,None,pygame.BLEND_RGBA_MULT)
    else:
        surf = srcimg
    if gm6.view_xscale!=1 or gm6.view_yscale!=1:
        surf = py_fasttint.scale(surf,gm6.view_xscale,gm6.view_yscale)
        x *= gm6.view_xscale
        y *= gm6.view_yscale
        xorig *= gm6.view_xscale
        yorig *= gm6.view_yscale
    __blended_blit(surf, (round(x+room.view_xport[room.view_current]-xorig), round(y+room.view_yport[room.view_current]-yorig)), alpha)

def draw_sprite_part(spr, idx, left, top, w, h, x, y):
    if idx!=-1:
        surf = spr.subimages[int(idx)]
    else:
        if self.image_index>=len(spr.subimages):
            print "WARNING: Sprite overflow."
            idx = int(self.image_index)%len(spr.subimages)
        else:
            idx = int(self.image_index)
        surf = spr.subimages[idx]
    __pygame_screen.blit(surf,(int(x),int(y)),(int(left),int(top),int(w),int(h)))

def __draw_text_to(xpos,ypos,s,dest):
    if type(xpos) is py_GMVar:
        xpos = xpos[0]
    if type(ypos) is py_GMVar:
        ypos = ypos[0]
    # XXX is this mimicing GM alignment behavior?
    if type(s)!=py_string:
        s = string(s)
    s2 = __string_unescape(s)
    surfaces = []
    totalheight = 0
    widest = 0
    for line in s2:
        surf = __fontcache[py_draw_props.font].render(line,1,py_draw_props.color)
        totalheight += surf.get_height()
        widest = max(surf.get_width(),widest)
        surfaces.append(surf)
    x, starty = __apply_text_alignment(xpos,ypos,widest,totalheight)
    y = starty
    for surf in surfaces:
        dest.blit(surf,(x,y))
        y += surf.get_height()
    if py_draw_props.font.bold:
        y = starty
        for surf in surfaces:
            dest.blit(surf,(x+1,y))
            y += surf.get_height()

def draw_text(xpos,ypos,s):
    if type(xpos) is py_GMVar:
        xpos = xpos[0]
    if type(ypos) is py_GMVar:
        ypos = ypos[0]
    # XXX is this mimicing GM alignment behavior?
    if type(s)!=py_string:
        s = string(s)
    xpos -= room.view_xview[room.view_current]
    ypos -= room.view_yview[room.view_current]
    s2 = __string_unescape(s)
    surfaces = []
    totalheight = 0
    widest = 0
    for line in s2:
        surf = __fontcache[py_draw_props.font].render(line,1,py_draw_props.color)
        totalheight += surf.get_height()
        widest = max(surf.get_width(),widest)
        surfaces.append(surf)
    x, starty = __apply_text_alignment(xpos,ypos,widest,totalheight)
    y = starty
    for surf in surfaces:
        __pygame_screen.blit(surf,(x,y))
        y += surf.get_height()
    if py_draw_props.font.bold:
        y = starty
        for surf in surfaces:
            __pygame_screen.blit(surf,(x+1,y))
            y += surf.get_height()

def draw_text_color(x,y,s,c1,c2,c3,c4,alpha):
    if type(s) is py_GMVar:
        s = s[0]
    if type(c1) is py_GMVar:
        c1 = c1[0]
    if type(c2) is py_GMVar:
        c2 = c2[0]
    if type(c3) is py_GMVar:
        c3 = c3[0]
    if type(c4) is py_GMVar:
        c4 = c4[0]
    if type(x) is py_GMVar:
        x = x[0]
    if type(y) is py_GMVar:
        y = y[0]
    x -= room.view_xview[room.view_current]
    y -= room.view_yview[room.view_current]
    if c1!=c2 or c2!=c3 or c3!=c4:
        grad_c1 = c1
        c1 = (255,255,255)
    else:
        grad_c1 = ()
    surf = __fontcache[py_draw_props.font].render(s,1,(c1[0]*alpha,c1[1]*alpha,c1[1]*alpha))
    if grad_c1:
        color = pygame.surface.Surface(surf.get_size())
        __draw_rectangle_color_to(color, 
            0, 0, surf.get_width(), surf.get_height(),
            c3, c4,
            grad_c1, c2, 
            0)
        surf.blit(color,(0,0),None,pygame.BLEND_RGB_MIN)
    #(py_draw_props.color[0]*alpha, py_draw_props.color[1]*alpha, py_draw_props.color[2]*alpha))
    align = __apply_text_alignment(x,y,surf.get_width(),surf.get_height())
    __pygame_screen.blit(surf, align)

def __render_text(s, w):
    # XXX should I count trailing linebreaks?
    # XXX also, string_height_ext needs to use this algorythm
    if type(s)!=py_string:
        s = string(s)
    lines = []
    i = 0
    linestart = 0
    while i<len(s):
        if s[i] in '#\015\012':
            lines.append(s[linestart:i])
            linestart = i+1
        i += 1
    if linestart<len(s):
        lines.append(s[linestart:])
    i = 0
    surfaces = []
    font = __fontcache[py_draw_props.font]
    spacewidth = 0
    while i<len(lines):
        surf = font.render(lines[i], 1, py_draw_props.color)
        #if py_draw_props.font.bold:
        #    surf2 = pygame.surface.Surface((surf.get_width()+1,surf.get_height()))
        #    surf2.blit(surf,(0,0))
        #    surf2.blit(surf,(1,0))
        #    surf = surf2
        if surf.get_width()>w: # XXX >w or >= w?
            surfw = surf.get_width()
            words = []
            lastspace = 0
            sublinstart = 0
            linewidth = 0
            j = 0
            while j<len(lines[i]):
                if lines[i][j]==' ':
                    surf = font.render(lines[i][sublinstart:j],1,py_draw_props.color)
                    if surf.get_width()>w:
                        if lastspace-sublinstart>0:
                            surf = font.render(lines[i][sublinstart:lastspace],
                                1, py_draw_props.color)
                            surfaces.append(surf)
                            sublinstart = lastspace+1
                            lastspace = j
                        else:
                            raise "Cannot wrap this text."
                            # XXX what does GM do when there are no spaces?
                    elif surf.get_width()<w:
                        # XXX may go boom wrapping a double-spaced text?
                        lastspace = j
                    else:
                        surfaces.append(surf)
                        lastspace = j
                        sublinstart = j+1
                j += 1
            if sublinstart<len(lines[i]):
                surf = font.render(lines[i][sublinstart:],
                    1, py_draw_props.color)
                if surf.get_width()>w:
                    if lastspace-sublinstart>0:
                        surf = font.render(lines[i][sublinstart:lastspace],
                            1, py_draw_props.color)
                        surfaces.append(surf)
                        sublinstart = lastspace+1
                        surf = font.render(lines[i][sublinstart:],1,py_draw_props.color)
                    else:
                        raise "Cannot wrap this text."
                        # XXX what does GM do when there are no spaces?
                surfaces.append(surf)
        else:
            surfaces.append(font.render(lines[i],
                1, py_draw_props.color))
        i += 1
    if py_draw_props.font.bold:
        for surf in surfaces:
            surf.blit(surf.copy(),(1,0))
    return surfaces


def draw_text_ext(x,y,s,sep,w):
    if type(s) is py_GMVar:
        s = s[0]
    if s==0:
        # XXX again, does draw_text_ext behave differently 
        return
    if w==-1:
        w = 1000000000 # This is a lie, but anyone making billion-pixel text
        # is going to run out of memory.
    surfaces = __render_text(s,w)
    widest = 0
    if sep==-1:
        total_height = 0
        for surf in surfaces:
            total_height+=surf.get_height()
            widest = max(surf.get_width(),widest)
    else:
        for surf in surfaces:
            widest = max(surf.get_width(),widest)
        total_height = len(surfaces)*sep
    y = __apply_text_valign(y, total_height)
    x -= room.view_xview[room.view_current]
    y -= room.view_yview[room.view_current]
    for surf in surfaces:
        __pygame_screen.blit(surf,(__apply_text_halign(x,widest),y))
        if sep==-1:
            y += surf.get_height()
        else:
            y += sep

def draw_triangle(x1,y1,x2,y2,x3,y3,outline):
    # XXX test.
    if outline:
        width=1
    else:
        width=0
    pygame.draw.polygon(__pygame_screen,py_draw_props.color,
        ((x1,y1),(x2,y2),(x3,y3)),width)
# XXX draw_triangle_color unimplimented
def draw_vertex(x,y):
    py_draw_props.primitive.vertices.append((x,y,py_draw_props.primitive.color,py_draw_props.primitive.alpha))
def draw_vertex_color(x,y,color,alpha):
    # XXX test.
    py_draw_props.primitive.vertices.append((x,y,color,alpha))
    py_draw_props.primitive.color = color
    py_draw_props.primitive.alpha = alpha
# XXX draw_vertex_texture
# XXX draw_vertex_texture_color

def py_tint_surface(surf,color):
    print "XXX need to impliment surface tints.",color
    return surf

def py_effect_controllers_create():
    global py_effect_controller_above
    py_effect_controller_above = py_ParticleSystem()
    py_effect_controller_above.depth = -100000
    global py_effect_controller_below
    py_effect_controller_below = py_ParticleSystem()
    py_effect_controller_below.depth = 100000

def py_effect_create_low(kind, x, y, size, color):
    if kind is ef_spark:
        if size==0:
            size = 0.5
        else:
            print "Unexpected ef_spark size",size
    elif kind is ef_star:
        if size==0:
            size = 0.4
        else:
            print "Unexpected ef_star size.",size
    elif kind is ef_cloud:
        if size!=2:
            print "Unexpected ef_cloud size."
            return
        else:
            size = 1.0
        if (not gm6.debug_fps_skippedframes) or int(py_ef_cloud_beancounter)<=3:
            ret = py_fasttint.CParticle_new_size_color(kind,x,y,size,__color_to_int(color))
            py_fasttint.CParticle_attach_beancounter(ret,py_ef_cloud_beancounter);
            return ret;
        return None
    else:
        print kind,size,color,1456
        size += 1.0
    return py_fasttint.CParticle_new_size_color(kind,x,y,size,__color_to_int(color))

def effect_clear():
    py_effect_controller_above.py_particles = []
    py_effect_controller_below.py_particles = []

def effect_create_above(kind,x,y,size,color):
    part = py_effect_create_low(kind, x, y, size, color)
    if part:
        py_effect_controller_above.py_particles.append(part)

def effect_create_below(kind,x,y,size,color):
    part = py_effect_create_low(kind, x, y, size, color)
    if part:
        py_effect_controller_below.py_particles.append(part)

def event_perform(evtype, numb):
    if evtype==ev_keypress:
        # XXX Should this be self or room?
        self.HandleKeypress(numb)
    elif evtype==ev_mouse:
        if numb is None:
            raise "Unexpected event subtype."
        elif numb == ev_left_press:
            if self.object_index.py_objectdata.py_LocalLeftPressedActionGroup:
                gm6.RunActionGroup(self.object_index.py_objectdata.py_LocalLeftPressedActionGroup)
        elif numb == ev_global_left_press:
            if self.object_index.py_objectdata.py_GlobLeftPressedActionGroup:
                gm6.RunActionGroup(self.object_index.py_objectdata.py_GlobLeftPressedActionGroup)
        elif numb == ev_global_left_button:
            if self.object_index.py_objectdata.py_GlobLeftButtonActionGroup:
                gm6.RunActionGroup(self.object_index.py_objectdata.py_GlobLeftButtonActionGroup)
        else:
            raise "Unwritten mouse event numb "+str(numb)
    elif evtype==ev_other:
        if numb>=10 and numb<=25:
            grp = self.object_index.py_objectdata.py_UserDefinedEvents[numb-10]
            if grp is not None:
                gm6.RunActionGroup(grp)
        else:
            raise "Unwritten event subtype for event_perform(ev_other)"
    else:
        raise "Cannot event_perform("+str(evtype)+','+str(numb)+')'

def execute_shell(s,flag):
    if type(s) is py_GMVar:
        s = s[0]
    if s.startswith('http://') or s.startswith('https://'):
        import webbrowser
        urlopener = webbrowser.get()
        try:
            if urlopener.args[0]=='"%s"':
                urlopener.args[0] = '%s'
        except:
            pass
        urlopener.open(s);
    else:
        print "Unimplimented type for execute_shell.\n"

def execute_string(s):
    if type(s) is py_GMVar:
        s = s[0]
    if type(s) in (int,float) and s==0:
        return
    py = gm6.tree_to_python(gm6.gml2syntaxtree(s))
    exec py in globals()

def file_bin_close(f):
    if type(f) is py_GMVar:
        f = f[0]
    f.close()

def file_bin_open(name, mode):
    if type(name) is py_GMVar:
        name = name[0]
    if mode==0:
        return renpy_glue.open_r(name)
    elif mode==1:
        return renpy_glue.open_w(name)
    elif mode==2:
        return renpy_glue.open_rw(name)
    else:
        raise "Invalid mode passed to file_bin_open."

def file_bin_position(f):
    if type(f) is py_GMVar:
        f = f[0]
    return f.tell()

def file_bin_rewrite(f):
    raise "file_bin_rewrite() unimplimented"

def file_bin_seek(f,pos):
    if type(f) is py_GMVar:
        f = f[0]
    f.seek(pos,0)

def file_bin_size(f):
    if type(f) is py_GMVar:
        f = f[0]
    opos = f.tell()
    f.seek(0,2)
    ret = f.tell()
    f.seek(opos,0)
    return ret

def file_bin_write_byte(f,b):
    if type(f) is py_GMVar:
        f = f[0]
    f.write(chr(b))

def file_bin_read_byte(f):
    if type(f) is py_GMVar:
        f = f[0]
    return ord(f.read(1))


# DOS file_attributes will not really make sense on posix.
def file_close():
    global __gm5_file
    __gm5_file.close()
    __gm5_file = None
def file_delete(fn):
    try:
        os.unlink(fn)
    except:
        print "Warning: unable to delete file",fn,"- this may indicate a bug in your GML."
def file_exists(fn):
    if type(fn) is py_GMVar:
        fn = fn[0]
    try:
        renpy_glue.open_r(fn).close()
    except:
        return False
    return True
def py_pat_to_regex(pat):
    regex = '^'
    for c in pat:
        if c in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789':
            regex += c
        elif c=='.':
            regex += '\\.'
        elif c=='*':
            regex += '(.*?)'
        else:
            regex += '\\'+c
    return regex+'$'
def file_find_close():
    global py_file_find_list
    py_file_find_list = []
    global py_file_find_pos
    py_file_find_pos = 0
def file_find_first(mask, attr):
    seppos = mask.rfind(os.sep)
    if os.altsep is not None:
        sepaltpos = mask.rfind(os.altsep)
        if sepaltpos>seppos:
            seppos = sepaltpos
    if seppos >= 0:
        filelist = renpy_glue.listdir(mask[:seppos])
    else:
        filelist = renpy_glue.listdir('.')
    global py_file_find_list
    if seppos==len(mask)-1:
        py_file_find_list = filelist
        return
    py_file_find_list = []
    pat = py_pat_to_regex(mask[seppos+1:])
    for file in filelist:
        if re.match(pat,file):
            py_file_find_list.append(file)
    global py_file_find_pos
    if py_file_find_list:
        py_file_find_pos = 1
        return py_file_find_list[0]
    py_file_find_pos = 0
    return ''
def file_find_next():
    global py_file_find_pos
    if py_file_find_pos<len(py_file_find_list):
        ret = py_file_find_list[py_file_find_pos]
        py_file_find_pos += 1
        return ret
    return ''
def file_open_read(fn):
    global __gm5_file
    __gm5_file = renpy_glue.open_r(fn)
def file_read_real():
    line = __gm5_file.readline()
    back = 0
    x = len(line)
    while x and line[x-1] in "\015\012":
        back += 1
        x -= 1
    __gm5_file.seek(__gm5_file.tell()-back)
    return float(line)
def file_read_string():
    line = __gm5_file.readline()
    back = 0
    x = len(line)
    while x and line[x-1] in "\015\012":
        back += 1
        x -= 1
    __gm5_file.seek(__gm5_file.tell()-back)
    return line[:x]
def file_readln():
    ret = __gm5_file.readline()
def file_text_close(f):
    f.close()
def file_text_eof():
    curpos = f.tell()
    f.seek(0,2)
    endpos = f.tell()
    if endpos==curpos:
        return True
    f.seek(curpos)
    return False
def file_text_open_append(filename):
    if type(filename) is py_GMVar:
        return renpy_glue.open_a(filename[0])
    return renpy_glue.open_a(filename)
def file_text_open_read(filename):
    if type(filename) is py_GMVar:
        filename = filename[0]
    return renpy_glue.open_r(filename)
def file_text_open_write(filename):
    if type(filename) is py_GMVar:
        return renpy_glue.open_w(filename[0])
    return renpy_glue.open_w(filename)
def file_text_read_real(f):
    return float(file_text_read_string(f))
def file_text_read_string(f):
    line = f.readline()
    back = 0
    x = len(line)
    while x and line[x-1] in "\015\012":
        back += 1
        x -= 1
    f.seek(f.tell()-back)
    return line[:x]
def file_text_readln(f):
    f.readline()
def file_text_write_real(f,x):
    if x==int(x):
        f.write(str(int(x)))
        return
    f.write(str(float(x)))
def file_text_write_string(f,s):
    if type(s) is py_GMVar:
        s = s[0]
    if type(s) in (float,int) and s==0:
        return
    f.write(s)
def file_text_writeln(f):
    f.write("\015\012")

def filename_path(filename):
    if type(filename) is py_GMVar:
        filename = filename[0]
    # XXX is this anything resembling what it's supposed to be?
    lastslash = filename.rfind(os.sep)
    if os.altsep is not None:
        lastslash = max(lastslash,filename.rfind(os.sep))
    if lastslash>=0:
        return filename[:lastslash+1]
    return './'

# Floor and ceil are the same in Python and GM.
floor = math.floor

def game_end():
    raise py_GameEndException

def game_restart():
    print "Warning: game_restart doesn't free up particle types."
    print "Was in room",room
    for tmproom in gm6.game.rooms:
        if tmproom is not None:
            tmproom.deactivated_objects = py_fasttint.InstanceList()
            tmproom.all_objects = py_fasttint.InstanceList()
            tmproom.stepped_objects = py_fasttint.InstanceList()
            tmproom.collisioned_objects = py_fasttint.InstanceList()
            tmproom.alarmed_objects = py_fasttint.InstanceList()
    for obj in gm6.game.objects:
        if obj is not None:
            obj.instances = []
    sound_stop_all()
    py_effect_controllers_create()
    global py_game_restart
    py_game_restart = True
    room_goto(gm6.game.execrooms[0])

def get_save_filename(filter, fname):
    filter = filter.split('|')
    global keyboard_string
    if type(fname) is py_GMVar:
        fname = fname[0]
    files = py_glob.glob(filter[1])
    return renpy_glue.get_save_filename(files,fname)

__highscores = [] # An array of score, name tuples.
__highscore_background = None
__highscore_border = None
__highscore_caption = "High score list"
__highscore_escape_prompt = 'Press escape'
__highscore_nobody = 'nobody'

def highscore_add(name,score):
    global __highscores
    i = len(__highscores)
    while i:
        i-=1
        if __highscores[i][0]<score:
            __highscores.insert(i,(score,name))
            if len(__highscores)>10:
                __highscores = __highscores[:10]
            return
    __highscores.append((score,name))

def highscore_clear():
    global __highscores
    __highscores = []

def highscore_name(place):
    if place>=len(__highscores):
        if gm6.game.default_undef_is_zero:
            return 0
    return __highscores[int(place)][0]

def highscore_set_background(bgidx):
    if type(bgidx) is py_GMVar:
        bgidx = bgidx[0]
    global __highscore_background
    __highscore_background = bgidx

def highscore_set_border(tf):
    global __highscore_border
    if tf:
        __highscore_border = True
    else:
        __highscore_border = False

def highscore_set_colors(back,new,other):
    global __highscore_back_color
    global __highscore_new_color
    global __highscore_other_color
    if type(back) is py_GMVar:
        __highscore_back_color = __to_rgb(back[0])
    else:
        __highscore_back_color = __to_rgb(back)
    if type(new) is py_GMVar:
        __highscore_new_color = __to_rgb(new[0])
    else:
        __highscore_new_color = __to_rgb(new)
    if type(other) is py_GMVar:
        __highscore_other_color = __to_rgb(other[0])
    else:
        __highscore_other_color = __to_rgb(other)

def highscore_set_font(fontname,size,style):
    if type(fontname) is py_GMVar:
        fontname = fontname[0]
    global __highscore_font
    __highscore_font = pygame.font.SysFont(fontname,int(size))
    if int(style)&1:
        __highscore_font.set_bold(1)
    if int(style)&2:
        __highscore_font.set_italic(1)

def highscore_set_strings(caption, nobody, escape):
    global __highscore_caption
    __highscore_caption = caption
    global __highscore_nobody
    __highscore_nobody = nobody
    global __highscore_escape_prompt
    __highscore_escape_prompt = escape

def __highscore_show_single(score,y,color):
    if score[1] is not None:
        name = score[1]
    else:
        name = __highscore_nobody
    __pygame_screen.blit(__highscore_font.render(name,1,color),(10,y))
    number = __highscore_font.render(str(int(score[0])),1,color)
    __pygame_screen.blit(number,((__pygame_screen.get_width()-10)-number.get_width(),y))

def __highscore_list_draw(newscore_idx,newscore):
    caption = __highscore_font.render(__highscore_caption,1,__highscore_other_color)
    __pygame_screen.blit(caption,((__pygame_screen.get_width()>>1)-(caption.get_width()>>1),10))
    y = 30
    lineheight = 20
    if newscore_idx is None:
        for score in __highscores:
            __highscore_show_single(score,y,__highscore_other_color)
            y += lineheight
    else:
        for score in __highscores[:newscore_idx]:
            __highscore_show_single(score,y,__highscore_other_color)
            y += lineheight
        __highscore_show_single((newscore,keyboard_string),y,__highscore_new_color)
        y += lineheight
        for score in __highscores[newscore_idx:]:
            __highscore_show_single(score,y,__highscore_other_color)
            y += lineheight
    caption = __highscore_font.render(__highscore_escape_prompt,1,__highscore_other_color)
    __pygame_screen.blit(caption,((__pygame_screen.get_width()>>1)-(caption.get_width()>>1),y))

def highscore_show(newscore):
    if type(newscore) is py_GMVar:
        newscore = newscore[0]
    bgidx = __highscore_background
    if type(bgidx) is float:
        bgidx = int(bgidx)
    bgsurf = None
    if type(bgidx) is int:
        if bgidx>=0:
            bgsurf = gm6.game.backgrounds[int(bgidx)].surf
            __pygame_screen.blit(bgsurf,(0,0))
    else:
        bgsurf = bgidx.surf
    if __highscore_border:
        print "Debug: highscore borders not yet implemented."
    x = 0
    newscore_idx = None
    print 2141,newscore,__highscores
    while x<len(__highscores):
        if newscore>__highscores[x][0]:
            print 'debug',newscore,'2141',__highscores
            newscore_idx = x
            break
        x += 1
    global keyboard_string
    old_keyboard_string = keyboard_string
    keyboard_string = ''
    __highscore_list_draw(newscore_idx,newscore)
    pygame.display.flip()
    io_clear()
    notdone = True
    while notdone:
        io_handle()
        if keyboard_check(vk_escape):
            notdone = False
        elif keyboard_check(vk_enter) and newscore_idx is not None:
            if keyboard_string.endswith('\015'):
                keyboard_string=keyboard_string[:-1]
            __highscores.insert(newscore_idx,(newscore,keyboard_string))
            __highscores.pop()
            newscore_idx = None
        if bgsurf:
            __pygame_screen.blit(bgsurf,(0,0))
        __highscore_list_draw(newscore_idx,newscore)
        pygame.display.flip()
    io_clear()
    keyboard_string = old_keyboard_string

def highscore_value(place):
    if place>=len(__highscores):
        return 0
    return __highscores[int(place)][0]

def instance_activate_object(obj):
    x = 0
    while x<len(room.deactivated_objects):
        i = room.deactivated_objects[x]
        if object_is_ancestor(i.object_index,obj):
            room.AddInstance(i)
            i.object_index.py_objectdata.instances.append(i)
            del(room.deactivated_objects[x])
        else:
            x += 1
def instance_change(obj, destroy_create):
    global self
    oldobj = self.object_index
    if destroy_create:
        if self.object_index.py_objectdata.events[1]:
            if len(self.object_index.py_objectdata.events[1])>1:
                raise "Too many destroy events?"
            gm6.RunActionGroupOn(
                self.object_index.py_objectdata.events[1][0],self)
    self.Remove()
    self.SetDepth(obj.py_objectdata.depth)
    self.py_Draw = 0
    self.object_index = obj
    #oldobj.py_objectdata.instances.remove(self)
    obj.py_objectdata.instances.append(self)
    room.AddInstance(self)
    if destroy_create:
        self.RunCreateEvent()
    return
def instance_create(x,y,obj):
    ret = py_ObjectInstance(obj)
    if type(x) is not py_GMVar:
        ret.x = ret.xstart = x
    else:
        ret.x = ret.xstart = x[0]
    if type(y) is not py_GMVar:
        ret.y = ret.ystart = y
    else:
        ret.y = ret.ystart = y[0]
    room.AddInstance(ret)
    ret.SetBoundingBox()
    ret.RunCreateEvent()
    return ret
def __instance_deactivate(instance):
    instance.Remove()
    if instance.py_inactive<1:
        instance.py_inactive = 1
    room.deactivated_objects.append(instance)

def instance_deactivate_object(obj):
    if type(obj)==py_ObjectInstance:
        __instance_deactivate(obj)
        return
    for o in tuple(obj):
        __instance_deactivate(o)

def instance_destroy():
    if type(self)==gm6.GM_Object:
        raise "Boom?"
    self.Destroy()
def instance_exists(obj):
    # Checks to see if an instance currently exists in the room.
    # returns false if the instance is deactivated.
    if type(obj) is py_GMVar:
        obj = obj[0]
    if type(obj) == float:
        obj = int(obj)
    if type(obj) in (int,gm6.InstanceID):
        if obj>=0:
            if obj in gm6.instances:
                if gm6.instances[obj] in room.all_objects:
                    return 1
                else:
                    print "Error: instance mapping exists, but instance doesn't."
                    return False
            elif obj<len(gm6.game.objects) and gm6.game.objects[obj]:
                raise "Instance_exists called on an object ("+str(obj)+")"
            else:
                return False
        elif obj==-1:
            return False
        else:
            raise "instance_exists on a negative instance ID."+str(obj)
    # XXX need to iterate over an object ID's children to find out if an
    # instance exists.
    if type(obj) == gm6.GM_Object:
        for i in obj:
            return True
        return False
    elif type(obj)==py_ObjectInstance:
        ret = obj.id in gm6.instances
        return ret
    elif obj is None:
        return False
    else:
        raise "Unexpected instance_exists call"+str(obj)
def instance_find(obj,n):
    n = int(n)
    i = 0
    for o in obj:
        if i==n:
            return o
        i += 1
    return noone
def instance_find_OLD(obj,n):
    if n>=len(obj.py_objectdata.instances):
        for child in gm6.ObjectChildrenDataGenerator(obj.py_objectdata):
            n -= len(child.instances)
            if n<len(child.instances):
                return child.instances[n]
        return noone
    return obj.instances[int(n)]
def instance_number(obj):
    if type(obj)==gm6.GM_Object:
        obj = obj.__dict__['py_objectdata']
    ret = len(obj.instances)
    for child in gm6.ObjectChildrenDataGenerator(obj):
        ret += len(child.instances)
    return ret
def instance_place(x,y,obj):
    selfbox = self.BoundingBox_At(x,y)
    under = []
    for o2 in obj:
        if o2.IsUnderBox(selfbox):
            under.append(o2)
    if len(under)==1:
        return under[0]
    elif len(under)==0:
        return noone
    else:
        # TODO what does GM do if there are multiple true results for instance_place?
        return under[0]
def instance_position(x,y,obj):
    for instance in obj:
        box = instance.BoundingBox()
        if x>=box[0] and x<box[2] and y>=box[1] and y<=box[3]:
            return instance
    return noone

def ini_close():
    py_current_ini_file.Close()
ini_open = __IniFile
def ini_read_real(section,key,default):
    return float(ini_read_string(section,key,default))
def ini_read_string(section,key,default):
    if type(section) is py_GMVar:
        section = section[0]
    if not section in py_current_ini_file.sections:
        return default
    if type(key) is py_GMVar:
        key = key[0]
    if not key in py_current_ini_file.sections[section]:
        return default
    return py_current_ini_file.sections[section][key]
def ini_write_real(section,key,value):
    if not section in py_current_ini_file.sections:
        py_current_ini_file.sections[section] = dict()
    py_current_ini_file.sections[section][key] = str(value)
def ini_write_string(section,key,value):
    if not section in py_current_ini_file.sections:
        py_current_ini_file.sections[section] = dict()
    py_current_ini_file.sections[section][key] = value

def io_clear():
    global py_keys_pressed
    py_keys_pressed = py_PythonAutovivifyingZeroArray()
    global mouse_button
    mouse_button = mb_none
    global mouse_lastbutton
    mouse_lastbutton = mb_none
def __io_handle_set_mouse_pos(pos):
    global mouse_x
    global mouse_y
    mouse_x, mouse_y = pos
    if room.view_enabled:
        mouse_x += room.view_xview[0]
        mouse_y += room.view_yview[0]
def io_handle():
    pygame.event.pump()
    ev = pygame.event.poll()
    global mouse_button
    global py_mouse_pressed
    while ev.type != pygame.NOEVENT:
        if ev.type==pygame.MOUSEBUTTONDOWN:
            __io_handle_set_mouse_pos(ev.pos)
            if ev.button==3:
                mouse_button |= 4
                py_mouse_pressed |= 4
            elif ev.button<3:
                mouse_button |= ev.button
                py_mouse_pressed |= ev.button
        elif ev.type==pygame.MOUSEBUTTONUP:
            __io_handle_set_mouse_pos(ev.pos)
            butt = ev.button
            if butt==3:
                butt = 4
            elif butt>=4:
                ev = pygame.event.poll()
                continue
            mouse_button &= ~butt
        elif ev.type==pygame.MOUSEMOTION:
            __io_handle_set_mouse_pos(ev.pos)
        elif ev.type==pygame.KEYDOWN:
            global keyboard_string
            if type(keyboard_string) is py_GMVar:
                keyboard_string = keyboard_string[0]
            if ev.key==pygame.K_BACKSPACE:
                if len(keyboard_string):
                    keyboard_string = keyboard_string[:-1]
            elif ev.key==pygame.K_TAB and ev.mod==pygame.KMOD_LMETA:
                gm6.handle_command_tab()
            else:
                keyboard_string += ev.unicode
                if len(keyboard_string)>1024:
                    keyboard_string = keyboard_string[-1024:]
            if py_translate_keys.has_key(ev.key):
                py_keys_pressed[py_translate_keys[ev.key]] = 1
            else:
                while ev.key>=len(py_keys_pressed):
                    py_keys_pressed.append(0)
                py_keys_pressed[ev.key] = 1
            # XXX what about unhandled keys?
        elif ev.type==pygame.KEYUP:
            if py_translate_keys.has_key(ev.key):
                py_keys_pressed[py_translate_keys[ev.key]] = 1
            # XXX what about unhandled ones?
        elif ev.type==gm6.renpy_PERIODIC:
            renpy.audio.audio.periodic()
        elif ev.type==pygame.ACTIVEEVENT:
            pass
        else:
            print "DEBUG: 1402: Unhandled event type"+str(ev)
        ev = pygame.event.poll()
    # XXX not sure if this should run events or not.  Guessing not.

def is_real(arg):
    if type(arg) is py_GMVar:
        arg = arg[0]
    return type(arg) in (float,int)

def keyboard_check(key):
    if type(key) is py_GMVar:
        key = key[0]
    return py_keys_pressed[key]

def keyboard_clear(key):
    py_keys_pressed[key] = 0

def lengthdir_x(distance, angle):
    return distance*math.sin(math.radians(angle+90))

def lengthdir_y(distance, angle):
    return distance*math.cos(math.radians(angle+90))

def make_color(r,g,b):
    return make_color_rgb(r,g,b)
# XXX make_color_hsv unimplimented
def make_color_rgb(r,g,b):
    return (r,g,b)

def merge_color(col1,col2,amount):
    if type(col1) is py_GMVar:
        col1 = col1[0]
    if type(col2) is py_GMVar:
        col2 = col2[0]
    amt1 = 1-amount
    return (
        int((col1[0]*amt1)+(col2[0]*amount)),
        int((col1[1]*amt1)+(col2[1]*amount)),
        int((col1[2]*amt1)+(col2[2]*amount)),
        )

def message_background(*args):
    if type(args[0]) is py_GMVar:
        print type(args[0][0])
        __message_props.background = args[0][0]
        return
    __message_props.background = args[0]
def message_button_font(*args):
    __message_props.button_font = __Fontspec(*args)
def message_button(spr):
    __message_props.button_spr = spr
def message_input_font(*args):
    __message_props.input_font = __Fontspec(*args)
def message_input_color(color):
    __message_props.input_color = color
def message_mouse_color(color):
    __message_props.button_font_hover_color = color
def message_size(w,h):
    __message_props.w = int(w)
    if not w:
        __message_props.w = __message_props.background.surf.get_width()
    __message_props.h = int(h)
def message_text_font(*args):
    __message_props.text_font = __Fontspec(*args)

def motion_add(direction, speed):
    moving = self.vspeed or self.hspeed
    # XXX when assignors for vhspeed are in, fix direction?
    self.__dict__['vspeed'] += speed*math.cos(math.radians(direction+90))
    self.__dict__['hspeed'] += speed*math.sin(math.radians(direction+90))
    self.RecalculateAngularSpeed()
    if self.vspeed or self.hspeed:
        if not moving:
            room.moving_objects.append(self)
    elif moving:
        room.moving_objects.remove(self)

def mouse_check_button(numb):
    if numb == mb_none and not mouse_button:
        return True
    elif numb == mb_left and mouse_button & mb_left:
        return True
    elif numb == mb_right and mouse_button & mb_right:
        return True
    elif numb == mb_middle and mouse_button & mb_middle:
        return True
    elif numb == mb_any and mouse_button:
        return True
    return False
def mouse_check_button_pressed(butt):
    print "1112, mouse_check_button_pressed",butt,mouse_button,py_mouse_pressed
    mouse_button = py_mouse_pressed
    if butt == mb_none and py_laststep_mouse_button == mouse_button:
        return True
    elif butt == mb_left and mouse_button & mb_left and not py_laststep_mouse_button & mb_left:
        return True
    elif butt == mb_right and mouse_button & mb_right and not py_laststep_mouse_button & mb_right:
        return True
    elif butt == mb_middle and mouse_button & mb_middle and not py_laststep_mouse_button & mb_middle:
        return True
    elif butt == mb_any and mouse_button > py_laststep_mouse_button:
        return True
    return False
def mouse_check_button_released(butt):
    mouse_button = py_mouse_released
    if butt == mb_none and py_laststep_mouse_button == mouse_button:
        return True
    elif butt == mb_left and py_laststep_mouse_button & mb_left and not mouse_button & mb_left:
        return True
    elif butt == mb_right and py_laststep_mouse_button & mb_right and not mouse_button & mb_right:
        return True
    elif butt == mb_middle and py_laststep_mouse_button & mb_middle and not mouse_button & mb_middle:
        return True
    elif butt == mb_any and mouse_button<py_laststep_mouse_button:
        return True
    return False
def mouse_clear(butt):
    # XXX mouse_clear may not work as desired.  This marks it no longer held
    # down, but does not interrupt any already-fired click-responses.
    mouse_button &= ~butt
    raise "Mouse_clear untested."
def mouse_wait(butt):
    raise "Mouse_wait unimplimented."

def move_bounce_all(precise):
    # I assume that 'adv' is 'precise collision checking'?
    print 'bouncing',self,self.hspeed,self.vspeed,'off',other,
    if not precise:
        if self.bbox_bottom>other.bbox_top:
            self.vspeed = -self.vspeed
        if self.bbox_top<other.bbox_bottom:
            self.vspeed = -self.vspeed
        if self.bbox_right>other.bbox_left:
            self.hspeed = -self.hspeed
        if self.bbox_left<other.bbox_right:
            self.hspeed = -self.hspeed
        print "Bounce done:",self.hspeed,self.vspeed
        self.RecalculateAngularSpeed()
        return
    raise "Precise bouncing not yet implemented."

def move_contact_solid(angle,maxdist):
    if maxdist<0:
        maxdist = room.room_width+room.room_height
    if not place_free(self.x,self.y):
        return
    angx = lengthdir_x(1,angle)
    angy = lengthdir_y(1,angle)
    dist = 0
    xdelt = 0
    ydelt = 0
    solids = []
    #solid_top = []
    #solid_left = []
    #solid_right = []
    #solid_bottom = []
    for instance in room.all_objects:
        if instance.object_index.py_objectdata.solid:
            if self.bbox_top-instance.bbox_bottom>maxdist:
                continue
            elif instance.bbox_top-self.bbox_bottom>maxdist:
                continue
            elif self.bbox_left-instance.bbox_right>maxdist:
                continue
            elif instance.bbox_left-self.bbox_right>maxdist:
                continue
            # XXX could probably improve performance further by culling
            # objects that are not in the right direction.
            solids.append(instance)
            #solid_top.append(instance.bbox_top)
            #solid_bottom.append(instance.bbox_bottom)
            #solid_left.append(instance.bbox_left)
            #solid_right.append(instance.bbox_right)
    if not solids:
        self.x += angx*maxdist  
        self.y += angy*maxdist
        self.SetBoundingBox()
        return
    selftop = self.bbox_top
    selfbottom = self.bbox_bottom
    selfleft = self.bbox_left
    selfright = self.bbox_right
    while dist<maxdist:
        dist += 1
        selftop += angy
        selfbottom += angy
        selfright += angx
        selfleft += angx
        box = (selftop,selfbottom,selfright,selfleft)
        for solid in solids:
            if solid.IsUnderBox(box):
                dist -= 1
                self.x += dist*angx
                self.y += dist*angy
                self.SetBoundingBox()
                return

def move_outside_all(angle,maxdist):
    global self
    if place_empty(self.x,self.y):
        return
    # Using 2-pixel steps because 1-pixel steps would involve mostly subpixel
    # checks.
    angx = lengthdir_x(1,angle)
    angy = lengthdir_y(1,angle)
    dist = 0
    xdelt = 0
    ydelt = 0
    while dist<=maxdist:
        dist += 1
        xdelt += angx
        ydelt += angy
        collid = False
        for instance in room.all_objects:
            if type(instance)==py_ParticleSystem:
                continue
            elif instance.bbox_left>=self.bbox_right+xdelt:
                continue
            elif instance.bbox_right<=self.bbox_left+xdelt:
                continue
            elif instance.bbox_top>=self.bbox_bottom+ydelt:
                continue
            elif instance.bbox_bottom<=self.bbox_top+ydelt:
                continue
            else:
                collid = True
                break
        if not collid:
            self.x += xdelt
            self.y += ydelt
            self.SetBoundingBox()
            return

def move_towards_point(x,y,distance):
    xd = x-self.x
    yd = y-self.y
    d = math.hypot(xd,yd)
    if d>distance:
        dd = distance/d
        self.x += xd*dd
        self.y += yd*dd
    else:
        self.x = x
        self.y = y
    self.SetBoundingBox()

def move_wrap(h,v,margin):
    global room
    global self
    # XXX where is it actually supposed to wrap to?  At the moment I'm
    # wrapping to the other margin.
    if h:
        if self.x<=-margin:
            self.x = room.room_width+margin-1
        elif self.x>=room.room_width+margin:
            self.x = 1-margin
    if v:
        if self.y<=-margin:
            self.y = room.room_height+margin-1
        elif self.y>=room.room_height+margin:
            self.y = 1-margin
    self.SetBoundingBox()

def mp_grid_add_instances(grid, obj, precise):
    # XXX support special object 'all'
    # XXX should actually forbid all OVERLAPPING cells, not just cells
    # containing the instance origin.  Also, should honour precise collision
    # checking flag (which only is meaningful if the instance sprite also uses
    # precise collision checking.)
    xoff = -grid.left
    yoff = -grid.top
    if type(obj)==gm6.GM_Object:
        totaladded = 0
        for instance in obj:
            grid.cells[int((instance.y+yoff)/grid.cellh)][int((instance.x+xoff)/grid.cellw)] = 1
            totaladded +=1
    elif type(obj)==py_ObjectData:
        raise "Boom!  Somehow py_ObjectData has gotten into GM space."
        instances = obj.instances
    else:
        instance = obj
        grid.cells[int((instance.y+yoff)/grid.cellh)][int((instance.x+xoff)/grid.cellw)] = 1

def mp_grid_clear_all(grid):
    for row in grid.cells:
        x = 0
        while x<len(row):
            row[x] = 0
            x += 1

def mp_grid_path_OLD(grid, path, xstart_px, ystart_px, xgoal, ygoal, allow_diag):
    # It currently appears to me that it's supposed to return an absolute path.
    xgoal = int((xgoal-grid.left)/grid.cellw)
    ygoal = int((ygoal-grid.top)/grid.cellh)
    # Adding this fixed walk-off-edge, but killed all mouse movement except
    # setclick.
    #xstart_px = int((xstart_px-grid.top)/grid.cellw)
    #ystart_px = int((ystart_px-grid.top)/grid.cellh)

    if ygoal>len(grid.cells) or xgoal>len(grid.cells[0]) or grid.cells[ygoal][xgoal]:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    xstart = int((xstart_px-grid.left)/grid.cellw)
    ystart = int((ystart_px-grid.top)/grid.cellh)
    if allow_diag:
        return __diag_mp_grid_path(grid, path, xstart, ystart, xgoal, ygoal, allow_diag)
    flood = []
    while len(flood)<len(grid.cells):
        flood.append([0,]*len(grid.cells[0]))
    flood[ygoal][xgoal] = 1024
    tocheck = [(xgoal,ygoal)]
    i = 0
    destval = 0
    while i<len(tocheck):
        x,y = tocheck[i]
        i += 1
        if x==xstart and y==ystart:
            destval = flood[y][x]
            continue
        else:
            val = flood[y][x]-1
        if val<=destval:
            continue
        if y:
            if (not grid.cells[y-1][x]) and flood[y-1][x]<val:
                flood[y-1][x] = val
                tocheck.append((x,y-1))
        if y<len(flood)-1:
            if (not grid.cells[y+1][x]) and flood[y+1][x]<val:
                flood[y+1][x] = val
                tocheck.append((x,y+1))
        if x:
            if (not grid.cells[y][x-1]) and flood[y][x-1]<val:
                flood[y][x-1] = val
                tocheck.append((x-1,y))
        if x<len(flood[0])-1:
            if (not grid.cells[y][x+1]) and flood[y][x+1]<val:
                flood[y][x+1] = val
                tocheck.append((x+1,y))
    if ystart>=len(flood) or ystart<0:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    if xstart>=len(flood[ystart]) or xstart<0:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    if not flood[ystart][xstart]:
        print "Warning: Could not pathfind to",xgoal,ygoal
        for line in flood:
            print line
        for line in grid.cells:
            print line
        # XXX does this clobber the existing path if you try to pathfind to
        # an impossible place?
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    x,y = xstart, ystart
    path.points = []
    val = flood[y][x]
    ox,oy = (x*grid.cellw)+grid.left,(y*grid.cellh)+grid.top
    xdelt,ydelt = ox-xstart_px,oy-ystart_px
    path.length = 0
    while x!=xgoal or y!=ygoal:
        newx = x
        newy = y
        if y>0 and flood[y-1][x]>val:
            val = flood[y-1][x]
            newy = y-1
        if y<len(flood)-1 and flood[y+1][x]>val:
            val = flood[y+1][x]
            newy = y+1
        if x>0 and flood[y][x-1]>val:
            val = flood[y][x-1]
            newx = x-1
            newy = y
        if x<len(flood[0])-1 and flood[y][x+1]>val:
            val = flood[y][x+1]
            newx = x+1
            newy = y
        px = int(grid.left+(newx*grid.cellw))
        py = int(grid.top+(newy*grid.cellh))
        path.points.append((px-xdelt,py-ydelt))
        path.length += point_distance(ox,oy,path.points[-1][0],path.points[-1][1])
        ox,oy = path.points[-1]
        if x==newx and y==newy:
            return False
        x,y = newx,newy
    if path.length:
        path.multiplier = 1.0/path.length
    else:
        path.multiplier = 1.0
    return True
def mp_grid_path(grid, path, xstart_px, ystart_px, xgoal, ygoal, allow_diag):
    # It currently appears to me that it's supposed to return an absolute path.
    xgoal = int((xgoal-grid.left)/grid.cellw)
    ygoal = int((ygoal-grid.top)/grid.cellh)
    # Adding this fixed walk-off-edge, but killed all mouse movement except
    # setclick.
    #xstart_px = int((xstart_px-grid.top)/grid.cellw)
    #ystart_px = int((ystart_px-grid.top)/grid.cellh)

    if ygoal>len(grid.cells) or xgoal>len(grid.cells[0]) or grid.cells[ygoal][xgoal]:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    xstart = int((xstart_px-grid.left)/grid.cellw)
    ystart = int((ystart_px-grid.top)/grid.cellh)
    if allow_diag:
        return __diag_mp_grid_path(grid, path, xstart, ystart, xgoal, ygoal, allow_diag)
    flood = []
    while len(flood)<len(grid.cells):
        flood.append([0,]*len(grid.cells[0]))
    flood[ygoal][xgoal] = 1024
    tocheck = [(xgoal,ygoal)]
    i = 0
    destval = 0
    while i<len(tocheck):
        x,y = tocheck[i]
        i += 1
        if x==xstart and y==ystart:
            destval = flood[y][x]
            continue
        else:
            val = flood[y][x]-1
        if val<=destval:
            continue
        if y:
            if (not grid.cells[y-1][x]) and flood[y-1][x]<val:
                flood[y-1][x] = val
                tocheck.append((x,y-1))
        if y<len(flood)-1:
            if (not grid.cells[y+1][x]) and flood[y+1][x]<val:
                flood[y+1][x] = val
                tocheck.append((x,y+1))
        if x:
            if (not grid.cells[y][x-1]) and flood[y][x-1]<val:
                flood[y][x-1] = val
                tocheck.append((x-1,y))
        if x<len(flood[0])-1:
            if (not grid.cells[y][x+1]) and flood[y][x+1]<val:
                flood[y][x+1] = val
                tocheck.append((x+1,y))
    if ystart>=len(flood) or ystart<0:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    if xstart>=len(flood[ystart]) or xstart<0:
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    if not flood[ystart][xstart]:
        print "Warning: Could not pathfind to",xgoal,ygoal
        for line in flood:
            print line
        for line in grid.cells:
            print line
        # XXX does this clobber the existing path if you try to pathfind to
        # an impossible place?
        path.points = []
        path.multiplier = 1
        path.length = 0
        return False
    x,y = xstart, ystart
    path.points = []
    val = flood[y][x]
    ox,oy = (x*grid.cellw)+grid.left,(y*grid.cellh)+grid.top
    #if ox!=xstart_px or oy!=ystart_px:
    path.points.append((ox+(grid.cellw*0.5),oy+(grid.cellh*0.5)))#(xstart_px,ystart_px))
    #xdelt,ydelt = ox-xstart_px,oy-ystart_px
    path.length = 0
    while x!=xgoal or y!=ygoal:
        newx = x
        newy = y
        if y>0 and flood[y-1][x]>val:
            val = flood[y-1][x]
            newy = y-1
        if y<len(flood)-1 and flood[y+1][x]>val:
            val = flood[y+1][x]
            newy = y+1
        if x>0 and flood[y][x-1]>val:
            val = flood[y][x-1]
            newx = x-1
            newy = y
        if x<len(flood[0])-1 and flood[y][x+1]>val:
            val = flood[y][x+1]
            newx = x+1
            newy = y
        px = int(grid.left+((0.5+newx)*grid.cellw))
        py = int(grid.top+((0.5+newy)*grid.cellh))
        path.points.append((px,py))#(px-xdelt,py-ydelt))
        path.length += point_distance(ox,oy,path.points[-1][0],path.points[-1][1])
        ox,oy = path.points[-1]
        if x==newx and y==newy:
            return False
        x,y = newx,newy
    if path.length:
        path.multiplier = 1.0/path.length
    else:
        path.multiplier = 1.0
    return True


def object_event_add(ind, ev, numb, code):
    python = gm6.tree_to_python(gm6.gml2syntaxtree(code),depth=1)
    ind.py_objectdata.AddEvent(ev,numb,python)
def object_event_clear(ind, ev, numb):
    x = 0
    ev = int(ev)
    if ev==6:
        if numb==53:
            ind.py_objectdata.py_GlobLeftPressedActionGroup = None
        else:
            raise "Unhandled mouse event, cannot clear."
        return
    while x<len(ind.py_objectdata.events[ev]):
        if ind.py_objectdata.events[ev][x].numb==numb:
            del ind.py_objectdata.events[ev][x]
        else:
            x += 1
def object_exists(ind):
    return type(ind)==gm6.GM_Object
# All the object_gets are untested.  XXX
def object_get_depth(ind):
    return ind.py_objectdata.depth
def object_get_mask(ind):
    return ind.py_objectdata.mask
def object_get_name(ind):
    return ind.py_objectdata.name
def object_get_parent(ind):
    return ind.py_objectdata.parent
def object_get_persistent(ind):
    return ind.py_objectdata.persistent
def object_get_solid(ind):
    return ind.py_objectdata.solid
def object_get_sprite(ind):
    return ind.py_objectdata.sprite
def object_get_visible(ind):
    return ind.py_objectdata.visible
def object_is_ancestor(o, possparent):
    if type(o) is py_GMVar:
        o = o[0]
    if type(possparent) is py_GMVar:
        possparent = possparent[0]
    # o will be a gm6.ObjectData object
    if o is possparent:
        return True
    parent = o.py_objectdata.parent_object_index
    if type(parent) in (int,float):
        if parent<0:
            return False
        #else:
        #    print "Resolving integer parent",parent,gm6.game.objects[int(parent)]
        parent = gm6.game.objects[int(parent)].gmobject
    while parent is not None:
        if parent is possparent:
            return True
        parent = parent.py_objectdata.parent_object_index
        if type(parent) in (int,float):
            if parent<0:
                parent = None
            else:
                parent = gm6.game.objects[parent].gmobject
    return False

py_ord = ord
def ord(s):
    if type(s) is py_GMVar:
        return py_ord(s[0])
    return py_ord(s)

def parameter_string(idx):
    if idx==0:
        if os.uname()[0]=='Darwin':
            path = py_string.rsplit(sys.argv[0],'/',maxsplit=1)
            return path[1]#path[0]+'/game/'+path[1]
        else:
            return sys.argv[0]
    return sys.argv[int(idx)]

def part_emitter_burst(ps,em,parttype,number):
    otype = em.part_type
    em.part_type=parttype
    while number>0:
        ps.py_particles.append(em.SpawnParticle())
        number -= 1
    em.part_type = otype

def part_emitter_create(ps):
    return ps.EmitterCreate()

def part_emitter_region(system, emitter, xmin, xmax, ymin, ymax, shape, distribution):
    emitter.xmin = int(xmin)
    emitter.xmax = int(xmax)
    emitter.ymin = int(ymin)
    emitter.ymax = int(ymax)
    emitter.shape = shape
    emitter.distribution = distribution

def part_emitter_stream(system, emitter, part_type, number):
    emitter.part_type = part_type
    emitter.number = number

def part_particles_create(ps, x, y, particletype, num):
    while num>0:
        num -= 1
        ps.py_particles.append(py_Particle(particletype,x,y))

part_system_create = py_ParticleSystem
def part_system_depth(ps,depth):
    ps.depth = depth
    room.ResortDrawnObjects()
def part_system_drawit(ps):
    raise "part_system_drawit unimplimented."
def part_system_position(ps,x,y):
    ps.x = x
    ps.y = y

def part_system_update(ps):
    ps.py_RealStep()

def part_type_alpha1(pt,alpha):
    pt.startalpha = pt.halfalpha = pt.endalpha = int(alpha)
    pt.Regenerate()
part_type_alpha2 = py_fasttint.part_type_alpha2
part_type_alpha3 = py_fasttint.part_type_alpha3
part_type_blend = py_fasttint.part_type_blend
def part_type_color2(pt,start,end):
    py_fasttint.part_type_color2(pt,
        __color_to_int(__to_rgb(start)),
        __color_to_int(interpolate_rgb(pt.startcolor,pt.endcolor)),
        __color_to_int(__to_rgb(end)))
def part_type_color3(pt,start,half,end):
    py_fasttint.part_type_color3(pt, 
        __color_to_int(__to_rgb(start)),
        __color_to_int(__to_rgb(half)),
        __color_to_int(__to_rgb(end)))
part_type_create = py_fasttint.part_type_create #py_ParticleType
part_type_direction = py_fasttint.part_type_direction
part_type_gravity = py_fasttint.part_type_gravity
part_type_life = py_fasttint.part_type_life
part_type_orientation = py_fasttint.part_type_orientation;
part_type_scale = py_fasttint.part_type_scale
part_type_shape = py_fasttint.part_type_shape
part_type_size = py_fasttint.part_type_size
part_type_speed = py_fasttint.part_type_speed
part_type_sprite = py_fasttint.part_type_sprite
def place_empty(x,y):
    # Returns true only if moving self to x,y will meet nobody (regardless of
    # solidity)
    # Untested!
    selfbox = self.BoundingBox(x=x,y=y)
    for o in room.all_objects:
        if o.IsUnderBox(selfbox) and o is not self:
            return False
    return True

def place_free(x,y):
    # XXX place_free checks to see if moving self to x,y will be collision-free.
    # Untested!
    selfbox = self.BoundingBox(x=x,y=y)
    for o in room.all_objects:
        if o.object_index.py_objectdata.solid and o.IsUnderBox(selfbox) and o is not self:
            return False
    return True

def place_meeting(x,y,obj):
    if self.object_index.sprite_index.precise_collision_checking:
        raise "Precise collision checking not yet supported."
    selfbox = self.BoundingBox(x=x,y=y)
    if type(obj)==py_ObjectInstance:
        return obj.IsUnderBox(selfbox)
    else:
        # XXX should only check collidable objects?
        for o2 in obj:
            if o2.IsUnderBox(selfbox):
                return True
    return False

def path_end():
    global self
    self.path_index = None
                
def path_start(path, speed, endaction, absolute):
    self.AttachPath(path, speed, endaction, absolute)

def point_direction(x1,y1,x2,y2):
    if x1-x2==0:
        return 0
    return math.degrees(math.atan(float(y1-y2)/float(x1-x2)))

def point_distance(x1,y1,x2,y2):
    return math.hypot(x1-x2,y1-y2)

def position_empty(x,y):
    for instance in room.all_objects:
        if instance.IsUnderPos(x,y):
            if instance.object_index.py_objectdata.solid:
                return False
    # XXX should only check collidable objects?
    return True

def position_meeting(x,y,obj):
    # The docs are wrong!  Position_meeting checks to see if something is
    # under the pos, not exactly at! - at least in GM6
    for o in obj:
        if o.IsUnderPos(x,y):
            return True
    return False

def random(x):
    ret = py_random.random()*x
    if ret==x:
        ret -= 0.000000001
    return ret

def real(s):
    if s=='\015\012' or s=='\015' or s=='\012' or s=='':
        return 0.0
    return float(s)

def __free_caches():
    for spr in gm6.game.sprites:
        if spr is not None:
            spr.scalecache.clear()
    for bg in gm6.game.backgrounds:
        if bg is not None:
            bg.scaled_data_cache.clear()
    py_fasttint.CParticle_type_cache_clear(ef_snow)
    py_fasttint.CParticle_type_cache_clear(ef_cloud)
    

def __room_goto_low():
    global room
    persistent = []
    killedcount = 0
    for instance in tuple(room.all_objects):
        instance.RoomEnd()
        if instance.persistent:
            persistent.append(instance)
            if room.room_persistent:
                instance.Remove()
        elif not room.room_persistent:
            instance.Remove(kill=True)
            killedcount += 1
    if not room.room_persistent:
        room.Leave()
    __free_caches()
    global __new_room
    room = __new_room
    for instance in persistent:
        room.AddInstance(instance)
        instance.object_index.py_objectdata.instances.append(instance)
    if room.created:
        print "Returning to persistent room.",room.name,room.room_persistent
    else:
        x = 0
        for vd in room.initial_view_data:
            room.view_xview[x] = vd.view_x
            room.view_yview[x] = vd.view_y
            room.view_object[x] = vd.objfollowing
            room.view_hborder[x] = vd.hbor
            room.view_vborder[x] = vd.vbor
            room.view_hview[x] = vd.view_h
            room.view_wview[x] = vd.view_w
            room.view_xport[x] = vd.port_x
            room.view_yport[x] = vd.port_y
            room.view_wport[x] = vd.port_w
            room.view_hport[x] = vd.port_h
            room.view_visible[x] = vd.visible
            x += 1
    # XXX Docs indicate that:
    # Creation of new instances takes place only the first time you visit a
    #   persistent room.
    # Room creation code runs every time you enter a room.  Is this true?
    room.Create()
    global mouse_x
    global mouse_y
    mouse_x,mouse_y = pygame.mouse.get_pos()
    mouse_x += room.view_xview[0]
    mouse_y += room.view_yview[0]
    global py_game_restart
    if py_game_restart:
        py_game_restart = False
        for obj in room.all_objects:
            if obj.object_index.py_objectdata.py_GameStartEvent:
                gm6.RunActionGroupOn(obj.object_index.py_objectdata.py_GameStartEvent,obj)
    for instance in room.all_objects:
        instance.RoomStart()

__crash_on_room_goto = []

def room_goto(roomnum):
    global __new_room
    if type(roomnum) is py_GMVar:
        roomnum = roomnum[0]
    if type(roomnum) in (int,float):
        roomnum = gm6.game.rooms[int(roomnum)]
    __new_room = roomnum
    if __new_room.name in __crash_on_room_goto:
        raise "Crash requested."

def room_goto_next():
    x = 0
    global room
    while x<len(gm6.game.rooms):
        if gm6.game.rooms[x] is room:
            if x+1<len(gm6.game.rooms):
                room_goto(x+1)
                return
            else:
                print "XXX what happens on room_goto_next after the last room?"
                room_goto(0)
                return
        x+=1
    raise "Can't find current room."

def room_restart():
    global __new_room
    __new_room = room
    __room_goto_low()

def screen_redraw():
    global __screen_redraw_in_progress
    if __screen_redraw_in_progress:
        return
    __screen_redraw_in_progress = True
    py_fasttint.Room_Draw(room,__pygame_screen)
    pygame.display.update()
    __screen_redraw_in_progess = False

def screen_refresh():
    pygame.display.flip()

def screen_save(fname):
    pygame.image.save(__pygame_screen,'screenshot.png')
def screen_save_part(fname,x,y,w,h):
    raise "XXX should save a screenshot."

def script_execute(scr,*args):
    if type(scr) is py_GMVar:
        scr = scr[0]
    try:
        py_script = str(scr)[10:]
        py_script = py_script[:py_script.find(' ')]
        locals = dict(args=args)
        exec py_script+'(*args)' in globals(),locals
    except py_GameEndException:
        raise sys.exc_type,sys.exc_value,sys.exc_traceback
    except:
        print "1054: script_execute: Error executing",scr,args
        for script in py_gm6_game.scripts:
            if script is not None and script.name==py_script:
                pyth = script.python(py_gm6_game)
                if len(pyth)>4096:
                    pyth = pyth[:4096]+'<TRUNCATED>'
                print '[[[',pyth,']]]'
        raise sys.exc_type,sys.exc_value,sys.exc_traceback


def show_message(s):
    if type(__message_props.background) is py_fasttint.BackgroundData:
        bg = __message_props.background.surf
    else:
        print "Debug: __message_props.background is not a BackgroundData object.",type(__message_props.background)
        bg = None
    sw,sh = __pygame_screen.get_size()
    mcorn = ((sw>>1)-(__message_props.w>>1),((sh>>1)-(__message_props.h>>1)))
    if bg:
        __pygame_screen.blit(bg,mcorn)
    ofont = py_draw_props.font
    ocolor = py_draw_props.color
    py_draw_props.font = gm6.debugger_font
    surfaces = __render_text(s,__message_props.w-30)
    ok = []
    if __message_props.button_font:
        py_draw_props.color = __message_props.button_font.color
    else:
        py_draw_props.color = (0,0,0)
    ok.append(__render_text('OK',100)[0])
    py_draw_props.color = __message_props.button_font_hover_color
    if py_draw_props.color is None:
        py_draw_props.color = (0,0,0)
    ok.append(__render_text('OK',100)[0])
    ok.append(ok[-1])
    py_draw_props.font = ofont
    py_draw_props.color = ocolor
    y = mcorn[1]+15
    for surf in surfaces:
        __pygame_screen.blit(surf,(mcorn[0]+15,y))
        y += surf.get_height()
    notdone = True
    if __message_props.button_spr:
        buttsurf = __message_props.button_spr.subimages[0]
    else:
        buttsurf = (pygame.surface.Surface((50,25)),
            pygame.surface.Surface((50,25)), pygame.surface.Surface((50,25)) )
        buttsurf[0].fill((200,200,200))
        buttsurf[1].fill((150,150,150))
        buttsurf[1].fill((220,220,220))
    buttsize = buttsurf[0].get_size()
    if bg:
        buttymax = mcorn[1]+__message_props.background.surf.get_height()-15
    else:
        buttymax = mcorn[1]+__message_props.h-15
    buttymin = buttymax-buttsize[1]
    buttxmin = (sw>>1)-(buttsize[0]>>1)
    buttxmax = buttxmin+buttsize[0]
    lastbuttframe = -1
    okpos = (buttxmin+(buttsize[0]>>1)-(ok[0].get_width()>>1),
        buttymin+(buttsize[1]>>1)-(ok[0].get_height()>>1))
    io_clear()
    while notdone:
        io_handle()
        mx = mouse_x - room.view_xview[0]
        my = mouse_y - room.view_yview[0]
        if mx<buttxmin or mx>buttxmax or my<buttymin or my>buttymax:
            currbuttframe = 0
        elif mouse_button & mb_any:
            currbuttframe = 2
        else:
            currbuttframe = 1
        if currbuttframe!=lastbuttframe:
            if lastbuttframe==2:
                io_clear()
                return
            lastbuttframe = currbuttframe
            __pygame_screen.blit(buttsurf[currbuttframe],(buttxmin,buttymin))
            __pygame_screen.blit(ok[currbuttframe],okpos)
            pygame.display.flip()

def sleep(dur):
    pygame.time.delay(int(dur))

def sound_loop(snd):
    if type(snd) in (int, float):
        snd = gm6.game.sounds[int(snd)]
    snd.Loop()

def sound_play(snd):
    if type(snd) is py_GMVar:
        snd = snd[0]
    if type(snd) in (int, float):
        snd = gm6.game.sounds[int(snd)]
    snd.Play()

def sound_stop(snd):
    if type(snd) in (int, float):
        snd = gm6.game.sounds[int(snd)]
    snd.Stop()

def sound_stop_all():
    if gm6.use_renpy:
        for chan in ('music','m1','m2','m3','m4','m5','m6','m7'):
            renpy.exports.music.stop(channel=chan)
            renpy.exports.sound.stop()
        return
    else:
        pygame.mixer.stop()

def sprite_add(filename, frames, precise, transparent, smooth, preload, xorig, yorig):
    if type(filename) is py_GMVar:
        filename = filename[0]
    try:
        surf = renpy_glue.readsurf(filename)
    except:
        print "Error loading",filename
        raise sys.exc_type,sys.exc_value,sys.exc_traceback
    ret = py_fasttint.GMSprite()
    ret.subimages = [surf,]
    ret.name='sprite:'+filename
    gm6.game.sprites.append(ret)
    return ret

def sprite_create_from_screen(x,y,w,h,precise,transparent,smooth,preload,xorig,yorig):
    ret = py_fasttint.GMSprite();
    #SpriteData(__pygame_screen.subsurface((x,y,w,h)).convert(),name="Screencap")
    ret.subimages = [ __pygame_screen.subsurface((x,y,w,h)).convert(), ]
    ret.w, ret.h = ret.subimages[0].get_size()
    gm6.game.sprites.append(ret)
    return ret
def sprite_delete(spr):
    while len(spr.subimages):
        spr.subimages.pop()
    #spr.subimages = []
    idx = 0
    while idx<len(gm6.game.sprites):
        if gm6.game.sprites[idx] is spr:
            del gm6.game.sprites[idx]
        else:
            idx += 1
def sprite_exists(spr):
    if type(spr) is py_GMVar:
        return type(spr[0]) is py_SpriteData
    return type(spr) is py_SpriteData
def sprite_get_bbox_bottom(spr):
    return spr.bottombound
def sprite_get_bbox_left(spr):
    return spr.leftbound
def sprite_get_bbox_mode(spr):
    return spr.bounding_box_type
def sprite_get_bbox_right(spr):
    return spr.rightbound
def sprite_get_bbox_top(spr):
    return spr.topbound
def sprite_get_height(spr):
    return spr.h
def sprite_get_name(spr):
    return spr.name
def sprite_get_number(spr):
    return len(spr.subimages)
def sprite_get_precice(spr):
    return spr.precise_collision_checking
def sprite_get_preload(spr):
    return spr.preload
def sprite_get_smooth(spr):
    return spr.smooth
# XXX sprite_get_texture - in Advanced Drawing Functions.
def sprite_get_transparent(spr):
    return spr.transparent
def sprite_get_width(spr):
    return spr.w
def sprite_get_xoffset(spr):
    # XXX guessing that this is really xorigin
    return spr.xorigin
def sprite_get_yoffset(spr):
    # XXX also guessed.
    return spr.yorigin
def __subimage_set_alpha_SURFARRAY(dest,src):
    # Assume that no alpha conversion is necessary.
    target_alpha = pygame.surfarray.pixels_alpha(dest)
    a = 0
    if src.get_bitsize()==8:
        print "Setting alpha for surface to 8-bit - bypassed."
        mask_vals = pygame.surfarray.pixels2d(src)
        bbound = len(mask_vals[0])
        abound = len(mask_vals)
        while a<abound:
            b = 0
            while b<bbound:
                target_alpha[a][b] = mask_vals[a][b]
                b += 1
            a += 1
        return
    mask_vals = pygame.surfarray.pixels3d(src)
    bbound = len(mask_vals[0])
    abound = len(mask_vals)
    while a<abound:
        b = 0
        while b<bbound:
            target_alpha[a][b] = mask_vals[a][b][1]
            b += 1
        a += 1

def __surf_apply_alpha(target, mask):
    if gm6.use_renpy:
        return renpy.display.im.cache.get(
            renpy.display.im.AlphaMask(
                renpy_glue.SurfImage(target),
                renpy_glue.SurfImage(mask)))
    try:
        # Assume that no alpha conversion is necessary.
        target_alpha = pygame.surfarray.pixels_alpha(target)
    except:
        target = target.convert_alpha()
        target_alpha = pygame.surfarray.pixels_alpha(target)
    a = 0
    if mask.get_bitsize()==8:
        print "Setting alpha for surface to 8-bit - bypassed."
        mask_vals = pygame.surfarray.pixels2d(src)
        bbound = len(mask_vals[0])
        abound = len(mask_vals)
        while a<abound:
            b = 0
            while b<bbound:
                target_alpha[a][b] = mask_vals[a][b]
                b += 1
            a += 1
        return target
    mask_vals = pygame.surfarray.pixels3d(mask)
    bbound = len(mask_vals[0])
    abound = len(mask_vals)
    while a<abound:
        b = 0
        while b<bbound:
            target_alpha[a][b] = mask_vals[a][b][1]
            b += 1
        a += 1
    return target

def sprite_set_alpha_from_sprite(target,mask):
    x = 0
    y = 0
    while x<len(target.subimages):
        target.subimages[x] = __surf_apply_alpha(target.subimages[x],
            mask.subimages[y])
        x += 1
        y += 1
        if y>=len(mask.subimages):
            y = 0

# string functions
def string(n):
    if type(n) in (py_GMVar, py_PythonAutovivifyingZeroArray, py_PythonAutovivifyingArray, list):
        n = n[0]
    if type(n)==float:
        if int(n)==n:
            return str(int(n))
    return str(n)
# string_char_at(s,x) hardcoded into s[int(x)] - XXX 1-based or 0-based

def string_char_at(s,idx):
    if type(s) is py_GMVar:
        s = s[0]
    ret = s[int(idx)-1]
    return ret
def string_copy(s,idx,count):
    if type(s) is py_GMVar:
        s = s[0]
    idx = int(idx)
    if idx>0:
        idx -= 1
    ret = s[idx:idx+int(count)]
    return ret
def string_count(tok,s):
    if type(s) is py_GMVar:
        s = s[0]
    if type(tok) is py_GMVar:
        tok = tok[0]
    return py_string.count(s,tok);
#string_count = py_string.count
def string_delete(s,idx,count):
    if type(s) is py_GMVar:
        s = s[0]
    idx = int(idx)-1
    ret = s[:idx]+s[idx+int(count):]
    return ret
def string_format(n,tot,dec):
    # The docs say that 'tot' is the total number of places and 'dec' the
    # number of decimal places.  In practice, at least on GM5, 'tot' appears
    # to be ignored entirely and only dec honored.
    num = round(n,int(dec))
    if dec:
        return str(num)
    else:
        return str(int(num))
def string_height(s):
    if type(s) is py_GMVar:
        s = s[0]
    s2 = __string_unescape(s)
    totalheight = 0
    for sub in s2:
        totalheight += __fontcache[py_draw_props.font].size(sub)[1]
    return totalheight
def string_height_ext(s, sep, w):
    if type(s) is py_GMVar:
        s = s[0]
    # XXX what does string_height_ext(0,sep,w) do?  Is it different if undef
    # vars are not 0?  Is it different for undef vars and real zero?
    if s==0:
        return 0
    if w==-1:
        w = 1000000000
    surfaces = __render_text(s,w)
    total_height = 0
    if sep==-1:
        for surf in surfaces:
            h = surf.get_height()
            total_height += h
        return total_height
    return len(surfaces)*sep
    
def string_insert(substr,s,idx):
    if type(substr) is py_GMVar:
        substr = substr[0]
    if type(s) is py_GMVar:
        s = s[0]
    idx = int(idx)-1
    ret = s[:idx]+substr+s[idx:]
    return ret
def string_length(s):
    if type(s) is py_GMVar:
        return len(s[0])
    return len(s)
#string_length = len
def string_lettersdigits(s):
    s2 = ''
    if type(s) is py_GMVar:
        s = s[0]
    for c in s:
        if c in 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789':
            s2 += c
    return s2
def string_lower(s):
    return s.lower()
def string_pos(sub, s):
    if type(s) is py_GMVar:
        s = s[0]
    ret = s.find(sub)+1
    return ret
def string_repeat(s,n):
    if type(s) is py_GMVar:
        s = s[0]
    return s*n
def string_replace(s,substr,newstr):
    if type(s) is py_GMVar:
        s = s[0]
    if type(substr) is py_GMVar:
        substr = substr[0]
    if s.find(substr)<0:
        return s
    if type(newstr) is py_GMVar:
        newstr = newstr[0]
    return s.replace(substr,newstr,1)
def string_replace_all(s,substr,newstr):
    if type(s) is py_GMVar:
        s = s[0]
    if type(substr) is py_GMVar:
        substr = substr[0]
    if s.find(substr)<0:
        return s
    if type(newstr) is py_GMVar:
        newstr = newstr[0]
    return s.replace(substr,newstr)
def string_width(s):
    if type(s) is py_GMVar:
        s = s[0]
    s2 = __string_unescape(s)
    widest = 0
    for sub in s2:
        widest = max(widest,__fontcache[py_draw_props.font].size(sub)[0])
    return widest

# XXX texture_get_width - in Advanced Drawing Functions
# XXX texture_get_height - also in Advanced Drawing Functions.
# XXX texture_preload(texture_id): put texture in video memory.
# XXX texture_set_blending
def texture_set_interpolation(linear):
    py_draw_props.interpolate_textures = linear
# XXX texture_set_repeat
# XXX texture_set_priority

def variable_local_array_set(varname, i, val):
    # XXX this probably should go.
    global self
    arr = self.__dict__.get(varname, None)
    if arr is None:
        self.__dict__[varname] = arr = py_GMVar(0)
    elif type(arr) is not py_GMVar:
        print arr,type(arr)
        raise "Invalid type: expected a GMVar."
    #if not type(arr) in (list, py_GMVar):
    #    arr = self.__dict__[varname] = py_GMVar(0)
    #while len(arr.list)<i:
    #    arr.list.append(0)
    #if len(arr.list)==i:
    #    arr.list.append(val)
    #else:
    arr[int(i)] = val

def variable_local_exists(varname):
    # This doesn't check for local variables; rather, it checks to see if
    # this variable exists on self.  (Or, presumably, the parent object.)
    return (varname in self.__dict__) or (varname in self.object_index.__dict__)

def window_default():
    # "XXX Warning should center window."
    pass

def window_get_fullscreen():
    return py_is_fullscreen
# window_set_fullscreen imported from rungm6

# effect constants created at the end, because they need all the part_type definitions.

ef_explosion = 0
ef_ring = 1
ef_ellipse = 2
ef_firework = 3
ef_smoke = 4
ef_smokeup = 5
ef_star = part_type_create()
part_type_sprite(ef_star,pt_shape_star,0,0,0)
part_type_size(ef_star,0.0,0.2,-0.05,0)
part_type_orientation(ef_star,0,71,0,0,0)
part_type_life(ef_star,15,30)
ef_spark = part_type_create()
part_type_sprite(ef_spark,pt_shape_spark,0,0,0)
part_type_alpha2(ef_spark,1.0,0.01)
part_type_size(ef_spark,0.0,0.5,-0.01,0)
part_type_life(ef_spark,10,40)
#part_type_blend(ef_spark,bm_add)
ef_flare = 8
ef_cloud = part_type_create()
py_ef_cloud_beancounter = py_fasttint.Beancounter()
part_type_alpha3(ef_cloud,0.3,1.0,0.3)
part_type_blend(ef_cloud,bm_add)
part_type_sprite(ef_cloud, pt_shape_cloud,0,0,0)
part_type_life(ef_cloud,70,70)
ef_rain = 10
ef_snow = part_type_create()
part_type_sprite(ef_snow, pt_shape_snow, 0,0,0)
