try:
    import renpy
    use_renpy = True
except:
    import posix
    import pwd
    use_renpy = False
import pygame
import os

if use_renpy:
    class SurfImage(renpy.display.im.ImageBase):
        def __init__(self, surf):
            renpy.display.im.ImageBase.__init__(self,surf)
            self.surf = surf.convert_alpha()
        def load(self,unscaled=False):
            return self.surf

if use_renpy:
    write_directory = renpy.exports.config.savedir+'/'
    read_directories = [
        write_directory, renpy.exports.config.searchpath[0]+'/'
        ]
else:
    write_directory = pwd.getpwuid(posix.getuid())[5]+'/.renpy/SpikyCaterpillarGMRuntimeGame/'
    read_directories = [
        write_directory,
        os.getcwd(),
        ]
    if not read_directories[0].endswith('/'):
        read_directories[0]+='/'
    if not read_directories[1].endswith('/'):
        read_directories[1]+='/'
    print "Not using Ren'Py; set read directories to",read_directories,'and write to',write_directory

def realname(fn):
    if fn.startswith('/'):
        return fn
    try:
        file(write_directory+fn,'r').close()
        return write_directory+fn
    except:
        for d in read_directories:
            try:
                file(d+fn,'r').close()
                return d+fn
            except:
                pass
    return fn

def mkpath(fn):
    if not fn.startswith('/'):
        fn = write_directory+fn
    path = fn.rsplit('/',1)
    try:
        os.makedirs(path[0])
    except:
        pass

def open_r(fn):
    if fn.startswith('/'):
        try:
            return file(fn,'rb')
        except:
            return None
    for d in read_directories:
        try:
            f = file(d+fn,'rb')
            return f
        except:
            pass
    return None

def open_a(fn):
    if not fn.startswith('/'):
        fn = write_directory+fn
    mkpath(fn)
    return file(fn,'ab')

def open_w(fn):
    if not fn.startswith('/'):
        fn = write_directory+fn
    mkpath(fn)
    return file(fn,'wb')


def open_rw(fn):
    if fn.startswith('/'):
        return file(fn)
    try:
        f = file(write_directory+fn,'rb+')
        return f
    except:
        for d in read_directories[1:]:
            try:
                f = file(d+fn,'rb')
                data = f.readlines()
                f.close()
                mkpath(fn)
                f = file(write_directory+fn,'wb+')
                f.writelines(data)
                f.seek(0,0)
                return f
            except:
                pass

def readsurf(fn,transparent=True,loud=True):
    if fn.startswith('/'):
        return  pygame.image.load(fn)
    for d in read_directories:
        ret = None
        try:
            ret = pygame.image.load(d+fn)
            break
        except:
            pass
    if not ret:
        if loud:
            print "Unable to load",fn
        return None
    #if transparent:
    #    return ret.convert_alpha()
    return ret.convert()

#if use_renpy:
#    listdir = os.listdir
#else:
def listdir(path):
    if path.startswith('./'):
        ret = os.listdir(write_directory+path)
        return ret
    if not path.startswith('/'):
        ret = os.listdir(read_directories[0]+path)
        return ret
    else:
        return os.listdir(path)
if use_renpy:
    import renpy.exports.ui as ui
def get_save_filename(files, defaultname):
  notdone = True
  try:
    while notdone:
        #ui.clear()
        #ui.fixed(background='#000d',xminimum=800,yminimum=600)
        renpy.exports.free_memory()
        ui.window(background=renpy.store.Frame('out/popupback.png',5,5))
        ui.vbox()
        ui.text("Save as:")
        for fn in files:
            ui.textbutton(fn,clicked=ui.returns(('set',fn)))
        inp = ui.input(default=defaultname)
        ui.hbox()
        ui.textbutton('Cancel',ui.returns(('cancel',)))
        ui.textbutton('Save',ui.returns(('save',)))
        ui.close()
        ui.close()
        #ui.close()
        renpy.exports.restart_interaction()
        pygame.display.flip()
        ret = ui.interact()
        if type(ret) is tuple:
            if ret[0]=='cancel':
                return ''
            elif ret[0]=='save':
                return inp.text[1]
            elif ret[0]=='set':
                defaultname = ret[1]
  except:
    return ''

if not use_renpy:
    def get_save_filename(files,defaultname):
        return defaultname

def unlink(fn):
    try:
        os.unlink(write_directory+fn)
    except:
        print "Warning: unable to delete file",fn,"- this may indicate a bug in your GML."
