# SuperSound compatibility
import pygame
import time
import gmapi
try:
    import renpy.exports as renpy
    use_renpy = True
except:
    use_renpy = False

renpy_glue = gmapi.renpy_glue

used_channels = dict()
def looponchan(fn,chan):
    fn = renpy_glue.realname(fn)
    used_channels[chan] = fn
    renpy.music.set_volume(1.0,channel=chan)
    renpy.music.play(fn,channel=chan)
def playonchan(fn,chan):
    fn = renpy_glue.realname(fn)
    used_channels[chan] = fn
    renpy.music.set_volume(1.0,channel=chan)
    renpy.sound.play(fn,channel=chan)
    

class SS_Compat_Object(object):
    volume = 0
    def __eq__(self,other):
        return self is other
    def __ne__(self,other):
        return self is not other
    def __init__(self, filename):
        self.filename = filename
        self.realname = renpy_glue.realname(filename)
        self.snd = None
    def SetSoundVol(self, vol):
        self.volume = vol/10000.0
        #realvol = max(6*(self.volume-0.85),0)
        #realvol = max(2.5*(self.volume-0.6),0)
        #realvol = max(4*(self.volume-0.75),0) - still too abrupt
        #realvol = max(6.6666666*(self.volume-0.85),0) - combat comes in too slow
        realvol = max(5*(self.volume-0.8),0)
        if not use_renpy:
            if self.snd is not None:
                self.snd.set_volume(realvol)
        else:
            for channel in used_channels:
		if used_channels[channel] == self.realname:
                    renpy.music.set_volume(max(realvol,0),channel=channel)
    def Load(snd):
        if snd.snd is None:
            snd.snd = pygame.mixer.Sound(snd.realname)

def SS_GetSoundLength(*args):
    # XXX should implement
    return 0
def SS_GetSoundPosition(*args):
    # XXX should implement
    return 0
def SS_GetSoundVol(handle):
    return handle.volume*10000
def SS_Init(*args):
    pass
def SS_IsHandleValid(handle):
    if type(handle) is gmapi.py_fasttint.GMVar:
        handle = handle[0]
    if type(handle) is SS_Compat_Object:
        return True
    return False
def SS_LoopSound(snd):
    if use_renpy:
	looponchan(snd.filename,available_channel())
        return
    snd.Load()
    snd.snd.play(loops=-1)
def SS_PlaySound(snd):
    if use_renpy:
        chan = available_channel()
        playonchan(snd.filename,chan)
        return
    snd.Load()
    snd.snd.play(loops=0)
def SS_SetSoundVol(snd, vol):
    if snd==-1:
        return
    snd.SetSoundVol(vol)
def SS_StopSound(snd):
    if use_renpy:
        nuke = []
        for c in used_channels:
	    if used_channels[c] == snd.realname:
                renpy.music.stop(channel=c)
                nuke.append(c)
        for c in nuke:
            del used_channels[c]
        return
    elif snd.snd is not None:
        snd.snd.stop()
    else:
        print "ERROR: Tried to stop nonexistent sound",snd.filename

def add_refs(d):
    d['SS_GetSoundLength'] = SS_GetSoundLength
    d['SS_GetSoundPosition'] = SS_GetSoundPosition
    d['SS_GetSoundVol'] = SS_GetSoundVol
    d['SS_Init'] = SS_Init
    d['SS_IsHandleValid'] = SS_IsHandleValid
    d['SS_LoadSound'] = SS_Compat_Object
    d['SS_LoopSound'] = SS_LoopSound
    d['SS_PlaySound'] = SS_PlaySound
    d['SS_SetSoundVol'] = SS_SetSoundVol
    d['SS_StopSound'] = SS_StopSound

def available_channel():
    for c in channels:
	if not c in used_channels:
	    return c
    for c in used_channels:
        if used_channels[c] is None:
            del used_channels[c]
            return c
    for c in used_channels:
        if renpy.exports.music.get_playing(c) is None:
            del used_channels[c]
            return c
    return channels[0]
    #channels.append('c'+str(len(channels)))
    #renpy.music.register_channel(channels[-1], 'music', None)

if use_renpy:
    channels = ['music','m1','m2','m3','m4','m5','m6','m7']
